@@ -64,8 +64,13 @@
 #define LUTf LUT<float>
 #define LUTi LUT<int>
 #define LUTu LUT<unsigned int>
+#define LUTd LUT<double>
 
 #include <cstring>
+#ifndef NDEBUG
+#include <glibmm.h>
+#include <fstream>
+#endif
 
 template<typename T>
 class LUT {
@@ -92,7 +97,8 @@ public:
 		maxs=size-2;
 	}
 
-	LUT(int s, T * source) {
+	LUT(int s, T * source, int flags = 0xfffffff) {
+		clip = flags;
 		data = new T[s];
 		owner = 1;
 		size = s;
@@ -104,9 +110,7 @@ public:
 
 	LUT(void) {
 		data = NULL;
-		owner = 1;
-		size = 0;
-		maxs=0;
+		reset();
 	}
 
 	~LUT() {
@@ -114,6 +118,10 @@ public:
 			delete[] data;
 	}
 
+	void setClip(int flags) {
+		clip = flags;
+	}
+
 	LUT<T> & operator=(const LUT<T> &rhs) {
 	    if (this != &rhs) {
 	      if (rhs.size>this->size)
@@ -132,7 +140,7 @@ public:
 	    return *this;
 	  }
 	// use with integer indices
-	T& operator[](int index) {
+	T& operator[](int index) const {
 		if (((unsigned int)index)<size) return data[index];
 		else
 		{
@@ -144,7 +152,7 @@ public:
 		
 	}
 	// use with float indices
-	T operator[](float index) {
+	T operator[](float index) const {
 		int idx = (int)index;  // don't use floor! The difference in negative space is no problems here
 		if (((unsigned int)idx) > maxs) {
 			if (idx<0)
@@ -165,8 +173,26 @@ public:
 		T p2 = data[idx + 1]-p1;
 		return (p1 + p2*diff);
 	}
+	
+	
+#ifndef NDEBUG
+	// Debug facility ; dump the content of the LUT in a file. No control of the filename is done
+	void dump(Glib::ustring fname) {
+		if (size) {
+		    Glib::ustring fname_ = fname + ".xyz"; // TopSolid'Design "plot" file format
+			std::ofstream f (fname_.c_str());
+			f << "$" << std::endl;;
+			for (unsigned int iter=0; iter<size; iter++) {
+				f << iter << ", " << data[iter] << ", 0." << std::endl;;
+			}
+			f << "$" << std::endl;;
+			f.close ();
+		}
+	}
+#endif
 
-	operator bool (void)
+
+	operator bool (void) const
 		{
 			return size>0;
 		}
@@ -174,6 +200,14 @@ public:
 	void clear(void) {
 		memset(data, 0, size * sizeof(T));
 	}
+
+	void reset(void) {
+		delete[] data;
+		data = NULL;
+		owner = 1;
+		size = 0;
+		maxs=0;
+    }
 };
 
 #endif /* LUT_H_ */
