@@ -37,12 +37,7 @@ using namespace std;
 
 namespace rtengine {
 
-	Curve::Curve () {
-		x = 0;
-		y = 0;
-		ypp = 0;
-		hashSize = 1000;  // has to be initiallised to the maximum value
-	}
+	Curve::Curve () : N(0), x(NULL), y(NULL), ypp(NULL), hashSize(1000 /* has to be initialized to the maximum value */ ) {}
 	
 	void Curve::AddPolygons ()
 	{
@@ -153,86 +148,181 @@ namespace rtengine {
 		}
 	}
 
+void CurveFactory::updatechroma (
+		const std::vector<double>& cccurvePoints,
+		LUTu & histogramC, LUTu & histogramCroppedC, LUTu & outBeforeCCurveHistogramC,//for chroma
+		int skip)
+{
+	LUTf dCcurve(65536,0);
+	float val;
+	for (int i=0; i<48000; i++) {//32768*1.414  + ...
+			val = (double)i / 47999.0;
+			dCcurve[i] = CLIPD(val);
+		}
+
+	outBeforeCCurveHistogramC.clear();
+	bool histNeededC = false;
+	
+
+	if (!cccurvePoints.empty() && cccurvePoints[0]!=0) {
+		if (outBeforeCCurveHistogramC /*&& histogramCropped*/)
+				histNeededC = true;	
+	}
+	for (int i=0; i<=48000; i++) {//32768*1.414  + ...
+		float val;
+			if (histNeededC) {
+				float hval = dCcurve[i];
+				int hi = (int)(255.0*CLIPD(hval)); //
+				outBeforeCCurveHistogramC[hi] += histogramC[i] ;
+			}
+	}
+}
+	
+	
+void CurveFactory::curveLightBrightColor (
+		procparams::ColorAppearanceParams::eTCModeId curveMode1, const std::vector<double>& curvePoints1,
+		procparams::ColorAppearanceParams::eTCModeId curveMode2, const std::vector<double>& curvePoints2,
+		procparams::ColorAppearanceParams::eCTCModeId curveMode3, const std::vector<double>& curvePoints3,
+		LUTu & histogram, LUTu & histogramCropped, LUTu & outBeforeCCurveHistogram,//for Luminance  
+		LUTu & histogramC, LUTu & histogramCroppedC, LUTu & outBeforeCCurveHistogramC,//for chroma
+		ColorAppearance & customColCurve1,
+		ColorAppearance & customColCurve2,
+		ColorAppearance & customColCurve3,
+		int skip)
+{
+	LUTf dcurve(65536,0);
+	LUTf dCcurve(65536,0);
+	
+	float val;
+	for (int i=0; i<32768; i++) {
+			val = (double)i / 32767.0;
+			dcurve[i] = CLIPD(val);
+		}
+	for (int i=0; i<48000; i++) {  //# 32768*1.414  approximation maxi for chroma
+			val = (double)i / 47999.0;
+			dCcurve[i] = CLIPD(val);
+		}
+
+	outBeforeCCurveHistogram.clear();
+	outBeforeCCurveHistogramC.clear();
+	bool histNeededC = false;
+	
+	bool histNeeded = false;
+	DiagonalCurve* tcurve = NULL;
+	customColCurve3.Reset();
+
+	if (!curvePoints3.empty() && curvePoints3[0]>DCT_Linear && curvePoints3[0]<DCT_Unchanged) {
+		tcurve = new DiagonalCurve (curvePoints3, CURVES_MIN_POLY_POINTS/skip);
+		if (outBeforeCCurveHistogramC /*&& histogramCropped*/)
+				histNeededC = true;
+		
+	}
+	if (tcurve) {
+		if (tcurve->isIdentity()) {
+			delete tcurve;
+			tcurve = NULL;
+		}
+		else
+			customColCurve3.Set(tcurve);
+		delete tcurve;
+		tcurve = NULL;
+	}
+
+	customColCurve2.Reset();
+
+	if (!curvePoints2.empty() && curvePoints2[0]>DCT_Linear && curvePoints2[0]<DCT_Unchanged) {
+		tcurve = new DiagonalCurve (curvePoints2, CURVES_MIN_POLY_POINTS/skip);
+		if (outBeforeCCurveHistogram /*&& histogramCropped*/)
+				histNeeded = true;
+		
+	}
+	if (tcurve) {
+		if (tcurve->isIdentity()) {
+			delete tcurve;
+			tcurve = NULL;
+		}
+		else
+			customColCurve2.Set(tcurve);
+		delete tcurve;
+		tcurve = NULL;
+	}
+	// create first curve if needed
+	customColCurve1.Reset();
+
+	if (!curvePoints1.empty() && curvePoints1[0]>DCT_Linear && curvePoints1[0]<DCT_Unchanged) {
+		tcurve = new DiagonalCurve (curvePoints1, CURVES_MIN_POLY_POINTS/skip);
+		if (outBeforeCCurveHistogram /*&& histogramCropped*/)
+				histNeeded = true;
+		
+	}
+	if (tcurve) {
+		if (tcurve->isIdentity()) {
+			delete tcurve;
+			tcurve = NULL;
+		}
+		else  {
+			customColCurve1.Set(tcurve);
+			delete tcurve;
+			tcurve = NULL;
+		}
+	}
+	for (int i=0; i<=32768; i++) {
+		float val;
+
+			if (histNeeded) {
+				float hval = dcurve[i];
+				int hi = (int)(255.0*CLIPD(hval));
+				outBeforeCCurveHistogram[hi] += histogram[i] ;
+			}
+	}
+	for (int i=0; i<=48000; i++) {//32768*1.414  + ...
+		float val;
+			if (histNeededC) {
+				float hval = dCcurve[i];
+				int hi = (int)(255.0*CLIPD(hval)); //
+				outBeforeCCurveHistogramC[hi] += histogramC[i] ;
+			}
+	}
+	
+	if (tcurve) delete tcurve;
+
+}
+
 	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-	void CurveFactory::complexsgnCurve (double saturation, bool satlimit, double satlimthresh,
-										const std::vector<double>& acurvePoints, const std::vector<double>& bcurvePoints,
-										LUTf & aoutCurve, LUTf & boutCurve, LUTf & satCurve, int skip) {
+	void CurveFactory::complexsgnCurve ( bool & autili,  bool & butili, bool & ccutili, bool & cclutili, double saturation, double rstprotection,
+										const std::vector<double>& acurvePoints, const std::vector<double>& bcurvePoints,const std::vector<double>& cccurvePoints,
+										const std::vector<double>& lccurvePoints, LUTf & aoutCurve, LUTf & boutCurve, LUTf & satCurve, LUTf & lhskCurve,
+										LUTu & histogramC, LUTu & histogramCroppedC, LUTu & outBeforeCCurveHistogram,//for chroma
+										int skip) {
 		
-		//colormult = chroma_scale for Lab manipulations
 		
 		//-----------------------------------------------------
 
 		bool needed;
 		DiagonalCurve* dCurve = NULL;
-
-		// check if contrast curve is needed
-		needed = (saturation<-0.0001 || saturation>0.0001);
-
-		// Filling the curve if needed
-		if (needed) {
-
-			//%%%%%%%%%%%%%%%%% Saturation curve's control points %%%%%%%%%%%%%%%%%
-		    std::vector<double> satcurvePoints;
-			satcurvePoints.push_back((double)DCT_NURBS);
-			if (saturation>0) {
-				double satslope = (0.5+2*saturation/500.0)/(0.5-2*saturation/500.0);
-				double scale = (satlimthresh/100.1);
-				if (!satlimit) scale=100/100.1;
-
-				satcurvePoints.push_back(0); //black point.  Value in [0 ; 1] range
-				satcurvePoints.push_back(0); //black point.  Value in [0 ; 1] range
-
-				//if (satlimit) {
-				satcurvePoints.push_back(0.5-0.5*scale); //toe point
-				satcurvePoints.push_back(0.5-0.5*scale); //value at toe point
-
-				satcurvePoints.push_back(0.5-(0.5/satslope)*scale); //toe point
-				satcurvePoints.push_back(0.5-0.5*scale); //value at toe point
-
-				satcurvePoints.push_back(0.5+(0.5/satslope)*scale); //shoulder point
-				satcurvePoints.push_back(0.5+0.5*scale); //value at shoulder point
-
-				satcurvePoints.push_back(0.5+0.5*scale); //shoulder point
-				satcurvePoints.push_back(0.5+0.5*scale); //value at shoulder point
-				/*} else {
-				 satcurvePoints.push_back(0.25+saturation/500.0); //toe point
-				 satcurvePoints.push_back(0.25-saturation/500.0); //value at toe point
-
-				 satcurvePoints.push_back(0.75-saturation/500.0); //shoulder point
-				 satcurvePoints.push_back(0.75+saturation/500.0); //value at shoulder point
-				 }*/
-
-				satcurvePoints.push_back(1); // white point
-				satcurvePoints.push_back(1); // value at white point
-			} else {
-				satcurvePoints.push_back(0);
-				satcurvePoints.push_back(-(saturation/200.0));
-
-				satcurvePoints.push_back(1);
-				satcurvePoints.push_back(1+saturation/200.0);
-			}
-			dCurve = new DiagonalCurve (satcurvePoints, CURVES_MIN_POLY_POINTS/skip);
-			//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
-			fillCurveArray(dCurve, satCurve, skip, needed);
-
-			delete dCurve;
-			dCurve = NULL;
+		LUTf dCcurve(65536,0);
+	
+		float val;
+		for (int i=0; i<48000; i++) {  //# 32768*1.414  approximation maxi for chroma
+				dCcurve[i] = (float)i / 47999.0;
 		}
-		else {
-			fillCurveArray(NULL, satCurve, skip, needed);
-		}
-
+		if (outBeforeCCurveHistogram)		
+			outBeforeCCurveHistogram.clear();
+		bool histNeededC = false;
 		//-----------------------------------------------------
 
 		needed = false;
 		// create a curve if needed
 		if (!acurvePoints.empty() && acurvePoints[0]!=0) {
 			dCurve = new DiagonalCurve (acurvePoints, CURVES_MIN_POLY_POINTS/skip);
-			if (dCurve && !dCurve->isIdentity())
+			if (dCurve && !dCurve->isIdentity()) {
 				needed = true;
+				autili=true;
+			}
 		}
 		fillCurveArray(dCurve, aoutCurve, skip, needed);
+		//if(autili) aoutCurve.dump("acurve");
+
 		if (dCurve) {
 			delete dCurve;
 			dCurve = NULL;
@@ -243,14 +333,55 @@ namespace rtengine {
 		needed = false;
 		if (!bcurvePoints.empty() && bcurvePoints[0]!=0) {
 			dCurve = new DiagonalCurve (bcurvePoints, CURVES_MIN_POLY_POINTS/skip);
-			if (dCurve && !dCurve->isIdentity())
+			if (dCurve && !dCurve->isIdentity()) {
 				needed = true;
+				butili=true;
+			}
 		}
 		fillCurveArray(dCurve, boutCurve, skip, needed);
 		if (dCurve) {
 			delete dCurve;
 			dCurve = NULL;
 		}
+		
+		//-----------------------------------------------
+		needed = false;
+		if (!cccurvePoints.empty() && cccurvePoints[0]!=0) {
+			dCurve = new DiagonalCurve (cccurvePoints, CURVES_MIN_POLY_POINTS/skip);
+			if (outBeforeCCurveHistogram /*&& histogramCropped*/)
+				histNeededC = true;
+			
+			if (dCurve && !dCurve->isIdentity())
+				{needed = true;ccutili=true;}
+		}
+		for (int i=0; i<=48000; i++) {//32768*1.414  + ...
+			float val;
+			if (histNeededC) {
+				float hval = dCcurve[i];
+				int hi = (int)(255.0*CLIPD(hval)); //
+				outBeforeCCurveHistogram[hi] += histogramC[i] ;
+			}
+		}
+		
+		fillCurveArray(dCurve, satCurve, skip, needed);
+		if (dCurve) {
+			delete dCurve;
+			dCurve = NULL;
+		}
+		//----------------------------
+		needed = false;
+		if (!lccurvePoints.empty() && lccurvePoints[0]!=0) {
+			dCurve = new DiagonalCurve (lccurvePoints, CURVES_MIN_POLY_POINTS/skip);
+			if (dCurve && !dCurve->isIdentity())
+				{needed = true;cclutili=true;}
+		}
+		fillCurveArray(dCurve, lhskCurve, skip, needed);
+		if (dCurve) {
+			delete dCurve;
+			dCurve = NULL;
+		}
+		
+		
 	}
 
 	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -259,9 +390,14 @@ namespace rtengine {
 
 	void CurveFactory::complexCurve (double ecomp, double black, double hlcompr, double hlcomprthresh,
 									 double shcompr, double br, double contr, double gamma_, bool igamma_,
-									 const std::vector<double>& curvePoints, LUTu & histogram, LUTu & histogramCropped,
+									 procparams::ToneCurveParams::eTCModeId curveMode, const std::vector<double>& curvePoints,
+									 procparams::ToneCurveParams::eTCModeId curveMode2, const std::vector<double>& curvePoints2,
+									 LUTu & histogram, LUTu & histogramCropped,
 									 LUTf & hlCurve, LUTf & shCurve, LUTf & outCurve,
-									 LUTu & outBeforeCCurveHistogram, int skip) {
+									 LUTu & outBeforeCCurveHistogram,
+									 ToneCurve & customToneCurve1,
+									 ToneCurve & customToneCurve2,
+									 int skip) {
 		
 		
 		//double def_mul = pow (2.0, defmul);
@@ -289,7 +425,7 @@ namespace rtengine {
 		LUTf dcurve(0x10000);
 		
 		// check if inverse gamma is needed at the end
-		bool needigamma = igamma_ && gamma_>0;
+		bool needigamma = igamma_ && gamma_>1.;
 		
 		// clear array that stores histogram valid before applying the custom curve
 		outBeforeCCurveHistogram.clear();
@@ -304,26 +440,27 @@ namespace rtengine {
 		if (br>0.00001 || br<-0.00001) {
 
 			std::vector<double> brightcurvePoints;
-			brightcurvePoints.push_back((double)DCT_NURBS);
+			brightcurvePoints.resize(9);
+			brightcurvePoints.at(0) = double(DCT_NURBS);
 
-			brightcurvePoints.push_back(0.); //black point.  Value in [0 ; 1] range
-			brightcurvePoints.push_back(0.); //black point.  Value in [0 ; 1] range
+			brightcurvePoints.at(1) = 0.; //black point.  Value in [0 ; 1] range
+			brightcurvePoints.at(2) = 0.; //black point.  Value in [0 ; 1] range
 			
 			if(br>0) {
-				brightcurvePoints.push_back(0.1); //toe point
-				brightcurvePoints.push_back(0.1+br/150.0); //value at toe point
+				brightcurvePoints.at(3) = 0.1; //toe point
+				brightcurvePoints.at(4) = 0.1+br/150.0; //value at toe point
 
-				brightcurvePoints.push_back(0.7); //shoulder point
-				brightcurvePoints.push_back(min(1.0,0.7+br/300.0)); //value at shoulder point
+				brightcurvePoints.at(5) = 0.7; //shoulder point
+				brightcurvePoints.at(6) = min(1.0,0.7+br/300.0); //value at shoulder point
 			} else {
-				brightcurvePoints.push_back(max(0.0,0.1-br/150.0)); //toe point
-				brightcurvePoints.push_back(0.1); //value at toe point
+				brightcurvePoints.at(3) = max(0.0,0.1-br/150.0); //toe point
+				brightcurvePoints.at(4) = 0.1; //value at toe point
 
-				brightcurvePoints.push_back(0.7-br/300.0); //shoulder point
-				brightcurvePoints.push_back(0.7); //value at shoulder point
+				brightcurvePoints.at(5) = 0.7-br/300.0; //shoulder point
+				brightcurvePoints.at(6) = 0.7; //value at shoulder point
 			}
-			brightcurvePoints.push_back(1.); // white point
-			brightcurvePoints.push_back(1.); // value at white point
+			brightcurvePoints.at(7) = 1.; // white point
+			brightcurvePoints.at(8) = 1.; // value at white point
 			
 			brightcurve = new DiagonalCurve (brightcurvePoints, CURVES_MIN_POLY_POINTS/skip);
 		}
@@ -369,7 +506,7 @@ namespace rtengine {
 			val = (double)i / 65535.0;
 			
 			// gamma correction
-			if (gamma_>1)
+			if (gamma_>1.)
 				val = gamma (val, gamma_, start, slope, mul, add);
 			
 			// apply brightness curve
@@ -407,19 +544,20 @@ namespace rtengine {
 			
 			//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 			std::vector<double> contrastcurvePoints;
-			contrastcurvePoints.push_back((double)DCT_NURBS);
+			contrastcurvePoints.resize(9);
+			contrastcurvePoints.at(0) = double(DCT_NURBS);
 			
-			contrastcurvePoints.push_back(0); //black point.  Value in [0 ; 1] range
-			contrastcurvePoints.push_back(0); //black point.  Value in [0 ; 1] range
+			contrastcurvePoints.at(1) = 0; //black point.  Value in [0 ; 1] range
+			contrastcurvePoints.at(2) = 0; //black point.  Value in [0 ; 1] range
 			
-			contrastcurvePoints.push_back(avg-avg*(0.6-contr/250.0)); //toe point
-			contrastcurvePoints.push_back(avg-avg*(0.6+contr/250.0)); //value at toe point
+			contrastcurvePoints.at(3) = avg-avg*(0.6-contr/250.0); //toe point
+			contrastcurvePoints.at(4) = avg-avg*(0.6+contr/250.0); //value at toe point
 			
-			contrastcurvePoints.push_back(avg+(1-avg)*(0.6-contr/250.0)); //shoulder point
-			contrastcurvePoints.push_back(avg+(1-avg)*(0.6+contr/250.0)); //value at shoulder point
+			contrastcurvePoints.at(5) = avg+(1-avg)*(0.6-contr/250.0); //shoulder point
+			contrastcurvePoints.at(6) = avg+(1-avg)*(0.6+contr/250.0); //value at shoulder point
 			
-			contrastcurvePoints.push_back(1); // white point
-			contrastcurvePoints.push_back(1); // value at white point
+			contrastcurvePoints.at(7) = 1.; // white point
+			contrastcurvePoints.at(8) = 1.; // value at white point
 			
 			DiagonalCurve* contrastcurve = new DiagonalCurve (contrastcurvePoints, CURVES_MIN_POLY_POINTS/skip);
 			//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -433,20 +571,52 @@ namespace rtengine {
 		}
 
 		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-		
-		// create a curve if needed
+
+		// create second curve if needed
 		bool histNeeded = false;
 		DiagonalCurve* tcurve = NULL;
-		if (!curvePoints.empty() && curvePoints[0]!=0) {
+		customToneCurve2.Reset();
+
+		if (!curvePoints2.empty() && curvePoints2[0]>DCT_Linear && curvePoints2[0]<DCT_Unchanged) {
+			tcurve = new DiagonalCurve (curvePoints2, CURVES_MIN_POLY_POINTS/skip);
+			if (outBeforeCCurveHistogram /*&& histogramCropped*/)
+				histNeeded = true;
+		}
+		if (tcurve) {
+			if (tcurve->isIdentity()) {
+				delete tcurve;
+				tcurve = NULL;
+			}
+			else
+				customToneCurve2.Set(tcurve);
+			delete tcurve;
+			tcurve = NULL;
+		}
+
+		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+		// create first curve if needed
+		customToneCurve1.Reset();
+
+		if (!curvePoints.empty() && curvePoints[0]>DCT_Linear && curvePoints[0]<DCT_Unchanged) {
 			tcurve = new DiagonalCurve (curvePoints, CURVES_MIN_POLY_POINTS/skip);
 			if (outBeforeCCurveHistogram /*&& histogramCropped*/)
 				histNeeded = true;
 		}
-		if (tcurve && tcurve->isIdentity()) {
-			delete tcurve;
-			tcurve = NULL;
+		if (tcurve) {
+			if (tcurve->isIdentity()) {
+				delete tcurve;
+				tcurve = NULL;
+			}
+			else if (curveMode != procparams::ToneCurveParams::TC_MODE_STD) {
+				customToneCurve1.Set(tcurve);
+				delete tcurve;
+				tcurve = NULL;
+			}
 		}
 
+		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 		for (int i=0; i<=0xffff; i++) {
 			float val;
 
@@ -464,7 +634,7 @@ namespace rtengine {
 			if (tcurve) {
 				val = tcurve->getVal (dcurve[i]);  // TODO: getVal(double) is very slow! Optimize with a LUTf
 			} else {
-				val = (dcurve[i]);
+				val = dcurve[i];
 			}
 
 			// if inverse gamma is needed, do it (standard sRGB inverse gamma is applied)
@@ -474,8 +644,7 @@ namespace rtengine {
 			outCurve[i] = (65535.0 * val);
 		}
 
-		if (tcurve)
-			delete tcurve;
+		if (tcurve) delete tcurve;
 
 		/*if (outBeforeCCurveHistogram) {
 			for (int i=0; i<256; i++) printf("i= %d bchist= %d \n",i,outBeforeCCurveHistogram[i]);
@@ -489,7 +658,7 @@ namespace rtengine {
 	
 	void CurveFactory::complexLCurve (double br, double contr, const std::vector<double>& curvePoints,
 									 LUTu & histogram, LUTu & histogramCropped, LUTf & outCurve,
-									 LUTu & outBeforeCCurveHistogram, int skip) {
+									 LUTu & outBeforeCCurveHistogram, int skip, bool & utili) {
 		
 		// curve without contrast
 		LUTf dcurve(65536,0);
@@ -504,28 +673,30 @@ namespace rtengine {
 		
 		// check if brightness curve is needed
 		if (br>0.00001 || br<-0.00001) {
+			utili=true;
 
 			std::vector<double> brightcurvePoints;
-			brightcurvePoints.push_back((double)((CurveType)DCT_NURBS));
+			brightcurvePoints.resize(9);
+			brightcurvePoints.at(0) = double(DCT_NURBS);
 
-			brightcurvePoints.push_back(0.); // black point.  Value in [0 ; 1] range
-			brightcurvePoints.push_back(0.); // black point.  Value in [0 ; 1] range
+			brightcurvePoints.at(1) = 0.; // black point.  Value in [0 ; 1] range
+			brightcurvePoints.at(2) = 0.; // black point.  Value in [0 ; 1] range
 			
 			if (br>0) {
-				brightcurvePoints.push_back(0.1); // toe point
-				brightcurvePoints.push_back(0.1+br/150.0); //value at toe point
+				brightcurvePoints.at(3) = 0.1; // toe point
+				brightcurvePoints.at(4) = 0.1+br/150.0; //value at toe point
 
-				brightcurvePoints.push_back(0.7); // shoulder point
-				brightcurvePoints.push_back(min(1.0,0.7+br/300.0)); //value at shoulder point
+				brightcurvePoints.at(5) = 0.7; // shoulder point
+				brightcurvePoints.at(6) = min(1.0,0.7+br/300.0); //value at shoulder point
 			} else {
-				brightcurvePoints.push_back(0.1-br/150.0); // toe point
-				brightcurvePoints.push_back(0.1); // value at toe point
+				brightcurvePoints.at(3) = 0.1-br/150.0; // toe point
+				brightcurvePoints.at(4) = 0.1; // value at toe point
 
-				brightcurvePoints.push_back(min(1.0,0.7-br/300.0)); // shoulder point
-				brightcurvePoints.push_back(0.7); // value at shoulder point
+				brightcurvePoints.at(5) = min(1.0,0.7-br/300.0); // shoulder point
+				brightcurvePoints.at(6) = 0.7; // value at shoulder point
 			}
-			brightcurvePoints.push_back(1.); // white point
-			brightcurvePoints.push_back(1.); // value at white point
+			brightcurvePoints.at(7) = 1.; // white point
+			brightcurvePoints.at(8) = 1.; // value at white point
 			
 			DiagonalCurve* brightcurve = new DiagonalCurve (brightcurvePoints, CURVES_MIN_POLY_POINTS/skip);
 			//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -550,13 +721,13 @@ namespace rtengine {
 				dcurve[i] = (float)i / 32767.0;
 			}
 		}
-		
 		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 		
 		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 		
 		// check if contrast curve is needed
 		if (contr>0.00001 || contr<-0.00001) {
+			utili=true;
 
 			// compute mean luminance of the image with the curve applied
 			int sum = 0;
@@ -570,22 +741,24 @@ namespace rtengine {
 			avg /= sum;
 			//sqavg /= sum;
 			//float stddev = sqrt(sqavg-avg*avg);
-			
+			//		printf("avg=%f\n",avg);
+
 			//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 			std::vector<double> contrastcurvePoints;
-			contrastcurvePoints.push_back((double)((CurveType)DCT_NURBS));
+			contrastcurvePoints.resize(9);
+			contrastcurvePoints.at(0) = double(DCT_NURBS);
 			
-			contrastcurvePoints.push_back(0.); // black point.  Value in [0 ; 1] range
-			contrastcurvePoints.push_back(0.); // black point.  Value in [0 ; 1] range
+			contrastcurvePoints.at(1) = 0.; // black point.  Value in [0 ; 1] range
+			contrastcurvePoints.at(2) = 0.; // black point.  Value in [0 ; 1] range
 			
-			contrastcurvePoints.push_back(avg-avg*(0.6-contr/250.0)); // toe point
-			contrastcurvePoints.push_back(avg-avg*(0.6+contr/250.0)); // value at toe point
+			contrastcurvePoints.at(3) = avg-avg*(0.6-contr/250.0); // toe point
+			contrastcurvePoints.at(4) = avg-avg*(0.6+contr/250.0); // value at toe point
 			
-			contrastcurvePoints.push_back(avg+(1-avg)*(0.6-contr/250.0)); // shoulder point
-			contrastcurvePoints.push_back(avg+(1-avg)*(0.6+contr/250.0)); // value at shoulder point
+			contrastcurvePoints.at(5) = avg+(1-avg)*(0.6-contr/250.0); // shoulder point
+			contrastcurvePoints.at(6) = avg+(1-avg)*(0.6+contr/250.0); // value at shoulder point
 			
-			contrastcurvePoints.push_back(1.); // white point
-			contrastcurvePoints.push_back(1.); // value at white point
+			contrastcurvePoints.at(7) = 1.; // white point
+			contrastcurvePoints.at(8) = 1.; // value at white point
 			
 			DiagonalCurve* contrastcurve = new DiagonalCurve (contrastcurvePoints, CURVES_MIN_POLY_POINTS/skip);
 			//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -613,6 +786,8 @@ namespace rtengine {
 		}
 
 		if (tcurve) {
+			utili=true;//if active
+
 			// L values go up to 32767, last stop is for highlight overflow
 			for (int i=0; i<32768; i++) {
 				float val;
@@ -652,7 +827,6 @@ namespace rtengine {
 		}*/
 	}
 
-
 	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -673,50 +847,43 @@ namespace rtengine {
 		}
 		
 		if (tcurve) {
-			for (int i=0; i<65536; i++) {				
+			if (!outCurve)
+				outCurve(65536, 0);
+			for (int i=0; i<65536; i++) {
 				// apply custom/parametric/NURBS curve, if any
 				float val = tcurve->getVal ((float)i/65536.0f);
 				outCurve[i] = (65536.0f * val);
 			}
-		}
-		else {
-			// Skip the slow getval method if no curve is used (or an identity curve)
-			for (int i=0; i<65536; i++) {
-				outCurve[i] = i;
-			}
-		}
-		
-		if (tcurve)
 			delete tcurve;
+		}
+		// let the LUTf empty for identity curves
+		else {
+			outCurve.reset();
+		}
 
 	}
 	
 	
 	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-	
-	
 
-LUTf CurveFactory::gammatab;
-LUTf CurveFactory::igammatab_srgb;
-LUTf CurveFactory::gammatab_srgb;
+void ColorAppearance::Reset() {
+    lutColCurve.reset();
+}
 
-void CurveFactory::init () {
-	
-	gammatab(65536,0);
-	igammatab_srgb(65536,0);
-	gammatab_srgb(65536,0);
+// Fill a LUT with X/Y, ranged 0xffff
+void ColorAppearance::Set(Curve *pCurve) {
+    lutColCurve(65536);
+    for (int i=0; i<65536; i++) lutColCurve[i] = pCurve->getVal(double(i)/65535.) * 65535.;
+}
 
-  for (int i=0; i<65536; i++)
-    gammatab_srgb[i] = (65535.0 * gamma2 (i/65535.0));
-  for (int i=0; i<65536; i++)
-    igammatab_srgb[i] = (65535.0 * igamma2 (i/65535.0));
-  for (int i=0; i<65536; i++)
-    gammatab[i] = (65535.0 * pow (i/65535.0, 0.454545));
-    
-/*    FILE* f = fopen ("c.txt", "wt");
-    for (int i=0; i<256; i++)
-        fprintf (f, "%g %g\n", i/255.0, clower (i/255.0, 2.0, 1.0));
-    fclose (f);*/
+void ToneCurve::Reset() {
+    lutToneCurve.reset();
+}
+
+// Fill a LUT with X/Y, ranged 0xffff
+void ToneCurve::Set(Curve *pCurve) {
+    lutToneCurve(65536);
+    for (int i=0; i<65536; i++) lutToneCurve[i] = pCurve->getVal(double(i)/65535.) * 65535.;
 }
 
 }
