@@ -21,6 +21,7 @@
 #define _DCP_
 
 #include "imagefloat.h"
+#include "curves.h"
 #include <glibmm.h>
 #include <map>
 #include <string>
@@ -47,6 +48,8 @@ namespace rtengine {
 
         int iHueStep, iValStep, iArrayCount;
 
+        AdobeToneCurve toneCurve;
+
         void ConvertDNGMatrix2XYZCAM(const double (*mColorMatrix)[3], double (*mXYZCAM)[3]);
 
         const HSBModify* GetBestProfile(DCPLightType preferredProfile,  double (*mXYZCAM)[3]) const;
@@ -54,11 +57,11 @@ namespace rtengine {
         DCPLightType GetLightType(short iLightSource) const;
 
     public:
-        DCPProfile(Glib::ustring fname);
+        DCPProfile(Glib::ustring fname, bool isRTProfile);
         ~DCPProfile();
 
-        void Apply(Imagefloat *pImg, DCPLightType preferredProfile, Glib::ustring workingSpace) const;
-        void Apply(Image16 *pImg, DCPLightType preferredProfile, Glib::ustring workingSpace) const;
+        void Apply(Imagefloat *pImg, DCPLightType preferredProfile, Glib::ustring workingSpace, float rawWhiteFac=1, bool useToneCurve=false) const;
+        void Apply(Image16 *pImg, DCPLightType preferredProfile, Glib::ustring workingSpace, bool useToneCurve) const;
     };
 
     class DCPStore {
@@ -75,7 +78,7 @@ namespace rtengine {
 
         bool isValidDCPFileName(Glib::ustring filename) const;
 
-        DCPProfile* getProfile(Glib::ustring filename);
+        DCPProfile* getProfile(Glib::ustring filename, bool isRTProfile=false);
         DCPProfile* getStdProfile(Glib::ustring camShortName);
         
         static DCPStore* getInstance();
