@@ -22,45 +22,57 @@
 #include "refreshmap.h"
 #include "simpleprocess.h"
 #include "../rtgui/ppversion.h"
+#include "colortemp.h"
 
 namespace rtengine {
 
 extern const Settings* settings;
 
 ImProcCoordinator::ImProcCoordinator ()
-    : workimg(NULL), awbComputed(false), ipf(&params, true), scale(10), lastHighDetail(false), allocated(false),
-    pW(-1), pH(-1), plistener(NULL),
-    imageListener(NULL), aeListener(NULL), hListener(NULL), resultValid(false),
-    changeSinceLast(0), updaterRunning(false), destroying(false) {
+    : workimg(NULL), awbComputed(false), ipf(&params, true), scale(10), highDetailPreprocessComputed(false),
+      highDetailRawComputed(false), allocated(false),
 
-    hltonecurve(65536,0);
-    shtonecurve(65536,2);//clip above
-    tonecurve(65536,0);//,1);
+      hltonecurve(65536,0),
+      shtonecurve(65536,2),//clip above
+      tonecurve(65536,0),//,1);
 
-    lumacurve(65536,0);
-    chroma_acurve(65536,0);
-    chroma_bcurve(65536,0);
-	satcurve(65536,0);
+      lumacurve(65536,0),
+      chroma_acurve(65536,0),
+      chroma_bcurve(65536,0),
+      satcurve(65536,0),
+      lhskcurve(65536,0),
 
-    vhist16(65536);
-    lhist16(65536); lhist16Cropped(65536);
-    histCropped(65536);
+      vhist16(65536),
+      lhist16(65536), lhist16Cropped(65536),
+      lhist16CAM(65536), lhist16CroppedCAM(65536),
+      lhist16CCAM(65536), lhist16CroppedCCAM(65536),
+	  lhist16CCAMAF(65536), lhist16ClabAF(65536),
+      histCropped(65536),
+	  lhist16Clad(65536),lhist16CroppedClad(65536),
 
-    histRed(256); histRedRaw(256);
-    histGreen(256); histGreenRaw(256);
-    histBlue(256); histBlueRaw(256);
-    histLuma(256);
-    histToneCurve(256);
-    histLCurve(256);
-    bcabhist(256);
-		
-    rCurve(65536,0);
-    rcurvehist(256); rcurvehistCropped(256); rbeforehist(256);
-    gCurve(65536,0);
-    gcurvehist(256); gcurvehistCropped(256); gbeforehist(256);		
-    bCurve(65536,0);
-    bcurvehist(256); bcurvehistCropped(256); bbeforehist(256);
-}
+      histRed(256), histRedRaw(256),
+      histGreen(256), histGreenRaw(256),
+      histBlue(256), histBlueRaw(256),
+      histLuma(256), histChroma(256),
+      histToneCurve(256),
+      histLCurve(256),
+      histCCurve(256),
+	  histLCAM(256),
+	  histCCAM(256),
+	  histClad(256),	
+      bcabhist(256),
+
+      rCurve(),
+      gCurve(),
+      bCurve(),
+      rcurvehist(256), rcurvehistCropped(256), rbeforehist(256),
+      gcurvehist(256), gcurvehistCropped(256), gbeforehist(256),
+      bcurvehist(256), bcurvehistCropped(256), bbeforehist(256),
+
+      pW(-1), pH(-1),
+      plistener(NULL), imageListener(NULL), aeListener(NULL), hListener(NULL),acListener(NULL),
+      resultValid(false), changeSinceLast(0), updaterRunning(false), destroying(false)
+    {}
 
 void ImProcCoordinator::assign (ImageSource* imgsrc) {
     this->imgsrc = imgsrc;
@@ -71,7 +83,7 @@ ImProcCoordinator::~ImProcCoordinator () {
     destroying = true;
     updaterThreadStart.lock ();
     if (updaterRunning && thread)
-        thread->join ();      
+        thread->join ();
     mProcessing.lock(); 
     mProcessing.unlock(); 
     freeAll ();
@@ -89,6 +101,7 @@ DetailedCrop* ImProcCoordinator::createCrop  () {
     return new Crop (this); 
 }
 
+
 // todo: bitmask containing desired actions, taken from changesSinceLast
 // cropCall: calling crop, used to prevent self-updates
 void ImProcCoordinator::updatePreviewImage (int todo, Crop* cropCall) {
@@ -117,13 +130,18 @@ void ImProcCoordinator::updatePreviewImage (int todo, Crop* cropCall) {
 		rp.ca_autocorrect  = false;
 		rp.hotdeadpix_filt = false;
 		rp.ccSteps = 0;
-		rp.all_enhance = false;
+		//rp.all_enhance = false;
 	}
 
     progress ("Applying white balance, color correction & sRGB conversion...",100*readyphase/numofphases);
-    if ( todo & M_PREPROC) {
+    // raw auto CA is bypassed if no high detail is needed, so we have to compute it when high detail is needed
+    if ( (todo & M_PREPROC) || (!highDetailPreprocessComputed && highDetailNeeded)) {
     	imgsrc->preprocess( rp, params.lensProf, params.coarse );
         imgsrc->getRAWHistogram( histRedRaw, histGreenRaw, histBlueRaw );
+        if (highDetailNeeded)
+        	highDetailPreprocessComputed = true;
+        else
+        	highDetailPreprocessComputed = false;
     }
 
     /*  
@@ -137,18 +155,28 @@ void ImProcCoordinator::updatePreviewImage (int todo, Crop* cropCall) {
         OR HLR gets disabled when Color method was selected
     */
     // If high detail (=100%) is newly selected, do a demosaic update, since the last was just with FAST
-    if ((todo & M_RAW)
-    	|| (!lastHighDetail && highDetailNeeded)
-    	|| (params.hlrecovery.enabled && params.hlrecovery.method!="Color" && imgsrc->IsrgbSourceModified())
-    	|| (!params.hlrecovery.enabled && params.hlrecovery.method=="Color" && imgsrc->IsrgbSourceModified())){
+    if (   (todo & M_RAW)
+        || (!highDetailRawComputed && highDetailNeeded)
+        || ( params.hlrecovery.enabled && params.hlrecovery.method!="Color" && imgsrc->IsrgbSourceModified())
+        || (!params.hlrecovery.enabled && params.hlrecovery.method=="Color" && imgsrc->IsrgbSourceModified()))
+    {
 
-    	if (settings->verbose) printf("Demosaic %s\n",rp.dmethod.c_str());
-    	imgsrc->demosaic( rp );
+        if (settings->verbose) printf("Demosaic %s\n",rp.dmethod.c_str());
+
+        imgsrc->demosaic( rp );
+
+        if (highDetailNeeded) {
+            highDetailRawComputed = true;
+            if (params.hlrecovery.enabled && params.hlrecovery.method=="Color") {
+                todo |= M_INIT;
+            }
+        }
+        else
+            highDetailRawComputed = false;
     }
-    lastHighDetail=highDetailNeeded;
 
 
-    if (todo & M_INIT) {
+    if (todo & (M_INIT|M_LINDENOISE)) {
         Glib::Mutex::Lock lock(minit);  // Also used in crop window
 
         imgsrc->HLRecovery_Global( params.hlrecovery ); // this handles Color HLRecovery
@@ -179,6 +207,18 @@ void ImProcCoordinator::updatePreviewImage (int todo, Crop* cropCall) {
         PreviewProps pp (0, 0, fw, fh, scale);
         setScale (scale);
         imgsrc->getImage (currWB, tr, orig_prev, pp, params.hlrecovery, params.icm, params.raw);
+        //ColorTemp::CAT02 (orig_prev, &params)	;
+
+		//imgsrc->convertColorSpace(orig_prev, params.icm, params.raw);
+
+        if (todo & M_LINDENOISE) {
+        	//printf("denoising!\n");
+			if (scale==1 && params.dirpyrDenoise.enabled) {
+				ipf.RGB_denoise(orig_prev, orig_prev, imgsrc->isRAW(), params.dirpyrDenoise, params.defringe, imgsrc->getDirPyrDenoiseExpComp());
+			}
+        }
+        imgsrc->convertColorSpace(orig_prev, params.icm, params.raw);
+
         ipf.firstAnalysis (orig_prev, &params, vhist16, imgsrc->getGamma());
     }
     readyphase++;
@@ -225,11 +265,13 @@ void ImProcCoordinator::updatePreviewImage (int todo, Crop* cropCall) {
         if (hListener) oprevi->calcCroppedHistogram(params, scale, histCropped);
 
         // complexCurve also calculated pre-curves histogram dependend on crop
+        ipf.g = imgsrc->getGamma();
+        ipf.iGamma = true;
         CurveFactory::complexCurve (params.toneCurve.expcomp, params.toneCurve.black/65535.0,
 									params.toneCurve.hlcompr, params.toneCurve.hlcomprthresh,
 									params.toneCurve.shcompr, params.toneCurve.brightness, params.toneCurve.contrast,
-									imgsrc->getGamma(), true, params.toneCurve.curve,
-									vhist16, histCropped, hltonecurve, shtonecurve, tonecurve, histToneCurve, scale==1 ? 1 : 1);
+									ipf.g, !ipf.iGamma, params.toneCurve.curveMode, params.toneCurve.curve, params.toneCurve.curveMode2, params.toneCurve.curve2,
+									vhist16, histCropped, hltonecurve, shtonecurve, tonecurve, histToneCurve, customToneCurve1, customToneCurve2, scale==1 ? 1 : 1);
 
         CurveFactory::RGBCurve (params.rgbCurves.rcurve, rCurve, scale==1 ? 1 : 1);
         CurveFactory::RGBCurve (params.rgbCurves.gcurve, gCurve, scale==1 ? 1 : 1);
@@ -238,72 +280,76 @@ void ImProcCoordinator::updatePreviewImage (int todo, Crop* cropCall) {
         // if it's just crop we just need the histogram, no image updates
         if ( todo!=CROP ) {
             ipf.rgbProc (oprevi, oprevl, hltonecurve, shtonecurve, tonecurve, shmap, params.toneCurve.saturation,
-                         rCurve, gCurve, bCurve);
+                         rCurve, gCurve, bCurve, customToneCurve1, customToneCurve2, params.toneCurve.expcomp, params.toneCurve.hlcompr, params.toneCurve.hlcomprthresh);
         }
 
         // compute L channel histogram
-        int x1, y1, x2, y2, pos;
+        int x1, y1, x2, y2, pos, poscc;
         params.crop.mapToResized(pW, pH, scale, x1, x2,  y1, y2); 
-
         lhist16.clear(); lhist16Cropped.clear();
+	    lhist16Clad.clear(); lhist16CroppedClad.clear();
+		lhist16ClabAF.clear();		
         for (int x=0; x<pH; x++)
             for (int y=0; y<pW; y++) {
                 pos=CLIP((int)(oprevl->L[x][y]));
                 lhist16[pos]++;
-
-                if (y>=y1 && y<y2 && x>=x1 && x<x2) lhist16Cropped[pos]++;
+                if (y>=y1 && y<y2 && x>=x1 && x<x2) {lhist16Cropped[pos]++;lhist16CroppedClad[poscc]++;}
             }
  
     }
     readyphase++;
-
+		utili=false;
+		autili=false;
+		butili=false;
+		ccutili=false;
+		cclutili=false;		
     if ((todo & M_LUMACURVE) || todo==CROP) {
-
         CurveFactory::complexLCurve (params.labCurve.brightness, params.labCurve.contrast, params.labCurve.lcurve, lhist16, lhist16Cropped,
-                                     lumacurve, histLCurve, scale==1 ? 1 : 16);
-    }
-
+                                     lumacurve, histLCurve, scale==1 ? 1 : 16, utili);
+    
+		}
     if (todo & M_LUMACURVE) {
-		CurveFactory::complexsgnCurve (params.labCurve.saturation, params.labCurve.enable_saturationlimiter, params.labCurve.saturationlimit,
-									   params.labCurve.acurve, params.labCurve.bcurve, chroma_acurve, chroma_bcurve, satcurve, scale==1 ? 1 : 16);
-	}
+		CurveFactory::complexsgnCurve (autili, butili,ccutili,cclutili, params.labCurve.chromaticity, params.labCurve.rstprotection,
+										params.labCurve.acurve, params.labCurve.bcurve,params.labCurve.cccurve,params.labCurve.lccurve, chroma_acurve, chroma_bcurve, satcurve,lhskcurve, 
+										lhist16Clad,lhist16CroppedClad, histCCurve, scale==1 ? 1 : 16);
 	
+	}
 	if (todo & (M_LUMINANCE+M_COLOR) ) {
 		nprevl->CopyFrom(oprevl);
-
-		ipf.EPDToneMap(nprevl,0,scale);
-
-		progress ("Applying Luminance Curve...",100*readyphase/numofphases);
-
-		ipf.luminanceCurve (nprevl, nprevl, lumacurve);
-
-		readyphase++;
+		
 		progress ("Applying Color Boost...",100*readyphase/numofphases);
-		ipf.chrominanceCurve (nprevl, nprevl, chroma_acurve, chroma_bcurve, satcurve/*, params.labCurve.saturation*/);
-		//ipf.colorCurve (nprevl, nprevl);
+	    int poscc;
+		
+		
+		ipf.chromiLuminanceCurve (pW,nprevl, nprevl, chroma_acurve, chroma_bcurve, satcurve,lhskcurve, lumacurve, utili, autili, butili, ccutili,cclutili, histCCurve);
 		ipf.vibrance(nprevl);
+	if((params.colorappearance.enabled && !params.colorappearance.tonecie) ||  (!params.colorappearance.enabled)) ipf.EPDToneMap(nprevl,5,1);
+		// for all treatments Defringe, Sharpening, Contrast detail , Microcontrast they are activated if "CIECAM" function are disabled
 		readyphase++;
 		if (scale==1) {
+			if((params.colorappearance.enabled && !settings->autocielab) || (!params.colorappearance.enabled)){		
             progress ("Denoising luminance impulse...",100*readyphase/numofphases);
             ipf.impulsedenoise (nprevl);
             readyphase++;
+			}
+			if((params.colorappearance.enabled && !settings->autocielab) || (!params.colorappearance.enabled)){
 			progress ("Defringing...",100*readyphase/numofphases);
-            ipf.defringe (nprevl);
-            readyphase++;
-            progress ("Denoising luma/chroma...",100*readyphase/numofphases);
-            ipf.dirpyrdenoise (nprevl);
-            readyphase++;
+			ipf.defringe (nprevl);
+			readyphase++;
+			}
 			if (params.sharpenEdge.enabled) {
                 progress ("Edge sharpening...",100*readyphase/numofphases);
 				ipf.MLsharpen (nprevl);
 		        readyphase++;
 			}
 			if (params.sharpenMicro.enabled) {
+				if(( params.colorappearance.enabled && !settings->autocielab) || (!params.colorappearance.enabled)){
                 progress ("Microcontrast...",100*readyphase/numofphases);			
 				ipf.MLmicrocontrast (nprevl);
 		        readyphase++;
+				}
 			}
-            if (params.sharpening.enabled) {
+			if(((params.colorappearance.enabled && !settings->autocielab) || (!params.colorappearance.enabled)) && params.sharpening.enabled) {			
                 progress ("Sharpening...",100*readyphase/numofphases);
                     
                 float **buffer = new float*[pH];
@@ -317,17 +363,88 @@ void ImProcCoordinator::updatePreviewImage (int todo, Crop* cropCall) {
                 delete [] buffer;
                 readyphase++;
             }
-
-            progress ("Pyramid equalizer...",100*readyphase/numofphases);
-            ipf.dirpyrequalizer (nprevl);
-            readyphase++;
+			if((params.colorappearance.enabled && !settings->autocielab) || (!params.colorappearance.enabled)){
+//			if(params.colorappearance.enabled && !params.colorappearance.sharpcie){
+				progress ("Pyramid equalizer...",100*readyphase/numofphases);
+				ipf.dirpyrequalizer (nprevl);
+				readyphase++;
+			}
+			
         }
-    }
-
+	    //L histo  and Chroma histo for ciecam
+		// histogram well be for Lab (Lch) values, because very difficult to do with J,Q, M, s, C
+        int x1, y1, x2, y2, pos, posc;
+        params.crop.mapToResized(pW, pH, scale, x1, x2,  y1, y2); 	
+	    lhist16CAM.clear(); lhist16CroppedCAM.clear();
+	    lhist16CCAM.clear(); lhist16CroppedCCAM.clear();
+		lhist16CCAMAF.clear();
+        for (int x=0; x<pH; x++)
+            for (int y=0; y<pW; y++) {
+                pos=CLIP((int)(nprevl->L[x][y]));
+                posc=CLIP((int)sqrt(nprevl->a[x][y]*nprevl->a[x][y] + nprevl->b[x][y]*nprevl->b[x][y]));
+	            if(!params.colorappearance.datacie) lhist16CCAM[posc]++;
+				if(!params.colorappearance.datacie)lhist16CAM[pos]++;
+                if (y>=y1 && y<y2 && x>=x1 && x<x2) {lhist16CroppedCAM[pos]++;lhist16CroppedCCAM[posc]++;}
+            }
+	LUTu dummy;
+	CurveFactory::curveLightBrightColor (
+					params.colorappearance.curveMode, params.colorappearance.curve,
+					params.colorappearance.curveMode2, params.colorappearance.curve2,
+					params.colorappearance.curveMode3, params.colorappearance.curve3,
+					lhist16CAM, lhist16CroppedCAM,histLCAM,	
+					lhist16CCAM, lhist16CroppedCCAM,histCCAM,						
+					customColCurve1,
+					customColCurve2, 
+					customColCurve3, 
+					scale==1 ? 1 : 1);
+	int  Iterates=0;
+	if(params.colorappearance.enabled){	
+	float fnum = imgsrc->getMetaData()->getFNumber  ();// F number
+	float fiso = imgsrc->getMetaData()->getISOSpeed () ;// ISO
+	float fspeed = imgsrc->getMetaData()->getShutterSpeed () ;//speed 
+	float fcomp = imgsrc->getMetaData()->getExpComp  ();//compensation + -
+	float adap2,adap;
+	double ada, ada2;
+	if(fnum < 0.3f || fiso < 5.f || fspeed < 0.00001f) {adap=adap=2000.f;ada=2000.;}//if no exif data or wrong
+	else {
+	float E_V = fcomp + log2 ((fnum*fnum) / fspeed / (fiso/100.f));
+	float expo2= params.toneCurve.expcomp;// exposure compensation in tonecurve ==> direct EV
+	E_V += expo2;
+	float expo1;//exposure raw white point
+	expo1=log2(params.raw.expos);//log2 ==>linear to EV
+	E_V += expo1;
+	adap2 = adap= powf(2.f, E_V-3.f);//cd / m2
+	ada=ada2=(double) adap;
+	//end calculation adaptation scene luminosity
+	}	
+	int begh=0;
+	int endh=pH;
+	float d;
+	double dd;
+	float **buffer = new float*[pH];
+       for (int i=0; i<pH; i++)
+         buffer[i] = new float[pW];
+	bool execsharp=false;
+	if(scale==1) execsharp=true;	
+	if(settings->ciecamfloat){ipf.ciecam_02float (ncie, adap, begh, endh, pW, 2, nprevl, &params, customColCurve1,customColCurve2,customColCurve3, histLCAM, histCCAM, 5, 1, (float**)buffer, execsharp, d);
+	if(params.colorappearance.autodegree && acListener && params.colorappearance.enabled) acListener->autoCamChanged(100.*(double)d);
+	if(params.colorappearance.autoadapscen && acListener && params.colorappearance.enabled) acListener->adapCamChanged(adap2);//real value of adapt scene luminosity
+	}
+	else {ipf.ciecam_02 (ncie, ada, begh, endh, pW, 2, nprevl, &params, customColCurve1,customColCurve2,customColCurve3, histLCAM, histCCAM, 5, 1, (float**)buffer, execsharp, dd);
+	if(params.colorappearance.autodegree && acListener && params.colorappearance.enabled) acListener->autoCamChanged(100.*dd);
+	if(params.colorappearance.autoadapscen && acListener && params.colorappearance.enabled) acListener->adapCamChanged(ada);
+	
+	}
+		for (int i=0; i<pH; i++)
+			delete [] buffer[i];
+			delete [] buffer;
+		readyphase++;
+	}
+	}
     // process crop, if needed
     for (size_t i=0; i<crops.size(); i++)
         if (crops[i]->hasListener () && cropCall != crops[i] )
-            crops[i]->update (todo);  // may call outselves
+            crops[i]->update (todo);  // may call ourselves
 
     progress ("Conversion to RGB...",100*readyphase/numofphases);
     if (todo!=CROP) {
@@ -360,7 +477,7 @@ void ImProcCoordinator::updatePreviewImage (int todo, Crop* cropCall) {
 
     if (hListener) {
         updateLRGBHistograms ();
-        hListener->histogramChanged (histRed, histGreen, histBlue, histLuma, histToneCurve, histLCurve, histRedRaw, histGreenRaw, histBlueRaw);
+        hListener->histogramChanged (histRed, histGreen, histBlue, histLuma, histToneCurve, histLCurve,histCCurve, histLCAM, histCCAM, histRedRaw, histGreenRaw, histBlueRaw, histChroma);
     }
 
     mProcessing.unlock ();
@@ -373,11 +490,12 @@ void ImProcCoordinator::freeAll () {
 
     if (allocated) {
         if (orig_prev!=oprevi)
-            delete oprevi;
+        delete oprevi;
         delete orig_prev;
         delete oprevl;
         delete nprevl;
-        
+        delete ncie;
+       
         if (imageListener) {
             imageListener->delImage (previmg);
         }
@@ -421,6 +539,7 @@ if (settings->verbose) printf ("setscale before lock\n");
         oprevi = orig_prev;
         oprevl = new LabImage (pW, pH);    
         nprevl = new LabImage (pW, pH);    
+        ncie = new CieImage (pW, pH);    
         previmg = new Image8 (pW, pH);
 		workimg = new Image8 (pW, pH);
         shmap = new SHMap (pW, pH, true);
@@ -463,10 +582,13 @@ void ImProcCoordinator::updateLRGBHistograms () {
         }
     }
 
-    histLuma.clear();
+    histLuma.clear();    
+	histChroma.clear();
     for (int i=y1; i<y2; i++)
         for (int j=x1; j<x2; j++) {
-            histLuma[(int)(nprevl->L[i][j]/128)]++;
+		  histChroma[(int)(sqrt(nprevl->a[i][j]*nprevl->a[i][j] + nprevl->b[i][j]*nprevl->b[i][j]))/188]++;//188 = 48000/256
+		  histLuma[(int)(nprevl->L[i][j]/128)]++;
+		
 		}
 	
 	/*for (int i=0; i<256; i++) {
@@ -548,7 +670,14 @@ void ImProcCoordinator::getAutoCrop (double ratio, int &x, int &y, int &w, int &
 
     mProcessing.lock ();
 
-    double fillscale = ipf.getTransformAutoFill (fullw, fullh);
+    LCPMapper *pLCPMap=NULL;
+    if (params.lensProf.lcpFile.length() && imgsrc->getMetaData()->getFocalLen()>0) {
+        LCPProfile *pLCPProf=lcpStore->getProfile(params.lensProf.lcpFile);
+        if (pLCPProf) pLCPMap=new LCPMapper(pLCPProf, imgsrc->getMetaData()->getFocalLen(), imgsrc->getMetaData()->getFocalLen35mm(), imgsrc->getMetaData()->getFocusDist(),
+            0, false, params.lensProf.useDist, fullw, fullh, params.coarse, imgsrc->getRotateDegree());
+    }
+
+    double fillscale = ipf.getTransformAutoFill (fullw, fullh, pLCPMap);
     if (ratio>0) {
         w = fullw * fillscale;
     	h = w / ratio;
@@ -579,7 +708,6 @@ void ImProcCoordinator::saveInputICCReference (const Glib::ustring& fname) {
 	ppar.hlrecovery.enabled = false;
 	ppar.icm.input = "(none)";
 	Imagefloat* im = new Imagefloat (fW, fH);
-	Image16* im16 = new Image16 (fW, fH);
 	imgsrc->preprocess( ppar.raw, ppar.lensProf, ppar.coarse );
 	imgsrc->demosaic(ppar.raw );
 	//imgsrc->getImage (imgsrc->getWB(), 0, im, pp, ppar.hlrecovery, ppar.icm, ppar.raw);
@@ -596,8 +724,11 @@ void ImProcCoordinator::saveInputICCReference (const Glib::ustring& fname) {
 	params.wb.temperature = currWB.getTemp ();
 	params.wb.green = currWB.getGreen ();
 	imgsrc->getImage (currWB, 0, im, pp, ppar.hlrecovery, ppar.icm, ppar.raw);
-	im16 = im->to16();
+	imgsrc->convertColorSpace(im, ppar.icm, params.raw);
+	Image16* im16 = im->to16();
+	delete im;
 	im16->saveTIFF (fname,16,true);
+	delete im16;
 	//im->saveJPEG (fname, 85);
 	mProcessing.unlock ();
 }
