@@ -21,7 +21,6 @@
 #include "dcp.h"
 #include "improcfun.h"
 #include "improccoordinator.h"
-#include "curves.h"
 #include "dfmanager.h"
 #include "ffmanager.h"
 #include "rtthumbnail.h"
@@ -43,8 +42,7 @@ int init (const Settings* s, Glib::ustring baseDir) {
 
     profileStore.init ();
     ProcParams::init ();
-    CurveFactory::init ();
-    ImProcFunctions::initMunsell();
+    Color::init();
     ImProcFunctions::initCache ();
     Thumbnail::initGamma ();
     delete lcmsMutex;
@@ -57,6 +55,7 @@ int init (const Settings* s, Glib::ustring baseDir) {
 void cleanup () {
 
     ProcParams::cleanup ();
+    Color::cleanup ();
     ImProcFunctions::cleanupCache ();
     Thumbnail::cleanupGamma ();
 }
