@@ -33,68 +33,86 @@ namespace rtengine {
 #define CLIPTOC(a,b,c,d) ((a)>=(b)?((a)<=(c)?(a):(d=true,(c))):(d=true,(b)))
 #define RT_PI 3.141592653589
 
-bool ImProcFunctions::transCoord (int W, int H, std::vector<Coord2D> &src, std::vector<Coord2D> &red,  std::vector<Coord2D> &green, std::vector<Coord2D> &blue, double ascaleDef) {
+bool ImProcFunctions::transCoord (int W, int H, const std::vector<Coord2D> &src, std::vector<Coord2D> &red,  std::vector<Coord2D> &green, std::vector<Coord2D> &blue, double ascaleDef,
+    const LCPMapper *pLCPMap) {
 
-    bool clipresize = true;
     bool clipped = false;
 
-    red.clear ();
-    green.clear ();
-    blue.clear ();
+    red.clear (); green.clear (); blue.clear ();
 
-    if (!needsCA() && !needsDistortion() && !needsRotation() && !needsPerspective()) {
-        if (clipresize) {
+    if (!needsCA() && !needsDistortion() && !needsRotation() && !needsPerspective() && (!params->lensProf.useDist || pLCPMap==NULL)) {
                 for (size_t i=0; i<src.size(); i++) {
                     red.push_back   (Coord2D (src[i].x, src[i].y));
                     green.push_back (Coord2D (src[i].x, src[i].y));
                     blue.push_back  (Coord2D (src[i].x, src[i].y));
                 }
-        }
+
         return clipped;
     }
 
-    double oW = W;
-    double oH = H;
+    double oW = W, oH = H;
 	double w2 = (double) oW  / 2.0 - 0.5;
 	double h2 = (double) oH  / 2.0 - 0.5;
-	double a = params->distortion.amount;
+    double maxRadius = sqrt( (double)( oW*oW + oH*oH ) ) / 2;
+
+    // auxiliary variables for distortion correction
+    bool needsDist = needsDistortion();  // for performance
+	double distAmount = params->distortion.amount;
+
+    // auxiliary variables for rotation
 	double cost = cos(params->rotate.degree * RT_PI/180.0);
 	double sint = sin(params->rotate.degree * RT_PI/180.0);
-	double maxRadius = sqrt( (double)( oW*oW + oH*oH ) ) / 2;
+
+    // auxiliary variables for vertical perspective correction
     double vpdeg = params->perspective.vertical / 100.0 * 45.0;
     double vpalpha = (90.0 - vpdeg) / 180.0 * RT_PI;
-    double vpteta  = fabs(vpalpha-RT_PI/2)<1e-3 ? 0.0 : acos ((vpdeg>0 ? 1.0 : -1.0) * sqrt((-oW*oW*tan(vpalpha)*tan(vpalpha) + (vpdeg>0 ? 1.0 : -1.0) * oW*tan(vpalpha)*sqrt(16*maxRadius*maxRadius+oW*oW*tan(vpalpha)*tan(vpalpha)))/(maxRadius*maxRadius*8)));
+    double vpteta  = fabs(vpalpha-RT_PI/2)<3e-4 ? 0.0 : acos ((vpdeg>0 ? 1.0 : -1.0) * sqrt((-oW*oW*tan(vpalpha)*tan(vpalpha) + (vpdeg>0 ? 1.0 : -1.0) * oW*tan(vpalpha)*sqrt(16*maxRadius*maxRadius+oW*oW*tan(vpalpha)*tan(vpalpha)))/(maxRadius*maxRadius*8)));
     double vpcospt = (vpdeg>=0 ? 1.0 : -1.0) * cos (vpteta), vptanpt = tan (vpteta);
+
+    // auxiliary variables for horizontal perspective correction
     double hpdeg = params->perspective.horizontal / 100.0 * 45.0;
     double hpalpha = (90.0 - hpdeg) / 180.0 * RT_PI;
-    double hpteta  = fabs(hpalpha-RT_PI/2)<1e-3 ? 0.0 : acos ((hpdeg>0 ? 1.0 : -1.0) * sqrt((-oH*oH*tan(hpalpha)*tan(hpalpha) + (hpdeg>0 ? 1.0 : -1.0) * oH*tan(hpalpha)*sqrt(16*maxRadius*maxRadius+oH*oH*tan(hpalpha)*tan(hpalpha)))/(maxRadius*maxRadius*8)));
+    double hpteta  = fabs(hpalpha-RT_PI/2)<3e-4 ? 0.0 : acos ((hpdeg>0 ? 1.0 : -1.0) * sqrt((-oH*oH*tan(hpalpha)*tan(hpalpha) + (hpdeg>0 ? 1.0 : -1.0) * oH*tan(hpalpha)*sqrt(16*maxRadius*maxRadius+oH*oH*tan(hpalpha)*tan(hpalpha)))/(maxRadius*maxRadius*8)));
     double hpcospt = (hpdeg>=0 ? 1.0 : -1.0) * cos (hpteta), hptanpt = tan (hpteta);
 
-	double ascale = ascaleDef>0 ? ascaleDef : (params->commonTrans.autofill ? getTransformAutoFill (oW, oH) : 1.0);
+	double ascale = ascaleDef>0 ? ascaleDef : (params->commonTrans.autofill ? getTransformAutoFill (oW, oH, pLCPMap) : 1.0);
 
 	for (size_t i=0; i<src.size(); i++) {
+        double x_d=src[i].x, y_d=src[i].y;
+        if (pLCPMap && params->lensProf.useDist) pLCPMap->correctDistortion(x_d,y_d);  // must be first transform
 
-		double y_d = ascale * (src[i].y - h2);
-		double x_d = ascale * (src[i].x - w2);
+		y_d = ascale * (y_d - h2);
+		x_d = ascale * (x_d - w2);
 
-        y_d = y_d * maxRadius / (maxRadius + x_d*hptanpt);
-        x_d = x_d * maxRadius * hpcospt / (maxRadius + x_d*hptanpt);
+        if (needsPerspective()) {
+            // horizontal perspective transformation
+            y_d *= maxRadius / (maxRadius + x_d*hptanpt);
+            x_d *= maxRadius * hpcospt / (maxRadius + x_d*hptanpt);
 
-        x_d = x_d * maxRadius / (maxRadius - y_d*vptanpt);
-        y_d = y_d * maxRadius * vpcospt / (maxRadius - y_d*vptanpt);
+            // vertical perspective transformation
+            x_d *= maxRadius / (maxRadius - y_d*vptanpt);
+            y_d *= maxRadius * vpcospt / (maxRadius - y_d*vptanpt);
+        }
 
+        // rotate
 		double Dx = x_d * cost - y_d * sint;
 		double Dy = x_d * sint + y_d * cost;
 
-		double r = sqrt(Dx*Dx + Dy*Dy) / maxRadius;
-		double s = 1.0 - a + a * r ;
+        // distortion correction
+        double s = 1;
+        if (needsDist) {
+            double r = sqrt(Dx*Dx + Dy*Dy) / maxRadius;  // sqrt is slow
+            s = 1.0 - distAmount + distAmount * r ;
+        }
+
+        // LCP CA is not reflected in preview (and very small), so don't add it here
 
 		red.push_back (Coord2D(Dx*(s+params->cacorrection.red)+w2, Dy*(s+params->cacorrection.red)+h2));
 		green.push_back (Coord2D(Dx*s+w2, Dy*s+h2));
 		blue.push_back (Coord2D(Dx*(s+params->cacorrection.blue)+w2, Dy*(s+params->cacorrection.blue)+h2));
 	}
 
-	if (clipresize) {
+    // Clip all points and track if they were any corrections
         for (size_t i=0; i<src.size(); i++) {
             red[i].x = CLIPTOC(red[i].x,0,W-1,clipped);
             red[i].y = CLIPTOC(red[i].y,0,H-1,clipped);
@@ -103,16 +121,19 @@ bool ImProcFunctions::transCoord (int W, int H, std::vector<Coord2D> &src, std::
             blue[i].x = CLIPTOC(blue[i].x,0,W-1,clipped);
             blue[i].y = CLIPTOC(blue[i].y,0,H-1,clipped);
         }
-    }
+
     return clipped;
 }
 
-bool ImProcFunctions::transCoord (int W, int H, int x, int y, int w, int h, int& xv, int& yv, int& wv, int& hv, double ascaleDef) {
+// Transform all corners and critical sidelines of an image
+bool ImProcFunctions::transCoord (int W, int H, int x, int y, int w, int h, int& xv, int& yv, int& wv, int& hv, double ascaleDef, const LCPMapper *pLCPMap) {
+    const int DivisionsPerBorder=32;
 
     int x1 = x, y1 = y;
     int x2 = x1 + w - 1;
     int y2 = y1 + h - 1;
 
+    // Build all edge points and half-way points
     std::vector<Coord2D> corners (8);
     corners[0].set (x1, y1);
     corners[1].set (x1, y2);
@@ -122,13 +143,15 @@ bool ImProcFunctions::transCoord (int W, int H, int x, int y, int w, int h, int&
     corners[5].set ((x1+x2)/2, y2);
     corners[6].set (x1, (y1+y2)/2);
     corners[7].set (x2, (y1+y2)/2);
-    int xstep = (x2-x1)/16;
+
+    // Add several steps inbetween
+    int xstep = (x2-x1)/DivisionsPerBorder;
     if (xstep<1) xstep = 1;
     for (int i=x1+xstep; i<=x2-xstep; i+=xstep) {
         corners.push_back (Coord2D (i, y1));
         corners.push_back (Coord2D (i, y2));
     }
-    int ystep = (y2-y1)/16;
+    int ystep = (y2-y1)/DivisionsPerBorder;
     if (ystep<1) ystep = 1;
     for (int i=y1+ystep; i<=y2-ystep; i+=ystep) {
         corners.push_back (Coord2D (x1, i));
@@ -137,13 +160,15 @@ bool ImProcFunctions::transCoord (int W, int H, int x, int y, int w, int h, int&
 
     std::vector<Coord2D> r, g, b;
 
-    bool result = transCoord (W, H, corners, r, g, b, ascaleDef);
+    bool clipped = transCoord (W, H, corners, r, g, b, ascaleDef, pLCPMap);
 
+    // Merge all R G Bs into one X/Y pool
     std::vector<Coord2D> transCorners;
     transCorners.insert (transCorners.end(), r.begin(), r.end());
     transCorners.insert (transCorners.end(), g.begin(), g.end());
     transCorners.insert (transCorners.end(), b.begin(), b.end());
 
+    // find the min/max of all coordinates, so the borders
     double x1d = transCorners[0].x;
     for (size_t i=1; i<transCorners.size(); i++)
         if (transCorners[i].x<x1d)
@@ -173,7 +198,7 @@ bool ImProcFunctions::transCoord (int W, int H, int x, int y, int w, int h, int&
     wv = x2v - x1v + 1;
     hv = y2v - y1v + 1;
 
-    return result;
+    return clipped;
 }
 
 void ImProcFunctions::transform (Imagefloat* original, Imagefloat* transformed, int cx, int cy, int sx, int sy, int oW, int oH, 
@@ -228,16 +253,18 @@ void ImProcFunctions::transformVignetteOnly (Imagefloat* original, Imagefloat* t
 		for (int x=0; x<transformed->width; x++) {
 			double vig_x_d = (double) (x + cx) - vig_w2 ;
 			double r = sqrt(vig_x_d*vig_x_d + vig_y_d*vig_y_d);
-			double vign = v + mul * tanh (b*(maxRadius-r) / maxRadius);
-			transformed->r[y][x] = original->r[y][x] / vign;
-			transformed->g[y][x] = original->g[y][x] / vign;
-			transformed->b[y][x] = original->b[y][x] / vign;
+			double vign = std::max(v + mul * tanh (b*(maxRadius-r) / maxRadius), 0.001);
+			transformed->r(y,x) = original->r(y,x) / vign;
+			transformed->g(y,x) = original->g(y,x) / vign;
+			transformed->b(y,x) = original->b(y,x) / vign;
 		}
 	}
 }
 
 // Transform WITH scaling (opt.) and CA, cubic interpolation
 #include "cubintch.cc"
+#include "cubint.cc"
+
 void ImProcFunctions::transformHighQuality (Imagefloat* original, Imagefloat* transformed, int cx, int cy, int sx, int sy, int oW, int oH, 
     const LCPMapper *pLCPMap, bool fullImage) {
 
@@ -248,14 +275,14 @@ void ImProcFunctions::transformHighQuality (Imagefloat* original, Imagefloat* tr
 	calcVignettingParams(oW, oH, params->vignetting, vig_w2, vig_h2, maxRadius, v, b, mul);
 
     float** chOrig[3];
-    chOrig[0] = original->r;
-    chOrig[1] = original->g;
-    chOrig[2] = original->b;
+    chOrig[0] = original->r.ptrs;
+    chOrig[1] = original->g.ptrs;
+    chOrig[2] = original->b.ptrs;
 
     float** chTrans[3];
-    chTrans[0] = transformed->r;
-    chTrans[1] = transformed->g;
-    chTrans[2] = transformed->b;
+    chTrans[0] = transformed->r.ptrs;
+    chTrans[1] = transformed->g.ptrs;
+    chTrans[2] = transformed->b.ptrs;
 
     // auxiliary variables for c/a correction
     double chDist[3];
@@ -274,23 +301,24 @@ void ImProcFunctions::transformHighQuality (Imagefloat* original, Imagefloat* tr
 	// auxiliary variables for vertical perspective correction
     double vpdeg = params->perspective.vertical / 100.0 * 45.0;
     double vpalpha = (90.0 - vpdeg) / 180.0 * RT_PI;
-    double vpteta  = fabs(vpalpha-RT_PI/2)<1e-3 ? 0.0 : acos ((vpdeg>0 ? 1.0 : -1.0) * sqrt((-SQR(oW*tan(vpalpha)) + (vpdeg>0 ? 1.0 : -1.0) *
+    double vpteta  = fabs(vpalpha-RT_PI/2)<3e-4 ? 0.0 : acos ((vpdeg>0 ? 1.0 : -1.0) * sqrt((-SQR(oW*tan(vpalpha)) + (vpdeg>0 ? 1.0 : -1.0) *
 																oW*tan(vpalpha)*sqrt(SQR(4*maxRadius)+SQR(oW*tan(vpalpha))))/(SQR(maxRadius)*8)));
     double vpcospt = (vpdeg>=0 ? 1.0 : -1.0) * cos (vpteta), vptanpt = tan (vpteta);
 
 	// auxiliary variables for horizontal perspective correction
     double hpdeg = params->perspective.horizontal / 100.0 * 45.0;
     double hpalpha = (90.0 - hpdeg) / 180.0 * RT_PI;
-    double hpteta  = fabs(hpalpha-RT_PI/2)<1e-3 ? 0.0 : acos ((hpdeg>0 ? 1.0 : -1.0) * sqrt((-SQR(oH*tan(hpalpha)) + (hpdeg>0 ? 1.0 : -1.0) *
+    double hpteta  = fabs(hpalpha-RT_PI/2)<3e-4 ? 0.0 : acos ((hpdeg>0 ? 1.0 : -1.0) * sqrt((-SQR(oH*tan(hpalpha)) + (hpdeg>0 ? 1.0 : -1.0) *
 																oH*tan(hpalpha)*sqrt(SQR(4*maxRadius)+SQR(oH*tan(hpalpha))))/(SQR(maxRadius)*8)));
     double hpcospt = (hpdeg>=0 ? 1.0 : -1.0) * cos (hpteta), hptanpt = tan (hpteta);
 
-	double ascale = params->commonTrans.autofill ? getTransformAutoFill (oW, oH) : 1.0;
+	double ascale = params->commonTrans.autofill ? getTransformAutoFill (oW, oH, fullImage ? pLCPMap : NULL) : 1.0;
 
     // smaller crop images are a problem, so only when processing fully
-    bool enableLCPCA   = pLCPMap && params->lensProf.useCA && fullImage; 
+            bool enableLCPCA   = pLCPMap && params->lensProf.useCA && fullImage && pLCPMap->enableCA; 
     bool enableLCPDist = pLCPMap && params->lensProf.useDist && fullImage;
     if (enableLCPCA) enableLCPDist=false;
+            bool enableCA = enableLCPCA || needsCA();
 
 	// main cycle
 	#pragma omp parallel for if (multiThread)
@@ -336,7 +364,7 @@ void ImProcFunctions::transformHighQuality (Imagefloat* original, Imagefloat* tr
                 r2=sqrt(vig_Dx*vig_Dx + vig_Dy*vig_Dy);
             }
 
-            for (int c=0; c<3; c++) {
+                    for (int c=0; c < (enableCA ? 3 : 1); c++) {
                 double Dx = Dxc * (s + chDist[c]);
                 double Dy = Dyc * (s + chDist[c]);
 
@@ -356,23 +384,40 @@ void ImProcFunctions::transformHighQuality (Imagefloat* original, Imagefloat* tr
 					// multiplier for vignetting correction
 					double vignmul = 1.0;
 					if (needsVignetting())
-						vignmul /= (v + mul * tanh (b*(maxRadius-s*r2) / maxRadius));
+						vignmul /= std::max(v + mul * tanh (b*(maxRadius-s*r2) / maxRadius), 0.001);
 
 					if (yc > 0 && yc < original->height-2 && xc > 0 && xc < original->width-2) {
                         // all interpolation pixels inside image
-                        interpolateTransformChannelsCubic (chOrig[c], xc-1, yc-1, Dx, Dy, &(chTrans[c][y][x]), vignmul);
+                        if (enableCA)
+                            interpolateTransformChannelsCubic (chOrig[c], xc-1, yc-1, Dx, Dy, &(chTrans[c][y][x]), vignmul);
+                        else
+                            interpolateTransformCubic (original, xc-1, yc-1, Dx, Dy, &(transformed->r(y,x)), &(transformed->g(y,x)), &(transformed->b(y,x)), vignmul);
                     } else { 
                         // edge pixels
 						int y1 = LIM(yc,   0, original->height-1);
 						int y2 = LIM(yc+1, 0, original->height-1);
 						int x1 = LIM(xc,   0, original->width-1);
 						int x2 = LIM(xc+1, 0, original->width-1);
+
+                                if (enableCA) {
                         chTrans[c][y][x] = vignmul * (chOrig[c][y1][x1]*(1.0-Dx)*(1.0-Dy) + chOrig[c][y1][x2]*Dx*(1.0-Dy) + chOrig[c][y2][x1]*(1.0-Dx)*Dy + chOrig[c][y2][x2]*Dx*Dy);
+                                } else {
+                                    transformed->r(y,x) = vignmul*(original->r(y1,x1)*(1.0-Dx)*(1.0-Dy) + original->r(y1,x2)*Dx*(1.0-Dy) + original->r(y2,x1)*(1.0-Dx)*Dy + original->r(y2,x2)*Dx*Dy);
+                                    transformed->g(y,x) = vignmul*(original->g(y1,x1)*(1.0-Dx)*(1.0-Dy) + original->g(y1,x2)*Dx*(1.0-Dy) + original->g(y2,x1)*(1.0-Dx)*Dy + original->g(y2,x2)*Dx*Dy);
+                                    transformed->b(y,x) = vignmul*(original->b(y1,x1)*(1.0-Dx)*(1.0-Dy) + original->b(y1,x2)*Dx*(1.0-Dy) + original->b(y2,x1)*(1.0-Dx)*Dy + original->b(y2,x2)*Dx*Dy);
 					}
 				}
-				else
+                        }
+                        else {
+                            if (enableCA) {
 					// not valid (source pixel x,y not inside source image, etc.)
 					chTrans[c][y][x] = 0;
+                            } else {
+                                transformed->r(y,x) = 0;
+                                transformed->g(y,x) = 0;
+                                transformed->b(y,x) = 0;
+                            }
+                        }
 			}
         }
     }
@@ -398,16 +443,16 @@ void ImProcFunctions::transformPreview (Imagefloat* original, Imagefloat* transf
 	// auxiliary variables for vertical perspective correction
     double vpdeg = params->perspective.vertical / 100.0 * 45.0;
     double vpalpha = (90 - vpdeg) / 180.0 * RT_PI;
-    double vpteta  = fabs(vpalpha-RT_PI/2)<1e-3 ? 0.0 : acos ((vpdeg>0 ? 1.0 : -1.0) * sqrt((-oW*oW*tan(vpalpha)*tan(vpalpha) + (vpdeg>0 ? 1.0 : -1.0) * oW*tan(vpalpha)*sqrt(16*maxRadius*maxRadius+oW*oW*tan(vpalpha)*tan(vpalpha)))/(maxRadius*maxRadius*8)));
+    double vpteta  = fabs(vpalpha-RT_PI/2)<3e-4 ? 0.0 : acos ((vpdeg>0 ? 1.0 : -1.0) * sqrt((-oW*oW*tan(vpalpha)*tan(vpalpha) + (vpdeg>0 ? 1.0 : -1.0) * oW*tan(vpalpha)*sqrt(16*maxRadius*maxRadius+oW*oW*tan(vpalpha)*tan(vpalpha)))/(maxRadius*maxRadius*8)));
     double vpcospt = (vpdeg>=0 ? 1.0 : -1.0) * cos (vpteta), vptanpt = tan (vpteta);
 
 	// auxiliary variables for horizontal perspective correction
     double hpdeg = params->perspective.horizontal / 100.0 * 45.0;
     double hpalpha = (90 - hpdeg) / 180.0 * RT_PI;
-    double hpteta  = fabs(hpalpha-RT_PI/2)<1e-3 ? 0.0 : acos ((hpdeg>0 ? 1.0 : -1.0) * sqrt((-oH*oH*tan(hpalpha)*tan(hpalpha) + (hpdeg>0 ? 1.0 : -1.0) * oH*tan(hpalpha)*sqrt(16*maxRadius*maxRadius+oH*oH*tan(hpalpha)*tan(hpalpha)))/(maxRadius*maxRadius*8)));
+    double hpteta  = fabs(hpalpha-RT_PI/2)<3e-4 ? 0.0 : acos ((hpdeg>0 ? 1.0 : -1.0) * sqrt((-oH*oH*tan(hpalpha)*tan(hpalpha) + (hpdeg>0 ? 1.0 : -1.0) * oH*tan(hpalpha)*sqrt(16*maxRadius*maxRadius+oH*oH*tan(hpalpha)*tan(hpalpha)))/(maxRadius*maxRadius*8)));
     double hpcospt = (hpdeg>=0 ? 1.0 : -1.0) * cos (hpteta), hptanpt = tan (hpteta);
 
-	double ascale = params->commonTrans.autofill ? getTransformAutoFill (oW, oH) : 1.0;
+	double ascale = params->commonTrans.autofill ? getTransformAutoFill (oW, oH, pLCPMap) : 1.0;
 
     // main cycle
 	#pragma omp parallel for if (multiThread)
@@ -468,13 +513,13 @@ void ImProcFunctions::transformPreview (Imagefloat* original, Imagefloat* transf
                 // multiplier for vignetting correction
             	double vignmul = 1.0;
                 if (needsVignetting())
-                	vignmul /= (v + mul * tanh (b*(maxRadius-s*r2) / maxRadius));
+                	vignmul /= std::max(v + mul * tanh (b*(maxRadius-s*r2) / maxRadius), 0.001);
 
                 if (yc < original->height-1 && xc < original->width-1) {  
                     // all interpolation pixels inside image
-                    transformed->r[y][x] = vignmul*(original->r[yc][xc]*(1.0-Dx)*(1.0-Dy) + original->r[yc][xc+1]*Dx*(1.0-Dy) + original->r[yc+1][xc]*(1.0-Dx)*Dy + original->r[yc+1][xc+1]*Dx*Dy);
-                    transformed->g[y][x] = vignmul*(original->g[yc][xc]*(1.0-Dx)*(1.0-Dy) + original->g[yc][xc+1]*Dx*(1.0-Dy) + original->g[yc+1][xc]*(1.0-Dx)*Dy + original->g[yc+1][xc+1]*Dx*Dy);
-                    transformed->b[y][x] = vignmul*(original->b[yc][xc]*(1.0-Dx)*(1.0-Dy) + original->b[yc][xc+1]*Dx*(1.0-Dy) + original->b[yc+1][xc]*(1.0-Dx)*Dy + original->b[yc+1][xc+1]*Dx*Dy);
+                    transformed->r(y,x) = vignmul*(original->r(yc,xc)*(1.0-Dx)*(1.0-Dy) + original->r(yc,xc+1)*Dx*(1.0-Dy) + original->r(yc+1,xc)*(1.0-Dx)*Dy + original->r(yc+1,xc+1)*Dx*Dy);
+                    transformed->g(y,x) = vignmul*(original->g(yc,xc)*(1.0-Dx)*(1.0-Dy) + original->g(yc,xc+1)*Dx*(1.0-Dy) + original->g(yc+1,xc)*(1.0-Dx)*Dy + original->g(yc+1,xc+1)*Dx*Dy);
+                    transformed->b(y,x) = vignmul*(original->b(yc,xc)*(1.0-Dx)*(1.0-Dy) + original->b(yc,xc+1)*Dx*(1.0-Dy) + original->b(yc+1,xc)*(1.0-Dx)*Dy + original->b(yc+1,xc+1)*Dx*Dy);
                 }
                 else { 
                     // edge pixels
@@ -482,36 +527,39 @@ void ImProcFunctions::transformPreview (Imagefloat* original, Imagefloat* transf
                 	int y2 = LIM(yc+1, 0, original->height-1);
                 	int x1 = LIM(xc,   0, original->width-1);
                 	int x2 = LIM(xc+1, 0, original->width-1);
-                    transformed->r[y][x] = vignmul*(original->r[y1][x1]*(1.0-Dx)*(1.0-Dy) + original->r[y1][x2]*Dx*(1.0-Dy) + original->r[y2][x1]*(1.0-Dx)*Dy + original->r[y2][x2]*Dx*Dy);
-                    transformed->g[y][x] = vignmul*(original->g[y1][x1]*(1.0-Dx)*(1.0-Dy) + original->g[y1][x2]*Dx*(1.0-Dy) + original->g[y2][x1]*(1.0-Dx)*Dy + original->g[y2][x2]*Dx*Dy);
-                    transformed->b[y][x] = vignmul*(original->b[y1][x1]*(1.0-Dx)*(1.0-Dy) + original->b[y1][x2]*Dx*(1.0-Dy) + original->b[y2][x1]*(1.0-Dx)*Dy + original->b[y2][x2]*Dx*Dy);
+                    transformed->r(y,x) = vignmul*(original->r(y1,x1)*(1.0-Dx)*(1.0-Dy) + original->r(y1,x2)*Dx*(1.0-Dy) + original->r(y2,x1)*(1.0-Dx)*Dy + original->r(y2,x2)*Dx*Dy);
+                    transformed->g(y,x) = vignmul*(original->g(y1,x1)*(1.0-Dx)*(1.0-Dy) + original->g(y1,x2)*Dx*(1.0-Dy) + original->g(y2,x1)*(1.0-Dx)*Dy + original->g(y2,x2)*Dx*Dy);
+                    transformed->b(y,x) = vignmul*(original->b(y1,x1)*(1.0-Dx)*(1.0-Dy) + original->b(y1,x2)*Dx*(1.0-Dy) + original->b(y2,x1)*(1.0-Dx)*Dy + original->b(y2,x2)*Dx*Dy);
                 }
             }
             else {
                 // not valid (source pixel x,y not inside source image, etc.)
-                transformed->r[y][x] = 0;
-                transformed->g[y][x] = 0;
-                transformed->b[y][x] = 0;
+                transformed->r(y,x) = 0;
+                transformed->g(y,x) = 0;
+                transformed->b(y,x) = 0;
             }
         }
     }
 }
 
-double ImProcFunctions::getTransformAutoFill (int oW, int oH) {
+double ImProcFunctions::getTransformAutoFill (int oW, int oH, const LCPMapper *pLCPMap) {
+    if (!needsCA() && !needsDistortion() && !needsRotation() && !needsPerspective() && (!params->lensProf.useDist || pLCPMap==NULL))
+        return 1;
 
-	double scaleU = 1.0;
-	double scaleL = 0.001;
-	while (scaleU - scaleL > 0.001) {
-		double scale = (scaleU + scaleL) / 2.0;
+	double scaleU = 2, scaleL = 0.001;  // upper and lower border, iterate inbetween
+
+	do {
+		double scale = (scaleU + scaleL) * 0.5;
 
         int orx, ory, orw, orh;
-        bool clipped = transCoord (oW, oH, 0, 0, oW, oH, orx, ory, orw, orh, scale);
+        bool clipped = transCoord (oW, oH, 0, 0, oW, oH, orx, ory, orw, orh, scale, pLCPMap);
 
         if (clipped)
         	scaleU = scale;
         else
         	scaleL = scale;
-	}
+	} while (scaleU - scaleL > 0.001);
+
 	return scaleL;
 }
 
