@@ -43,7 +43,7 @@ using namespace rtexif;
 
 LCPModelCommon::LCPModelCommon() {
     focLenX=focLenY=-1; imgXCenter=imgYCenter=0.5;
-    x0=y0=fx=fy=0;
+    x0=y0=fx=fy=meanErr=0; badErr=false;
     for (int i=0;i<5;i++) param[i]=0;
     scaleFac=1;
 }
@@ -53,7 +53,7 @@ bool LCPModelCommon::empty() const {
 }
 
 void LCPModelCommon::print() const {
-    printf("focLen %g/%g; imgCenter %g/%g; scale %g\n",focLenX,focLenY,imgXCenter,imgYCenter,scaleFac);
+    printf("focLen %g/%g; imgCenter %g/%g; scale %g; err %g\n",focLenX,focLenY,imgXCenter,imgYCenter,scaleFac,meanErr);
     printf("xy0 %g/%g  fxy %g/%g\n",x0,y0,fx,fy);
     printf("param: %g/%g/%g/%g/%g\n",param[0],param[1],param[2],param[3],param[4]);
 }
@@ -67,6 +67,7 @@ void LCPModelCommon::merge(const LCPModelCommon& a, const LCPModelCommon& b, flo
     imgXCenter = facA * a.imgXCenter + facB * b.imgXCenter;
     imgYCenter = facA * a.imgYCenter + facB * b.imgYCenter;
     scaleFac   = facA * a.scaleFac   + facB * b.scaleFac;
+    meanErr    = facA * a.meanErr    + facB * b.meanErr;
 
     for (int i=0;i<5;i++) param[i]= facA * a.param[i] + facB * b.param[i];
 }
@@ -103,8 +104,9 @@ LCPPersModel::LCPPersModel() {
 
 // mode: 0=distortion, 1=vignette, 2=CA
 bool LCPPersModel::hasModeData(int mode) const {
-    return (mode==0 && !vignette.empty()) || (mode==1 && !base.empty())
-        || (mode==2 && !chromRG.empty() && !chromG.empty() && !chromBG.empty());
+    return (mode==0 && !vignette.empty() && !vignette.badErr) || (mode==1 && !base.empty() && !base.badErr)
+        || (mode==2 && !chromRG.empty() && !chromG.empty() && !chromBG.empty() &&
+            !chromRG.badErr && !chromG.badErr && !chromBG.badErr);
 }
 
 void LCPPersModel::print() const {
@@ -142,6 +144,8 @@ LCPMapper::LCPMapper(LCPProfile* pProf, float focalLength, float focalLength35mm
         pProf->calcParams(2, focalLength, focusDist, aperture, &chrom[0], &chrom[1], &chrom[2]);
         for (int i=0;i<3;i++) chrom[i].prepareParams(fullWidth, fullHeight, focalLength, focalLength35mm, pProf->sensorFormatFactor, swapXY, mirrorX, mirrorY);
     }
+
+    enableCA = !vignette && focusDist>0;
 }
 
 void LCPMapper::correctDistortion(double& x, double& y) const {
@@ -162,6 +166,8 @@ void LCPMapper::correctDistortion(double& x, double& y) const {
 }
 
 void LCPMapper::correctCA(double& x, double& y, int channel) const {
+    if (!enableCA) return;
+
     double rsqr, xgreen, ygreen;
 
     // First calc the green channel like normal distortion
@@ -171,7 +177,7 @@ void LCPMapper::correctCA(double& x, double& y, int channel) const {
     // Green contains main distortion, just like base
     if (useCADist) {
         const float* aDist = chrom[1].param;
-    double rsqr      = xd*xd+yd*yd;
+        double rsqr      = xd*xd+yd*yd;
         double xfac=aDist[swapXY?3:4], yfac=aDist[swapXY?4:3];
 
         double commonFac = (((aDist[2]*rsqr + aDist[1])*rsqr + aDist[0])*rsqr + 1.)
@@ -229,8 +235,8 @@ LCPProfile::LCPProfile(Glib::ustring fname) {
 
     isFisheye=inCamProfiles=firstLIDone=inPerspect=inAlternateLensID=false;
     sensorFormatFactor=1;
-    for (int i=0;i<2000;i++) aPersModel[i]=NULL;
-    persModelCount=0;
+    for (int i=0;i<MaxPersModelCount;i++) aPersModel[i]=NULL;
+    persModelCount=0; *inInvalidTag=0;
 
     FILE *pFile = safe_g_fopen(fname, "rb");
 
@@ -245,6 +251,69 @@ LCPProfile::LCPProfile(Glib::ustring fname) {
     fclose(pFile);
 
     XML_ParserFree(parser);
+
+    //printf("Parsing %s\n", fname.c_str());
+    // Two phase filter: first filter out the very rough ones, that distord the average a lot
+    // force it, even if there are few frames (community profiles)
+    filterBadFrames(2.0, 0);
+    // from the non-distorded, filter again on new average basis, but only if there are enough frames left
+    filterBadFrames(1.5, 100);
+}
+
+// from all frames not marked as bad already, take average and filter out frames with higher deviation than this if there are enough values
+int LCPProfile::filterBadFrames(double maxAvgDevFac, int minFramesLeft) {
+    // take average error per type, then calculated the maximum deviation allowed
+    double errBase=0, errChrom=0, errVignette=0;
+    int baseCount=0, chromCount=0, vignetteCount=0;
+    for (int pm=0;pm<MaxPersModelCount && aPersModel[pm];pm++) {
+        if (aPersModel[pm]->hasModeData(0)) {
+            errVignette+=aPersModel[pm]->vignette.meanErr;
+            vignetteCount++;
+        }
+
+        if (aPersModel[pm]->hasModeData(1)) {
+            errBase+=aPersModel[pm]->base.meanErr;
+            baseCount++;
+        }
+
+        if (aPersModel[pm]->hasModeData(2)) {
+            errChrom+=std::max(std::max(aPersModel[pm]->chromRG.meanErr,aPersModel[pm]->chromG.meanErr),aPersModel[pm]->chromBG.meanErr);
+            chromCount++;
+        }
+    }
+
+    // Only if we have enough frames, filter out errors
+    int filtered=0;
+
+    if (baseCount+chromCount+vignetteCount>=minFramesLeft) {
+        if (baseCount>0) errBase/=(double)baseCount;
+        if (chromCount>0) errChrom/=(double)chromCount;
+        if (vignetteCount>0) errVignette/=(double)vignetteCount;
+
+        // Now mark all the bad ones as bad, and hasModeData will return false;
+        for (int pm=0;pm<MaxPersModelCount && aPersModel[pm];pm++) {
+            if (aPersModel[pm]->hasModeData(0) && aPersModel[pm]->vignette.meanErr > maxAvgDevFac * errVignette) {
+                aPersModel[pm]->vignette.badErr=true;
+                filtered++;
+            }
+
+            if (aPersModel[pm]->hasModeData(1) && aPersModel[pm]->base.meanErr > maxAvgDevFac * errBase) {
+                aPersModel[pm]->base.badErr=true;
+                filtered++;
+            }
+
+            if (aPersModel[pm]->hasModeData(2) && 
+                (aPersModel[pm]->chromRG.meanErr > maxAvgDevFac * errChrom || aPersModel[pm]->chromG.meanErr > maxAvgDevFac * errChrom 
+                 || aPersModel[pm]->chromBG.meanErr > maxAvgDevFac * errChrom)) {
+                aPersModel[pm]->chromRG.badErr=aPersModel[pm]->chromG.badErr=aPersModel[pm]->chromBG.badErr=true;
+                filtered++;
+            }
+        }
+
+        //printf("Filtered %.1f%% frames for maxAvgDevFac %g leaving %i\n", filtered*100./(baseCount+chromCount+vignetteCount), maxAvgDevFac, baseCount+chromCount+vignetteCount-filtered);
+    }
+
+    return filtered;
 }
 
 
@@ -276,7 +345,7 @@ void LCPProfile::calcParams(int mode, float focalLength, float focusDist, float
         pLow=pHigh;
     else if (!pHigh) 
         pHigh=pLow;
-    else if (mode==0 || focusDist>0) {
+    else {
         // Pass 2: We have some, so take the best aperture for vignette and best focus for CA and distortion
         // there are usually several frame per focal length. In the end pLow will have both flen and apterure/focdis below the target,
         // and vice versa pHigh
@@ -285,29 +354,52 @@ void LCPProfile::calcParams(int mode, float focalLength, float focusDist, float
         for (int pm=0;pm<persModelCount;pm++) {
             float aper=aPersModel[pm]->aperture; // float aperLog=log(aper);
             float focDist=aPersModel[pm]->focDist; float focDistLog=log(focDist)+euler;
+            double meanErr;
 
             if (aPersModel[pm]->hasModeData(mode)) {
                 if (mode==0) {
-                    // by aperture, and max out focus distance
+                    meanErr=aPersModel[pm]->vignette.meanErr;
+
+                    // by aperture (vignette), and max out focus distance
                     // tests showed doing this by log(aperture) is not as advisable
-                    if (aPersModel[pm]->focLen==bestFocLenLow && ((aper>=aperture && aper<pLow->aperture && pLow->aperture > aperture)
+                    if (aPersModel[pm]->focLen==bestFocLenLow && (
+                        (aper==aperture && pLow->vignette.meanErr>meanErr)
+                        || (aper>=aperture && aper<pLow->aperture && pLow->aperture > aperture)
                             || (aper<=aperture && (pLow->aperture>aperture || fabs(aperture-aper)<fabs(aperture - pLow->aperture))))) {
                     pLow=aPersModel[pm];
                 }
-                    if (aPersModel[pm]->focLen==bestFocLenHigh && ((aper<=aperture && aper>pHigh->aperture && pHigh->aperture < aperture)
+                    if (aPersModel[pm]->focLen==bestFocLenHigh && (
+                        (aper==aperture && pHigh->vignette.meanErr>meanErr)
+                        || (aper<=aperture && aper>pHigh->aperture && pHigh->aperture < aperture)
                             || (aper>=aperture && (pHigh->aperture<aperture || fabs(aperture-aper)<fabs(aperture - pHigh->aperture))))) {
                     pHigh=aPersModel[pm];
                 }
                 } else {
+                    meanErr = (mode==1 ? aPersModel[pm]->base.meanErr : aPersModel[pm]->chromG.meanErr);
+
+                    if (focusDist>0) {
                     // by focus distance
-                    if (aPersModel[pm]->focLen==bestFocLenLow && ((focDist>=focusDist && focDist<pLow->focDist && pLow->focDist > focusDist)
+                        if (aPersModel[pm]->focLen==bestFocLenLow && (
+                            (focDist==focusDist && (mode==1 ? pLow->base.meanErr : pLow->chromG.meanErr)>meanErr)
+                            || (focDist>=focusDist && focDist<pLow->focDist && pLow->focDist > focusDist)
                          || (focDist<=focusDist && (pLow->focDist>focusDist || fabs(focusDistLog-focDistLog)<fabs(focusDistLog - (log(pLow->focDist)+euler)))))) {
                         pLow=aPersModel[pm];
                     }
-                    if (aPersModel[pm]->focLen==bestFocLenHigh && ((focDist<=focusDist && focDist>pHigh->focDist && pHigh->focDist < focusDist)
+                        if (aPersModel[pm]->focLen==bestFocLenHigh && (
+                            (focDist==focusDist && (mode==1 ? pHigh->base.meanErr : pHigh->chromG.meanErr)>meanErr)
+                            || (focDist<=focusDist && focDist>pHigh->focDist && pHigh->focDist < focusDist)
                          || (focDist>=focusDist && (pHigh->focDist<focusDist || fabs(focusDistLog-focDistLog)<fabs(focusDistLog - (log(pHigh->focDist)+euler)))))) {
                         pHigh=aPersModel[pm];
                     }
+                    } else {
+                        // no focus distance available, just error
+                        if (aPersModel[pm]->focLen==bestFocLenLow && (mode==1 ? pLow->base.meanErr : pLow->chromG.meanErr)>meanErr) {
+                            pLow=aPersModel[pm];
+                        }
+                        if (aPersModel[pm]->focLen==bestFocLenHigh && (mode==1 ? pHigh->base.meanErr : pHigh->chromG.meanErr)>meanErr) {
+                            pHigh=aPersModel[pm];
+                        }
+                    }
                 }
             }
         }
@@ -366,12 +458,16 @@ void XMLCALL LCPProfile::XmlStartHandler(void *pLCPProfile, const char *el, cons
     LCPProfile *pProf=static_cast<LCPProfile*>(pLCPProfile);
     bool parseAttr=false;
 
+    if (*pProf->inInvalidTag) return;  // We ignore everything in dirty tag till it's gone
+
     // clean up tagname
     const char* src=strrchr(el,':');
     if (src==NULL) src=const_cast<char*>(el); else src++;
 
     strcpy(pProf->lastTag,src);
 
+    if (!strcmp("VignetteModelPiecewiseParam",src)) strcpy(pProf->inInvalidTag,src);
+
     if (!strcmp("CameraProfiles",src)) pProf->inCamProfiles=true;
     if (!strcmp("AlternateLensIDs",src)) pProf->inAlternateLensID=true;
     if (!pProf->inCamProfiles || pProf->inAlternateLensID) return;
@@ -424,7 +520,7 @@ void XMLCALL LCPProfile::XmlStartHandler(void *pLCPProfile, const char *el, cons
 void XMLCALL LCPProfile::XmlTextHandler(void *pLCPProfile, const XML_Char *s, int len) {
     LCPProfile *pProf=static_cast<LCPProfile*>(pLCPProfile);
 
-    if (!pProf->inCamProfiles || pProf->inAlternateLensID) return;
+    if (!pProf->inCamProfiles || pProf->inAlternateLensID || *pProf->inInvalidTag) return;
 
     // Check if it contains non-whitespaces (there are several calls to this for one tag unfortunately)
     bool onlyWhiteSpace=true;
@@ -490,6 +586,8 @@ void XMLCALL LCPProfile::XmlTextHandler(void *pLCPProfile, const XML_Char *s, in
         pProf->pCurCommon->imgYCenter=atof(raw);
     else if (!strcmp("ScaleFactor",tag)) 
         pProf->pCurCommon->scaleFac=atof(raw);
+    else if (!strcmp("ResidualMeanError",tag)) 
+        pProf->pCurCommon->meanErr=atof(raw);
     else if (!strcmp("RadialDistortParam1",tag) || !strcmp("VignetteModelParam1",tag)) 
         pProf->pCurCommon->param[0]=atof(raw);
     else if (!strcmp("RadialDistortParam2",tag) || !strcmp("VignetteModelParam2",tag)) 
@@ -500,13 +598,17 @@ void XMLCALL LCPProfile::XmlTextHandler(void *pLCPProfile, const XML_Char *s, in
         pProf->pCurCommon->param[3]=atof(raw);
     else if (!strcmp("RadialDistortParam5",tag) || !strcmp("TangentialDistortParam2",tag)) 
         pProf->pCurCommon->param[4]=atof(raw);    
-    else if (!strcmp("RadialDistortParam6",tag) || !strcmp("TangentialDistortParam3",tag)) 
-        pProf->pCurCommon->param[5]=atof(raw);    
 }
 
 void XMLCALL LCPProfile::XmlEndHandler(void *pLCPProfile, const char *el) {
     LCPProfile *pProf=static_cast<LCPProfile*>(pLCPProfile);
 
+    // We ignore everything in dirty tag till it's gone
+    if (*pProf->inInvalidTag) {
+        if (strstr(el,pProf->inInvalidTag)) *pProf->inInvalidTag=0;
+        return;
+    }
+
     if (strstr(el,":CameraProfiles")) pProf->inCamProfiles=false;
     if (strstr(el,":AlternateLensIDs")) pProf->inAlternateLensID=false;
 
