@@ -34,6 +34,8 @@ namespace rtengine {
        float focLenX, focLenY, imgXCenter, imgYCenter;
        float param[5];  // k1..k5, resp. alpha1..5
        float scaleFac;  // alpha0
+       double meanErr;
+       bool badErr;
 
        double x0,y0,fx,fy;  // prepared params
 
@@ -61,7 +63,7 @@ namespace rtengine {
     class LCPProfile {
         // Temporary data for parsing
         bool inCamProfiles,firstLIDone,inPerspect,inAlternateLensID;
-        char lastTag[256];
+        char lastTag[256],inInvalidTag[256];
         LCPPersModel* pCurPersModel;
         LCPModelCommon* pCurCommon;
 
@@ -69,6 +71,8 @@ namespace rtengine {
         static void XMLCALL XmlTextHandler (void *pLCPProfile, const XML_Char *s, int len);
         static void XMLCALL XmlEndHandler  (void *pLCPProfile, const char *el);
 
+        int filterBadFrames(double maxAvgDevFac, int minFramesLeft);
+
     public:
         // Common data
         Glib::ustring profileName, lensPrettyName, cameraPrettyName, lens, camera;  // lens/camera(=model) can be auto-matched with DNG
@@ -77,7 +81,8 @@ namespace rtengine {
         int persModelCount;
 
         // The correction frames
-        LCPPersModel* aPersModel[2000];  // Do NOT use std::list or something, it's buggy in GCC!
+        static const int MaxPersModelCount=3000;
+        LCPPersModel* aPersModel[MaxPersModelCount];  // Do NOT use std::list or something, it's buggy in GCC!
 
         LCPProfile(Glib::ustring fname);
 
@@ -112,6 +117,8 @@ namespace rtengine {
         LCPModelCommon chrom[3];  // in order RedGreen/Green/BlueGreen
 
     public:
+        bool enableCA;  // is the mapper capable if CA correction?
+
         // precalculates the mapper.
         LCPMapper(LCPProfile* pProf, float focalLength, float focalLength35mm, float focusDist, float aperture, bool vignette, bool useCADistP, int fullWidth, int fullHeight,
             const CoarseTransformParams& coarse, int rawRotationDeg);
