@@ -32,7 +32,7 @@ enum ProcEvent {
     EvExpComp=7,
     EvHLCompr=8,
     EvSHCompr=9,
-    EvToneCurve=10,
+    EvToneCurve1=10,
     EvAutoExp=11,
     EvClip=12,
     EvLBrightness=13,
@@ -58,18 +58,18 @@ enum ProcEvent {
     EvLCPUseDist=33,
     EvLCPUseVign=34,
     EvLCPUseCA=35,
-    EvCBBoost=36,     // obsolete
+    EvFixedExp=36,
     EvWBMethod=37,
     EvWBTemp=38,
     EvWBGreen=39,
-    EvCShiftA=40,     // obsolete
-    EvCShiftB=41,     // obsolete
-    EvLDNEnabled=42,  // obsolete
+    EvToneCurveMode1=40,
+    EvToneCurve2=41,
+    EvToneCurveMode2=42,
     EvLDNRadius=43,   // obsolete
     EvLDNEdgeTolerance=44, // obsolete
     EvCDNEnabled=45,  // obsolete
-    EvCDNRadius=46,   // obsolete
-    EvCDNEdgeTolerance=47, // obsolete
+    EvBlendCMSMatrix=46,
+    EvDCPToneCurve=47,
     EvPrefProfile=48,
     EvSHEnabled=49,
     EvSHHighlights=50,
@@ -107,7 +107,7 @@ enum ProcEvent {
     EvSHHighQuality=82,
     EvPerspCorr=83,
     EvLCPFile=84,
-    EvEqlEnabled=85,// obsolete
+    EvRGBrCurveLumamode=85,
     EvIDNEnabled=86,
     EvIDNThresh=87,
     EvDPDNEnabled=88,
@@ -132,9 +132,9 @@ enum ProcEvent {
     EvHLComprThreshold=107,
     EvResizeBoundingBox=108,
     EvResizeAppliesTo=109,
-    EvLAvoidClip=110,
-    EvLSatLimiter=111,
-    EvLSatLimit=112,
+    EvLAvoidColorShift=110,
+    EvLSatLimiter=111,      // obsolete
+    EvLRSTProtection=112,
     EvDemosaicDCBIter=113,
     EvDemosaicFalseColorIter=114,
     EvDemosaicDCBEnhanced=115,
@@ -171,7 +171,7 @@ enum ProcEvent {
 	EvSharpenEdgeThreechannels=146,
 	EvSharpenMicroEnabled=147,
 	EvSharpenMicroMatrix=148,
-	EvDemosaicALLEnhanced=149,
+	EvDemosaicALLEnhanced=149,    // Disabled but not removed for now, may be reintroduced some day
 	EvVibranceEnabled=150,
 	EvVibrancePastels=151,
 	EvVibranceSaturated=152,
@@ -188,7 +188,50 @@ enum ProcEvent {
 	EvRGBgCurve=163,
 	EvRGBbCurve=164,
 	EvNeutralExp=165,
-	NUMOFEVENTS=166
+	EvLBWtoning=166,
+	EvLCCCurve=167,
+	EvLCHCurve=168,
+	EvVibranceSkinTonesCurve=169,
+	EvLLCCurve=170,
+	EvLLCredsk=171,
+	EvDPDNLdetail=172,
+	EvCATEnabled=173,
+	EvCATDegree=174,
+	EvCATMethodsur=175,
+	EvCATAdapscen=176,
+	EvCATAdapLum=177,
+	EvCATMethodWB=178,
+	EvCATJLight=179,
+	EvCATChroma=180,
+	EvCATAutoDegree=181,
+	EvCATContrast=182,
+	EvCATsurr=183,
+	EvCATgamut=184,
+	EvCATMethodalg=185,
+	EvCATRstpro=186,
+	EvCATQbright=187,
+	EvCATQContrast=188,
+	EvCATSChroma=189,
+	EvCATMChroma=190,
+	EvCAThue=191,
+	EvCATCurve1=192,
+	EvCATCurve2=193,
+	EvCATCurveMode1=194,
+	EvCATCurveMode2=195,
+	EvCATCurve3=196,
+	EvCATCurveMode3=197,
+	EvCATdatacie=198,
+	EvCATtonecie=199,
+	EvDPDNredchro=200,
+	EvDPDNbluechro=201,
+	EvDPDNmet=202,
+//	EvDPDNperform=201,
+	EvDemosaicLMMSEIter=203,
+	EvCATbadpix=204,
+	EvCATAutoAdap=205,
+	EvPFCurve=206,	
+	NUMOFEVENTS=207
 };
-}    
-#endif    
+}
+#endif
+
