@@ -21,22 +21,180 @@
 
 #include <glibmm.h>
 #include <vector>
+#include <cassert>
+#include <cstdio>
+#include <cmath>
 
 class ParamsEdited;
 
 namespace rtengine {
 namespace procparams {
 
+template <typename T>
+class Threshold {
+    public:
+        T value[4];
+
+    protected:
+        bool initEq1;
+        bool _isDouble;
+#ifndef NDEBUG
+        unsigned int part[5];
+#endif
+
+    public:
+        Threshold (T val1, T val2, bool startAtOne) {
+            initEq1 = startAtOne;
+            value[0] = val1;
+            value[1] = val2;
+            value[2] = T(0);
+            value[3] = T(0);
+            _isDouble = false;
+        }
+
+        Threshold (T val1, T val2, T val3, T val4, bool startAtOne) {
+            initEq1 = startAtOne;
+            value[0] = val1;
+            value[1] = val2;
+            value[2] = val3;
+            value[3] = val4;
+            _isDouble = true;
+        }
+
+        // for convenience, since 'values' is public
+        void setValues(T val1, T val2) {
+            value[0] = val1;
+            value[1] = val2;
+        }
+
+        // for convenience, since 'values' is public
+        void setValues(T val1, T val2, T val3, T val4) {
+            value[0] = val1;
+            value[1] = val2;
+            value[2] = val3;
+            value[3] = val4;
+        }
+
+        bool isDouble() const { return _isDouble; }
+
+        // RT: Type of the returned value
+        // RV: Type of the value on the X axis
+        // RV2: Type of the maximum value on the Y axis
+        template <typename RT, typename RV, typename RV2>
+        RT multiply(RV x, RV2 yMax) const {
+        	double val = double(x);
+            if (initEq1) {
+                if (_isDouble) {
+                	if (val == double(value[2]) && double(value[2]) == double(value[3]))
+                		// this handle the special case where the 2 right values are the same, then bottom one is sent back,
+                		// useful if one wants to keep the bottom value even beyond the x max bound
+                        return RT(0.);
+                    if (val >= double(value[3]))
+                        return RT(yMax);
+                    if (val > double(value[2]))
+                        return RT(double(yMax)*(val-double(value[2]))/(double(value[3])-double(value[2])));
+                }
+                if (val >= double(value[0]))
+                    return RT(0);
+                if (val > double(value[1]))
+                    return RT(double(yMax)*(1.-(val-double(value[0]))/(double(value[1])-double(value[0]))));
+                return RT(yMax);
+            }
+            else {
+                if (_isDouble) {
+                	if (val == double(value[2]) && double(value[2]) == double(value[3]))
+                		// this handle the special case where the 2 right values are the same, then top one is sent back,
+                		// useful if one wants to keep the top value even beyond the x max bound
+                        return RT(yMax);
+                    if (val >= double(value[2]))
+                        return RT(0);
+                    if (val > double(value[3]))
+                        return RT(double(yMax)*(1.-(val-double(value[3]))/(double(value[2])-double(value[3]))));
+                }
+                if (val >= double(value[1]))
+                    return RT(yMax);
+                if (val > double(value[0]))
+                    return RT(double(yMax)*(val-double(value[0]))/(double(value[1])-double(value[0])));
+                return RT(0);
+            }
+        }
+
+        // RT: Type of the returned value
+        // RV: Type of the value on the X axis
+        /*template <typename RT, typename RV>
+        RT getRatio(RV val) const {
+        	double val = double(val);
+            if (initEq1) {
+                if (_isDouble) {  // assuming that simple thresholds will be more frequent
+                    if (val >= double(value[3]))
+                        return RT(1);
+                    if (val > double(value[2]))
+                        return (val-double(value[2]))/(double(value[3])-double(value[2]));
+                }
+                if (val >= double(value[1]))
+                    return RT(0);
+                if (val > double(value[0]))
+                    return 1.-(val-double(value[0]))/(double(value[1])-double(value[0]));
+                return RT(1);
+            }
+            else {
+                if (_isDouble) {  // assuming that simple thresholds will be more frequent
+                    if (val >= double(value[3]))
+                        return RT(0);
+                    if (val > double(value[2]))
+                        return 1.-(val-double(value[2]))/(double(value[3])-double(value[2]));
+                }
+                if (val >= double(value[1]))
+                    return RT(1);
+                if (val > double(value[0]))
+                    return (val-double(value[0]))/(double(value[1])-double(value[0]));
+                return RT(0);
+            }
+        }*/
+
+        Threshold<T> & operator= (const Threshold<T> &rhs) {
+            value[0] = rhs.value[0];
+            value[1] = rhs.value[1];
+            value[2] = rhs.value[2];
+            value[3] = rhs.value[3];
+            initEq1 = rhs.initEq1;
+            _isDouble = rhs._isDouble;
+    	    return *this;
+        }
+
+        bool operator== (const Threshold<T> &rhs) const {
+            if (_isDouble)
+            	return fabs(value[0]-rhs.value[0])<1e-10
+            	    && fabs(value[1]-rhs.value[1])<1e-10
+            	    && fabs(value[2]-rhs.value[2])<1e-10
+            	    && fabs(value[3]-rhs.value[3])<1e-10;
+            else
+            	return fabs(value[0]-rhs.value[0])<1e-10
+            	    && fabs(value[1]-rhs.value[1])<1e-10;
+        }
+};
+
 /**
   * Parameters of the tone curve 
   */
 class ToneCurveParams {
 
     public:
+
+        enum eTCModeId {
+            TC_MODE_STD,               // Standard modes, the curve is applied on all component individually
+            TC_MODE_WEIGHTEDSTD,       // Weighted standard mode
+            TC_MODE_FILMLIKE,          // Film-like mode, as defined in Adobe's reference code
+            TC_MODE_SATANDVALBLENDING  // Modify the Saturation and Value channel
+        };
+
         bool        autoexp;
         double      clip;
         double      expcomp;
         std::vector<double>   curve;
+        std::vector<double>   curve2;
+        eTCModeId   curveMode;
+        eTCModeId   curveMode2;
         int         brightness;
         int         black;
         int         contrast;
@@ -55,12 +213,16 @@ class LCurveParams {
         std::vector<double>   lcurve;
         std::vector<double>   acurve;
         std::vector<double>   bcurve;
+        std::vector<double>   cccurve;
+        std::vector<double>   chcurve;
+        std::vector<double>   lccurve;
         int     brightness;
         int     contrast;
-        int     saturation;
-        bool    avoidclip;
-        bool    enable_saturationlimiter;
-        double  saturationlimit;
+        int     chromaticity;
+        bool    avoidcolorshift;
+        double  rstprotection;
+        bool    bwtoning;
+		bool    lcredsk;
 };
 
 /**
@@ -69,6 +231,7 @@ class LCurveParams {
 class RGBCurvesParams {
 
     public:
+        bool lumamode;
         std::vector<double>   rcurve;
         std::vector<double>   gcurve;
         std::vector<double>   bcurve;
@@ -80,20 +243,22 @@ class RGBCurvesParams {
 class SharpeningParams {
 
     public:
-        bool    enabled;
-        double  radius;
-        int     amount;
-        int     threshold;
-        bool    edgesonly;
-        double  edges_radius;
-        int     edges_tolerance;
-        bool    halocontrol;
-        int     halocontrol_amount;
-        Glib::ustring method;
-        int     deconvamount;
-        double  deconvradius;
-        int     deconviter;
-        int     deconvdamping;
+        bool           enabled;
+        double         radius;
+        int            amount;
+        Threshold<int> threshold;
+        bool           edgesonly;
+        double         edges_radius;
+        int            edges_tolerance;
+        bool           halocontrol;
+        int            halocontrol_amount;
+        Glib::ustring  method;
+        int            deconvamount;
+        double         deconvradius;
+        int            deconviter;
+        int            deconvdamping;
+
+        SharpeningParams() : threshold(20, 80, 2000, 1200, false) {};
 };
 class SharpenEdgeParams {
   public:
@@ -116,13 +281,16 @@ class SharpenMicroParams {
 class VibranceParams {
 
     public:
-        bool    enabled;
-        int     pastels;
-        int     saturated;
-        int     psthreshold;
-        bool    protectskins;
-        bool    avoidcolorshift;
-        bool    pastsattog;
+        bool           enabled;
+        int            pastels;
+        int            saturated;
+        Threshold<int> psthreshold;
+        bool           protectskins;
+        bool           avoidcolorshift;
+        bool           pastsattog;
+        std::vector<double> skintonescurve;
+
+        VibranceParams() : psthreshold(0, 75,  false) {};
 };
 
 /**
@@ -169,7 +337,7 @@ public:
 class WBParams {
 
     public:
-	    static std::vector<WBEntry*> wbEntries;
+        static std::vector<WBEntry*> wbEntries;
         Glib::ustring   method;
         int             temperature;
         double          green;
@@ -178,6 +346,58 @@ class WBParams {
         static void     cleanup();
 };
 
+    /**
+     * Parameters of colorappearance
+     */
+    class ColorAppearanceParams {
+
+    public:
+        enum eTCModeId {
+            TC_MODE_LIGHT,    // Lightness mode
+            TC_MODE_BRIGHT,   // Brightness mode
+        };
+
+        enum eCTCModeId {
+            TC_MODE_CHROMA,   // chroma mode
+            TC_MODE_SATUR,    // saturation mode
+            TC_MODE_COLORF,   // colorfullness mode
+       };
+
+        bool          enabled;
+        int           degree;
+        bool          autodegree;
+        std::vector<double> curve;
+        std::vector<double> curve2;
+        std::vector<double> curve3;
+        eTCModeId     curveMode;
+        eTCModeId     curveMode2;
+        eCTCModeId    curveMode3;
+
+        Glib::ustring surround;
+        double        adapscen;
+        bool          autoadapscen;
+
+        double        adaplum;
+        int           badpixsl;
+        Glib::ustring wbmodel;
+        Glib::ustring algo;
+        double        contrast;
+        double        qcontrast;
+        double        jlight;
+        double        qbright;
+        double        chroma;
+        double        schroma;
+        double        mchroma;
+        double        colorh;
+        double        rstprotection;
+        bool          surrsource;
+        bool          gamut;
+  //      bool          badpix;
+        bool          datacie;
+        bool          tonecie;
+  //      bool          sharpcie;
+    };
+
 /**
   * Parameters of the color shift
   */
@@ -210,50 +430,55 @@ class WBParams {
         bool    edgesensitive;
         int		amount;
 };*/
-	
-	/**
-	 * Parameters of defringing
-	 */
-	class DefringeParams {
-		
+
+/**
+ * Parameters of defringing
+ */
+class DefringeParams {
+
     public:
         bool    enabled;
         double  radius;
         int     threshold;
-	};
-	
-	
-	/**
-	 * Parameters of impulse denoising
-	 */
-	class ImpulseDenoiseParams {
-		
-    public:
-        bool    enabled;
-		int		thresh;
+        std::vector<double> huecurve;
+};
+
+/**
+  * Parameters of impulse denoising
+  */
+class ImpulseDenoiseParams {
 
-	};
-	
-	/**
-	 * Parameters of the directional pyramid denoising
-	 */
-	class DirPyrDenoiseParams {
-		
     public:
         bool    enabled;
-        int		luma;
-        int     chroma;
-		float	gamma;
-	};
+        int     thresh;
+
+};
+
+/**
+ * Parameters of the directional pyramid denoising
+ */
+class DirPyrDenoiseParams {
+
+    public:
+        bool    enabled;
+        bool    perform;
+        double  luma;
+        double  Ldetail;
+        double  chroma;
+        double  redchro;
+        double  bluechro;
+        double  gamma;
+        Glib::ustring dmethod;
+};
 
 //EPD related parameters.
 class EPDParams{
-public:
-	bool enabled;
-	double Strength;
-	double EdgeStopping;
-	double Scale;
-	int ReweightingIterates;
+    public:
+        bool   enabled;
+        double Strength;
+        double EdgeStopping;
+        double Scale;
+        int    ReweightingIterates;
 };
 
 /**
@@ -343,8 +568,8 @@ public:
 class PerspectiveParams {
 
     public:
-		int  horizontal;
-		int  vertical;
+		double  horizontal;
+		double  vertical;
 };
 
 /**
@@ -413,6 +638,7 @@ class ColorManagementParams {
 
     public:
         Glib::ustring input;
+        bool          toneCurve;
         bool          blendCMSMatrix;
         short preferredProfile;
         Glib::ustring working;
@@ -465,8 +691,11 @@ class HSVEqualizerParams {
 class RAWParams {
 
     public:
-		enum eMethod{eahd,hphd,vng4,dcb,amaze,ahd,fast,
+	//	enum eMethod{eahd,hphd,vng4,dcb,amaze,ahd,IGV_noise,fast,
+	//				numMethods }; // This MUST be the last enum
+		enum eMethod{amaze,igv,lmmse,eahd,hphd,vng4,dcb,ahd,fast,
 					numMethods }; // This MUST be the last enum
+					
 		static const char *methodstring[numMethods];
 
 		enum eFlatFileBlurType{/*parametric,*/area_ff,v_ff,h_ff,vh_ff,
@@ -501,9 +730,10 @@ class RAWParams {
         int ccSteps;
         Glib::ustring dmethod;
         int dcb_iterations;
-        bool dcb_enhance;
-        bool all_enhance;
+        int lmmse_iterations;
 		
+        bool dcb_enhance;
+        //bool all_enhance;
 };
 
 /**
@@ -521,13 +751,14 @@ class ProcParams {
         VibranceParams          vibrance;        ///< Vibrance parameters
         //ColorBoostParams        colorBoost;      ///< Color boost parameters
         WBParams                wb;              ///< White balance parameters
+        ColorAppearanceParams   colorappearance;
         //ColorShiftParams        colorShift;      ///< Color shift parameters
         //LumaDenoiseParams       lumaDenoise;     ///< Luminance denoising parameters
         //ColorDenoiseParams      colorDenoise;    ///< Color denoising parameters
         DefringeParams          defringe;        ///< Defringing parameters
         ImpulseDenoiseParams    impulseDenoise;  ///< Impulse denoising parameters
         DirPyrDenoiseParams     dirpyrDenoise;   ///< Directional Pyramid denoising parameters
-        EPDParams					  edgePreservingDecompositionUI;
+        EPDParams               edgePreservingDecompositionUI;
         SHParams                sh;              ///< Shadow/highlight enhancement parameters
         CropParams              crop;            ///< Crop parameters
         CoarseTransformParams   coarse;          ///< Coarse transformation (90, 180, 270 deg rotation, h/v flipping) parameters
@@ -542,7 +773,6 @@ class ProcParams {
         HRecParams              hlrecovery;      ///< Highlight recovery parameters
         ResizeParams            resize;          ///< Resize parameters
         ColorManagementParams   icm;             ///< profiles/color spaces used during the image processing
-		
         RAWParams               raw;             ///< RAW parameters before demosaicing
         DirPyrEqualizerParams   dirpyrequalizer; ///< directional pyramid equalizer parameters
         HSVEqualizerParams      hsvequalizer;    ///< hsv equalizer parameters
@@ -626,7 +856,7 @@ class PartialProfile {
         void clearGeneral   ();
         int  load           (Glib::ustring fName);
         void set            (bool v);
-        void applyTo        (ProcParams *destParams) const ;
+        const void applyTo  (ProcParams *destParams) const ;
 };
 
 }
