@@ -24,6 +24,9 @@
 #include "dcp.h"
 #include "array2D.h"
 #include "curves.h"
+#include <fftw3.h>
+#include "color.h"
+#include "iimage.h"
 #include "../rtgui/cacheimagedata.h"
 
 #define HR_SCALE 2
@@ -62,6 +65,7 @@ class RawImageSource : public ImageSource {
         static LUTf invGrad;  // for fast_demosaic
         static LUTf initInvGrad ();
         static bool findInputProfile(Glib::ustring inProfile, cmsHPROFILE embedded, std::string camName, DCPProfile **dcpProf, cmsHPROFILE& in);
+        static void colorSpaceConversion (Imagefloat* im, ColorManagementParams &cmp, float rawWhitePoint, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], const std::string &camName);
 
     protected:
         Glib::Mutex getImageMutex;  // locks getImage
@@ -70,8 +74,6 @@ class RawImageSource : public ImageSource {
         ColorTemp wb;
         ProgressListener* plistener;
         float scale_mul[4]; // multiplier for each color
-        float cblack[4];// black
-        float scale_mu_l[4];// copy of scale_mul, for saturation
         float c_black[4]; // copy of cblack Dcraw for black level
         float cblacksom[4];
         double camwb_red;
@@ -103,14 +105,14 @@ class RawImageSource : public ImageSource {
         double* cache;
         int threshold;
 
-        float** rawData;  // holds preprocessed pixel values, rowData[i][j] corresponds to the ith row and jth column
+        array2D<float> rawData;  // holds preprocessed pixel values, rowData[i][j] corresponds to the ith row and jth column
 
         // the interpolated green plane:
-        float** green; 
+        array2D<float> green; 
         // the interpolated red plane:
-        float** red;
+        array2D<float> red;
         // the interpolated blue plane:
-        float** blue;
+        array2D<float> blue;
 
 
         void hphd_vertical       (float** hpmap, int col_from, int col_to);
@@ -119,7 +121,7 @@ class RawImageSource : public ImageSource {
         void processFalseColorCorrectionThread (Imagefloat* im, int row_from, int row_to);
         void hlRecovery          (std::string method, float* red, float* green, float* blue, int i, int sx1, int width, int skip, const RAWParams &raw, float* hlmax);
         int  defTransform        (int tran);
-        void rotateLine          (float* line, float** channel, int tran, int i, int w, int h);
+        void rotateLine          (float* line, PlanarPtr<float> &channel, int tran, int i, int w, int h);
         void transformRect       (PreviewProps pp, int tran, int &sx1, int &sy1, int &width, int &height, int &fw);
         void transformPosition   (int x, int y, int tran, int& tx, int& ty);
 
@@ -140,7 +142,8 @@ class RawImageSource : public ImageSource {
         void        flushRawData      ();
         void        flushRGB          ();
         void        HLRecovery_Global (HRecParams hrp);
-        void        refinement_lassus ();
+        void        refinement_lassus (int PassCount);
+        void        refinement(int PassCount);
 
         bool        IsrgbSourceModified() {return rgbSourceModified;} // tracks whether cached rgb output of demosaic has been modified
 
@@ -151,24 +154,33 @@ class RawImageSource : public ImageSource {
         void        getImage    (ColorTemp ctemp, int tran, Imagefloat* image, PreviewProps pp, HRecParams hrp, ColorManagementParams cmp, RAWParams raw);
         ColorTemp   getWB       () { return wb; }
         ColorTemp   getAutoWB   ();
-        ColorTemp   getSpotWB   (std::vector<Coord2D> red, std::vector<Coord2D> green, std::vector<Coord2D>& blue, int tran);
-        bool        isWBProviderReady () { return rawData != NULL; };
+        ColorTemp   getSpotWB   (std::vector<Coord2D> &red, std::vector<Coord2D> &green, std::vector<Coord2D> &blue, int tran);
+        bool        isWBProviderReady () { return rawData; }
 
         double      getDefGain  () { return defGain; }
 
-        double      getGamma    () { return CurveFactory::sRGBGamma; }
+        double      getGamma    () { return Color::sRGBGamma; }
 
         void        getFullSize (int& w, int& h, int tr = TR_NONE);
         void        getSize     (int tran, PreviewProps pp, int& w, int& h);
         int         getRotateDegree() const { return ri->get_rotateDegree(); }
 
         ImageData*  getImageData () { return idata; }
+        ImageMatrices* getImageMatrices () { return &imatrices; }
+        bool        isRAW() const { return true; }
+
         void        setProgressListener (ProgressListener* pl) { plistener = pl; }
         void        getAutoExpHistogram (LUTu & histogram, int& histcompr);
         void        getRAWHistogram (LUTu & histRedRaw, LUTu & histGreenRaw, LUTu & histBlueRaw);
 
-        static void colorSpaceConversion16 (Image16* im, ColorManagementParams cmp, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], std::string camName, double& defgain);
-        static void colorSpaceConversion (Imagefloat* im, ColorManagementParams cmp, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], std::string camName, double& defgain);
+        void convertColorSpace(Imagefloat* image, ColorManagementParams cmp, RAWParams raw);
+        //static void colorSpaceConversion16 (Image16*    im, ColorManagementParams cmp, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], std::string camName);
+        static void colorSpaceConversion   (Imagefloat* im, ColorManagementParams cmp, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], std::string camName) {
+            colorSpaceConversion (im, cmp, 0.0f, embedded, camprofile, cam, camName);
+        }
+        static void colorSpaceConversion   (Imagefloat* im, ColorManagementParams cmp, RAWParams raw, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], std::string camName) {
+            colorSpaceConversion (im, cmp, float(raw.expos), embedded, camprofile, cam, camName);
+        }
         static void inverse33 (const double (*coeff)[3], double (*icoeff)[3]);
 
         void boxblur2(float** src, float** dst, int H, int W, int box );
@@ -203,6 +215,8 @@ class RawImageSource : public ImageSource {
         int  findHotDeadPixel( PixelsMap &bpMap, float thresh);
 
         void cfa_linedn (float linenoiselevel);//Emil's line denoise
+        void cfa_tile_denoise (fftwf_complex * fcfablox, int vblk, int hblk, int numblox_H, int numblox_W, float noisevar, float * rolloff );
+        void cfa_output_tile_row (float *cfabloxrow, float ** cfahipassdn, float ** tilemask_out, int height, int width, int top, int blkrad);
 
         void green_equilibrate (float greenthresh);//Emil's green equilibration
 
@@ -211,11 +225,16 @@ class RawImageSource : public ImageSource {
         void hphd_demosaic();
         void vng4_demosaic();
         void ppg_demosaic();
+        void jdl_interpolate_omp();
+        void igv_interpolate(int winw, int winh);
+        void lmmse_interpolate_omp(int winw, int winh, int iterations);
+
         void amaze_demosaic_RT(int winx, int winy, int winw, int winh);//Emil's code for AMaZE
-        void fast_demosaic(int winx, int winy, int winw, int winh);//Emil's code for fast demosaicing
+        void fast_demosaic(int winx, int winy, int winw, int winh );//Emil's code for fast demosaicing
         void dcb_demosaic(int iterations, bool dcb_enhance);
         void ahd_demosaic(int winx, int winy, int winw, int winh);
-        void border_interpolate(int border, float (*image)[4], int start = 0, int end = 0);
+        void border_interpolate(unsigned int border, float (*image)[4], unsigned int start = 0, unsigned int end = 0);
+        void border_interpolate2(int winw, int winh, int lborders);
         void dcb_initTileLimits(int &colMin, int &rowMin, int &colMax, int &rowMax, int x0, int y0, int border);
         void fill_raw( float (*cache )[4], int x0, int y0, float** rawData);
         void fill_border( float (*cache )[4], int border, int x0, int y0);
