@@ -30,21 +30,22 @@
 
 // Elementary functions that can be done to
 // the preview image when an event occurs
-#define M_PREPROC    (1<<9)
-#define M_RAW        (1<<8)
-#define M_INIT       (1<<7)
-#define M_TRANSFORM  (1<<6)
-#define M_BLURMAP    (1<<5)
-#define M_AUTOEXP    (1<<4)
-#define M_RGBCURVE   (1<<3)
-#define M_LUMACURVE  (1<<2)
-#define M_LUMINANCE  (1<<1)
-#define M_COLOR      (1<<0)
+#define M_PREPROC     (1<<10)
+#define M_RAW         (1<<9)
+#define M_INIT        (1<<8)
+#define M_LINDENOISE  (1<<7)
+#define M_TRANSFORM   (1<<6)
+#define M_BLURMAP     (1<<5)
+#define M_AUTOEXP     (1<<4)
+#define M_RGBCURVE    (1<<3)
+#define M_LUMACURVE   (1<<2)
+#define M_LUMINANCE   (1<<1)
+#define M_COLOR       (1<<0)
 
 // Bitfield of functions to do to the preview image when an event occurs
 // Use those or create new ones for your new events
-#define FIRST           (M_PREPROC|M_RAW|M_INIT|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)  // without HIGHQUAL
-#define ALL             (M_PREPROC|M_RAW|M_INIT|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)  // without HIGHQUAL
+#define FIRST           (M_PREPROC|M_RAW|M_INIT|M_LINDENOISE|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)  // without HIGHQUAL
+#define ALL             (M_PREPROC|M_RAW|M_INIT|M_LINDENOISE|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)  // without HIGHQUAL
 #define TRANSFORM       (M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
 #define RETINEX         (M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
 #define AUTOEXP         (M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
@@ -53,10 +54,10 @@
 #define SHARPENING       M_LUMINANCE
 #define IMPULSEDENOISE   M_LUMINANCE
 #define DEFRINGE         M_LUMINANCE
-#define WHITEBALANCE    (M_INIT|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
-#define DEMOSAIC        (M_RAW|M_INIT|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
-#define DARKFRAME       (M_PREPROC|M_RAW|M_INIT|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
-#define FLATFIELD       (M_PREPROC|M_RAW|M_INIT|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
+#define WHITEBALANCE    (M_INIT|M_LINDENOISE|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
+#define DEMOSAIC        (M_RAW|M_INIT|M_LINDENOISE|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
+#define DARKFRAME       (M_PREPROC|M_RAW|M_INIT|M_LINDENOISE|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
+#define FLATFIELD       (M_PREPROC|M_RAW|M_INIT|M_LINDENOISE|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
 #define DIRPYRDENOISE   (M_COLOR|M_LUMINANCE)
 #define CROP             M_MINUPDATE
 #define RESIZE           M_VOID
@@ -66,7 +67,7 @@
 #define OUTPUTPROFIL    (M_COLOR|M_LUMINANCE)
 #define GAMMA           (M_COLOR|M_LUMINANCE)
 #define NONE             0
-#define ALLNORAW        (M_INIT|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
+#define ALLNORAW        (M_INIT|M_LINDENOISE|M_TRANSFORM|M_BLURMAP|M_AUTOEXP|M_RGBCURVE|M_LUMACURVE|M_LUMINANCE|M_COLOR)
 
 extern int refreshmap[];
 #endif    
