@@ -26,9 +26,13 @@ namespace rtengine {
         public:
             Glib::ustring   iccDirectory;           ///< The directory containing the possible output icc profiles
             int             colorimetricIntent;     ///< Colorimetric intent used at color space conversions
-            Glib::ustring   monitorProfile;         ///< ICC profile of the monitor (full path recommended)
+			int				viewingdevice; 			// white of output device (D50...D65..)
+			int				viewingdevicegrey; 			// level of grey output device
+ 
+			Glib::ustring   monitorProfile;         ///< ICC profile of the monitor (full path recommended)
 			bool            autoMonitorProfile;     ///< Try to auto-determine the correct monitor color profile
-
+			bool			autocielab;
+			bool			rgbcurveslumamode_gamut;// controls gamut enforcement for RGB curves in lumamode
             bool            verbose;
             Glib::ustring   darkFramesPath;         ///< The default directory for dark frames
             Glib::ustring   flatFieldsPath;         ///< The default directory for flat fields
@@ -43,8 +47,14 @@ namespace rtengine {
 			Glib::ustring   srgb;					// default name of SRGB space profile
 			Glib::ustring   srgb10;					// default name of SRGB space profile
 			
-			bool		    gamutICC;           // 
-			
+			bool            gamutICC;
+			bool            gamutLch;
+			bool			ciecamfloat;
+			int             protectred;
+			double          protectredh;
+			bool			ciebadpixgauss;
+			int             CRI_color; // N for display Lab value  ; 0 disabled
+
         /** Creates a new instance of Settings.
           * @return a pointer to the new Settings instance. */
             static Settings* create  ();
