@@ -28,7 +28,12 @@
 #include <iostream>
 #include "rawimagesource.h"
 #include "../rtgui/ppversion.h"
+//#include "mytime.h"
+
 #undef THREAD_PRIORITY_NORMAL
+#ifdef _OPENMP
+#include <omp.h>
+#endif
 
 namespace rtengine {
 extern const Settings* settings;
@@ -54,7 +59,7 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
     }
     procparams::ProcParams& params = job->pparams;
 
-    // aquire image from imagesource
+    // acquire image from imagesource
     ImageSource* imgsrc = ii->getImageSource ();
 
     int tr = TR_NONE;
@@ -87,6 +92,8 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
 			params.crop.h = fh-params.crop.y;
 		}
 	}
+//    MyTime t1,t2;
+//    t1.set();
 
     ImProcFunctions ipf (&params, true);
 
@@ -108,8 +115,19 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
     if (pl) pl->setProgress (0.45);
 
 
+    // perform luma/chroma denoise
+//	CieImage *cieView;    
+
+    LabImage* labView = new LabImage (fw,fh);
+    if (params.dirpyrDenoise.enabled) {
+		ipf.RGB_denoise(baseImg, baseImg, imgsrc->isRAW(), params.dirpyrDenoise, params.defringe, imgsrc->getDirPyrDenoiseExpComp());
+    }
+    imgsrc->convertColorSpace(baseImg, params.icm, params.raw);
+
     // perform first analysis
     LUTu hist16 (65536);
+    LUTu hist16C (65536);
+	
     ipf.firstAnalysis (baseImg, &params, hist16, imgsrc->getGamma());
 
     // perform transform (excepted resizing)
@@ -146,7 +164,7 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
     }
 
     // at this stage, we can flush the raw data to free up quite an important amount of memory
-    // commented out because it make the application crash when batch processing...
+    // commented out because it makes the application crash when batch processing...
     // TODO: find a better place to flush rawData and rawRGB
     //imgsrc->flushRawData();
     //imgsrc->flushRGB();
@@ -155,21 +173,28 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
     LUTf curve2 (65536,0);
 	LUTf curve (65536,0);
 	LUTf satcurve (65536,0);
+	LUTf lhskcurve (65536,0);
+	LUTf lumacurve(65536,0);
+
 	LUTf rCurve (65536,0);
 	LUTf gCurve (65536,0);
 	LUTf bCurve (65536,0);
 	LUTu dummy;
 
-    CurveFactory::complexCurve (expcomp, black/65535.0, params.toneCurve.hlcompr, params.toneCurve.hlcomprthresh, params.toneCurve.shcompr, bright, params.toneCurve.contrast, imgsrc->getGamma(), true, params.toneCurve.curve, 
-        hist16, dummy, curve1, curve2, curve, dummy);
+	ToneCurve customToneCurve1, customToneCurve2;
+	ColorAppearance customColCurve1, customColCurve2,customColCurve3 ;
+
+	ipf.g = imgsrc->getGamma();
+	ipf.iGamma = true;
+	CurveFactory::complexCurve (expcomp, black/65535.0, hlcompr, hlcomprthresh, params.toneCurve.shcompr, bright, contr, ipf.g, !ipf.iGamma,
+	                            params.toneCurve.curveMode, params.toneCurve.curve, params.toneCurve.curveMode2, params.toneCurve.curve2,
+	                            hist16, dummy, curve1, curve2, curve, dummy, customToneCurve1, customToneCurve2);
 	
 	CurveFactory::RGBCurve (params.rgbCurves.rcurve, rCurve, 1);
 	CurveFactory::RGBCurve (params.rgbCurves.gcurve, gCurve, 1);
 	CurveFactory::RGBCurve (params.rgbCurves.bcurve, bCurve, 1);
 
-	LabImage* labView = new LabImage (fw,fh);
-
-    ipf.rgbProc (baseImg, labView, curve1, curve2, curve, shmap, params.toneCurve.saturation, rCurve, gCurve, bCurve);
+    ipf.rgbProc (baseImg, labView, curve1, curve2, curve, shmap, params.toneCurve.saturation, rCurve, gCurve, bCurve, customToneCurve1, customToneCurve2, expcomp, hlcompr, hlcomprthresh);
 
     // Freeing baseImg because not used anymore
     delete baseImg;
@@ -182,49 +207,149 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
     if (pl) 
         pl->setProgress (0.5);
 
+	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+	// start tile processing...???
 
-    // luminance histogram update
-    hist16.clear();
+    hist16.clear();  hist16C.clear();
+	if(params.labCurve.contrast !=0) {//only use hist16 for contrast
+	
+#ifdef _OPENMP
+#pragmaompparallelshared(hist16,labView, fh, fw)
+#endif
+{
+#ifdef _OPENMP	
+#pragma omp for schedule(static)
+#endif
     for (int i=0; i<fh; i++)
-        for (int j=0; j<fw; j++)
+        for (int j=0; j<fw; j++){
             hist16[CLIP((int)((labView->L[i][j])))]++;
+			}
+}
+	
+	
+	}
+	bool utili=false;
+	bool autili=false;
+	bool butili=false;
+	bool ccutili=false;
+	bool cclutili=false;	
+	CurveFactory::complexLCurve (params.labCurve.brightness, params.labCurve.contrast, params.labCurve.lcurve,hist16, hist16, lumacurve, dummy, 1, utili);
 
-    // luminance processing
+	CurveFactory::complexsgnCurve (autili, butili, ccutili, cclutili, params.labCurve.chromaticity, params.labCurve.rstprotection,
+								   params.labCurve.acurve, params.labCurve.bcurve, params.labCurve.cccurve,params.labCurve.lccurve,curve1, curve2, satcurve,lhskcurve, 
+								   hist16C, hist16C,dummy,	
+								   1);
 
-	ipf.EPDToneMap(labView);
+	ipf.chromiLuminanceCurve (1,labView, labView, curve1, curve2, satcurve,lhskcurve,lumacurve, utili, autili, butili, ccutili,cclutili,dummy);
+	
+ 	if((params.colorappearance.enabled && !params.colorappearance.tonecie) || (!params.colorappearance.enabled))ipf.EPDToneMap(labView,5,1);
+	
 
-	CurveFactory::complexLCurve (params.labCurve.brightness, params.labCurve.contrast, params.labCurve.lcurve, hist16, hist16, curve, dummy, 1);
-
-	CurveFactory::complexsgnCurve (params.labCurve.saturation, params.labCurve.enable_saturationlimiter, params.labCurve.saturationlimit,
-								   params.labCurve.acurve, params.labCurve.bcurve, curve1, curve2, satcurve, 1);
-	ipf.luminanceCurve (labView, labView, curve);
-	ipf.chrominanceCurve (labView, labView, curve1, curve2, satcurve);
 	ipf.vibrance(labView);
 
-	ipf.impulsedenoise (labView);
-	ipf.defringe (labView);
-	ipf.dirpyrdenoise (labView);
+	if((params.colorappearance.enabled && !settings->autocielab) || (!params.colorappearance.enabled)) ipf.impulsedenoise (labView);
+	// for all treatments Defringe, Sharpening, Contrast detail ,Microcontrast they are activated if "CIECAM" function are disabled
+
+	if((params.colorappearance.enabled && !settings->autocielab) || (!params.colorappearance.enabled)) ipf.defringe (labView);
+	
 	if (params.sharpenEdge.enabled) {
 		 ipf.MLsharpen(labView);
 	}
 	if (params.sharpenMicro.enabled) {
-		ipf.MLmicrocontrast (labView);
+		if((params.colorappearance.enabled && !settings->autocielab) ||  (!params.colorappearance.enabled)) ipf.MLmicrocontrast (labView);//!params.colorappearance.sharpcie
 	}
-    if (params.sharpening.enabled) {
-        float** buffer = new float*[fh];
-        for (int i=0; i<fh; i++)
-            buffer[i] = new float[fw];
+	
+	if(((params.colorappearance.enabled && !settings->autocielab) || (!params.colorappearance.enabled)) && params.sharpening.enabled) {			
+                    
+        float **buffer = new float*[fh];
+            for (int i=0; i<fh; i++)
+                buffer[i] = new float[fw];
 
         ipf.sharpening (labView, (float**)buffer);
 
-        for (int i=0; i<fh; i++)
-            delete [] buffer[i];
-        delete [] buffer; buffer=NULL;
+            for (int i=0; i<fh; i++)
+                delete [] buffer[i];
+                delete [] buffer;
     }
-
 	// directional pyramid equalizer
-    ipf.dirpyrequalizer (labView);//TODO: this is the luminance tonecurve, not the RGB one
+	if((params.colorappearance.enabled && !settings->autocielab)  || !params.colorappearance.enabled) ipf.dirpyrequalizer (labView);//TODO: this is the luminance tonecurve, not the RGB one
+	
+	//Colorappearance and tone-mapping associated
+	
+	int f_w=1,f_h=1;
+	int begh = 0, endh = fh;
+	if(params.colorappearance.tonecie || params.colorappearance.enabled){f_w=fw;f_h=fh;}
+	CieImage *cieView = new CieImage (f_w,(f_h));
+	begh=0;
+	endh=fh;
+	CurveFactory::curveLightBrightColor (
+					params.colorappearance.curveMode, params.colorappearance.curve,
+					params.colorappearance.curveMode2, params.colorappearance.curve2,
+					params.colorappearance.curveMode3, params.colorappearance.curve3,
+					hist16, hist16,dummy,
+					hist16C, hist16C,dummy,	
+					customColCurve1,
+					customColCurve2,
+					customColCurve3, 					
+					1);
+	float adap2,adap;
+	double ada, ada2;
+	if(params.colorappearance.enabled){	
+	float fnum = imgsrc->getMetaData()->getFNumber  ();// F number
+	float fiso = imgsrc->getMetaData()->getISOSpeed () ;// ISO
+	float fspeed = imgsrc->getMetaData()->getShutterSpeed () ;//speed 
+	float fcomp = imgsrc->getMetaData()->getExpComp  ();//compensation + -
+	if(fnum < 0.3f || fiso < 5.f || fspeed < 0.00001f) {adap=adap2=2000.f;ada=ada2=2000.;}//if no exif data or wrong
+	else {
+	float E_V = fcomp + log2 ((fnum*fnum) / fspeed / (fiso/100.f));
+	float expo2= params.toneCurve.expcomp;// exposure compensation in tonecurve ==> direct EV
+	E_V += expo2;
+	float expo1;//exposure raw white point
+	expo1=log2(params.raw.expos);//log2 ==>linear to EV
+	E_V += expo1;
+	adap2 = adap= powf(2.f, E_V-3.f);//cd / m2
+	ada=ada2=(double) adap;
+	}			
+    if (params.sharpening.enabled) {
+		float d;
+		double dd;
+		
+        float** buffer = new float*[fh];
+        for (int i=0; i<fh; i++)
+            buffer[i] = new float[fw];	
+	if(settings->ciecamfloat) ipf.ciecam_02float (cieView, adap, begh, endh,1,2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, 5, 1, (float**)buffer, true, d);
+	else ipf.ciecam_02 (cieView, ada, begh, endh,1,2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, 5, 1, (float**)buffer, true, dd);
 
+	        for (int i=0; i<fh; i++)
+            delete [] buffer[i];
+			delete [] buffer;
+			}
+			else {
+			int f_h=2,f_w=2;
+			float d;
+			
+			double dd;
+	        float** buffer = new float*[f_h];
+			for (int i=0; i<f_h; i++)
+            buffer[i] = new float[f_w];
+if(settings->ciecamfloat) ipf.ciecam_02float (cieView, adap, begh, endh,1,2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, 5, 1, (float**)buffer, true, d);
+else ipf.ciecam_02 (cieView, adap, begh, endh,1, 2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, 5, 1, (float**)buffer, true, dd);
+
+	        for (int i=0; i<f_h; i++)
+            delete [] buffer[i];
+			delete [] buffer;
+			}
+	}	
+    delete cieView;
+    cieView = NULL;
+	
+	
+	
+
+	// end tile processing...???
+	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
     if (pl) pl->setProgress (0.60);
 
@@ -245,7 +370,7 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
     if(params.icm.gamma != "default" || params.icm.freegamma) { // if select gamma output between BT709, sRGB, linear, low, high, 2.2 , 1.8
         cmsMLU *DescriptionMLU, *CopyrightMLU, *DmndMLU, *DmddMLU;// for modification TAG
 
-        cmsToneCurve* GammaTRC[3];
+        cmsToneCurve* GammaTRC[3] = { NULL, NULL, NULL };
         cmsFloat64Number Parameters[7];
         double ga0,ga1,ga2,ga3,ga4,ga5,ga6;
        // wchar_t string[80] ;
@@ -399,6 +524,7 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
 				
             }
         }
+        if (GammaTRC[0]) cmsFreeToneCurve(GammaTRC[0]);
     }
     else {
         // if Default gamma mode: we use the profile selected in the "Output profile" combobox;
@@ -514,6 +640,9 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
              readyImg->setOutputProfile (NULL,0);
         }
     }
+//    t2.set();
+//    if( settings->verbose )
+//           printf("Total:- %d usec\n", t2.etime(t1));
 
     if (!job->initialImage)
         ii->decreaseRef ();
@@ -521,7 +650,17 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
     delete job;
     if (pl)
         pl->setProgress (0.75);
-
+/*	curve1.reset();curve2.reset();
+	curve.reset();
+	satcurve.reset();
+	lhskcurve.reset();
+	
+	rCurve.reset();
+	gCurve.reset();
+	bCurve.reset();
+	hist16.reset();
+	hist16C.reset();
+*/
     return readyImg;
 }
 
