@@ -120,11 +120,15 @@ void BatchToolPanelCoordinator::initSession () {
 
 		if (selected.size()==1) {
 
+			for (size_t i=0; i<toolPanels.size(); i++)
+				toolPanels.at(i)->setMultiImage(false);
+
 			toneCurve->setAdjusterBehavior (false, false, false, false, false, false, false, false);
 			lcurve->setAdjusterBehavior (false, false, false);
 			whitebalance->setAdjusterBehavior (false, false);
 			vibrance->setAdjusterBehavior (false, false, false);
-			vignetting->setAdjusterBehavior (false);
+			vignetting->setAdjusterBehavior (false, false, false, false);
+			colorappearance->setAdjusterBehavior (false, false, false, false, false, false, false, false, false, false, false, false, false);
 			rotate->setAdjusterBehavior (false);
 			distortion->setAdjusterBehavior (false);
 			perspective->setAdjusterBehavior (false);
@@ -137,18 +141,21 @@ void BatchToolPanelCoordinator::initSession () {
 			chmixer->setAdjusterBehavior (false);
 			shadowshighlights->setAdjusterBehavior (false, false, false);
 			dirpyrequalizer->setAdjusterBehavior (false);
-			dirpyrdenoise->setAdjusterBehavior (false, false);
+			dirpyrdenoise->setAdjusterBehavior (false, false,false,false,false,false);
 			preprocess->setAdjusterBehavior (false, false);
 			rawcacorrection->setAdjusterBehavior (false);
 			rawexposure->setAdjusterBehavior (false, false, false);
 		}
 		else {
 
+			for (size_t i=0; i<toolPanels.size(); i++)
+				toolPanels.at(i)->setMultiImage(true);
+
 			toneCurve->setAdjusterBehavior (options.baBehav[ADDSET_TC_EXPCOMP], options.baBehav[ADDSET_TC_HLCOMPAMOUNT],options.baBehav[ADDSET_TC_HLCOMPTHRESH], options.baBehav[ADDSET_TC_BRIGHTNESS], options.baBehav[ADDSET_TC_BLACKLEVEL],options.baBehav[ADDSET_TC_SHCOMP], options.baBehav[ADDSET_TC_CONTRAST], options.baBehav[ADDSET_TC_SATURATION]);
-			lcurve->setAdjusterBehavior (options.baBehav[ADDSET_LC_BRIGHTNESS], options.baBehav[ADDSET_LC_CONTRAST], options.baBehav[ADDSET_LC_SATURATION]);
+			lcurve->setAdjusterBehavior (options.baBehav[ADDSET_LC_BRIGHTNESS], options.baBehav[ADDSET_LC_CONTRAST], options.baBehav[ADDSET_LC_CHROMATICITY]);
 			whitebalance->setAdjusterBehavior (options.baBehav[ADDSET_WB_TEMPERATURE], options.baBehav[ADDSET_WB_GREEN]);
-			vibrance->setAdjusterBehavior (options.baBehav[ADDSET_VIBRANCE_PASTELS], options.baBehav[ADDSET_VIBRANCE_SATURATED], options.baBehav[ADDSET_VIBRANCE_PSTHRESHOLD]);
-			vignetting->setAdjusterBehavior (options.baBehav[ADDSET_VIGN_AMOUNT]);
+			vignetting->setAdjusterBehavior (options.baBehav[ADDSET_VIGN_AMOUNT], options.baBehav[ADDSET_VIGN_RADIUS], options.baBehav[ADDSET_VIGN_STRENGTH], options.baBehav[ADDSET_VIGN_CENTER]);
+			colorappearance->setAdjusterBehavior (options.baBehav[ADDSET_CAT_DEGREE], options.baBehav[ADDSET_CAT_ADAPTSCENE], options.baBehav[ADDSET_CAT_ADAPTVIEWING],options.baBehav[ADDSET_CAT_BADPIX], options.baBehav[ADDSET_CAT_LIGHT], options.baBehav[ADDSET_CAT_CHROMA],options.baBehav[ADDSET_CAT_CONTRAST],options.baBehav[ADDSET_CAT_RSTPRO],options.baBehav[ADDSET_CAT_BRIGHT],options.baBehav[ADDSET_CAT_CONTRAST_Q],options.baBehav[ADDSET_CAT_CHROMA_S],options.baBehav[ADDSET_CAT_CHROMA_M],options.baBehav[ADDSET_CAT_HUE]);
 			rotate->setAdjusterBehavior (options.baBehav[ADDSET_ROTATE_DEGREE]);
 			distortion->setAdjusterBehavior (options.baBehav[ADDSET_DIST_AMOUNT]);
 			perspective->setAdjusterBehavior (options.baBehav[ADDSET_PERSPECTIVE]);
@@ -161,7 +168,7 @@ void BatchToolPanelCoordinator::initSession () {
 			chmixer->setAdjusterBehavior (options.baBehav[ADDSET_CHMIXER]);
 			shadowshighlights->setAdjusterBehavior (options.baBehav[ADDSET_SH_HIGHLIGHTS], options.baBehav[ADDSET_SH_SHADOWS], options.baBehav[ADDSET_SH_LOCALCONTRAST]);
 			dirpyrequalizer->setAdjusterBehavior (options.baBehav[ADDSET_DIRPYREQ]);
-			dirpyrdenoise->setAdjusterBehavior (options.baBehav[ADDSET_DIRPYRDN_CHLUM], options.baBehav[ADDSET_DIRPYRDN_GAMMA]);
+			dirpyrdenoise->setAdjusterBehavior (options.baBehav[ADDSET_DIRPYRDN_LUMA],options.baBehav[ADDSET_DIRPYRDN_LUMDET],options.baBehav[ADDSET_DIRPYRDN_CHROMA],options.baBehav[ADDSET_DIRPYRDN_CHROMARED],options.baBehav[ADDSET_DIRPYRDN_CHROMABLUE], options.baBehav[ADDSET_DIRPYRDN_GAMMA]);
 			preprocess->setAdjusterBehavior (options.baBehav[ADDSET_PREPROCESS_LINEDENOISE], options.baBehav[ADDSET_PREPROCESS_GREENEQUIL]);
 			rawcacorrection->setAdjusterBehavior (options.baBehav[ADDSET_RAWCACORR]);
 			rawexposure->setAdjusterBehavior (options.baBehav[ADDSET_RAWEXPOS_LINEAR], options.baBehav[ADDSET_RAWEXPOS_PRESER], options.baBehav[ADDSET_RAWEXPOS_BLACKS]);
@@ -180,7 +187,7 @@ void BatchToolPanelCoordinator::initSession () {
 
 			if (options.baBehav[ADDSET_LC_BRIGHTNESS])  pparams.labCurve.brightness = 0;
 			if (options.baBehav[ADDSET_LC_CONTRAST])  pparams.labCurve.contrast = 0;
-			if (options.baBehav[ADDSET_LC_SATURATION])  pparams.labCurve.saturation = 0;
+			if (options.baBehav[ADDSET_LC_CHROMATICITY])  pparams.labCurve.chromaticity = 0;
 
 			if (options.baBehav[ADDSET_SHARP_AMOUNT])  pparams.sharpening.amount = 0;
 			if (options.baBehav[ADDSET_SHARPENEDGE_AMOUNT])  pparams.sharpenEdge.amount = 0;
@@ -197,6 +204,20 @@ void BatchToolPanelCoordinator::initSession () {
 			if (options.baBehav[ADDSET_VIBRANCE_PASTELS])  pparams.vibrance.pastels = 0;
 			if (options.baBehav[ADDSET_VIBRANCE_SATURATED])  pparams.vibrance.saturated = 0;
 
+			if (options.baBehav[ADDSET_CAT_DEGREE])  pparams.colorappearance.degree = 0;
+			if (options.baBehav[ADDSET_CAT_ADAPTSCENE])  pparams.colorappearance.adapscen = 0;
+			if (options.baBehav[ADDSET_CAT_ADAPTVIEWING])  pparams.colorappearance.adaplum = 0;
+			if (options.baBehav[ADDSET_CAT_BADPIX])  pparams.colorappearance.badpixsl = 0;
+			if (options.baBehav[ADDSET_CAT_LIGHT])  pparams.colorappearance.jlight = 0;
+			if (options.baBehav[ADDSET_CAT_BRIGHT])  pparams.colorappearance.qbright = 0;
+			if (options.baBehav[ADDSET_CAT_CHROMA])  pparams.colorappearance.chroma = 0;
+			if (options.baBehav[ADDSET_CAT_CHROMA_S])  pparams.colorappearance.schroma = 0;
+			if (options.baBehav[ADDSET_CAT_CHROMA_M])  pparams.colorappearance.mchroma = 0;
+			if (options.baBehav[ADDSET_CAT_RSTPRO])  pparams.colorappearance.rstprotection = 0;
+			if (options.baBehav[ADDSET_CAT_CONTRAST])  pparams.colorappearance.contrast = 0;
+			if (options.baBehav[ADDSET_CAT_CONTRAST_Q])  pparams.colorappearance.qcontrast = 0;
+			if (options.baBehav[ADDSET_CAT_HUE])  pparams.colorappearance.colorh = 0;
+
 			if (options.baBehav[ADDSET_FREE_OUPUT_GAMMA])  pparams.icm.gampos = 0;
 			if (options.baBehav[ADDSET_FREE_OUTPUT_SLOPE])  pparams.icm.slpos = 0;
 			
@@ -211,9 +232,18 @@ void BatchToolPanelCoordinator::initSession () {
 			if (options.baBehav[ADDSET_CA])  pparams.cacorrection.red = 0;
 			if (options.baBehav[ADDSET_CA])  pparams.cacorrection.blue = 0;
 			if (options.baBehav[ADDSET_VIGN_AMOUNT])  pparams.vignetting.amount = 0;
+			if (options.baBehav[ADDSET_VIGN_RADIUS])  pparams.vignetting.radius = 0;
+			if (options.baBehav[ADDSET_VIGN_STRENGTH])  pparams.vignetting.strength = 0;
+			if (options.baBehav[ADDSET_VIGN_CENTER])  pparams.vignetting.centerX = 0;
+			if (options.baBehav[ADDSET_VIGN_CENTER])  pparams.vignetting.centerY = 0;
 
 			if (options.baBehav[ADDSET_DIRPYREQ]) for (int i=0; i<5; i++) pparams.dirpyrequalizer.mult[i] = 0;
-			if (options.baBehav[ADDSET_DIRPYRDN_CHLUM])  pparams.dirpyrDenoise.luma = pparams.dirpyrDenoise.chroma = 0;
+			if (options.baBehav[ADDSET_DIRPYRDN_LUMA]) pparams.dirpyrDenoise.luma = 0;
+			
+			if (options.baBehav[ADDSET_DIRPYRDN_CHROMA]) pparams.dirpyrDenoise.chroma = 0;
+			if (options.baBehav[ADDSET_DIRPYRDN_CHROMARED]) pparams.dirpyrDenoise.redchro = 0;
+			if (options.baBehav[ADDSET_DIRPYRDN_CHROMABLUE]) pparams.dirpyrDenoise.bluechro = 0;
+//			pparams.dirpyrDenoise.Ldetail = pparams.dirpyrDenoise.luma = pparams.dirpyrDenoise.chroma = 0;
 			if (options.baBehav[ADDSET_DIRPYRDN_GAMMA])  pparams.dirpyrDenoise.gamma = 0;
 
 			if (options.baBehav[ADDSET_PREPROCESS_GREENEQUIL])  pparams.raw.greenthresh = 0;
@@ -227,6 +257,9 @@ void BatchToolPanelCoordinator::initSession () {
 		for (size_t i=0; i<toolPanels.size(); i++) {
 			toolPanels[i]->setDefaults (&pparams, &pparamsEdited);
 			toolPanels[i]->read (&pparams, &pparamsEdited);
+			// TODO: autoOpenCurve has been disabled because initSession is called on each parameter change from the editor panel,
+			// if the thumbnail remains selected in the DirectoryBrowser (i.e. always, unless the user think about deselecting it)
+			//toolPanels[i]->autoOpenCurve();
 		}
 		for (size_t i=0; i<paramcListeners.size(); i++)
 			// send this initial state to the History
@@ -308,7 +341,7 @@ void BatchToolPanelCoordinator::procParamsChanged (Thumbnail* thm, int whoChange
 
 /*
  * WARNING: profileChange is actually called by the History only.
- *          Using aProfile panel in the batch tool panel editor is actually
+ *          Using a Profile panel in the batch tool panel editor is actually
  *          not supported by BatchToolPanelCoordinator::profileChange!
  */
 void BatchToolPanelCoordinator::profileChange  (const rtengine::procparams::PartialProfile* nparams, rtengine::ProcEvent event, const Glib::ustring& descr, const ParamsEdited* paramsEdited) {
@@ -325,8 +358,9 @@ void BatchToolPanelCoordinator::profileChange  (const rtengine::procparams::Part
 
 
     for (size_t i=0; i<toolPanels.size(); i++)
-    	// writing the values to the GUI
+        // writing the values to the GUI
         toolPanels[i]->read (&pparams, &pparamsEdited);
+        // I guess we don't want to automatically unfold curve editors here...
 
     somethingChanged = true;
 
