@@ -155,6 +155,7 @@ void CacheManager::deleteEntry (const Glib::ustring& fname) {
 	    if (r==openEntries.end() && md5!="") {
 			safe_g_remove (getCacheFileName ("data", fname, md5) + ".txt");
 			safe_g_remove (getCacheFileName ("profiles", fname, md5) + paramFileExtension);
+			safe_g_remove (getCacheFileName ("images", fname, md5) + ".rtti");
 			safe_g_remove (getCacheFileName ("images", fname, md5) + ".cust16");
 			safe_g_remove (getCacheFileName ("images", fname, md5) + ".cust");
 			safe_g_remove (getCacheFileName ("images", fname, md5) + ".jpg");
@@ -167,6 +168,7 @@ void CacheManager::deleteEntry (const Glib::ustring& fname) {
 	    if (md5!="") {
 	        safe_g_remove (getCacheFileName ("data", fname, md5) + ".txt");
 	        safe_g_remove (getCacheFileName ("profiles", fname, md5) + paramFileExtension);
+	        safe_g_remove (getCacheFileName ("images", fname, md5) + ".rtti");
 	        safe_g_remove (getCacheFileName ("images", fname, md5) + ".cust16");
 	        safe_g_remove (getCacheFileName ("images", fname, md5) + ".cust");
 	        safe_g_remove (getCacheFileName ("images", fname, md5) + ".jpg");
@@ -183,6 +185,7 @@ void CacheManager::clearFromCache (const Glib::ustring& fname, bool leavenotrace
 			safe_g_remove (getCacheFileName ("data", fname, md5) + ".txt");
 			safe_g_remove (getCacheFileName ("profiles", fname, md5) + paramFileExtension);
 		}
+		safe_g_remove (getCacheFileName ("images", fname, md5) + ".rtti");
 		safe_g_remove (getCacheFileName ("images", fname, md5) + ".cust16");
 		safe_g_remove (getCacheFileName ("images", fname, md5) + ".cust");
 		safe_g_remove (getCacheFileName ("images", fname, md5) + ".jpg");
@@ -198,6 +201,7 @@ void CacheManager::renameEntry (const std::string& oldfilename, const std::strin
     std::string newmd5 = getMD5 (newfilename);
 
     safe_g_rename (getCacheFileName ("profiles", oldfilename, oldmd5) + paramFileExtension, (getCacheFileName ("profiles", newfilename, newmd5) + paramFileExtension).c_str());
+    safe_g_rename (getCacheFileName ("images", oldfilename, oldmd5) + ".rtti", getCacheFileName ("images", newfilename, newmd5) + ".rtti");
     safe_g_rename (getCacheFileName ("images", oldfilename, oldmd5) + ".cust16", getCacheFileName ("images", newfilename, newmd5) + ".cust16");
     safe_g_rename (getCacheFileName ("images", oldfilename, oldmd5) + ".cust", getCacheFileName ("images", newfilename, newmd5) + ".cust");
     safe_g_rename (getCacheFileName ("images", oldfilename, oldmd5) + ".jpg", getCacheFileName ("images", newfilename, newmd5) + ".jpg");
@@ -329,6 +333,7 @@ void CacheManager::applyCacheSizeLimitation () {
         std::sort (flist.begin(), flist.end());
         while (flist.size() > options.maxCacheEntries) {
             safe_g_remove (Glib::build_filename (Glib::build_filename (baseDir, "data"), flist.front().fname) + ".txt");
+            safe_g_remove (Glib::build_filename (Glib::build_filename (baseDir, "images"), flist.front().fname) + ".rtti");
             safe_g_remove (Glib::build_filename (Glib::build_filename (baseDir, "images"), flist.front().fname) + ".cust16");
             safe_g_remove (Glib::build_filename (Glib::build_filename (baseDir, "images"), flist.front().fname) + ".cust");
             safe_g_remove (Glib::build_filename (Glib::build_filename (baseDir, "images"), flist.front().fname) + ".jpg");
