@@ -37,6 +37,8 @@ class RefreshSpinHelper {
 
 Crop::Crop (): Gtk::VBox(), FoldableToolPanel(this) {
 
+  set_border_width(4);
+
   clistener = NULL;
 
   maxw = 3000;
@@ -46,7 +48,7 @@ Crop::Crop (): Gtk::VBox(), FoldableToolPanel(this) {
   enabled->set_active (false);
   pack_start(*enabled);
 
-  pack_start(*Gtk::manage (new  Gtk::HSeparator()));
+  pack_start(*Gtk::manage (new  Gtk::HSeparator()), Gtk::PACK_EXPAND_WIDGET, 4);
 
   Gtk::HBox* hb1 = Gtk::manage (new Gtk::HBox ());
 
@@ -186,6 +188,7 @@ Crop::Crop (): Gtk::VBox(), FoldableToolPanel(this) {
   guide->append_text (M("TP_CROP_GTHARMMEANS4"));
   guide->append_text (M("TP_CROP_GTGRID"));
   guide->append_text (M("TP_CROP_GTEPASSPORT"));
+  guide->append_text (M("TP_CROP_GTFRAME"));
   guide->set_active (0);
 
   w->set_range (0, maxw);
@@ -294,6 +297,8 @@ void Crop::read (const ProcParams* pp, const ParamsEdited* pedited) {
         guide->set_active (7);
     else if (pp->crop.guide == "ePassport")
         guide->set_active (8);
+    else if (pp->crop.guide == "Frame")
+        guide->set_active (9);
 
     x->set_value (pp->crop.x);
     y->set_value (pp->crop.y);
@@ -376,12 +381,14 @@ void Crop::write (ProcParams* pp, ParamsEdited* pedited) {
     pp->crop.guide = "Grid";
   else if (guide->get_active_row_number()==8)
     pp->crop.guide = "ePassport";
+  else if (guide->get_active_row_number()==9)
+    pp->crop.guide = "Frame";
 
     if (pedited) {
         pedited->crop.enabled       = !enabled->get_inconsistent();
-        pedited->crop.ratio         = ratio->get_active_row_number() != 8;
+        pedited->crop.ratio         = ratio->get_active_row_number() != 9;
         pedited->crop.orientation   = orientation->get_active_row_number() != 2;
-        pedited->crop.guide         = guide->get_active_row_number() != 8;
+        pedited->crop.guide         = guide->get_active_row_number() != 9;
         pedited->crop.fixratio      = !fixr->get_inconsistent();
         pedited->crop.w             = wDirty;
         pedited->crop.h             = hDirty;
