@@ -35,8 +35,13 @@ struct ZoomStep {
     int czoom;
 };
 
-ZoomStep zoomSteps[] = {{" 10%",  0.1,     10},
+ZoomStep zoomSteps[] = {
+		                {"  5%",  0.05,    20},
+		                {"6.7%",  1.0/15.0,15},
+		                {"  8%",  1.0/12.0,12},
+		                {" 10%",  0.1,     10},
                         {"12.5%", 0.125,   8},
+                        {" 14%",  1.0/7.0, 7},
                         {"16.6%", 1.0/6.0, 6},
                         {" 20%",  0.2,     5},
                         {" 25%",  0.25,    4},
@@ -50,8 +55,8 @@ ZoomStep zoomSteps[] = {{" 10%",  0.1,     10},
                         {"600%",  6.0,     6000},
                         {"700%",  7.0,     7000},
                         {"800%",  8.0,     8000}};
-#define MAXZOOMSTEPS 14
-#define ZOOM11INDEX  7
+#define MAXZOOMSTEPS 18
+#define ZOOM11INDEX  11
 
 CropWindow::CropWindow (ImageArea* parent, rtengine::StagedImageProcessor* ipc_, bool isLowUpdatePriority_) 
     : onResizeArea(false), deleted(false), fitZoomEnabled(true), fitZoom(false), isLowUpdatePriority(isLowUpdatePriority_),
@@ -236,10 +241,11 @@ void CropWindow::buttonPress (int button, int type, int bstate, int x, int y) {
             zoom11 ();
         state = SNormal;
     }
-    else if (button==1 && type==GDK_2BUTTON_PRESS && onArea (CropBorder, x, y)) {
+  //below code is no longer working/needed after adding buttons for each of the backColor values
+  /*else if (button==1 && type==GDK_2BUTTON_PRESS && onArea (CropBorder, x, y)) {
         backColor = (backColor+1) % 3;
         options.bgcolor = backColor;
-    }
+    }*/
     else if (button==1 && type==GDK_BUTTON_PRESS && state==SNormal && onArea (CropToolBar, x, y)) {
         if (!decorated || !buttonSet.pressNotify (x, y)) {
             state = SCropWinMove;
@@ -477,11 +483,9 @@ void CropWindow::pointerMoved (int x, int y) {
 		int mx, my;
 		translateCoord (x, y, mx, my);
 		if (!onArea (CropImage, x, y) || !cropHandler.cropPixbuf) {
-		//	pmlistener->pointerMoved (false, mx, my, -1, -1, -1);
+            cropHandler.getFullImageSize(mx,my);
 			pmlistener->pointerMoved (false, cropHandler.colorParams.working, mx, my, -1, -1, -1);
-                        if (pmhlistener) {
-                           pmhlistener->pointerMoved (false, cropHandler.colorParams.working, mx, my, -1, -1, -1);
-                        }
+            if (pmhlistener) pmhlistener->pointerMoved (false, cropHandler.colorParams.working, mx, my, -1, -1, -1);
                 }
 		else {
 			/*Glib::Mutex::Lock lock(cropHandler.cimg);
@@ -638,6 +642,8 @@ void CropWindow::expose (Cairo::RefPtr<Cairo::Context> cr) {
     int x = xpos, y = ypos, h = height, w = width;
  
     // draw the background
+    backColor = iarea->previewModePanel->GetbackColor();
+    options.bgcolor = backColor;
     if (backColor==0) {
         Gdk::Color cback = iarea->get_style()->get_bg(Gtk::STATE_NORMAL);
         cr->set_source_rgb (cback.get_red_p(), cback.get_green_p(), cback.get_blue_p());
@@ -681,21 +687,17 @@ void CropWindow::expose (Cairo::RefPtr<Cairo::Context> cr) {
     //t3.set ();
 			bool showcs = iarea->indClippedPanel->showClippedShadows();
 			bool showch = iarea->indClippedPanel->showClippedHighlights();
-			bool showR  = iarea->previewModePanel->showR();
-			bool showG  = iarea->previewModePanel->showG();
-			bool showB  = iarea->previewModePanel->showB();
-			bool showL  = iarea->previewModePanel->showL();
+			bool showR  = iarea->previewModePanel->showR(); // will show clipping if R channel is clipped
+			bool showG  = iarea->previewModePanel->showG(); // will show clipping if G channel is clipped
+			bool showB  = iarea->previewModePanel->showB(); // will show clipping if B channel is clipped
+			bool showL  = iarea->previewModePanel->showL(); // will show clipping if L value   is clipped
 			bool showFocusMask  = iarea->previewModePanel->showFocusMask();
-			// additional flags to control clipping indicators for individual channels and across all channels
-			bool showclippedAll = (!showR && !showG && !showB && !showL);
-			bool showclippedR = showR || showL || showclippedAll;
-			bool showclippedG = showG || showL || showclippedAll;
-			bool showclippedB = showB || showL || showclippedAll;
+			bool showclippedAny = (!showR && !showG && !showB && !showL); // will show clipping if any of RGB chanels is clipped
 
-            // If ALT was pressed, auto-enable highlight and shadow
+            // While the Right-side ALT is pressed, auto-enable highlight and shadow clipping indicators
             // TODO: Add linux/MacOS specific functions for alternative
             #ifdef WIN32
-            if (GetKeyState(VK_MENU)<0) {
+            if (GetKeyState(VK_RMENU)<0) {
                 showcs=true; showch=true;
             }
             #endif
@@ -767,7 +769,7 @@ void CropWindow::expose (Cairo::RefPtr<Cairo::Context> cr) {
                                     	// Band2 @ blur_radius2
                                     	if (kh>=-blur_radius2 && kh<=blur_radius2 && k>=-blur_radius2 && k<=blur_radius2){
                                     		sum_L2 += curL;
-						sumsq_L2 += SQR(curL);
+                                    		sumsq_L2 += SQR(curL);
                                     	}
                                     }
                                 }
@@ -779,8 +781,8 @@ void CropWindow::expose (Cairo::RefPtr<Cairo::Context> cr) {
                                 avg_L2 = sum_L2/kernel_size2;
 
 
-				stdDev_L = sqrt(sumsq_L/kernel_size - SQR(avg_L));
-				stdDev_L2 = sqrt(sumsq_L2/kernel_size2 - SQR(avg_L2));
+                                stdDev_L = sqrt(sumsq_L/kernel_size - SQR(avg_L));
+                                stdDev_L2 = sqrt(sumsq_L2/kernel_size2 - SQR(avg_L2));
 
                                 //TODO: try to normalize by average L of the entire (preview) image
 
@@ -815,29 +817,36 @@ void CropWindow::expose (Cairo::RefPtr<Cairo::Context> cr) {
     
                             changedHL=false;
                             changedSH=false;
+
+                            // for efficiency, pre-calculate currWS_L as it may be needed in both
+                            // if (showch) and if (showcs) branches
+                            int currWS_L;
+                            if (showL && (showch||showcs)) currWS_L = (int)(0.299*currWS[0]+0.587*currWS[1]+0.114*currWS[2]);
     
                             if (showch) {
                                 delta=0; changedHL=false;
-    
-                                if (currWS[0]>=options.highlightThreshold && showclippedR) { delta += 255-currWS[0]; changedHL=true; }
-                                if (currWS[1]>=options.highlightThreshold && showclippedG) { delta += 255-currWS[1]; changedHL=true; }
-                                if (currWS[2]>=options.highlightThreshold && showclippedB) { delta += 255-currWS[2]; changedHL=true; }
-    
+
+                            	if (currWS[0]>=options.highlightThreshold && (showR || showclippedAny)) { delta += 255-currWS[0]; changedHL=true; }
+								if (currWS[1]>=options.highlightThreshold && (showG || showclippedAny)) { delta += 255-currWS[1]; changedHL=true; }
+								if (currWS[2]>=options.highlightThreshold && (showB || showclippedAny)) { delta += 255-currWS[2]; changedHL=true; }
+								if (currWS_L>= options.highlightThreshold && showL)                     { delta += 255-currWS_L ; changedHL=true; }
+
                                 if (changedHL) { 
                                     delta *= HighlightFac; 
-                                    if (showclippedAll) curr[0]=curr[1]=curr[2]=delta; // indicate clipped highlights in gray
+                                    if (showclippedAny) curr[0]=curr[1]=curr[2]=delta; // indicate clipped highlights in gray
                                     else {curr[0]=255; curr[1]=curr[2]=delta;}         // indicate clipped highlights in red
                                 }
                             }
                             if (showcs) {
                                 delta=0; changedSH=false;
     
-                                if (currWS[0]<=options.shadowThreshold && showclippedR) { delta += currWS[0]; changedSH=true; }
-                                if (currWS[1]<=options.shadowThreshold && showclippedG) { delta += currWS[1]; changedSH=true; }
-                                if (currWS[2]<=options.shadowThreshold && showclippedB) { delta += currWS[2]; changedSH=true; }
-                                    
+                                if (currWS[0]<=options.shadowThreshold && (showR || showclippedAny)) { delta += currWS[0]; changedSH=true; }
+                                if (currWS[1]<=options.shadowThreshold && (showG || showclippedAny)) { delta += currWS[1]; changedSH=true; }
+                                if (currWS[2]<=options.shadowThreshold && (showB || showclippedAny)) { delta += currWS[2]; changedSH=true; }
+                                if (currWS_L <=options.shadowThreshold && showL)                     { delta += currWS_L ; changedSH=true; }
+
                                 if (changedSH) {                            
-                                    if (showclippedAll) {
+                                    if (showclippedAny) {
                                         delta = 255 - (delta * ShawdowFac);
                                         curr[0]=curr[1]=curr[2]=delta;      // indicate clipped shadows in gray
                                     }
