@@ -24,10 +24,13 @@ using namespace rtengine::procparams;
 
 DirPyrEqualizer::DirPyrEqualizer () : Gtk::VBox(), FoldableToolPanel(this) {
 
+    set_border_width(4);
+
     enabled = Gtk::manage (new Gtk::CheckButton (M("GENERAL_ENABLED")));
     enabled->set_active (true);
     pack_start(*enabled);
     enaConn = enabled->signal_toggled().connect( sigc::mem_fun(*this, &DirPyrEqualizer::enabledToggled) );
+	enabled->set_tooltip_markup (M("TP_SHARPENING_TOOLTIP"));
 
     Gtk::HSeparator *separator1 = Gtk::manage (new  Gtk::HSeparator());
     pack_start(*separator1, Gtk::PACK_SHRINK, 2);
@@ -56,11 +59,14 @@ DirPyrEqualizer::DirPyrEqualizer () : Gtk::VBox(), FoldableToolPanel(this) {
     {
         Glib::ustring ss;
         ss = Glib::ustring::format(i);
-        if(i == 0)
+        if(i == 0) {
             ss += Glib::ustring::compose(" (%1)", M("TP_DIRPYREQUALIZER_LUMAFINEST"));
-        if(i == 3)
-            ss += Glib::ustring::compose(" (%1)", M("TP_DIRPYREQUALIZER_LUMACOARSEST"));
-		multiplier[i] = Gtk::manage ( new Adjuster (ss, 0, 4, 0.01, 1.0) );
+            multiplier[i] = Gtk::manage ( new Adjuster (ss, 0, 4, 0.01, 1.0) );
+        } else {
+	    if(i == 3)
+                ss += Glib::ustring::compose(" (%1)", M("TP_DIRPYREQUALIZER_LUMACOARSEST"));    
+            multiplier[i] = Gtk::manage ( new Adjuster (ss, 0, 4, 0.01, 1.0) );
+	}
         multiplier[i]->setAdjusterListener(this);
         pack_start(*multiplier[i]);
     }
@@ -68,7 +74,7 @@ DirPyrEqualizer::DirPyrEqualizer () : Gtk::VBox(), FoldableToolPanel(this) {
 	Gtk::HSeparator *separator3 = Gtk::manage (new  Gtk::HSeparator());
     pack_start(*separator3, Gtk::PACK_SHRINK, 2);
 	
-	multiplier[4] = Gtk::manage ( new Adjuster (M("TP_DIRPYREQUALIZER_THRESHOLD"), 0, 1, 0.01, 0.0) );
+	multiplier[4] = Gtk::manage ( new Adjuster (M("TP_DIRPYREQUALIZER_THRESHOLD"), 0, 1, 0.01, 0.2) );
 	multiplier[4]->setAdjusterListener(this);
 	pack_start(*multiplier[4]);
 
