@@ -27,6 +27,7 @@
 #include "../rtengine/imagesource.h"
 #include "soundman.h"
 #include "rtimage.h"
+#include <iostream>
 
 using namespace rtengine::procparams;
 
@@ -90,7 +91,6 @@ EditorPanel::EditorPanel (FilePanel* filePanel)
     Gtk::VSeparator* vsepz = Gtk::manage (new Gtk::VSeparator ());
     Gtk::VSeparator* vsepi = Gtk::manage (new Gtk::VSeparator ());
     Gtk::VSeparator* vseph = Gtk::manage (new Gtk::VSeparator ());
-    Gtk::VSeparator* vsep1 = Gtk::manage (new Gtk::VSeparator ());
 
     hidehp = Gtk::manage (new Gtk::ToggleButton ());
 
@@ -107,13 +107,16 @@ EditorPanel::EditorPanel (FilePanel* filePanel)
         hidehp->set_image (*iHistoryShow);
     }
 
-    tbTopPanel_1 = new Gtk::ToggleButton ();
-    iTopPanel_1_Show = new RTImage ("panel-to-bottom.png");
-    iTopPanel_1_Hide = new RTImage ("panel-to-top.png");
-    tbTopPanel_1->set_relief(Gtk::RELIEF_NONE);
-    tbTopPanel_1->set_active (true);
-    tbTopPanel_1->set_tooltip_markup (M("MAIN_TOOLTIP_SHOWHIDETP1"));
-    tbTopPanel_1->set_image (*iTopPanel_1_Hide);
+    tbTopPanel_1 = NULL;
+    if (!simpleEditor && filePanel) {
+        tbTopPanel_1 = new Gtk::ToggleButton ();
+        iTopPanel_1_Show = new RTImage ("panel-to-bottom.png");
+        iTopPanel_1_Hide = new RTImage ("panel-to-top.png");
+        tbTopPanel_1->set_relief(Gtk::RELIEF_NONE);
+        tbTopPanel_1->set_active (true);
+        tbTopPanel_1->set_tooltip_markup (M("MAIN_TOOLTIP_SHOWHIDETP1"));
+        tbTopPanel_1->set_image (*iTopPanel_1_Hide);
+    }
 
     tbRightPanel_1 = new Gtk::ToggleButton ();
     iRightPanel_1_Show = new RTImage ("panel-to-left.png");
@@ -139,8 +142,11 @@ EditorPanel::EditorPanel (FilePanel* filePanel)
     toolBarPanel->pack_start (*tpc->getToolBar(), Gtk::PACK_SHRINK, 1);
     toolBarPanel->pack_start (*vsept, Gtk::PACK_SHRINK, 2);
 
-    toolBarPanel->pack_end   (*tbTopPanel_1, Gtk::PACK_SHRINK, 1);
-    toolBarPanel->pack_end   (*vsep1, Gtk::PACK_SHRINK, 2);
+    if (tbTopPanel_1) {
+        toolBarPanel->pack_end   (*tbTopPanel_1, Gtk::PACK_SHRINK, 1);
+        Gtk::VSeparator* vsep1 = Gtk::manage (new Gtk::VSeparator ());
+        toolBarPanel->pack_end   (*vsep1, Gtk::PACK_SHRINK, 2);
+    }
     toolBarPanel->pack_end   (*tpc->coarse, Gtk::PACK_SHRINK, 2);
     toolBarPanel->pack_end   (*vsepcl, Gtk::PACK_SHRINK, 2);
     toolBarPanel->pack_end   (*iareapanel->imageArea->indClippedPanel, Gtk::PACK_SHRINK, 0);
@@ -218,6 +224,35 @@ EditorPanel::EditorPanel (FilePanel* filePanel)
     iops->pack_end (*iareapanel->imageArea->zoomPanel, Gtk::PACK_SHRINK, 1);
     iops->pack_end (*vsepz3, Gtk::PACK_SHRINK, 2);
 
+    // Navigation buttons
+    Gtk::Image *navPrevImage = Gtk::manage (new RTImage ("nav-prev.png"));
+    navPrevImage->set_padding(0,0);
+    navPrev = Gtk::manage (new Gtk::Button ());
+    navPrev->add(*navPrevImage);
+    navPrev->set_relief(Gtk::RELIEF_NONE);
+    navPrev->set_tooltip_markup(M("MAIN_BUTTON_NAVPREV_TOOLTIP"));
+    
+    Gtk::Image *navNextImage = Gtk::manage (new RTImage ("nav-next.png"));
+    navNextImage->set_padding(0,0);
+    navNext = Gtk::manage (new Gtk::Button ());
+    navNext->add(*navNextImage);
+    navNext->set_relief(Gtk::RELIEF_NONE);
+    navNext->set_tooltip_markup(M("MAIN_BUTTON_NAVNEXT_TOOLTIP"));
+
+    Gtk::Image *navSyncImage = Gtk::manage (new RTImage ("nav-sync.png"));
+    navSyncImage->set_padding(0,0);
+    navSync = Gtk::manage (new Gtk::Button ());
+    navSync->add(*navSyncImage);
+    navSync->set_relief(Gtk::RELIEF_NONE);
+    navSync->set_tooltip_markup(M("MAIN_BUTTON_NAVSYNC_TOOLTIP"));    
+    
+    if (!simpleEditor && !options.tabbedUI){
+	    iops->pack_end (*Gtk::manage(new Gtk::VSeparator()), Gtk::PACK_SHRINK, 0);
+        iops->pack_end (*navNext, Gtk::PACK_SHRINK, 0);
+        iops->pack_end (*navSync, Gtk::PACK_SHRINK, 0);
+        iops->pack_end (*navPrev, Gtk::PACK_SHRINK, 0);
+    }
+    
     editbox->pack_start (*Gtk::manage(new Gtk::HSeparator()), Gtk::PACK_SHRINK, 0);
     editbox->pack_start (*iops, Gtk::PACK_SHRINK, 0);
     editbox->show_all ();
@@ -258,7 +293,7 @@ EditorPanel::EditorPanel (FilePanel* filePanel)
     updateHistogramPosition (0, options.histogramPosition);
 
     show_all ();
-
+/*
     // save as dialog
     if (safe_file_test (options.lastSaveAsPath, Glib::FILE_TEST_IS_DIR))
         saveAsDialog = new SaveAsDialog (options.lastSaveAsPath);
@@ -266,7 +301,7 @@ EditorPanel::EditorPanel (FilePanel* filePanel)
         saveAsDialog = new SaveAsDialog (safe_get_user_picture_dir());
 
     saveAsDialog->set_default_size (options.saveAsDialogWidth, options.saveAsDialogHeight);
-
+*/
     // connect listeners
     profilep->setProfileChangeListener (tpc);
     history->setProfileChangeListener (tpc);
@@ -286,12 +321,17 @@ EditorPanel::EditorPanel (FilePanel* filePanel)
     info->signal_toggled().connect( sigc::mem_fun(*this, &EditorPanel::info_toggled) );
     beforeAfter->signal_toggled().connect( sigc::mem_fun(*this, &EditorPanel::beforeAfterToggled) );
     hidehp->signal_toggled().connect( sigc::mem_fun(*this, &EditorPanel::hideHistoryActivated) );
-    tbTopPanel_1->signal_toggled().connect( sigc::mem_fun(*this, &EditorPanel::tbTopPanel_1_toggled) );
     tbRightPanel_1->signal_toggled().connect( sigc::mem_fun(*this, &EditorPanel::tbRightPanel_1_toggled) );
     saveimgas->signal_pressed().connect( sigc::mem_fun(*this, &EditorPanel::saveAsPressed) );
     queueimg->signal_pressed().connect( sigc::mem_fun(*this, &EditorPanel::queueImgPressed) );
     sendtogimp->signal_pressed().connect( sigc::mem_fun(*this, &EditorPanel::sendToGimpPressed) );
+    navPrev->signal_pressed().connect( sigc::mem_fun(*this, &EditorPanel::openPreviousEditorImage) );
+    navNext->signal_pressed().connect( sigc::mem_fun(*this, &EditorPanel::openNextEditorImage) );
+    navSync->signal_pressed().connect( sigc::mem_fun(*this, &EditorPanel::syncFileBrowser) );
+
     ShowHideSidePanelsconn = tbShowHideSidePanels->signal_toggled().connect ( sigc::mem_fun(*this, &EditorPanel::toggleSidePanels), true);
+    if (tbTopPanel_1)
+        tbTopPanel_1->signal_toggled().connect( sigc::mem_fun(*this, &EditorPanel::tbTopPanel_1_toggled) );
 }
 
 EditorPanel::~EditorPanel () {
@@ -329,9 +369,13 @@ EditorPanel::~EditorPanel () {
     delete ppframe;
     delete leftbox;
     delete vboxright;
-    delete saveAsDialog;
+    //delete saveAsDialog;
     if(catalogPane)
         delete catalogPane;
+
+    if (!iTopPanel_1_Show) delete iTopPanel_1_Show;
+    if (!iTopPanel_1_Hide) delete iTopPanel_1_Hide;
+
 }
 
 void EditorPanel::leftPaneButtonReleased(GdkEventButton *event) {
@@ -354,6 +398,13 @@ void EditorPanel::rightPaneButtonReleased(GdkEventButton *event) {
 	}*/
 }
 
+void EditorPanel::writeOptions() {
+	if (profilep)
+		profilep->writeOptions();
+	if (tpc)
+		tpc->writeOptions();
+}
+
 void EditorPanel::setAspect () {
 	int winW, winH;
 	parent->get_size(winW, winH);
@@ -382,6 +433,7 @@ void EditorPanel::open (Thumbnail* tmb, rtengine::InitialImage* isrc) {
     openThm->increaseRef ();
 
     fname=openThm->getFileName();
+    lastSaveAsFileName = removeExtension (Glib::path_get_basename (fname));
 
     previewHandler = new PreviewHandler ();
 
@@ -408,6 +460,7 @@ void EditorPanel::open (Thumbnail* tmb, rtengine::InitialImage* isrc) {
     // initialize profile
     Glib::ustring defProf = openThm->getType()==FT_Raw ? options.defProfRaw : options.defProfImg;
     profilep->initProfile (defProf, ldprof);
+    profilep->setInitialFileName (fname);
 
     openThm->addThumbnailListener (this);
     info_toggled ();
@@ -692,8 +745,8 @@ void EditorPanel::info_toggled () {
         }
 
         infoString3 = Glib::ustring::compose ("<span size=\"small\">%1</span><span>%2</span>",
-        		Glib::path_get_dirname(openThm->getFileName()) + G_DIR_SEPARATOR_S,
-        		Glib::path_get_basename(openThm->getFileName()));
+        				escapeHtmlChars(Glib::path_get_dirname(openThm->getFileName())) + G_DIR_SEPARATOR_S,
+        				escapeHtmlChars(Glib::path_get_basename(openThm->getFileName()))  );
 
         infoString = Glib::ustring::compose ("%1\n%2\n%3",infoString1, infoString2, infoString3);
     }
@@ -748,6 +801,9 @@ void EditorPanel::tbRightPanel_1_toggled () {
 }
 
 void EditorPanel::tbTopPanel_1_visible (bool visible){
+	if (!tbTopPanel_1)
+		return;
+
 	if (visible)
 		tbTopPanel_1->show();
 	else
@@ -757,9 +813,8 @@ void EditorPanel::tbTopPanel_1_visible (bool visible){
 void EditorPanel::tbTopPanel_1_toggled () {
 
 	if (catalogPane){ // catalogPane does not exist in multitab mode
-		tbTopPanel_1_Active = tbTopPanel_1->get_active();
 
-		if (tbTopPanel_1_Active){
+		if (tbTopPanel_1->get_active()){
 			catalogPane->show();
 			tbTopPanel_1->set_image (*iTopPanel_1_Hide);
 		}
@@ -772,6 +827,10 @@ void EditorPanel::tbTopPanel_1_toggled () {
 	}
 }
 
+/*
+ * WARNING: Take care of the simpleEditor value when adding or modifying shortcut keys,
+ *          since handleShortcutKey is now also triggered in simple editor mode
+ */
 bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
 
     bool ctrl = event->state & GDK_CONTROL_MASK;
@@ -782,10 +841,12 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
     // Editor Layout
     switch(event->keyval) {
         case GDK_L:
-		    tbTopPanel_1->set_active (!tbTopPanel_1->get_active()); // toggle top panel
+			if (tbTopPanel_1)
+				tbTopPanel_1->set_active (!tbTopPanel_1->get_active()); // toggle top panel
 		    if (ctrl) hidehp->set_active (!hidehp->get_active()); // toggle History (left panel)
 			if (alt) tbRightPanel_1->set_active (!tbRightPanel_1->get_active()); // toggle right panel
 			return true;
+			break;
     	case GDK_l:
     		if (!shift && !alt /*&& !ctrl*/){
     			hidehp->set_active (!hidehp->get_active()); // toggle History (left panel)
@@ -800,16 +861,19 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
 				tbRightPanel_1->set_active (!tbRightPanel_1->get_active());
 				return true;
 			}
+			break;
         case GDK_m: // Maximize preview panel: hide top AND right AND history panels
         	if (!ctrl && !alt) {
         		toggleSidePanels();
         		return true;
         	}
+			break;
         case GDK_M: // Maximize preview panel: hide top AND right AND history panels AND (fit image preview)
         	if (!ctrl && !alt) {
         		toggleSidePanelsZoomFit();
         		return true;
         	}
+			break;
     }
 
     if (!alt){
@@ -838,10 +902,20 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
 				case GDK_underscore:
                     iareapanel->imageArea->zoomPanel->zoomOutClicked();
 					return true;
-				case GDK_1:
+				case GDK_z://GDK_1
                     iareapanel->imageArea->zoomPanel->zoom11Clicked();
 					return true;
 
+				case GDK_8: //background color of the preview 0
+				    iareapanel->imageArea->previewModePanel->togglebackColor0();
+				    return true;
+				case GDK_9: //background color of the preview 1
+					iareapanel->imageArea->previewModePanel->togglebackColor1();
+					return true;
+				case GDK_0: //background color of the preview 2
+					iareapanel->imageArea->previewModePanel->togglebackColor2();
+					return true;
+
 				case GDK_r: //preview mode Red
                     iareapanel->imageArea->previewModePanel->toggleR();
 					return true;
@@ -871,16 +945,33 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
 				case GDK_F5:
 					openThm->openDefaultViewer(event->state & GDK_SHIFT_MASK ? 2 : 1);
 					return true;
+				case GDK_y: // synchronize filebrowser with image in Editor
+			    	if (!simpleEditor && fPanel && !fname.empty()){
+	    				fPanel->fileCatalog->selectImage(fname, false);
+			    		return true;
+			    	}
+					break; // to avoid gcc complain
+			    case GDK_x: // clear filters and synchronize filebrowser with image in Editor
+			    	if (!simpleEditor && fPanel && !fname.empty()){
+	    				fPanel->fileCatalog->selectImage(fname, true);
+			    		return true;
+			    	}
+					break; // to avoid gcc complain
 			}
 		}
 		else {
 			// With control
 			switch (event->keyval) {
+				case GDK_S:
+					saveProfile();
+					setProgressStr(M("PROGRESSBAR_PROCESSING_PROFILESAVED"));
+					return true;
 				case GDK_s:
 					saveAsPressed();
 					return true;
 				case GDK_q:
-					queueImgPressed();
+					if (!simpleEditor)
+						queueImgPressed();
 					return true;
 				case GDK_e:
 					sendToGimpPressed();
@@ -897,12 +988,43 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
 			}
 		} //if (!ctrl)
     } //if (!alt)
+    
+    if (alt){
+    	switch (event->keyval) {
+			case GDK_s:
+				history->addBookmarkPressed ();
+				setProgressStr(M("PROGRESSBAR_SNAPSHOT_ADDED"));
+				return true;
+		}
+    }
+    
+    if (shift){
+    	switch (event->keyval) {
+			case GDK_F3: // open Previous image from Editor's perspective
+				if (!simpleEditor && fPanel && !fname.empty()){
+	    			EditorPanel::openPreviousEditorImage();
+		    		return true;
+		    	}
+				break; // to avoid gcc complain
+		    case GDK_F4: // open next image from Editor's perspective
+				if (!simpleEditor && fPanel && !fname.empty()){
+	    			EditorPanel::openNextEditorImage();
+		    		return true;
+		    	}
+				break; // to avoid gcc complain
+		}
+    }
 
-    if(tpc->getToolBar()->handleShortcutKey(event))
+    if(tpc->getToolBar() && tpc->getToolBar()->handleShortcutKey(event))
         return true;
     if(tpc->handleShortcutKey(event))
         return true;
 
+    if (!simpleEditor && fPanel){
+        if (fPanel->handleShortcutKey(event))
+        return true;
+    }
+
     return false;
 }
 
@@ -932,7 +1054,7 @@ bool EditorPanel::idle_saveImage (ProgressConnector<rtengine::IImage16*> *pc, Gl
             ld->startFunc (sigc::bind(sigc::mem_fun(img, &rtengine::IImage16::saveAsPNG), fname, sf.pngCompression, sf.pngBits),
     			            sigc::bind(sigc::mem_fun(*this,&EditorPanel::idle_imageSaved), ld, img, fname, sf));
         else if (sf.format=="jpg")
-            ld->startFunc (sigc::bind(sigc::mem_fun(img, &rtengine::IImage16::saveAsJPEG), fname, sf.jpegQuality),
+            ld->startFunc (sigc::bind(sigc::mem_fun(img, &rtengine::IImage16::saveAsJPEG), fname, sf.jpegQuality, sf.jpegSubSamp),
     			            sigc::bind(sigc::mem_fun(*this,&EditorPanel::idle_imageSaved), ld, img, fname, sf));
     } else {
 		Glib::ustring msg_ = Glib::ustring("<b>") + fname + ": Error during image processing\n</b>";
@@ -1006,42 +1128,54 @@ BatchQueueEntry* EditorPanel::createBatchQueueEntry () {
 void EditorPanel::saveAsPressed () {
 	if (!ipc || !openThm) return;
 	bool fnameOK = false;
-	Glib::ustring fname;
+	Glib::ustring fnameOut;
+
+	SaveAsDialog* saveAsDialog;
+	if (safe_file_test (options.lastSaveAsPath, Glib::FILE_TEST_IS_DIR))
+		saveAsDialog = new SaveAsDialog (options.lastSaveAsPath);
+	else
+		saveAsDialog = new SaveAsDialog (safe_get_user_picture_dir());
+
+	saveAsDialog->set_default_size (options.saveAsDialogWidth, options.saveAsDialogHeight);
+	saveAsDialog->setInitialFileName (lastSaveAsFileName);
+	saveAsDialog->setImagePath (fname);
 
-	saveAsDialog->setInitialFileName (removeExtension (Glib::path_get_basename (openThm->getFileName())));
 	do {
-		saveAsDialog->run ();
-		if (saveAsDialog->getResponse()==Gtk::RESPONSE_CANCEL)
-			return;
+		int result = saveAsDialog->run ();
 
 		// The SaveAsDialog ensure that a filename has been specified
-		fname = saveAsDialog->getFileName ();
+		fnameOut = saveAsDialog->getFileName ();
 
 		options.lastSaveAsPath = saveAsDialog->getDirectory ();
-		options.saveAsDialogWidth = saveAsDialog->get_width();
-		options.saveAsDialogHeight = saveAsDialog->get_height();
-
+		options.saveAsDialogWidth = saveAsDialog->get_width ();
+		options.saveAsDialogHeight = saveAsDialog->get_height ();
+		options.autoSuffix = saveAsDialog->getAutoSuffix ();
+		options.saveMethodNum = saveAsDialog->getSaveMethodNum ();
+		lastSaveAsFileName = Glib::path_get_basename (removeExtension (fnameOut));
 		SaveFormat sf = saveAsDialog->getFormat ();
-
 		options.saveFormat = sf;
-		options.autoSuffix = saveAsDialog->getAutoSuffix();
+		options.forceFormatOpts = saveAsDialog->getForceFormatOpts ();
+
+		if (result != Gtk::RESPONSE_OK)
+			break;
 
 		if (saveAsDialog->getImmediately ()) {
 			// separate filename and the path to the destination directory
-			Glib::ustring dstdir = Glib::path_get_dirname (fname);
-			Glib::ustring dstfname = Glib::path_get_basename (removeExtension(fname));
+			Glib::ustring dstdir = Glib::path_get_dirname (fnameOut);
+			Glib::ustring dstfname = Glib::path_get_basename (removeExtension(fnameOut));
+			Glib::ustring dstext = getExtension (fnameOut);
 
 			if (saveAsDialog->getAutoSuffix()) {
 
 				Glib::ustring fnameTemp;
 				for (int tries=0; tries<100; tries++) {
 					if (tries==0)
-						fnameTemp = Glib::ustring::compose ("%1.%2", Glib::build_filename (dstdir,  dstfname), sf.format);
+						fnameTemp = Glib::ustring::compose ("%1.%2", Glib::build_filename (dstdir,  dstfname), dstext);
 					else
-						fnameTemp = Glib::ustring::compose ("%1-%2.%3", Glib::build_filename (dstdir,  dstfname), tries, sf.format);
+						fnameTemp = Glib::ustring::compose ("%1-%2.%3", Glib::build_filename (dstdir,  dstfname), tries, dstext);
 
 					if (!safe_file_test (fnameTemp, Glib::FILE_TEST_EXISTS)) {
-						fname = fnameTemp;
+						fnameOut = fnameTemp;
 						fnameOK = true;
 						break;
 					}
@@ -1049,13 +1183,7 @@ void EditorPanel::saveAsPressed () {
 			}
 			// check if it exists
 			if (!fnameOK) {
-				if (safe_file_test (fname, Glib::FILE_TEST_EXISTS)) {
-					Glib::ustring msg_ = Glib::ustring("<b>") + fname + ": " + M("MAIN_MSG_ALREADYEXISTS") + "\n" + M("MAIN_MSG_QOVERWRITE") + "</b>";
-					Gtk::MessageDialog msgd (*parent, msg_, true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO, true);
-					int response = msgd.run ();
-					fnameOK = (response==Gtk::RESPONSE_YES);
-				}
-				else fnameOK = true;
+				fnameOK = confirmOverwrite (*saveAsDialog, fnameOut);
 			}
 
 			if (fnameOK) {
@@ -1066,21 +1194,24 @@ void EditorPanel::saveAsPressed () {
 
 				ProgressConnector<rtengine::IImage16*> *ld = new ProgressConnector<rtengine::IImage16*>();
 				ld->startFunc(sigc::bind(sigc::ptr_fun(&rtengine::processImage), job, err, parent->getProgressListener(), options.tunnelMetaData ),
-							  sigc::bind(sigc::mem_fun( *this,&EditorPanel::idle_saveImage ),ld,fname,sf ));
+							  sigc::bind(sigc::mem_fun( *this,&EditorPanel::idle_saveImage ),ld,fnameOut,sf ));
 				saveimgas->set_sensitive(false);
 				sendtogimp->set_sensitive(false);
 			}
 		}
 		else {
 			BatchQueueEntry* bqe = createBatchQueueEntry ();
-			bqe->outFileName = fname;
+			bqe->outFileName = fnameOut;
 			bqe->saveFormat = saveAsDialog->getFormat ();
+			bqe->forceFormatOpts = saveAsDialog->getForceFormatOpts ();
 			parent->addBatchQueueJob (bqe, saveAsDialog->getToHeadOfQueue ());
 			fnameOK = true;
 		}
 		// ask parent to redraw file browser
 		// ... or does it automatically when the tab is switched to it
 	} while (!fnameOK);
+
+	saveAsDialog->hide();
 }
 
 void EditorPanel::queueImgPressed () {
@@ -1102,6 +1233,22 @@ void EditorPanel::sendToGimpPressed () {
     sendtogimp->set_sensitive(false);
 }
 
+
+void EditorPanel::openPreviousEditorImage() {
+	if (!simpleEditor && fPanel && !fname.empty())
+		fPanel->fileCatalog->openNextPreviousEditorImage(fname, true, NAV_PREVIOUS);
+}
+
+void EditorPanel::openNextEditorImage() {
+	if (!simpleEditor && fPanel && !fname.empty())
+		fPanel->fileCatalog->openNextPreviousEditorImage(fname, true, NAV_NEXT);
+}
+
+void EditorPanel::syncFileBrowser() { // synchronize filebrowser with image in Editor
+	if (!simpleEditor && fPanel && !fname.empty())
+		fPanel->fileCatalog->selectImage(fname, true);
+}
+
 bool EditorPanel::idle_sendToGimp( ProgressConnector<rtengine::IImage16*> *pc){
 
 	rtengine::IImage16* img = pc->returnValue();
@@ -1164,9 +1311,14 @@ bool EditorPanel::idle_sentToGimp(ProgressConnector<int> *pc,rtengine::IImage16*
 								if ( safe_file_test(executable, (Glib::FILE_TEST_EXISTS|Glib::FILE_TEST_IS_EXECUTABLE)) ) {
 									success = safe_spawn_command_line_async (cmdLine);
 								}
+#elif defined __APPLE__
+                cmdLine = Glib::ustring("open -a /Applications/GIMP.app \'") + filename + Glib::ustring("\'");
+                success = safe_spawn_command_line_async (cmdLine);
+                std::cout << cmdLine << std::endl;
 #else
-								cmdLine = Glib::ustring("gimp-remote ") + Glib::ustring(" \"") + filename + Glib::ustring("\"");
+								cmdLine = Glib::ustring("gimp \"") + filename + Glib::ustring("\"");
 								success = safe_spawn_command_line_async (cmdLine);
+                std::cout << cmdLine << std::endl;
 #endif
 								if (!success){
 #ifdef WIN32
@@ -1180,11 +1332,13 @@ bool EditorPanel::idle_sentToGimp(ProgressConnector<int> *pc,rtengine::IImage16*
 												ver--;
 										}
 #elif defined __APPLE__
-										cmdLine = Glib::ustring("gimp \"") + filename + Glib::ustring("\"");
+                    cmdLine = Glib::ustring("open -a /Applications/Gimp.app/Contents/Resources/start \'") + filename + Glib::ustring("\'");
 										success = safe_spawn_command_line_async (cmdLine);
+                    std::cout << cmdLine << std::endl;
 #else
-										cmdLine = Glib::ustring("gimp \"") + filename + Glib::ustring("\"");
+										cmdLine = Glib::ustring("gimp-remote \"") + filename + Glib::ustring("\"");
 										success = safe_spawn_command_line_async (cmdLine);
+                    std::cout << cmdLine << std::endl;
 #endif
 								}
 						}
@@ -1202,6 +1356,7 @@ bool EditorPanel::idle_sentToGimp(ProgressConnector<int> *pc,rtengine::IImage16*
 								cmdLine = Glib::ustring("\"") + Glib::build_filename(options.psDir,"Photoshop.exe") + Glib::ustring("\" \"") + filename + Glib::ustring("\"");
 		#endif
 								success = safe_spawn_command_line_async (cmdLine);
+                std::cout << cmdLine << std::endl;
 #endif
 						}
 						else if (options.editorToSendTo==3) {
@@ -1212,11 +1367,12 @@ bool EditorPanel::idle_sentToGimp(ProgressConnector<int> *pc,rtengine::IImage16*
 								}
 #else
 		#ifdef __APPLE__
-								cmdLine = options.customEditorProg + filename;
+								cmdLine = options.customEditorProg + Glib::ustring(" \"") + filename + Glib::ustring("\"");
 		#else
 								cmdLine = Glib::ustring("\"") + options.customEditorProg + Glib::ustring("\" \"") + filename + Glib::ustring("\"");
 		#endif
 								success = safe_spawn_command_line_async (cmdLine);
+                std::cout << cmdLine << std::endl;
 #endif
 						}
 
@@ -1331,18 +1487,22 @@ void EditorPanel::tbBeforeLock_toggled () {
 	tbBeforeLock->get_active()? tbBeforeLock->set_image (*iBeforeLockON) : tbBeforeLock->set_image (*iBeforeLockOFF);
 }
 
-void EditorPanel::histogramChanged (LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma, LUTu & histToneCurve, LUTu & histLCurve,
-    LUTu & histRedRaw, LUTu & histGreenRaw, LUTu & histBlueRaw) {
+void EditorPanel::histogramChanged (LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma, LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM, LUTu & histCCAM,
+    LUTu & histRedRaw, LUTu & histGreenRaw, LUTu & histBlueRaw , LUTu & histChroma) {
 
-    if (histogramPanel) histogramPanel->histogramChanged (histRed, histGreen, histBlue, histLuma, histRedRaw, histGreenRaw, histBlueRaw);
-    tpc->updateCurveBackgroundHistogram (histToneCurve, histLCurve, histRed, histGreen, histBlue, histLuma);
+    if (histogramPanel) histogramPanel->histogramChanged (histRed, histGreen, histBlue, histLuma, histRedRaw, histGreenRaw, histBlueRaw, histChroma);
+    tpc->updateCurveBackgroundHistogram (histToneCurve, histLCurve, histCCurve, histLCAM, histCCAM, histRed, histGreen, histBlue, histLuma);
 }
 
 bool EditorPanel::CheckSidePanelsVisibility() {
-	if(tbTopPanel_1->get_active()==false && tbRightPanel_1->get_active()==false && hidehp->get_active()==false)
-		return false;
-	else
+	if (tbTopPanel_1) {
+		if(tbTopPanel_1->get_active()==false && tbRightPanel_1->get_active()==false && hidehp->get_active()==false)
+			return false;
 		return true;
+	}
+	if(tbRightPanel_1->get_active()==false && hidehp->get_active()==false)
+		return false;
+	return true;
 }
 void EditorPanel::toggleSidePanels(){
 	// Maximize preview panel:
@@ -1351,7 +1511,8 @@ void EditorPanel::toggleSidePanels(){
 	bool bAllSidePanelsVisible;
 	bAllSidePanelsVisible= CheckSidePanelsVisibility();
 
-	tbTopPanel_1->set_active (!bAllSidePanelsVisible);
+	if (tbTopPanel_1)
+		tbTopPanel_1->set_active (!bAllSidePanelsVisible);
 	tbRightPanel_1->set_active (!bAllSidePanelsVisible);
 	hidehp->set_active (!bAllSidePanelsVisible);
 	if (bAllSidePanelsVisible == false)
