@@ -49,6 +49,9 @@ class EditorPanel : public Gtk::VBox,
                     public ThumbnailListener,
                     public HistoryBeforeLineListener,
                     public rtengine::HistogramListener {
+    private:
+
+        Glib::ustring lastSaveAsFileName;
 
     protected:      
         Gtk::ProgressBar  *progressLabel;
@@ -58,8 +61,6 @@ class EditorPanel : public Gtk::VBox,
         Gtk::ToggleButton* tbTopPanel_1;
         Gtk::ToggleButton* tbRightPanel_1;
         Gtk::ToggleButton* tbBeforeLock;
-        bool tbTopPanel_1_Active;
-        bool tbRightPanel_1_Active;
         //bool bAllSidePanelsVisible;
         Gtk::ToggleButton* beforeAfter;
         Gtk::HPaned* hpanedl;
@@ -77,6 +78,9 @@ class EditorPanel : public Gtk::VBox,
         Gtk::Button* queueimg;
         Gtk::Button* saveimgas;
         Gtk::Button* sendtogimp;
+        Gtk::Button* navSync;
+        Gtk::Button* navNext;
+        Gtk::Button* navPrev;
 
         ImageAreaPanel* iareapanel;
         PreviewHandler* previewHandler;
@@ -97,14 +101,14 @@ class EditorPanel : public Gtk::VBox,
         HistogramPanel* histogramPanel;
         ToolPanelCoordinator* tpc;
         RTWindow* parent;
-        SaveAsDialog* saveAsDialog;
+        //SaveAsDialog* saveAsDialog;
         BatchToolPanelCoordinator* btpCoordinator;        
         FilePanel* fPanel;
       
         bool firstProcessingDone;
     
         Thumbnail* openThm;  // may get invalid on external delete event
-        Glib::ustring fname;  // must be safed seperately
+        Glib::ustring fname;  // must be saved separately
 
         rtengine::InitialImage* isrc;
         rtengine::StagedImageProcessor* ipc;
@@ -140,6 +144,7 @@ class EditorPanel : public Gtk::VBox,
         void rightPaneButtonReleased(GdkEventButton *event);
 
         void setParent (RTWindow* p) { parent = p; }
+        void writeOptions();
 
         // progresslistener interface
         void setProgress (double p);
@@ -159,8 +164,8 @@ class EditorPanel : public Gtk::VBox,
         void historyBeforeLineChanged (const rtengine::procparams::ProcParams& params);
         
         // HistogramListener
-        void histogramChanged (LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma, LUTu & histToneCurve, LUTu & histLCurve,
-            LUTu & histRedRaw, LUTu & histGreenRaw, LUTu & histBlueRaw);
+        void histogramChanged (LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma, LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM, LUTu & histCCAM,
+            LUTu & histRedRaw, LUTu & histGreenRaw, LUTu & histBlueRaw, LUTu & histChroma);
 
         // event handlers
         void info_toggled ();
@@ -172,6 +177,9 @@ class EditorPanel : public Gtk::VBox,
         void saveAsPressed ();
         void queueImgPressed ();
         void sendToGimpPressed ();
+        void openNextEditorImage ();
+        void openPreviousEditorImage ();
+        void syncFileBrowser ();
 
         void tbTopPanel_1_visible (bool visible);
         bool CheckSidePanelsVisibility();
