@@ -94,7 +94,10 @@ FileBrowser::FileBrowser ()
     		pmenu->attach (*Gtk::manage(colorlabel[i] = new Gtk::ImageMenuItem (M(Glib::ustring::compose("%1%2","FILEBROWSER_POPUPCOLORLABEL",i)))), 0, 1, p, p+1); p++;
     	}
     }
-    for (int i=1; i<=5; i++){//set color label images
+
+   	//set color label images
+   	colorlabel[0]->set_image(*Gtk::manage(new RTImage ("cglabel0.png")));
+    for (int i=1; i<=5; i++){
     	colorlabel[i]->set_image(*Gtk::manage(new RTImage (Glib::ustring::compose("%1%2%3","clabel",i,".png"))));
     }
         
@@ -268,11 +271,31 @@ FileBrowser::FileBrowser ()
     execcustprof->signal_activate().connect (sigc::bind(sigc::mem_fun(*this, &FileBrowser::menuItemActivated), execcustprof));    
     clearprof->signal_activate().connect (sigc::bind(sigc::mem_fun(*this, &FileBrowser::menuItemActivated), clearprof));
     cachemenu->signal_activate().connect (sigc::bind(sigc::mem_fun(*this, &FileBrowser::menuItemActivated), cachemenu));
+
+
+
+    // A separate pop-up menu for Color Labels
+    int c = 0;
+    pmenuColorLabels = new Gtk::Menu ();
+    for (int i=0; i<=5; i++){
+    	pmenuColorLabels->attach (*Gtk::manage(colorlabel_pop[i] = new Gtk::ImageMenuItem (M(Glib::ustring::compose("%1%2","FILEBROWSER_POPUPCOLORLABEL",i)))), 0, 1, c, c+1); c++;
+	}
+	//set color label images
+	colorlabel_pop[0]->set_image(*Gtk::manage(new RTImage ("cglabel0.png")));
+	for (int i=1; i<=5; i++){
+		colorlabel_pop[i]->set_image(*Gtk::manage(new RTImage (Glib::ustring::compose("%1%2%3","clabel",i,".png"))));
+	}
+	pmenuColorLabels->show_all ();
+
+    // Bind to event handlers
+    for (int i=0; i<=5; i++)
+    	colorlabel_pop[i]->signal_activate().connect (sigc::bind(sigc::mem_fun(*this, &FileBrowser::menuColorlabelActivated), colorlabel_pop[i]));
 }
 
 FileBrowser::~FileBrowser ()
 {
     delete pmenu;
+    delete pmenuColorLabels;
     delete[] amiExtProg;
 }
 
@@ -499,6 +522,18 @@ void FileBrowser::close () {
     lastClicked = NULL;
 }
 
+void FileBrowser::menuColorlabelActivated (Gtk::MenuItem* m) {
+
+    std::vector<FileBrowserEntry*> tbe;
+    tbe.push_back (static_cast<FileBrowserEntry*>(colorLabel_actionData));
+    
+    for (int i=0; i<6; i++)
+        if (m==colorlabel_pop[i]) {
+            colorlabelRequested (tbe, i);
+            return;
+        }
+}
+
 void FileBrowser::menuItemActivated (Gtk::MenuItem* m) {
 
     std::vector<FileBrowserEntry*> mselected;
@@ -613,16 +648,33 @@ void FileBrowser::menuItemActivated (Gtk::MenuItem* m) {
 			}
     	}
     }else if( m==thisIsDF){
-    	if( !options.rtSettings.darkFramesPath.empty() && Gio::File::create_for_path(options.rtSettings.darkFramesPath)->query_exists() ){
-		for (size_t i=0; i<mselected.size(); i++){
-				Glib::RefPtr<Gio::File> file = Gio::File::create_for_path ( mselected[i]->filename );
-				if( !file )continue;
-				Glib::ustring destName = options.rtSettings.darkFramesPath+ "/" + file->get_basename();
-				Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path ( destName );
-				file->move(  dest );
+    	if( !options.rtSettings.darkFramesPath.empty()) {
+    		if (Gio::File::create_for_path(options.rtSettings.darkFramesPath)->query_exists() ){
+    			for (size_t i=0; i<mselected.size(); i++){
+					Glib::RefPtr<Gio::File> file = Gio::File::create_for_path ( mselected[i]->filename );
+					if( !file )continue;
+					Glib::ustring destName = options.rtSettings.darkFramesPath+ "/" + file->get_basename();
+					Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path ( destName );
+					file->move(  dest );
+				}
+				// Reinit cache
+				rtengine::dfm.init( options.rtSettings.darkFramesPath );
 			}
-			// Reinit cache
-			rtengine::dfm.init( options.rtSettings.darkFramesPath );
+			else {
+				// Target directory creation failed, we clear the darkFramesPath setting
+				options.rtSettings.darkFramesPath.clear();
+				Glib::ustring msg_ = Glib::ustring::compose (M("MAIN_MSG_PATHDOESNTEXIST"), options.rtSettings.darkFramesPath)
+				                     +"\n\n"+M("MAIN_MSG_OPERATIONCANCELLED");
+				Gtk::MessageDialog msgd (msg_, true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+				msgd.set_title(M("TP_DARKFRAME_LABEL"));
+				msgd.run ();
+    		}
+    	}
+    	else {
+			Glib::ustring msg_ = M("MAIN_MSG_SETPATHFIRST")+"\n\n"+M("MAIN_MSG_OPERATIONCANCELLED");
+			Gtk::MessageDialog msgd (msg_, true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+			msgd.set_title(M("TP_DARKFRAME_LABEL"));
+			msgd.run ();
     	}
     }
     else if (m==autoFF){
@@ -653,16 +705,33 @@ void FileBrowser::menuItemActivated (Gtk::MenuItem* m) {
     	}
     }
     else if( m==thisIsFF){
-    	if( !options.rtSettings.flatFieldsPath.empty() && Gio::File::create_for_path(options.rtSettings.flatFieldsPath)->query_exists() ){
-		for (size_t i=0; i<mselected.size(); i++){
-				Glib::RefPtr<Gio::File> file = Gio::File::create_for_path ( mselected[i]->filename );
-				if( !file )continue;
-				Glib::ustring destName = options.rtSettings.flatFieldsPath+ "/" + file->get_basename();
-				Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path ( destName );
-				file->move(  dest );
+    	if( !options.rtSettings.flatFieldsPath.empty()) {
+    		if (Gio::File::create_for_path(options.rtSettings.flatFieldsPath)->query_exists() ){
+    			for (size_t i=0; i<mselected.size(); i++){
+					Glib::RefPtr<Gio::File> file = Gio::File::create_for_path ( mselected[i]->filename );
+					if( !file )continue;
+					Glib::ustring destName = options.rtSettings.flatFieldsPath+ "/" + file->get_basename();
+					Glib::RefPtr<Gio::File> dest = Gio::File::create_for_path ( destName );
+					file->move(  dest );
+				}
+				// Reinit cache
+				rtengine::ffm.init( options.rtSettings.flatFieldsPath );
 			}
-			// Reinit cache
-			rtengine::ffm.init( options.rtSettings.flatFieldsPath );
+    		else {
+				// Target directory creation failed, we clear the flatFieldsPath setting
+				options.rtSettings.flatFieldsPath.clear();
+				Glib::ustring msg_ = Glib::ustring::compose (M("MAIN_MSG_PATHDOESNTEXIST"), options.rtSettings.flatFieldsPath)
+				                     +"\n\n"+M("MAIN_MSG_OPERATIONCANCELLED");
+				Gtk::MessageDialog msgd (msg_, true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+				msgd.set_title(M("TP_FLATFIELD_LABEL"));
+				msgd.run ();
+			}
+		}
+		else {
+			Glib::ustring msg_ = M("MAIN_MSG_SETPATHFIRST")+"\n\n"+M("MAIN_MSG_OPERATIONCANCELLED");
+			Gtk::MessageDialog msgd (msg_, true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+			msgd.set_title(M("TP_FLATFIELD_LABEL"));
+			msgd.run ();
     	}
     }
     else if (m==copyprof)
@@ -774,38 +843,51 @@ void FileBrowser::openDefaultViewer (int destination) {
 
 bool FileBrowser::keyPressed (GdkEventKey* event) {
 
-    if ((event->keyval==GDK_C || event->keyval==GDK_c) && event->state & GDK_CONTROL_MASK) {
+    bool ctrl  = event->state & GDK_CONTROL_MASK;
+    bool shift = event->state & GDK_SHIFT_MASK;
+    bool alt   = event->state & GDK_MOD1_MASK;
+
+    if ((event->keyval==GDK_C || event->keyval==GDK_c) && ctrl) {
         copyProfile ();
         return true;
     }
-    else if ((event->keyval==GDK_V || event->keyval==GDK_v) && event->state & GDK_CONTROL_MASK && !(event->state & GDK_SHIFT_MASK)) {
+    else if ((event->keyval==GDK_V || event->keyval==GDK_v) && ctrl && !shift) {
         pasteProfile ();
         return true;
     }
-    else if ((event->keyval==GDK_V || event->keyval==GDK_v) && event->state & GDK_CONTROL_MASK && event->state & GDK_SHIFT_MASK) {
+    else if ((event->keyval==GDK_V || event->keyval==GDK_v) && ctrl && shift) {
         partPasteProfile ();
         return true;
     }
-    else if (event->keyval==GDK_Delete && !(event->state & GDK_SHIFT_MASK)) {
+    else if (event->keyval==GDK_Delete && !shift) {
         menuItemActivated (trash);
         return true;
     }
-    else if (event->keyval==GDK_Delete && event->state & GDK_SHIFT_MASK) {
+    else if (event->keyval==GDK_Delete && shift) {
         menuItemActivated (untrash);
         return true;
     }
-    else if ((event->keyval==GDK_Q || event->keyval==GDK_q) && event->state & GDK_CONTROL_MASK) {
+    else if ((event->keyval==GDK_Q || event->keyval==GDK_q) && ctrl) {
         menuItemActivated (develop);
         return true;
     }
-    else if ((event->keyval==GDK_A || event->keyval==GDK_a) && event->state & GDK_CONTROL_MASK) {
+    else if ((event->keyval==GDK_A || event->keyval==GDK_a) && ctrl) {
         menuItemActivated (selall);
         return true;
     }
-    else if (event->keyval==GDK_F2 && !(event->state & GDK_CONTROL_MASK)) {
+    else if (event->keyval==GDK_F2 && !ctrl) {
         menuItemActivated (rename);
         return true;
     }
+    else if (event->keyval==GDK_F3 && !(ctrl || shift || alt)) {  // open Previous image from FileBrowser perspective
+    	FileBrowser::openPrevImage ();
+        return true;
+    }
+    else if (event->keyval==GDK_F4 && !(ctrl || shift || alt)) {  // open Next image from FileBrowser perspective
+    	FileBrowser::openNextImage ();
+        return true;
+    }
+
     else if (event->keyval==GDK_F5) {
         int dest = 1;
         if (event->state & GDK_SHIFT_MASK)
@@ -824,13 +906,59 @@ bool FileBrowser::keyPressed (GdkEventKey* event) {
         scrollPage(GDK_SCROLL_DOWN);
         return true;
     }
-        
+    
+    else if (shift && !ctrl && !alt) { // rank
+        switch(event->keyval) {
+            case GDK_asciitilde:
+                requestRanking (0);
+                return true;
+            case GDK_exclam:
+                requestRanking (1);
+                return true;
+            case GDK_at:
+                requestRanking (2);
+                return true;
+            case GDK_numbersign:
+                requestRanking (3);
+                return true;
+            case GDK_dollar:
+                requestRanking (4);
+                return true;
+            case GDK_percent:
+                requestRanking (5);
+                return true;
+        }
+    }
+    
+    else if (shift && ctrl && !alt) { // color labels
+        switch(event->keyval) {
+            case GDK_asciitilde:
+                requestColorLabel (0);
+                return true;
+            case GDK_exclam:
+                requestColorLabel (1);
+                return true;
+            case GDK_at:
+                requestColorLabel (2);
+                return true;
+            case GDK_numbersign:
+                requestColorLabel (3);
+                return true;
+            case GDK_dollar:
+                requestColorLabel (4);
+                return true;
+            case GDK_percent:
+                requestColorLabel (5);
+                return true;
+        }
+    }
+
     return false;
 }
 
 void FileBrowser::applyMenuItemActivated (Glib::ustring ppname) {
 
-    rtengine::procparams::PartialProfile* partProfile = profileStore.getProfile (ppname);
+    const rtengine::procparams::PartialProfile* partProfile = profileStore.getProfile (ppname);
     if (partProfile->pparams && !selected.empty()) {
 	    for (size_t i=0; i<selected.size(); i++)
             (static_cast<FileBrowserEntry*>(selected[i]))->thumbnail->setProcParams (*partProfile->pparams, partProfile->pedited, FILEBROWSER);
@@ -843,7 +971,7 @@ void FileBrowser::applyPartialMenuItemActivated (Glib::ustring ppname) {
 	if (!tbl || selected.empty())
 		return;
 
-	rtengine::procparams::PartialProfile* srcProfiles = profileStore.getProfile (ppname);
+	const rtengine::procparams::PartialProfile* srcProfiles = profileStore.getProfile (ppname);
 
 	if (srcProfiles->pparams) {
 		if (partialPasteDlg.run()==Gtk::RESPONSE_OK) {
@@ -922,8 +1050,24 @@ bool FileBrowser::checkFilter (ThumbBrowserEntryBase* entryb) { // true -> entry
     	FileName = FileName.uppercase();
     	//printf("FileBrowser::checkFilter FileName = '%s'; find() result= %i \n",FileName.c_str(), FileName.find(filter.queryFileName.uppercase()));
     	
-    	if (FileName.find(filter.queryFileName.uppercase())==-1)
-    		 return false;
+    	// Consider that queryFileName consist of comma separated values (FilterString)
+    	// Evaluate if ANY of these FilterString are contained in the filename
+    	// This will construct OR filter within the filter.queryFileName
+    	int iFilenameMatch=0;
+    	std::vector<Glib::ustring> vFilterStrings = Glib::Regex::split_simple(",", filter.queryFileName.uppercase());
+    	for(int i=0; i<vFilterStrings.size(); i++) {
+    		// ignore empty vFilterStrings. Otherwise filter will always return true if
+    		// e.g. filter.queryFileName ends on "," and will stop being a filter
+    		if (!vFilterStrings.at(i).empty()){
+    			if (FileName.find(vFilterStrings.at(i))!=-1) iFilenameMatch++;
+    		}
+    	}
+    	if (iFilenameMatch==0) //none of the vFilterStrings found in FileName
+        	 return false;
+
+    	/*experimental Regex support, this is unlikely to be useful to photographers*/
+    	//bool matchfound=Glib::Regex::match_simple(filter.queryFileName.uppercase(),FileName);
+    	//if (!matchfound) return false;
     }
 
     // check exif filter
@@ -1029,6 +1173,22 @@ void FileBrowser::colorlabelRequested (std::vector<FileBrowserEntry*> tbe, int c
     applyFilter (filter);
 }
 
+void FileBrowser::requestRanking(int rank){
+    std::vector<FileBrowserEntry*> mselected;
+    for (size_t i=0; i<selected.size(); i++)
+      mselected.push_back (static_cast<FileBrowserEntry*>(selected[i]));
+
+    rankingRequested (mselected, rank);
+}
+
+void FileBrowser::requestColorLabel(int colorlabel){
+    std::vector<FileBrowserEntry*> mselected;
+    for (size_t i=0; i<selected.size(); i++)
+      mselected.push_back (static_cast<FileBrowserEntry*>(selected[i]));
+
+    colorlabelRequested (mselected, colorlabel);
+}
+
 void FileBrowser::buttonPressed (LWButton* button, int actionCode, void* actionData) {
 
     if (actionCode>=0 && actionCode<=5) { // rank
@@ -1050,54 +1210,159 @@ void FileBrowser::buttonPressed (LWButton* button, int actionCode, void* actionD
         else
             fromTrashRequested (tbe);
     }
+    else if (actionCode==8 && tbl) { // color label
+    	// show popup menu
+    	colorLabel_actionData = actionData;// this will be reused when pmenuColorLabels is clicked
+    	pmenuColorLabels->popup (3, this->eventTime);
+    }
 }
 
 void FileBrowser::openNextImage () {
-	// TODO: Check for Linux
-	#ifdef WIN32
-	Glib::RWLock::ReaderLock l(entryRW);
-	#endif
+    // TODO: Check for Linux
+    #ifdef WIN32
+    Glib::RWLock::ReaderLock l(entryRW);
+    #endif
 
-    if (!fd.empty()) {
-	    for (size_t i=fd.size()-1; i>0; i--)
-            if (editedFiles.find (fd[i]->filename)!=editedFiles.end()) 
-                if (i<fd.size()-1 && tbl) {
-                    std::vector<Thumbnail*> entries;
-		    entries.push_back ((static_cast<FileBrowserEntry*>(fd[i+1]))->thumbnail);
-                    tbl->openRequested (entries);
-                    return;
+    if (!fd.empty() && selected.size()>0 && !options.tabbedUI) {
+
+        for (size_t i=0; i<fd.size()-1; i++){
+            if (selected[0]->thumbnail->getFileName()==fd[i]->filename) {// located 1-st image in current selection
+                if (i<fd.size() && tbl) {
+                    // find the first not-filtered-out (next) image
+                    for (size_t k=i+1; k<fd.size(); k++){
+                        if (!fd[k]->filtered/*checkFilter (fd[k])*/){
+                            // clear current selection
+                            for (size_t j=0; j<selected.size(); j++)
+                                selected[j]->selected = false;
+                            selected.clear ();
+
+                            // set new selection
+                            fd[k]->selected = true;
+                            selected.push_back (fd[k]);
+                            //queue_draw ();
+                            notifySelectionListener ();
+
+                            // scroll to the selected position
+                            double h1, v1;
+                            getScrollPosition(h1,v1);
+
+                            double h2=selected[0]->getStartX();
+                            double v2=selected[0]->getStartY();
+
+                            // scroll only when selected[0] is outside of the displayed bounds
+                            if (h2+fd[k]->getMinimalWidth()-h1 > get_width())
+                                setScrollPosition(h2-(get_width()-fd[k]->getMinimalWidth()),v2);
+                            if (h1>h2)
+                                setScrollPosition(h2,v2);
+
+                            // open the selected image
+                            std::vector<Thumbnail*> entries;
+                            entries.push_back ((static_cast<FileBrowserEntry*>(fd[k]))->thumbnail);
+                            tbl->openRequested (entries);
+                            return;
+                        }
+                    }
                 }
-        if (tbl) {
-            std::vector<Thumbnail*> entries;
-	    entries.push_back ((static_cast<FileBrowserEntry*>(fd[0]))->thumbnail);
-            tbl->openRequested (entries);
+            }
         }
     }
 }
 
 void FileBrowser::openPrevImage () {
-	// TODO: Check for Linux
-	#ifdef WIN32
-	Glib::RWLock::ReaderLock l(entryRW);
-	#endif
+    // TODO: Check for Linux
+    #ifdef WIN32
+    Glib::RWLock::ReaderLock l(entryRW);
+    #endif
 
-    if (!fd.empty()) {
-	    for (size_t i=0; i<fd.size(); i++)
-            if (editedFiles.find (fd[i]->filename)!=editedFiles.end()) 
+    if (!fd.empty() && selected.size()>0 && !options.tabbedUI) {
+
+        for (size_t i=1; i<fd.size(); i++){
+            if (selected[0]->thumbnail->getFileName()==fd[i]->filename) {// located 1-st image in current selection
                 if (i>0 && tbl) {
-                    std::vector<Thumbnail*> entries;
-		    entries.push_back ((static_cast<FileBrowserEntry*>(fd[i-1]))->thumbnail);
-                    tbl->openRequested (entries);
-                    return;
+                    // find the first not-filtered-out (previous) image
+                    for (size_t k=i-1; k>=0; k--){
+                        if (!fd[k]->filtered/*checkFilter (fd[k])*/){
+                            // clear current selection
+                            for (size_t j=0; j<selected.size(); j++)
+                                selected[j]->selected = false;
+                            selected.clear ();
+
+                            // set new selection
+                            fd[k]->selected = true;
+                            selected.push_back (fd[k]);
+                            //queue_draw ();
+                            notifySelectionListener ();
+
+                            // scroll to the selected position
+                            double h1, v1;
+                            getScrollPosition(h1,v1);
+
+                            double h2=selected[0]->getStartX();
+                            double v2=selected[0]->getStartY();
+
+                            // scroll only when selected[0] is outside of the displayed bounds
+                            if (h2+fd[k]->getMinimalWidth()-h1 > get_width())
+                                setScrollPosition(h2-(get_width()-fd[k]->getMinimalWidth()),v2);
+                            if (h1>h2)
+                                setScrollPosition(h2,v2);
+
+                            // open the selected image
+                            std::vector<Thumbnail*> entries;
+                            entries.push_back ((static_cast<FileBrowserEntry*>(fd[k]))->thumbnail);
+                            tbl->openRequested (entries);
+                            return;
+                        }
+                    }
                 }
-        if (tbl) {
-            std::vector<Thumbnail*> entries;
-	    entries.push_back ((static_cast<FileBrowserEntry*>(fd[fd.size()-1]))->thumbnail);
-            tbl->openRequested (entries);
+            }
         }
     }
 }
 
+
+void FileBrowser::selectImage (Glib::ustring fname) {
+
+    // need to clear the filter in filecatalog
+
+    if (!fd.empty() && !options.tabbedUI) {
+        for (size_t i=0; i<fd.size(); i++){
+            if (fname==fd[i]->filename && !fd[i]->filtered) {
+                // matching file found for sync
+
+                // clear current selection
+                for (size_t j=0; j<selected.size(); j++)
+                    selected[j]->selected = false;
+                selected.clear ();
+
+                // set new selection
+                fd[i]->selected = true;
+                selected.push_back (fd[i]);
+                queue_draw ();
+                notifySelectionListener ();
+
+                // scroll to the selected position
+                double h=selected[0]->getStartX();
+                double v=selected[0]->getStartY();
+                setScrollPosition(h,v);
+                
+                return;
+            }
+        }
+    }
+}
+
+void FileBrowser::openNextPreviousEditorImage (Glib::ustring fname, eRTNav nextPrevious) {
+
+    // let FileBrowser acquire Editor's perspective
+    selectImage (fname);
+
+    // now switch to the requested image
+    if (nextPrevious==NAV_NEXT)
+        openNextImage();
+    else if (nextPrevious==NAV_PREVIOUS)
+        openPrevImage();
+}
+
 int refreshThumbImagesUI (void* data) {
   (static_cast<FileBrowser*>(data))->_thumbRearrangementNeeded ();
     return 0;
