@@ -94,6 +94,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
     hbToolBar1->pack_start (*hbBrowsePath, Gtk::PACK_EXPAND_WIDGET,0);
 
     BrowsePath->signal_activate().connect (sigc::mem_fun(*this, &FileCatalog::buttonBrowsePathPressed)); //respond to the Enter key
+    BrowsePath->signal_key_press_event().connect(sigc::mem_fun(*this, &FileCatalog::BrowsePath_key_pressed));
 
     //setup Query
     iQueryClear = new RTImage("gtk-close-small.png");
@@ -114,6 +115,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
     hbToolBar1->pack_start (*hbQuery, Gtk::PACK_SHRINK,0);
 
     Query->signal_activate().connect (sigc::mem_fun(*this, &FileCatalog::executeQuery)); //respond to the Enter key
+    Query->signal_key_press_event().connect(sigc::mem_fun(*this, &FileCatalog::Query_key_pressed));
 
     // if NOT a single row toolbar
     if (!options.FileBrowserToolbarSingleRow) pack_start (*hbToolBar1, Gtk::PACK_SHRINK,0);
@@ -146,7 +148,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
     bFilterClear->set_relief (Gtk::RELIEF_NONE);
     bFilterClear->set_tooltip_markup (M("FILEBROWSER_SHOWDIRHINT"));
     bFilterClear->signal_button_press_event().connect (sigc::mem_fun(*this, &FileCatalog::capture_event),false);
-    bCateg[0] = bFilterClear->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bFilterClear));
+    bCateg[0] = bFilterClear->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bFilterClear, true));
     buttonBar->pack_start (*bFilterClear, Gtk::PACK_SHRINK);
     buttonBar->pack_start (*Gtk::manage(new Gtk::VSeparator), Gtk::PACK_SHRINK);
 
@@ -161,7 +163,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
     bUnRanked->set_image (*igUnRanked);
     bUnRanked->set_relief (Gtk::RELIEF_NONE);
     bUnRanked->set_tooltip_markup (M("FILEBROWSER_SHOWUNRANKHINT"));
-    bCateg[1] = bUnRanked->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bUnRanked));    
+    bCateg[1] = bUnRanked->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bUnRanked, true));
     fltrRankbox->pack_start (*bUnRanked, Gtk::PACK_SHRINK);
     bUnRanked->signal_button_press_event().connect (sigc::mem_fun(*this, &FileCatalog::capture_event),false);
 
@@ -174,7 +176,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
         bRank[i]->set_image (*igranked[i]);
         bRank[i]->set_relief (Gtk::RELIEF_NONE);
         fltrRankbox->pack_start (*bRank[i], Gtk::PACK_SHRINK);
-        bCateg[i+2] = bRank[i]->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bRank[i]));    
+        bCateg[i+2] = bRank[i]->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bRank[i], true));    
         bRank[i]->signal_button_press_event().connect (sigc::mem_fun(*this, &FileCatalog::capture_event),false);
     }  
 
@@ -185,7 +187,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
     bUnCLabeled->set_image (*igUnCLabeled);
     bUnCLabeled->set_relief (Gtk::RELIEF_NONE);
     bUnCLabeled->set_tooltip_markup (M("FILEBROWSER_SHOWUNCOLORHINT"));
-    bCateg[7] = bUnCLabeled->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bUnCLabeled));
+    bCateg[7] = bUnCLabeled->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bUnCLabeled, true));
     fltrLabelbox->pack_start (*bUnCLabeled, Gtk::PACK_SHRINK);
     bUnCLabeled->signal_button_press_event().connect (sigc::mem_fun(*this, &FileCatalog::capture_event),false);
 
@@ -198,7 +200,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
 		bCLabel[i]->set_image (*igCLabeled[i]);
 		bCLabel[i]->set_relief (Gtk::RELIEF_NONE);
 		fltrLabelbox->pack_start (*bCLabel[i], Gtk::PACK_SHRINK);
-		bCateg[i+8] = bCLabel[i]->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bCLabel[i]));
+		bCateg[i+8] = bCLabel[i]->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bCLabel[i], true));
 		bCLabel[i]->signal_button_press_event().connect (sigc::mem_fun(*this, &FileCatalog::capture_event),false);
     }
 
@@ -237,7 +239,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
 		bEdited[i]->set_relief (Gtk::RELIEF_NONE);
 		fltrEditedBox->pack_start (*bEdited[i], Gtk::PACK_SHRINK);
 		//13, 14
-		bCateg[i+13] = bEdited[i]->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bEdited[i]));
+		bCateg[i+13] = bEdited[i]->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bEdited[i], true));
 		bEdited[i]->signal_button_press_event().connect (sigc::mem_fun(*this, &FileCatalog::capture_event),false);
     }
     bEdited[0]->set_tooltip_markup (M("FILEBROWSER_SHOWEDITEDNOTHINT"));
@@ -256,7 +258,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
 		bRecentlySaved[i]->set_relief (Gtk::RELIEF_NONE);
 		fltrRecentlySavedBox->pack_start (*bRecentlySaved[i], Gtk::PACK_SHRINK);
 		//15, 16
-		bCateg[i+15] = bRecentlySaved[i]->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bRecentlySaved[i]));
+		bCateg[i+15] = bRecentlySaved[i]->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bRecentlySaved[i], true));
 		bRecentlySaved[i]->signal_button_press_event().connect (sigc::mem_fun(*this, &FileCatalog::capture_event),false);
     }
     bRecentlySaved[0]->set_tooltip_markup (M("FILEBROWSER_SHOWRECENTLYSAVEDNOTHINT"));
@@ -276,7 +278,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
     bTrash->set_image (*iTrashEmpty);
     bTrash->set_relief (Gtk::RELIEF_NONE);
     bTrash->set_tooltip_markup (M("FILEBROWSER_SHOWTRASHHINT"));   
-    bCateg[17] = bTrash->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bTrash));
+    bCateg[17] = bTrash->signal_toggled().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::categoryButtonToggled), bTrash, true));
     bTrash->signal_button_press_event().connect (sigc::mem_fun(*this, &FileCatalog::capture_event),false);
     buttonBar->pack_start (*bTrash, Gtk::PACK_SHRINK);
     buttonBar->pack_start (*Gtk::manage(new Gtk::VSeparator), Gtk::PACK_SHRINK);
@@ -486,7 +488,7 @@ void FileCatalog::dirSelected (const Glib::ustring& dirname, const Glib::ustring
         previewsToLoad = 0;
         previewsLoaded = 0;
         // if openfile exists, we have to open it first (it is a command line argument)
-        if (openfile!="") 
+        if (!openfile.empty())
             addAndOpenFile (openfile);
 
 		selectedDirectory = dir->get_parse_name();
@@ -502,7 +504,11 @@ void FileCatalog::dirSelected (const Glib::ustring& dirname, const Glib::ustring
         }
 
         _refreshProgressBar ();
-        filepanel->loadingThumbs(M("PROGRESSBAR_LOADINGTHUMBS"),0);
+        if (previewsToLoad == 0) {
+        	filepanel->loadingThumbs(M("PROGRESSBAR_NOIMAGES"),0);
+        } else {
+        	filepanel->loadingThumbs(M("PROGRESSBAR_LOADINGTHUMBS"),0);
+        }
 
 #ifdef WIN32
       wdMonitor = new WinDirMonitor (selectedDirectory, this);
@@ -642,6 +648,17 @@ void FileCatalog::previewsFinishedUI () {
  	fileBrowser->applyFilter (getFilter());  // refresh total image count
     _refreshProgressBar();
     filepanel->loadingThumbs(M("PROGRESSBAR_READY"),0);
+
+    if (!imageToSelect_fname.empty()){
+        fileBrowser->selectImage(imageToSelect_fname);
+        imageToSelect_fname = "";
+    }
+
+    if (!refImageForOpen_fname.empty() && actionNextPrevious!=NAV_NONE){
+        fileBrowser->openNextPreviousEditorImage(refImageForOpen_fname,actionNextPrevious);
+        refImageForOpen_fname = "";
+        actionNextPrevious = NAV_NONE;
+    }
 }
 
 void FileCatalog::previewsFinished (int dir_id) {
@@ -745,6 +762,10 @@ void FileCatalog::deleteRequested  (std::vector<FileBrowserEntry*> tbe, bool inc
 			if (inclBatchProcessed) {
 			    Glib::ustring procfName = Glib::ustring::compose ("%1.%2", BatchQueue::calcAutoFileNameBase(fname), options.saveFormatBatch.format);
 				if (safe_file_test (procfName, Glib::FILE_TEST_EXISTS)) safe_g_remove (procfName);
+
+				// delete paramfile if found
+				Glib::ustring procfNameParamFile = Glib::ustring::compose ("%1.%2.out%3", BatchQueue::calcAutoFileNameBase(fname), options.saveFormatBatch.format, paramFileExtension);
+				if (safe_file_test (procfNameParamFile, Glib::FILE_TEST_EXISTS)) safe_g_remove (procfNameParamFile);
 			}
 
             previewsLoaded--;
@@ -876,10 +897,11 @@ void FileCatalog::developRequested (std::vector<FileBrowserEntry*> tbe, bool fas
 				if (options.fastexport_bypass_dirpyrDenoise      ) params.dirpyrDenoise.enabled       = false;
 				if (options.fastexport_bypass_sh_hq              ) params.sh.hq                       = false;
 				if (options.fastexport_bypass_dirpyrequalizer    ) params.dirpyrequalizer.enabled     = false;
-				if (options.fastexport_bypass_raw_all_enhance    ) params.raw.all_enhance             = false;
+				//if (options.fastexport_bypass_raw_all_enhance    ) params.raw.all_enhance             = false;
 				if (options.fastexport_bypass_raw_ccSteps        ) params.raw.ccSteps                 = 0;
 				if (options.fastexport_bypass_raw_dcb_iterations ) params.raw.dcb_iterations          = 0;
 				if (options.fastexport_bypass_raw_dcb_enhance    ) params.raw.dcb_enhance             = false;
+				if (options.fastexport_bypass_raw_lmmse_iterations ) params.raw.lmmse_iterations          = 0;
 				if (options.fastexport_bypass_raw_ca             ) {params.raw.ca_autocorrect =false; params.raw.cared=0; params.raw.cablue=0;}
 				if (options.fastexport_bypass_raw_linenoise      ) params.raw.linenoise               = 0;
 				if (options.fastexport_bypass_raw_greenthresh    ) params.raw.greenthresh             = 0;
@@ -962,7 +984,7 @@ void FileCatalog::renameRequested  (std::vector<FileBrowserEntry*> tbe) {
 					continue;
 				// if no extension is given, concatenate the extension of the original file
 				Glib::ustring ext = getExtension (nBaseName);
-				if (ext=="")
+				if (ext.empty())
 					nBaseName += "." + getExtension (baseName);
 				Glib::ustring nfname = Glib::build_filename (dirName, nBaseName);
 
@@ -1047,7 +1069,7 @@ void FileCatalog::clearFromCacheRequested  (std::vector<FileBrowserEntry*> tbe,
 	 }
 }
 
-void FileCatalog::categoryButtonToggled (Gtk::ToggleButton* b) {
+void FileCatalog::categoryButtonToggled (Gtk::ToggleButton* b, bool isMouseClick) {
     
     //was control key pressed
     bool control_down = modifierKey & GDK_CONTROL_MASK;
@@ -1055,11 +1077,16 @@ void FileCatalog::categoryButtonToggled (Gtk::ToggleButton* b) {
     //was shift key pressed
     bool shift_down   = modifierKey & GDK_SHIFT_MASK;
 
+    // The event is process here, we can clear modifierKey now, it'll be set again on the next even
+    modifierKey = 0;
+
 	for (int i=0; i<18; i++)
 		bCateg[i].block (true);
 
-	//button is already toggled when entering this function, so we switch it back to its initial state
-    b->set_active(!b->get_active());
+    // button already toggled when entering this function from a mouse click, so
+    // we switch it back to its initial state.
+    if (isMouseClick)
+	b->set_active(!b->get_active());
 
     //if both control and shift keys were pressed, do nothing
     if (!(control_down && shift_down)) {
@@ -1080,10 +1107,24 @@ void FileCatalog::categoryButtonToggled (Gtk::ToggleButton* b) {
 			if (categoryButtons[i] == b) toggled_button = i;
 		}
 
-		//if no modifier key were pressed, we can switch-on the button, and clear all others
+		// if no modifier key is pressed, 
 		if (!(control_down || shift_down)) {
-			for (int i=0; i<18; i++) {
-				categoryButtons[i]->set_active (i==toggled_button);
+			// if we're deselecting the only star still active
+			if (toggled_stars_count == 1 && (buttons & (1 << toggled_button))) {
+				// activate clear-filters
+				categoryButtons[0]->set_active (true);
+				// deactivate the toggled filter
+				categoryButtons[toggled_button]->set_active (false);
+			}
+			// if we're deselecting trash
+			else if (toggled_button == 17 && (buttons & (1 << toggled_button))) {
+				categoryButtons[0]->set_active (true);	
+				categoryButtons[17]->set_active (false);
+			}
+			else {
+				// activate the toggled filter, deactivate the rest
+				for (int i=0; i<18; i++)
+					categoryButtons[i]->set_active (i==toggled_button);
 			}
 		}
 		//modifier key allowed only for stars and color labels
@@ -1091,7 +1132,7 @@ void FileCatalog::categoryButtonToggled (Gtk::ToggleButton* b) {
 			if (control_down) {
 				//control is pressed
 				if (toggled_stars_count == 1 && (buttons & (1 << toggled_button))) {
-					//we're deselecting the only star still active, so we activate the folder filter
+					//we're deselecting the only star still active, so we activate clear-filters
 					categoryButtons[0]->set_active(true);
 					//and we deselect the toggled star
 					categoryButtons[toggled_button]->set_active (false);
@@ -1315,7 +1356,7 @@ void FileCatalog::filterChanged () {
 
 void FileCatalog::reparseDirectory () {
 
-    if (selectedDirectory=="")
+    if (selectedDirectory.empty())
         return;
 
     if (!safe_file_test (selectedDirectory, Glib::FILE_TEST_IS_DIR)) {
@@ -1505,6 +1546,22 @@ void FileCatalog::executeQuery(){
 		FileCatalog::filterChanged ();
 }
 
+bool FileCatalog::Query_key_pressed (GdkEventKey *event){
+
+    bool shift = event->state & GDK_SHIFT_MASK;
+
+    switch (event->keyval) {
+    case GDK_Escape:
+        // Clear Query if the Escape character is pressed within it
+        if (!shift){
+            FileCatalog::buttonQueryClearPressed ();
+            return true;
+        }
+    default:
+        return false;
+    }
+}
+
 void FileCatalog::updateFBQueryTB (bool singleRow) {
 	hbToolBar1->reference();
 	if (singleRow) {
@@ -1540,7 +1597,7 @@ void FileCatalog::buttonBrowsePathPressed () {
 		DecodedPathPrefix = safe_get_user_picture_dir();
 	}
 
-	if (DecodedPathPrefix!=""){
+	if (!DecodedPathPrefix.empty()){
 		BrowsePathValue = Glib::ustring::compose ("%1%2",DecodedPathPrefix,BrowsePathValue.substr (1,BrowsePath->get_text_length()-1));
 		BrowsePath->set_text(BrowsePathValue);
 	}
@@ -1555,6 +1612,24 @@ void FileCatalog::buttonBrowsePathPressed () {
 		buttonBrowsePath->set_image (*iRefreshRed);
 }
 
+bool FileCatalog::BrowsePath_key_pressed (GdkEventKey *event){
+
+    bool shift = event->state & GDK_SHIFT_MASK;
+
+    switch (event->keyval) {
+    case GDK_Escape:
+    // On Escape character Reset BrowsePath to selectedDirectory
+        if (!shift){
+            BrowsePath->set_text(selectedDirectory);
+            // place cursor at the end
+            BrowsePath->select_region(BrowsePath->get_text_length(), BrowsePath->get_text_length());
+            return true;
+        }
+    default:
+        return false;
+    }
+}
+
 void FileCatalog::tbLeftPanel_1_visible (bool visible){
 	if (visible)
 		tbLeftPanel_1->show();
@@ -1605,6 +1680,65 @@ void FileCatalog::toggleSidePanels(){
 	tbRightPanel_1->set_active (!bAllSidePanelsVisible);
 }
 
+void FileCatalog::selectImage (Glib::ustring fname, bool clearFilters) {
+
+	Glib::ustring dirname = Glib::path_get_dirname(fname);
+	if (!dirname.empty()){
+		BrowsePath->set_text(dirname);
+
+		
+		if (clearFilters){ // clear all filters
+			Query->set_text("");
+			categoryButtonToggled(bFilterClear,false);
+			// disable exif filters
+			if (filterPanel->isEnabled()) filterPanel->setEnabled (false);
+		}
+		
+		if (BrowsePath->get_text()!=selectedDirectory){
+			// reload or refresh thumbs and select image
+			buttonBrowsePathPressed ();
+			// the actual selection of image will be handled asynchronously at the end of FileCatalog::previewsFinishedUI
+			imageToSelect_fname = fname;
+		}
+		else{
+			// FileCatalog::filterChanged ();//this will be replaced by queue_draw() in fileBrowser->selectImage
+			fileBrowser->selectImage(fname);
+			imageToSelect_fname = "";
+		}
+	}
+}
+
+
+void FileCatalog::openNextPreviousEditorImage (Glib::ustring fname, bool clearFilters, eRTNav nextPrevious) {
+
+	Glib::ustring dirname = Glib::path_get_dirname(fname);
+	if (!dirname.empty()){
+		BrowsePath->set_text(dirname);
+
+
+		if (clearFilters){ // clear all filters
+			Query->set_text("");
+			categoryButtonToggled(bFilterClear,false);
+			// disable exif filters
+			if (filterPanel->isEnabled()) filterPanel->setEnabled (false);
+		}
+
+		if (BrowsePath->get_text()!=selectedDirectory){
+			// reload or refresh thumbs and select image
+			buttonBrowsePathPressed ();
+			// the actual selection of image will be handled asynchronously at the end of FileCatalog::previewsFinishedUI
+			refImageForOpen_fname = fname;
+			actionNextPrevious = nextPrevious;
+		}
+		else{
+			// FileCatalog::filterChanged ();//this was replace by queue_draw() in fileBrowser->selectImage
+			fileBrowser->openNextPreviousEditorImage(fname,nextPrevious);
+			refImageForOpen_fname = "";
+			actionNextPrevious = NAV_NONE;
+		}
+	}
+}
+
 bool FileCatalog::handleShortcutKey (GdkEventKey* event) {
 
     bool ctrl = event->state & GDK_CONTROL_MASK;
@@ -1628,65 +1762,78 @@ bool FileCatalog::handleShortcutKey (GdkEventKey* event) {
 			return true;
     }
 
-    if (!alt) {
+    if (shift){
+        switch(event->keyval) {
+            case GDK_Escape:
+                BrowsePath->set_text(selectedDirectory);
+                // set focus on something neutral, this is useful to remove focus from BrowsePath and Query
+                // when need to execute a shortcut, which otherwise will be typed into those fields
+                filepanel->grab_focus();
+                return true;
+        }
+    }
+
+    if (!alt && !shift) { // shift is reserved for ranking
         switch(event->keyval) {
 			case GDK_grave:
-				categoryButtonToggled(bUnRanked);
+				categoryButtonToggled(bUnRanked,false);
 				return true;
             case GDK_1:
-                categoryButtonToggled(bRank[0]);
+                categoryButtonToggled(bRank[0],false);
                 return true;
             case GDK_2:
-                categoryButtonToggled(bRank[1]);
+                categoryButtonToggled(bRank[1],false);
                 return true;
             case GDK_3:
-                categoryButtonToggled(bRank[2]);
+                categoryButtonToggled(bRank[2],false);
                 return true;
             case GDK_4:
-                categoryButtonToggled(bRank[3]);
+                categoryButtonToggled(bRank[3],false);
                 return true;
             case GDK_5:
-                categoryButtonToggled(bRank[4]);
+                categoryButtonToggled(bRank[4],false);
                 return true;
             case GDK_6:
-                categoryButtonToggled(bEdited[0]);
+                categoryButtonToggled(bEdited[0],false);
                 return true;
             case GDK_7:
-                categoryButtonToggled(bEdited[1]);
+                categoryButtonToggled(bEdited[1],false);
                 return true;
 
             case GDK_Return:
             case GDK_KP_Enter:
-                FileCatalog::buttonBrowsePathPressed ();
-                return true;
+                if (BrowsePath->is_focus()){
+                    FileCatalog::buttonBrowsePathPressed ();
+                    return true;
+                }
         }
     }
     
-    if (alt) {
+    if (alt && !shift) { // shift is reserved for color labeling
         switch(event->keyval) {
-        	case GDK_grave:
-				categoryButtonToggled(bUnCLabeled);
-				return true;
+            case GDK_grave:
+                 categoryButtonToggled(bUnCLabeled,false);
+                 return true;
             case GDK_1:
-                categoryButtonToggled(bCLabel[0]);
+                categoryButtonToggled(bCLabel[0],false);
                 return true;
             case GDK_2:
-                categoryButtonToggled(bCLabel[1]);
+                categoryButtonToggled(bCLabel[1],false);
                 return true;
             case GDK_3:
-                categoryButtonToggled(bCLabel[2]);
+                categoryButtonToggled(bCLabel[2],false);
                 return true;
             case GDK_4:
-                categoryButtonToggled(bCLabel[3]);
+                categoryButtonToggled(bCLabel[3],false);
                 return true;
             case GDK_5:
-                categoryButtonToggled(bCLabel[4]);
+                categoryButtonToggled(bCLabel[4],false);
                 return true;
             case GDK_6:
-                categoryButtonToggled(bRecentlySaved[0]);
+                categoryButtonToggled(bRecentlySaved[0],false);
                 return true;
             case GDK_7:
-                categoryButtonToggled(bRecentlySaved[1]);
+                categoryButtonToggled(bRecentlySaved[1],false);
                 return true;
         }
     }
@@ -1695,16 +1842,16 @@ bool FileCatalog::handleShortcutKey (GdkEventKey* event) {
         switch(event->keyval) {
 			case GDK_d:
 			case GDK_D:
-				categoryButtonToggled(bFilterClear);
+				categoryButtonToggled(bFilterClear,false);
 				return true;
 			case GDK_t:
 			case GDK_T:
-				categoryButtonToggled(bTrash);
+				categoryButtonToggled(bTrash,false);
 				return true;
         }
     }
 
-    if (!ctrl) {
+    if (!ctrl || (alt && !options.tabbedUI)) {
         switch(event->keyval) {
 
 			case GDK_bracketright:
@@ -1728,22 +1875,21 @@ bool FileCatalog::handleShortcutKey (GdkEventKey* event) {
                 return true;
         }
     }
-    else { // with Ctrl
+    if (ctrl && !alt) { 
         switch (event->keyval) {
-			case GDK_o:
-				if (!alt){
-					BrowsePath->select_region(0, BrowsePath->get_text_length());
-					BrowsePath->grab_focus();
-					return true;
-				}
-			case GDK_f:
-				if (!alt){
-					Query->select_region(0, Query->get_text_length());
-					Query->grab_focus();
-					return true;
-				}
+            case GDK_o:
+                BrowsePath->select_region(0, BrowsePath->get_text_length());
+                BrowsePath->grab_focus();
+                return true;
+            case GDK_f:
+                Query->select_region(0, Query->get_text_length());
+                Query->grab_focus();
+                return true;
         }
     }
 
+    if (fileBrowser->keyPressed(event))
+        return true;
+
     return false;
 }
