@@ -74,6 +74,9 @@ class FileCatalog : public Gtk::VBox,
         int selectedDirectoryId;
         bool enabled;
         bool inTabMode;  // Tab mode has e.g. different progress bar handling
+        Glib::ustring imageToSelect_fname;
+        Glib::ustring refImageForOpen_fname; // Next/previous for Editor's perspective
+        eRTNav actionNextPrevious;
 
         FileSelectionListener* listener;
         FileSelectionChangeListener* fslistener;
@@ -210,7 +213,7 @@ class FileCatalog : public Gtk::VBox,
 				void setFilterPanel (FilterPanel* fpanel);
 				void setExportPanel (ExportPanel* expanel);
 				void exifInfoButtonToggled();
-                void categoryButtonToggled (Gtk::ToggleButton* b);
+                void categoryButtonToggled (Gtk::ToggleButton* b, bool isMouseClick);
                 bool capture_event(GdkEventButton* event);
                 void filterChanged ();
                 void runFilterDialog ();
@@ -223,8 +226,10 @@ class FileCatalog : public Gtk::VBox,
                 void zoomOut ();
 
                 void buttonBrowsePathPressed ();
+                bool BrowsePath_key_pressed (GdkEventKey *event);
                 void buttonQueryClearPressed ();
                 void executeQuery ();
+                bool Query_key_pressed(GdkEventKey *event);
                 void updateFBQueryTB (bool singleRow);
 
                 void tbLeftPanel_1_toggled ();
@@ -233,7 +238,9 @@ class FileCatalog : public Gtk::VBox,
                 void tbRightPanel_1_visible (bool visible);
 
                 void openNextImage () { fileBrowser->openNextImage(); }
-                void openPrevImage () { fileBrowser->openPrevImage(); }               
+                void openPrevImage () { fileBrowser->openPrevImage(); }
+                void selectImage (Glib::ustring fname, bool clearFilters);
+                void openNextPreviousEditorImage (Glib::ustring fname, bool clearFilters, eRTNav nextPrevious);
 
                 bool handleShortcutKey (GdkEventKey* event);
 
