@@ -49,6 +49,7 @@ FilePanel::FilePanel () : parent(NULL) {
     dirpaned->pack1 (*placespaned, false, true);
 
     tpc = new BatchToolPanelCoordinator (this);
+    tpc->removeWbTool();
     fileCatalog = Gtk::manage ( new FileCatalog (tpc->coarse, tpc->getToolBar(),this) );
     ribbonPane = Gtk::manage ( new Gtk::Paned() );
     ribbonPane->add(*fileCatalog);
@@ -247,7 +248,7 @@ bool FilePanel::handleShortcutKey (GdkEventKey* event) {
         }
     }
     
-    if(tpc->getToolBar()->handleShortcutKey(event))
+    if(tpc->getToolBar() && tpc->getToolBar()->handleShortcutKey(event))
         return true;
     
     if(tpc->handleShortcutKey(event))
