@@ -20,12 +20,45 @@
 #include "guiutils.h"
 #include "options.h"
 #include "../rtengine/utils.h"
+#include "../rtengine/safegtk.h"
 #include "rtimage.h"
+#include "multilangmgr.h"
 
 #include <assert.h>
 
 using namespace std;
 
+Glib::ustring escapeHtmlChars(const Glib::ustring &src) {
+
+    // Sources chars to be escaped
+    static const Glib::ustring srcChar("&<>");
+
+    // Destination strings, in the same order than the source
+    static std::vector<Glib::ustring> dstChar(3);
+    dstChar.at(0) = "&amp;";
+    dstChar.at(1) = "&lt;";
+    dstChar.at(2) = "&gt;";
+
+    // Copying the original string, that will be modified
+    Glib::ustring dst(src);
+
+    // Iterating all chars of the copy of the source string
+    for (size_t i=0; i<dst.length();) {
+
+        // Looking out if it's part of the characters to be escaped
+        size_t pos = srcChar.find_first_of(dst.at(i), 0);
+
+        if (pos != Glib::ustring::npos) {
+            // If yes, replacing the char in the destination string
+            dst.replace(i, 1, dstChar.at(pos));
+            // ... and going forward  by the length of the new string
+            i += dstChar.at(pos).length();
+        }
+        else ++i;
+    }
+    return dst;
+}
+
 bool removeIfThere (Gtk::Container* cont, Gtk::Widget* w, bool increference) {
 
     Glib::ListHandle<Gtk::Widget*> list = cont->get_children ();
@@ -53,7 +86,8 @@ Glib::ustring removeExtension (const Glib::ustring& filename) {
 
     Glib::ustring bname = Glib::path_get_basename(filename);
     size_t lastdot = bname.find_last_of ('.');
-    if (lastdot!=bname.npos)
+    size_t lastwhitespace = bname.find_last_of (" \t\f\v\n\r");
+    if (lastdot!=bname.npos && (lastwhitespace==bname.npos || lastdot > lastwhitespace))
         return filename.substr (0, filename.size()-(bname.size()-lastdot));
     else
         return filename;
@@ -63,12 +97,30 @@ Glib::ustring getExtension (const Glib::ustring& filename) {
 
     Glib::ustring bname = Glib::path_get_basename(filename);
     size_t lastdot = bname.find_last_of ('.');
-    if (lastdot!=bname.npos)
+    size_t lastwhitespace = bname.find_last_of (" \t\f\v\n\r");
+    if (lastdot!=bname.npos && (lastwhitespace==bname.npos || lastdot > lastwhitespace))
         return filename.substr (filename.size()-(bname.size()-lastdot)+1, filename.npos);
     else
         return "";
 }
 
+bool confirmOverwrite (Gtk::Window& parent, const std::string& filename) {
+    bool safe = true;
+    if (safe_file_test (filename, Glib::FILE_TEST_EXISTS)) {
+        Glib::ustring msg_ = Glib::ustring ("<b>\"") + Glib::path_get_basename (filename) + "\": "
+                             + M("MAIN_MSG_ALREADYEXISTS") + "</b>\n" + M("MAIN_MSG_QOVERWRITE");
+        Gtk::MessageDialog msgd (parent, msg_, true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO, true);
+        safe = (msgd.run () == Gtk::RESPONSE_YES);
+    }
+    return safe;
+}
+
+void writeFailed (Gtk::Window& parent, const std::string& filename) {
+    Glib::ustring msg_ = Glib::ustring::compose(M("MAIN_MSG_WRITEFAILED"), filename);
+    Gtk::MessageDialog msgd (parent, msg_, true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+    msgd.run ();
+}
+
 void drawCrop (Cairo::RefPtr<Cairo::Context> cr, int imx, int imy, int imw, int imh, int startx, int starty, double scale, const rtengine::procparams::CropParams& cparams) {
 
     cr->set_line_width (0.);
@@ -80,7 +132,14 @@ void drawCrop (Cairo::RefPtr<Cairo::Context> cr, int imx, int imy, int imw, int
     double c2x = (cparams.x+cparams.w-1-startx)*scale;
     double c2y = (cparams.y+cparams.h-1-starty)*scale;
 
+    // crop overlay color, linked with crop windows background
+    if (options.bgcolor==0)
     cr->set_source_rgba (options.cutOverlayBrush[0], options.cutOverlayBrush[1], options.cutOverlayBrush[2], options.cutOverlayBrush[3]);
+    else if (options.bgcolor==1)
+        cr->set_source_rgb (0,0,0);
+    else if (options.bgcolor==2)
+        cr->set_source_rgb (1,1,1);
+    
 
     cr->rectangle (imx, imy, imw, c1y);
     cr->rectangle (imx, imy+c2y, imw, imh-c2y);
@@ -475,3 +534,89 @@ void TextOrIcon::switchTo(TOITypes type) {
 	}
 	show_all();
 }
+
+BackBuffer::BackBuffer() {
+	x = y = w = h = 0;
+	dirty = true;
+}
+
+bool BackBuffer::setDrawRectangle(Glib::RefPtr<Gdk::Window> window, int newX, int newY, int newW, int newH) {
+	bool newSize = w!=newW || h!=newH;
+
+	x = newX;
+	y = newY;
+	w = newW;
+	h = newH;
+
+	// WARNING: we're assuming that the surface type won't change during all the execution time of RT. I guess it may be wrong when the user change the gfx card display settings!?
+	if (newSize && window) {
+		// allocate a new Surface
+		if (newW>0 && newH>0) {
+			surface = window->create_similar_surface(Cairo::CONTENT_COLOR, w, h);
+		}
+		else {
+			// at least one dimension is null, so we delete the Surface
+			surface.clear();
+			// and we reset all dimensions
+			x = y = w = h = 0;
+		}
+		dirty = true;
+	}
+	return dirty;
+}
+
+/*
+ * Copy the backbuffer to a Gdk::Window
+ */
+void BackBuffer::copySurface(Glib::RefPtr<Gdk::Window> window, GdkRectangle *rectangle) {
+	if (surface && window) {
+		// TODO: look out if window can be different on each call, and if not, store a reference to the window
+		Cairo::RefPtr<Cairo::Context> crSrc = window->create_cairo_context();
+		Cairo::RefPtr<Cairo::Surface> destSurface = crSrc->get_target();
+
+		// now copy the off-screen Surface to the destination Surface
+		Cairo::RefPtr<Cairo::Context> crDest = Cairo::Context::create(destSurface);
+		crDest->set_source(surface, x, y);
+		crDest->set_line_width(0.);
+		if (rectangle)
+			crDest->rectangle(rectangle->x, rectangle->y, rectangle->width, rectangle->height);
+		else
+			crDest->rectangle(x, y, w, h);
+		crDest->fill();
+	}
+}
+
+/*
+ * Copy the BackBuffer to another BackBuffer
+ */
+void BackBuffer::copySurface(BackBuffer *destBackBuffer, GdkRectangle *rectangle) {
+	if (surface && destBackBuffer) {
+		// now copy the off-screen Surface to the destination Surface
+		Cairo::RefPtr<Cairo::Context> crDest = Cairo::Context::create(destBackBuffer->getSurface());
+		crDest->set_source(surface, x, y);
+		crDest->set_line_width(0.);
+		if (rectangle)
+			crDest->rectangle(rectangle->x, rectangle->y, rectangle->width, rectangle->height);
+		else
+			crDest->rectangle(x, y, w, h);
+		crDest->fill();
+	}
+}
+
+/*
+ * Copy the BackBuffer to another Cairo::Surface
+ */
+void BackBuffer::copySurface(Cairo::RefPtr<Cairo::Surface> destSurface, GdkRectangle *rectangle) {
+	if (surface && destSurface) {
+		// now copy the off-screen Surface to the destination Surface
+		Cairo::RefPtr<Cairo::Context> crDest = Cairo::Context::create(destSurface);
+		crDest->set_source(surface, x, y);
+		crDest->set_line_width(0.);
+		if (rectangle)
+			crDest->rectangle(rectangle->x, rectangle->y, rectangle->width, rectangle->height);
+		else
+			crDest->rectangle(x, y, w, h);
+		crDest->fill();
+	}
+}
+
