@@ -23,6 +23,7 @@
 #include <cstring>
 #include "../rtengine/LUT.h"
 #include "rtimage.h"
+#include "../rtengine/improccoordinator.h"
 
 extern Glib::ustring argv0;
 extern Options options;
@@ -33,7 +34,7 @@ extern Options options;
 // HistogramPanel
 HistogramPanel::HistogramPanel () {
 
-   histogramArea = Gtk::manage (new HistogramArea ());
+   histogramArea = Gtk::manage (new HistogramArea (this));
    histogramRGBArea = Gtk::manage (new HistogramRGBArea ());
    histogramRGBArea->show();
 
@@ -47,42 +48,54 @@ HistogramPanel::HistogramPanel () {
    greenImage = new RTImage ("histGreen.png");
    blueImage  = new RTImage ("histBlue.png");
    valueImage = new RTImage ("histValue.png");
+   chroImage  = new RTImage ("histChro.png");
    rawImage   = new RTImage ("histRaw.png");
+   fullImage  = new RTImage ("histFull.png");
    barImage   = new RTImage ("histBar.png");
 
    redImage_g   = new RTImage ("histRedg.png");
    greenImage_g = new RTImage ("histGreeng.png");
    blueImage_g  = new RTImage ("histBlueg.png");
    valueImage_g = new RTImage ("histValueg.png");
+   chroImage_g  = new RTImage ("histChrog.png");
    rawImage_g   = new RTImage ("histRawg.png");
+   fullImage_g  = new RTImage ("histFullg.png");
    barImage_g   = new RTImage ("histBarg.png");
 
    showRed   = Gtk::manage (new Gtk::ToggleButton ());
    showGreen = Gtk::manage (new Gtk::ToggleButton ());
    showBlue  = Gtk::manage (new Gtk::ToggleButton ());
    showValue = Gtk::manage (new Gtk::ToggleButton ());
+   showChro  = Gtk::manage (new Gtk::ToggleButton ());
    showRAW   = Gtk::manage (new Gtk::ToggleButton ());
+   showFull  = Gtk::manage (new Gtk::ToggleButton ());
    showBAR   = Gtk::manage (new Gtk::ToggleButton ());
 
    showRed->set_name("histButton");   showRed->set_can_focus(false);
    showGreen->set_name("histButton"); showGreen->set_can_focus(false);
    showBlue->set_name("histButton");  showBlue->set_can_focus(false);
    showValue->set_name("histButton"); showValue->set_can_focus(false);
+   showChro->set_name("histButton");  showChro->set_can_focus(false);
    showRAW->set_name("histButton");   showRAW->set_can_focus(false);
+   showFull->set_name("fullButton");  showFull->set_can_focus(false);
    showBAR->set_name("histButton");   showBAR->set_can_focus(false);
 
    showRed->set_relief (Gtk::RELIEF_NONE);
    showGreen->set_relief (Gtk::RELIEF_NONE);
    showBlue->set_relief (Gtk::RELIEF_NONE);
    showValue->set_relief (Gtk::RELIEF_NONE);
+   showChro->set_relief (Gtk::RELIEF_NONE);
    showRAW->set_relief (Gtk::RELIEF_NONE);
+   showFull->set_relief (Gtk::RELIEF_NONE);
    showBAR->set_relief (Gtk::RELIEF_NONE);
 
    showRed->set_tooltip_text   (M("HISTOGRAM_TOOLTIP_R"));
    showGreen->set_tooltip_text (M("HISTOGRAM_TOOLTIP_G"));
    showBlue->set_tooltip_text  (M("HISTOGRAM_TOOLTIP_B"));
    showValue->set_tooltip_text (M("HISTOGRAM_TOOLTIP_L"));
+   showChro->set_tooltip_text  (M("HISTOGRAM_TOOLTIP_CHRO"));
    showRAW->set_tooltip_text   (M("HISTOGRAM_TOOLTIP_RAW"));
+   showFull->set_tooltip_text  (M("HISTOGRAM_TOOLTIP_FULL"));
    showBAR->set_tooltip_text   (M("HISTOGRAM_TOOLTIP_BAR"));
 
    buttonVBox = Gtk::manage (new Gtk::VBox (false, 2));
@@ -90,31 +103,37 @@ HistogramPanel::HistogramPanel () {
    showGreen->set_active (true);
    showBlue->set_active (true);
    showValue->set_active (true);
+   showChro->set_active (false);//unactive by default
+
    showRAW->set_active (false);
+   showFull->set_active (!options.histogramFullMode);
    showBAR->set_active (options.histogramBar);
 
-   showRed->set_image(*redImage);
-   showGreen->set_image(*greenImage);
-   showBlue->set_image(*blueImage);
-   showValue->set_image(*valueImage);
-   showRAW->set_image(*rawImage_g);
-   if (showBAR->get_active())
-     showBAR->set_image(*barImage);
-   else
-     showBAR->set_image(*barImage_g);
+   showRed->set_image   (showRed->get_active()   ? *redImage   : *redImage_g);
+   showGreen->set_image (showGreen->get_active() ? *greenImage : *greenImage_g);
+   showBlue->set_image  (showBlue->get_active()  ? *blueImage  : *blueImage_g);
+   showValue->set_image (showValue->get_active() ? *valueImage : *valueImage_g);
+   showChro->set_image  (showChro->get_active()   ? *chroImage : *chroImage_g);
+   showRAW->set_image   (showRAW->get_active()   ? *rawImage   : *rawImage_g);
+   showFull->set_image  (showFull->get_active()  ? *fullImage  : *fullImage_g);
+   showBAR->set_image   (showBAR->get_active()   ? *barImage   : *barImage_g);
 
    showRed->signal_toggled().connect( sigc::mem_fun(*this, &HistogramPanel::red_toggled), showRed );
    showGreen->signal_toggled().connect( sigc::mem_fun(*this, &HistogramPanel::green_toggled), showGreen );
    showBlue->signal_toggled().connect( sigc::mem_fun(*this, &HistogramPanel::blue_toggled), showBlue );
    showValue->signal_toggled().connect( sigc::mem_fun(*this, &HistogramPanel::value_toggled), showValue );
+   showChro->signal_toggled().connect( sigc::mem_fun(*this, &HistogramPanel::chro_toggled), showChro );
    showRAW->signal_toggled().connect( sigc::mem_fun(*this, &HistogramPanel::raw_toggled), showRAW );
+   showFull->signal_toggled().connect( sigc::mem_fun(*this, &HistogramPanel::full_toggled), showFull );
    showBAR->signal_toggled().connect( sigc::mem_fun(*this, &HistogramPanel::bar_toggled), showBAR );
 
    buttonVBox->pack_start (*showRed, Gtk::PACK_SHRINK, 0);
    buttonVBox->pack_start (*showGreen, Gtk::PACK_SHRINK, 0);
    buttonVBox->pack_start (*showBlue, Gtk::PACK_SHRINK, 0);
    buttonVBox->pack_start (*showValue, Gtk::PACK_SHRINK, 0);
-   buttonVBox->pack_start (*showRAW, Gtk::PACK_SHRINK, 0);
+   buttonVBox->pack_start (*showChro, Gtk::PACK_SHRINK, 0);
+	buttonVBox->pack_start (*showRAW, Gtk::PACK_SHRINK, 0);
+   buttonVBox->pack_start (*showFull, Gtk::PACK_SHRINK, 0);
    buttonVBox->pack_start (*showBAR, Gtk::PACK_SHRINK, 0);
 
    // Put the button vbox next to the window's border to be less disturbing
@@ -137,15 +156,20 @@ HistogramPanel::~HistogramPanel () {
   delete greenImage;
   delete blueImage;
   delete valueImage;
+  delete chroImage;
   delete rawImage;
+  delete fullImage;
   delete barImage;
 
   delete redImage_g;
   delete greenImage_g;
   delete blueImage_g;
   delete valueImage_g;
+  delete chroImage_g;
   delete rawImage_g;
+  delete fullImage_g;
   delete barImage_g;
+  
 }
 
 void HistogramPanel::resized (Gtk::Allocation& req) {
@@ -197,28 +221,41 @@ void HistogramPanel::value_toggled () {
     showValue->set_image(showValue->get_active() ? *valueImage : *valueImage_g);
     rgbv_toggled();
 }
+void HistogramPanel::chro_toggled () {
+	removeIfThere(showChro, chroImage, false);
+	removeIfThere(showChro, chroImage_g, false);
+    showChro->set_image(showChro->get_active() ? *chroImage : *chroImage_g);
+    rgbv_toggled();
+}
+
 void HistogramPanel::raw_toggled () {
 	if (showRAW->get_active()) {
       showRAW->set_image(*rawImage);
       showValue->set_sensitive(false);
+      showChro->set_sensitive(false);
 	}
 	else {
       showRAW->set_image(*rawImage_g);
       showValue->set_sensitive(true);
+      showChro->set_sensitive(true);
 	}
     rgbv_toggled();
 }
+void HistogramPanel::full_toggled () {
+    options.histogramFullMode = !showFull->get_active();
+    showFull->set_image(showFull->get_active() ? *fullImage : *fullImage_g);
+    rgbv_toggled();
+}
 void HistogramPanel::bar_toggled () {
     showBAR->set_image(showBAR->get_active() ? *barImage : *barImage_g);
     rgbv_toggled();
 }
-
 void HistogramPanel::rgbv_toggled () {
   // Update Display
-  histogramArea->updateOptions (showRed->get_active(), showGreen->get_active(), showBlue->get_active(), showValue->get_active(), showRAW->get_active());
+  histogramArea->updateOptions (showRed->get_active(), showGreen->get_active(), showBlue->get_active(), showValue->get_active(), showRAW->get_active(), showFull->get_active(), showChro->get_active());
   histogramArea->queue_draw ();
 
-  histogramRGBArea->updateOptions (showRed->get_active(), showGreen->get_active(), showBlue->get_active(), showValue->get_active(), showRAW->get_active(), showBAR->get_active());
+  histogramRGBArea->updateOptions (showRed->get_active(), showGreen->get_active(), showBlue->get_active(), showValue->get_active(), showRAW->get_active(), showBAR->get_active(), showChro->get_active());
   histogramRGBArea->renderRGBMarks (0,0,0);
   histogramArea->queue_draw ();
 }
@@ -265,12 +302,18 @@ void HistogramPanel::reorder (Gtk::AlignmentEnum align) {
 		reorder_child(*buttonVBox, 1);
 }
 
+// FullModeListener interface:
+void HistogramPanel::toggle_button_full () {
+    showFull->set_active (!showFull->get_active ());
+    showFull->set_image(showFull->get_active() ? *fullImage : *fullImage_g);
+}
+
 //
 //
 //
 // HistogramRGBArea
-HistogramRGBArea::HistogramRGBArea () :
-  frozen(false), valid(false), needRed(true), needGreen(true), needBlue(true), needLuma(true), rawMode(false), showMode(options.histogramBar), barDisplayed(options.histogramBar) {
+HistogramRGBArea::HistogramRGBArea () ://needChroma unactive by default
+  frozen(false), valid(false), needRed(true), needGreen(true), needBlue(true), needLuma(true), rawMode(false), showMode(options.histogramBar), barDisplayed(options.histogramBar), needChroma(false){
 
   harih = new HistogramRGBAreaIdleHelper;
   harih->harea = this;
@@ -364,6 +407,14 @@ void HistogramRGBArea::renderRGBMarks (int r, int g, int b) {
       cr->line_to((int)((r+g+b)/3*(winw/256.0)), winh-0);
       cr->stroke();
     }
+    if (needChroma) {
+      // Luma
+      cr->set_source_rgb(1.0, 1.0, 1.0);
+      cr->move_to((int)((r+g+b)/3*(winw/256.0)), 0);
+      cr->line_to((int)((r+g+b)/3*(winw/256.0)), winh-0);
+      cr->stroke();
+    }
+	
   }
 }
 
@@ -407,7 +458,7 @@ void HistogramRGBArea::update (int valh, int rh, int  gh, int bh) {
     g_idle_add (histrgbupdate, harih);
 }
 
-void HistogramRGBArea::updateOptions (bool r, bool g, bool b, bool l, bool raw, bool bar) {
+void HistogramRGBArea::updateOptions (bool r, bool g, bool b, bool l, bool raw, bool bar, bool c) {
 
     needRed   = r;
     needGreen = g;
@@ -415,6 +466,7 @@ void HistogramRGBArea::updateOptions (bool r, bool g, bool b, bool l, bool raw,
     needLuma  = l;
     rawMode   = raw;
     showMode  = bar;
+	needChroma = c;
 
     // Histogram RGB BAR button logic goes here
 
@@ -495,13 +547,14 @@ void HistogramRGBArea::styleChanged (const Glib::RefPtr<Gtk::Style>& style) {
 //
 //
 // HistogramArea
-HistogramArea::HistogramArea () : 
-    valid(false), showFull(true), oldwidth(-1), needLuma(true), needRed(true), needGreen(true), needBlue(true), rawMode(false) {
+HistogramArea::HistogramArea (FullModeListener *fml) : //needChroma unactive by default
+    valid(false), fullMode(options.histogramFullMode), myFullModeListener(fml), oldwidth(-1), needLuma(true), needRed(true), needGreen(true), needBlue(true), rawMode(false), needChroma(false) {
 
     lhist(256);
     rhist(256);
     ghist(256);
     bhist(256);
+    chist(256);
 
     haih = new HistogramAreaIdleHelper;
     haih->harea = this;
@@ -520,13 +573,15 @@ HistogramArea::~HistogramArea () {
 
 }
 
-void HistogramArea::updateOptions (bool r, bool g, bool b, bool l, bool raw) {
+void HistogramArea::updateOptions (bool r, bool g, bool b, bool l, bool raw, bool full, bool c) {
 
     needRed   = r;
     needGreen = g;
     needBlue  = b;
     needLuma  = l;
     rawMode   = raw;
+    fullMode  = !full;
+	needChroma = c;
 
     renderHistogram ();
 }
@@ -552,10 +607,10 @@ int histupdateUI (void* data) {
     return 0;
 }
 
-void HistogramArea::update (LUTu &histRed, LUTu &histGreen, LUTu &histBlue, LUTu &histLuma, LUTu &histRedRaw, LUTu &histGreenRaw, LUTu &histBlueRaw) {
+void HistogramArea::update (LUTu &histRed, LUTu &histGreen, LUTu &histBlue, LUTu &histLuma, LUTu &histRedRaw, LUTu &histGreenRaw, LUTu &histBlueRaw, LUTu &histChroma) {
 	
     if (histRed) {
-        lhist=histLuma;
+        lhist=histLuma;chist=histChroma;
         rhist=histRed; ghist=histGreen; bhist=histBlue;
         rhistRaw=histRedRaw; ghistRaw=histGreenRaw; bhistRaw=histBlueRaw;
 
@@ -598,6 +653,8 @@ void HistogramArea::renderHistogram () {
     for (int i=1; i<255; i++) {
         if (needLuma && lhist[i]>fullhistheight)
 	        fullhistheight = lhist[i];
+        if (needChroma && chist[i]>fullhistheight)
+	        fullhistheight = chist[i];
         if (needRed && (rawMode?rhistRaw:rhist)[i]>fullhistheight)
 	        fullhistheight = rh[i];
         if (needGreen && (rawMode?ghistRaw:ghist)[i]>fullhistheight)
@@ -608,23 +665,17 @@ void HistogramArea::renderHistogram () {
 
     int realhistheight = fullhistheight;
 
-    if (!showFull) {    
-        int area1thres = 0;
-        int area2thres = 0;
+    if (!fullMode) {
         int area = 0;
         for (int i=0; i<fullhistheight; i++) {
             for (int j=0; j<256; j++)
-                if ((needLuma && !rawMode && lhist[j]>i) || (needRed && rh[j]>i) || (needGreen && gh[j]>i) || (needBlue && bh[j]>i))
+                if ((needLuma && !rawMode && lhist[j]>i) || (needChroma && !rawMode && chist[j]>i) || (needRed && rh[j]>i) || (needGreen && gh[j]>i) || (needBlue && bh[j]>i))
                     area++;
-            if (area1thres==0 && (double)area / (256*(i+1)) < 0.3)
-                area1thres = i;
-            if (area2thres==0 && (double)area / (256*(i+1)) < 0.3)
-                area2thres = i;
-            if (area1thres && area2thres)
+            if ((double)area / (256*(i+1)) < 0.3) {
+                realhistheight = i;
                 break;
+            }
         }
-        if (area1thres>0 && area2thres>0 && area1thres<fullhistheight)
-            realhistheight = area2thres;
     }
     
     if (realhistheight<winh-2)
@@ -639,12 +690,23 @@ void HistogramArea::renderHistogram () {
     if (needLuma && !rawMode) {
         drawCurve(cr, lhist, realhistheight, winw, winh);
         cr->set_source_rgb (0.75, 0.75, 0.75);
-        cr->fill_preserve ();
+		cr->fill_preserve ();
         cr->set_source_rgb (0.5, 0.5, 0.5);
-    	cr->stroke ();
+   	cr->stroke ();
 
         drawMarks(cr, lhist, realhistheight, winw, ui, oi);
     }
+    if (needChroma && !rawMode) {
+        drawCurve(cr, chist, realhistheight, winw, winh);
+        cr->set_source_rgb (0.6, 0.6, 0.6);
+	//	cr->fill_preserve ();
+     //   cr->set_source_rgb (0.2, 0.2, 0.1);
+   	cr->stroke ();
+
+        drawMarks(cr, lhist, realhistheight, winw, ui, oi);
+    }
+	
+	
     if (needRed) {
         drawCurve(cr, rh, realhistheight, winw, winh);
         cr->set_source_rgb (1.0, 0.0, 0.0);
@@ -762,7 +824,10 @@ bool HistogramArea::on_expose_event(GdkEventExpose* event) {
 bool HistogramArea::on_button_press_event (GdkEventButton* event) {
 
     if (event->type==GDK_2BUTTON_PRESS && event->button==1) {
-        showFull = !showFull;
+        fullMode = !fullMode;
+        options.histogramFullMode = fullMode;
+        if (myFullModeListener)
+            myFullModeListener->toggle_button_full ();
         renderHistogram ();
         queue_draw ();
     }
