@@ -30,6 +30,7 @@
 #include "rtwindow.h"
 #include <cstring>
 #include <cstdlib>
+#include <locale.h>
 #include "options.h"
 #include "soundman.h"
 #include "rtimage.h"
@@ -65,6 +66,7 @@ int processLineParams( int argc, char **argv );
 
 int main(int argc, char **argv)
 {
+    setlocale(LC_ALL,"");
 #ifdef BUILD_BUNDLE
     char exname[512] = {0};
     Glib::ustring exePath;
@@ -298,36 +300,44 @@ int processLineParams( int argc, char **argv )
 			default:
 			{
 				Glib::ustring pparamsExt = paramFileExtension.substr(1);
-				std::cerr << "RawTherapee, V" << VERSION << std::endl;
-				std::cerr << "Copyright (c)2004-2012 Gabor Horvath <hgabor@rawtherapee.com>"<< std::endl << std::endl;
-				std::cerr << "Usage:"<< std::endl;
-				std::cerr << Glib::path_get_basename(argv[0]) << " [<selected dir>] : start RT GUI browser inside dir."<< std::endl;
-				std::cerr << Glib::path_get_basename(argv[0]) << " <file> : start GUI editor with file."<< std::endl;
-				std::cerr << Glib::path_get_basename(argv[0]) << " -c <inputDir>|<file list> : convert files in batch with default parameters."<< std::endl<< std::endl;
-				std::cerr << "Other options used with -c (that must be last option) "<< std::endl;
-				std::cerr << Glib::path_get_basename(argv[0]) <<" [-o <output> | -O <output>] [-s|-S] [-p <file>] [-d] [-j[1-100]|-t|-n] -Y -c <input>"<< std::endl;
-				std::cerr << " -o <outputFile>|<outputDir> : select output directory."<< std::endl;
-				std::cerr << " -O <outputFile>|<outputDir> : select output dir and copy " << pparamsExt << " file into it"<< std::endl;
-				std::cerr << " -s : include the " << pparamsExt << " file next to the input file (with same name) to build the image's parameters"<< std::endl;
-				std::cerr << "      ex: for IMG001.NEF there should be IMG001.NEF." << pparamsExt << " in the same dir" << std::endl;
-				std::cerr << "      if absent use default" << std::endl;
-				std::cerr << " -S : like -s but skip if " << pparamsExt << " file not found." << std::endl;
-				std::cerr << " -p <file." << pparamsExt << "> : specify " << pparamsExt << " file to be used for all conversions."<< std::endl;
-				std::cerr << "                 you can specify as much -p option as you want (see the note below)."<< std::endl;
-				std::cerr << " -d : use the default Raw or Image " << pparamsExt << " file to build the image's parameters"<< std::endl;
-				std::cerr << " -j[compression] : specify output to be jpeg.(default)"<< std::endl;
-				std::cerr << " -t : specify output to be uncompressed tiff."<< std::endl;
-				std::cerr << " -t1: specify output to be compressed tiff."<< std::endl;
-				std::cerr << " -n : specify output to be png."<< std::endl;
-				std::cerr << " -Y : overwrite output if present."<< std::endl<<std::endl;
-				std::cerr << "NOTE: Your " << pparamsExt << " files can be incomplete, RawTherapee will set the values like that:"<< std::endl;
-				std::cerr << "   - the " << pparamsExt << " file is built with internal default values;"<< std::endl;
-				std::cerr << "   - then overrided by those found in the default Raw or Image " << pparamsExt << " file (if -d has been set);"<< std::endl;
-				std::cerr << "   - then overrided by those found in the " << pparamsExt << " files provided by -p,"<< std::endl;
-				std::cerr << "     each one overriding the previous values;" << std::endl;
-				std::cerr << "   * then overrided by the sidecar file if -s is set and if the file exists;"<< std::endl;
-				std::cerr << "     the time where the sidecar file is used depend of the position of the -s switch"<< std::endl;
-				std::cerr << "     in the command line regarding to the -p parameters (e.g. \"-p first." << pparamsExt << " -p second." << pparamsExt << " -s -p fourth." << pparamsExt << "\")"<< std::endl;
+        std::cerr << "RawTherapee, version " << VERSION << std::endl << std::endl;
+        std::cerr << "Usage:" << std::endl;
+        std::cerr << "  " << Glib::path_get_basename(argv[0]) << " [<selected dir>]   Start File Browser inside directory." << std::endl;
+        std::cerr << "  " << Glib::path_get_basename(argv[0]) << " <file>             Start Image Editor with file." << std::endl;
+        std::cerr << "  " << Glib::path_get_basename(argv[0]) << " -c <dir>|<files>   Convert files in batch with default parameters." << std::endl << std::endl;
+        std::cerr << "Other options used with -c (-c must be the last option):" << std::endl;
+        std::cerr << Glib::path_get_basename(argv[0]) << " [-o <output>|-O <output>] [-s|-S] [-p <files>] [-d] [-j[1-100]|-t|-t1|-n] -Y -c <input>" << std::endl;
+        std::cerr << "  -o <file>|<dir>  Select output file or directory." << std::endl;
+        std::cerr << "  -O <file>|<dir>  Select output file or directory and copy " << pparamsExt << " file into it." << std::endl;
+        std::cerr << "  -s               Include the " << pparamsExt << " file next to the input file (with the same" << std::endl;
+        std::cerr << "                   name) to build the image parameters," << std::endl;
+        std::cerr << "                   e.g. for photo.raw there should be a photo.raw." << pparamsExt << " file in" << std::endl;
+        std::cerr << "                   the same directory. If the file does not exist, internal" << std::endl;
+        std::cerr << "                   default (neutral) values (not those in Default." << pparamsExt << ") will be" << std::endl;
+        std::cerr << "                   used." << std::endl;
+        std::cerr << "  -S               Like -s but skip if the " << pparamsExt << " file does not exist." << std::endl;
+        std::cerr << "  -p <file.pp3>    Specify " << pparamsExt << " file to be used for all conversions." << std::endl;
+        std::cerr << "                   You can specify as many -p options as you like (see" << std::endl;
+        std::cerr << "                   description below)." << std::endl;
+        std::cerr << "  -d               Use the default raw or non-raw " << pparamsExt << " file as set in" << std::endl;
+        std::cerr << "                   Preferences > Image Processing > Default Image Processing Parameters" << std::endl;
+        std::cerr << "  -j[1-100]        Specify output to be JPEG (on by default). Optionally add" << std::endl;
+        std::cerr << "                   compression 1-100." << std::endl;
+        std::cerr << "  -t               Specify output to be uncompressed 16-bit TIFF." << std::endl;
+        std::cerr << "  -t1              Specify output to be compressed 16-bit TIFF (ZIP compression)." << std::endl;
+        std::cerr << "  -n               Specify output to be compressed 16-bit PNG." << std::endl;
+        std::cerr << "  -Y               Overwrite output if present." << std::endl<<std::endl;
+        std::cerr << "Your " << pparamsExt << " files can be incomplete, RawTherapee will set the values as follows:" << std::endl;
+        std::cerr << "  1- A new profile is created using internal default (neutral) values" <<std::endl;
+        std::cerr << "     (hard-coded into RawTherapee)," << std::endl;
+        std::cerr << "  2- then overridden by those found in the default raw or non-raw " << pparamsExt << " file" << std::endl;
+        std::cerr << "     (if -d has been set)," << std::endl;
+        std::cerr << "  3- then overridden by those found in the " << pparamsExt << " files provided by -p, each one" << std::endl;
+        std::cerr << "     overriding the previous values," << std::endl;
+        std::cerr << "  4- then overridden by the sidecar file if -s is set and if the file exists;" << std::endl;
+        std::cerr << "     the time where the sidecar file is used depends on the position of the -s" << std::endl;
+        std::cerr << "     switch in the command line relative to the -p parameters," << std::endl;
+        std::cerr << "     e.g. -p first." << pparamsExt << " -p second." << pparamsExt << " -s -p fourth." << pparamsExt << std::endl;
 				return -1;
 			}
 			}
