@@ -36,7 +36,9 @@
 #include <Shlobj.h>
 #endif
 
+// User's settings directory, including images' profiles if used
 Glib::ustring Options::rtdir;
+// User's cached datas' directory
 Glib::ustring Options::cacheBaseDir;
 
 Options options;
@@ -147,12 +149,18 @@ void Options::updatePaths() {
         lastRgbCurvesDir = preferredPath;
     if (lastLabCurvesDir.empty() || !safe_file_test (lastLabCurvesDir, Glib::FILE_TEST_EXISTS) || !safe_file_test (lastLabCurvesDir, Glib::FILE_TEST_IS_DIR))
         lastLabCurvesDir = preferredPath;
+    if (lastPFCurvesDir.empty() || !safe_file_test (lastPFCurvesDir, Glib::FILE_TEST_EXISTS) || !safe_file_test (lastPFCurvesDir, Glib::FILE_TEST_IS_DIR))
+        lastPFCurvesDir = preferredPath;
     if (lastHsvCurvesDir.empty() || !safe_file_test (lastHsvCurvesDir, Glib::FILE_TEST_EXISTS) || !safe_file_test (lastHsvCurvesDir, Glib::FILE_TEST_IS_DIR))
         lastHsvCurvesDir = preferredPath;
     if (lastToneCurvesDir.empty() || !safe_file_test (lastToneCurvesDir, Glib::FILE_TEST_EXISTS) || !safe_file_test (lastToneCurvesDir, Glib::FILE_TEST_IS_DIR))
         lastToneCurvesDir = preferredPath;
     if (lastProfilingReferenceDir.empty() || !safe_file_test (lastProfilingReferenceDir, Glib::FILE_TEST_EXISTS) || !safe_file_test (lastProfilingReferenceDir, Glib::FILE_TEST_IS_DIR))
         lastProfilingReferenceDir = preferredPath;
+    if (lastVibranceCurvesDir.empty() || !safe_file_test (lastVibranceCurvesDir, Glib::FILE_TEST_EXISTS) || !safe_file_test (lastVibranceCurvesDir, Glib::FILE_TEST_IS_DIR))
+        lastVibranceCurvesDir = preferredPath;
+    if (loadSaveProfilePath.empty() || !safe_file_test (loadSaveProfilePath, Glib::FILE_TEST_EXISTS) || !safe_file_test (loadSaveProfilePath, Glib::FILE_TEST_IS_DIR))
+        loadSaveProfilePath = preferredPath;
 }
 
 Glib::ustring Options::getPreferredProfilePath() {
@@ -184,26 +192,30 @@ Glib::ustring Options::findProfilePath(Glib::ustring &profName) {
 
 void Options::setDefaults () {
 
-	font = "sans, 10";
+    font = "sans, 8";
     windowWidth = 900;
     windowHeight = 560;
     windowMaximized = false;
+    saveAsDialogWidth = 600;
+    saveAsDialogHeight = 600;
     savesParamsAtExit = true;
     saveFormat.format = "jpg";
-    saveFormat.jpegQuality = 100;
+    saveFormat.jpegQuality = 90;
+    saveFormat.jpegSubSamp = 2;
     saveFormat.pngCompression = 6;
     saveFormat.pngBits = 8;
     saveFormat.tiffBits = 8;
     saveFormat.tiffUncompressed = true;
-    saveFormat.saveParams = true;		// was false
+    saveFormat.saveParams = true;
 
     saveFormatBatch.format = "jpg";
-    saveFormatBatch.jpegQuality = 100;
+    saveFormatBatch.jpegQuality = 90;
+    saveFormatBatch.jpegSubSamp = 2;
     saveFormatBatch.pngCompression = 6;
     saveFormatBatch.pngBits = 8;
     saveFormatBatch.tiffBits = 8;
     saveFormatBatch.tiffUncompressed = true;
-    saveFormatBatch.saveParams = true;		// was false
+    saveFormatBatch.saveParams = true;
 
     savePathTemplate = "%p1/converted/%f";
     savePathFolder = "";
@@ -212,7 +224,7 @@ void Options::setDefaults () {
     defProfImg = DEFPROFILE_IMG;
     dateFormat = "%y-%m-%d";
     adjusterDelay = 0;
-    startupDir = STARTUPDIR_LAST;		// was STARTUPDIR_HOME ; an empty startupPath is now correctly handled (open in the Home dir)
+    startupDir = STARTUPDIR_LAST;
     startupPath = "";
     useBundledProfiles = true;
     dirBrowserWidth = 200;
@@ -222,8 +234,8 @@ void Options::setDefaults () {
     toolPanelWidth = 300;
     browserToolPanelWidth = 300;
     browserToolPanelHeight = 300;
-    historyPanelWidth = 230;			// was 150
-    lastScale = 5;						// was 4
+    historyPanelWidth = 230;
+    lastScale = 5;
     panAccelFactor = 5;
     lastCropSize = 1;
     fbOnlyRaw = false;
@@ -236,34 +248,35 @@ void Options::setDefaults () {
     profilePath = "profiles";
     loadSaveProfilePath = "";			// will be corrected in load as otherwise construction fails
     version = "0.0.0.0";				// temporary value; will be correctly set in RTWindow::on_realize
-    thumbSize = 240;					// was 80
+    thumbSize = 240;
     thumbSizeTab = 80;
     showHistory = true;
     showFilePanelState = 0;				// Not used anymore ; was the thumb strip state
-    showInfo = true;					// was false
-    cropPPI = 600;						// was 300
+    showInfo = true;
+    cropPPI = 600;
     showClippedHighlights = false;
     showClippedShadows = false;
     highlightThreshold = 253;			// was 254
     shadowThreshold = 8;				// was 0
     bgcolor = 0;
-    blinkClipped = false;				// was true
+    blinkClipped = false;
     language = DefaultLanguage;
     languageAutoDetect= langMgr.isOSLanguageDetectSupported();
     lastSaveAsPath = "";
     overwriteOutputFile = false;		// if TRUE, existing output JPGs/PNGs are overwritten, instead of adding ..-1.jpg, -2.jpg etc.
     theme = "25-Gray-Gray";
-    slimUI = false;		// TODO: Should this be TRUE for worst case screen resolution or FALSE for nicer interface by default ???
-    useSystemTheme = true;
+    slimUI = false;
+    useSystemTheme = false;
     maxThumbnailHeight = 400;
-    maxCacheEntries = 20000;			// was 10000
-    thumbnailFormat = FT_Custom;		// was FT_Custom16
+    maxCacheEntries = 20000;
     thumbInterp = 1;
     autoSuffix = false;
+    forceFormatOpts = false;
+    saveMethodNum = 0;				// 0->immediate, 1->putToQueuHead, 2->putToQueueTail
     saveParamsFile = true;				// was false, but saving the procparams files next to the file make more sense when reorganizing file tree than in a cache
     saveParamsCache = false;			// there's no need to save the procparams files in a cache if saveParamsFile is true
     paramsLoadLocation = PLL_Input;		// was PLL_Cache
-    procQueueEnabled = false;			// was true
+    procQueueEnabled = false;
     gimpDir = "";
     psDir = "";
     customEditorProg = "";
@@ -287,17 +300,23 @@ void Options::setDefaults () {
     overlayedFileNames = true;
     internalThumbIfUntouched = true; 	// if TRUE, only fast, internal preview images are taken if the image is not edited yet
     showFileNames = true;
-    tabbedUI = true;					// was false;
+    tabbedUI = true;
     multiDisplayMode = 0;
     tunnelMetaData = false;
     histogramPosition = 2;
     histogramBar = true;
+    histogramFullMode = false;
+
+    rgbDenoiseThreadLimit = 0;
+
+    filledProfile = false;
+
     showProfileSelector = true;
-    FileBrowserToolbarSingleRow = true;
+    FileBrowserToolbarSingleRow = false;
     hideTPVScrollbar = false;
     UseIconNoText = true;
     whiteBalanceSpotSize = 8;
-    squareDetailWindow = false;
+    squareDetailWindow = true;
     menuGroupRank = true;
     menuGroupLabel = true;
     menuGroupFileOperations = true;
@@ -313,10 +332,11 @@ void Options::setDefaults () {
     fastexport_bypass_dirpyrDenoise      = true;
     fastexport_bypass_sh_hq              = true;
     fastexport_bypass_dirpyrequalizer    = true;
-    fastexport_bypass_raw_all_enhance    = true;
+    //fastexport_bypass_raw_all_enhance    = true;
     fastexport_bypass_raw_ccSteps        = true;
     fastexport_bypass_raw_dcb_iterations = true;
     fastexport_bypass_raw_dcb_enhance    = true;
+    fastexport_bypass_raw_lmmse_iterations = true;
     fastexport_bypass_raw_ca             = true;
     fastexport_bypass_raw_linenoise      = true;
     fastexport_bypass_raw_greenthresh    = true;
@@ -340,6 +360,10 @@ void Options::setDefaults () {
 
     sndEnable=true;
     sndLngEditProcDoneSecs=3.0;
+#ifdef __linux__
+    sndBatchQueueDone = "complete";
+    sndLngEditProcDone = "window-attention";
+#endif
 
     // Reminder: 0 = SET mode, 1 = ADD mode
     int babehav[] = {
@@ -353,24 +377,27 @@ void Options::setDefaults () {
 			0,  // ADDSET_LC_BRIGHTNESS
 			0,  // ADDSET_LC_CONTRAST
 			0,  // ADDSET_SHARP_AMOUNT
-			//0,  // ADDSET_LD_EDGETOLERANCE -- From obsolete and removed tool
 			0,  // ADDSET_WB_TEMPERATURE
 			0,  // ADDSET_WB_GREEN
-			//0,  // ADDSET_CBOOST_AMOUNT -- From obsolete and removed tool
-			//0,  // ADDSET_CS_BLUEYELLOW -- From obsolete and removed tool
-			//0,  // ADDSET_CS_GREENMAGENTA -- From obsolete and removed tool
 			0,  // ADDSET_ROTATE_DEGREE
 			0,  // ADDSET_DIST_AMOUNT
 			0,  // ADDSET_PERSPECTIVE
 			0,  // ADDSET_CA
 			0,  // ADDSET_VIGN_AMOUNT
-			0,  // ADDSET_LC_SATURATION
+			0,  // ADDSET_VIGN_RADIUS
+			0,  // ADDSET_VIGN_STRENGTH
+			0,  // ADDSET_VIGN_CENTER
+			0,  // ADDSET_LC_CHROMATICITY
 			0,  // ADDSET_TC_SATURATION
 			0,  // ADDSET_TC_HLCOMPAMOUNT
 			0,  // ADDSET_TC_HLCOMPTHRESH
 			0,  // ADDSET_TC_SHCOMP
 			0,  // ADDSET_DIRPYREQ
-			0,  // ADDSET_DIRPYRDN_CHLUM
+			0,  // ADDSET_DIRPYRDN_LUMA	
+			0,  // ADDSET_DIRPYRDN_LUDET			
+			0,  // ADDSET_DIRPYRDN_CHROMA
+			0,  // ADDSET_DIRPYRDN_CHROMARED
+			0,  // ADDSET_DIRPYRDN_CHROMABLUE			
 			0,  // ADDSET_DIRPYRDN_GAMMA
 			0,  // ADDSET_CHMIXER
 			0,  // ADDSET_PREPROCESS_GREENEQUIL
@@ -385,25 +412,36 @@ void Options::setDefaults () {
 			0,  // ADDSET_SHARPENMICRO_UNIFORMITY
 			0,  // ADDSET_VIBRANCE_PASTELS
 			0,  // ADDSET_VIBRANCE_SATURATED
-			0,   // ADDSET_VIBRANCE_PSTHRESHOLD
-			0,   // ADDSET_FREE_OUPUT_GAMMA
-			0,   // ADDSET_FREE_OUTPUT_SLOPE
+			0,  // ADDSET_FREE_OUPUT_GAMMA
+			0,  // ADDSET_FREE_OUTPUT_SLOPE
+			0,  // ADDSET_CAT_DEGREE
+			0,  // ADDSET_CAT_ADAPSCEN
+			0,  // ADDSET_CAT_ADAPLUM
+			0,  // ADDSET_CAT_BADPIX
+			0,  // ADDSET_CAT_JLIGHT
+			0,  // ADDSET_CAT_CHROMA
+ 			0,  // ADDSET_CAT_CONTRAST
 
 	};
     baBehav = std::vector<int> (babehav, babehav+ADDSET_PARAM_NUM);
     
     rtSettings.darkFramesPath = "";
-	rtSettings.flatFieldsPath = "";
+    rtSettings.flatFieldsPath = "";
 #ifdef WIN32
 	const gchar* sysRoot = g_getenv("SystemRoot");  // Returns e.g. "c:\Windows"
 	if (sysRoot!=NULL) 
 		rtSettings.iccDirectory = Glib::ustring(sysRoot) + Glib::ustring("\\System32\\spool\\drivers\\color");
 	else 
 		rtSettings.iccDirectory = "C:\\WINDOWS\\System32\\spool\\drivers\\color";
+#elif defined __APPLE__
+    rtSettings.iccDirectory = "/library/ColorSync/Profiles/Displays";
 #else
     rtSettings.iccDirectory = "/usr/share/color/icc";
 #endif
     rtSettings.colorimetricIntent = 1;
+	rtSettings.viewingdevice=0;
+	rtSettings.viewingdevicegrey=3;
+	
     rtSettings.monitorProfile = "";
     rtSettings.autoMonitorProfile = false;
     rtSettings.adobe = "RT_Medium_gsRGB"; // put the name of yours profiles (here windows)
@@ -416,8 +454,15 @@ void Options::setDefaults () {
     rtSettings.beta = "BetaRGB";
     rtSettings.best = "BestRGB";
     rtSettings.verbose = false;
-	rtSettings.gamutICC = true;
-
+    rtSettings.gamutICC = true;
+    rtSettings.gamutLch = true;
+    rtSettings.ciecamfloat = true;
+    rtSettings.protectred = 60;
+    rtSettings.protectredh = 0.3;
+    rtSettings.CRI_color =0;
+	rtSettings.autocielab=true;
+//	rtSettings.ciebadpixgauss=false;	
+	rtSettings.rgbcurveslumamode_gamut=true;
 	lastIccDir = rtSettings.iccDirectory;
 	lastDarkframeDir = rtSettings.darkFramesPath;
 	lastFlatfieldDir = rtSettings.flatFieldsPath;
@@ -429,15 +474,16 @@ void Options::setDefaults () {
 	// at the end of the "updatePaths" method.
 	lastRgbCurvesDir = "";
 	lastLabCurvesDir = "";
+	lastPFCurvesDir = "";
 	lastHsvCurvesDir = "";
 	lastToneCurvesDir = "";
+	lastVibranceCurvesDir = "";
 	lastProfilingReferenceDir = "";
 }
 
 Options* Options::copyFrom (Options* other) {
-
-    *this = *other;
-	return this;
+  *this = *other;
+  return this;
 }
 
 void Options::filterOutParsedExtensions () {
@@ -456,7 +502,7 @@ int Options::readFromFile (Glib::ustring fname) {
         if (!keyFile.load_from_file (fname)) 
             return 1;
     }
-    catch (Glib::FileError) {
+    catch (Glib::FileError &err) {
         return 1;
     }
 
@@ -497,6 +543,7 @@ if (keyFile.has_group ("External Editor")) {
 if (keyFile.has_group ("Output")) { 
     if (keyFile.has_key ("Output", "Format"))           saveFormat.format          = keyFile.get_string ("Output", "Format");
     if (keyFile.has_key ("Output", "JpegQuality"))      saveFormat.jpegQuality     = keyFile.get_integer ("Output", "JpegQuality");
+    if (keyFile.has_key ("Output", "JpegSubSamp"))      saveFormat.jpegSubSamp     = keyFile.get_integer ("Output", "JpegSubSamp");
     if (keyFile.has_key ("Output", "PngCompression"))   saveFormat.pngCompression  = keyFile.get_integer ("Output", "PngCompression");
     if (keyFile.has_key ("Output", "PngBps"))           saveFormat.pngBits         = keyFile.get_integer ("Output", "PngBps");
     if (keyFile.has_key ("Output", "TiffBps"))          saveFormat.tiffBits        = keyFile.get_integer ("Output", "TiffBps");
@@ -506,6 +553,7 @@ if (keyFile.has_group ("Output")) {
 
     if (keyFile.has_key ("Output", "FormatBatch"))           saveFormatBatch.format          = keyFile.get_string ("Output", "FormatBatch");
     if (keyFile.has_key ("Output", "JpegQualityBatch"))      saveFormatBatch.jpegQuality     = keyFile.get_integer ("Output", "JpegQualityBatch");
+    if (keyFile.has_key ("Output", "JpegSubSampBatch"))      saveFormatBatch.jpegSubSamp     = keyFile.get_integer ("Output", "JpegSubSampBatch");
     if (keyFile.has_key ("Output", "PngCompressionBatch"))   saveFormatBatch.pngCompression  = keyFile.get_integer ("Output", "PngCompressionBatch");
     if (keyFile.has_key ("Output", "PngBpsBatch"))           saveFormatBatch.pngBits         = keyFile.get_integer ("Output", "PngBpsBatch");
     if (keyFile.has_key ("Output", "TiffBpsBatch"))          saveFormatBatch.tiffBits        = keyFile.get_integer ("Output", "TiffBpsBatch");
@@ -516,10 +564,12 @@ if (keyFile.has_group ("Output")) {
     if (keyFile.has_key ("Output", "PathTemplate"))     savePathTemplate           = keyFile.get_string ("Output", "PathTemplate");
     if (keyFile.has_key ("Output", "PathFolder"))       savePathFolder             = keyFile.get_string ("Output", "PathFolder");
     if (keyFile.has_key ("Output", "AutoSuffix"))       autoSuffix                 = keyFile.get_boolean("Output", "AutoSuffix");
+    if (keyFile.has_key ("Output", "ForceFormatOpts"))  forceFormatOpts            = keyFile.get_boolean("Output", "ForceFormatOpts");
+    if (keyFile.has_key ("Output", "SaveMethodNum"))    saveMethodNum              = keyFile.get_integer("Output", "SaveMethodNum");
     if (keyFile.has_key ("Output", "UsePathTemplate"))  saveUsePathTemplate        = keyFile.get_boolean("Output", "UsePathTemplate");
     if (keyFile.has_key ("Output", "LastSaveAsPath"))   lastSaveAsPath             = keyFile.get_string ("Output", "LastSaveAsPath");
-	if (keyFile.has_key ("Output", "OverwriteOutputFile"))  overwriteOutputFile    = keyFile.get_boolean("Output", "OverwriteOutputFile");
-	if (keyFile.has_key ("Output", "TunnelMetaData"))   tunnelMetaData             = keyFile.get_boolean("Output", "TunnelMetaData");
+    if (keyFile.has_key ("Output", "OverwriteOutputFile"))  overwriteOutputFile    = keyFile.get_boolean("Output", "OverwriteOutputFile");
+    if (keyFile.has_key ("Output", "TunnelMetaData"))   tunnelMetaData             = keyFile.get_boolean("Output", "TunnelMetaData");
 }
 
 if (keyFile.has_group ("Profiles")) { 
@@ -528,6 +578,7 @@ if (keyFile.has_group ("Profiles")) {
     if (keyFile.has_key ("Profiles", "LoadSaveProfilePath"))    loadSaveProfilePath  = keyFile.get_string  ("Profiles", "LoadSaveProfilePath");
     if (keyFile.has_key ("Profiles", "RawDefault"))             defProfRaw           = keyFile.get_string  ("Profiles", "RawDefault");
     if (keyFile.has_key ("Profiles", "ImgDefault"))             defProfImg           = keyFile.get_string  ("Profiles", "ImgDefault");
+    if (keyFile.has_key ("Profiles", "FilledProfile"))          filledProfile        = keyFile.get_boolean ("Profiles", "FilledProfile");
     if (keyFile.has_key ("Profiles", "SaveParamsWithFile"))     saveParamsFile       = keyFile.get_boolean ("Profiles", "SaveParamsWithFile");
     if (keyFile.has_key ("Profiles", "SaveParamsToCache"))      saveParamsCache      = keyFile.get_boolean ("Profiles", "SaveParamsToCache");
     if (keyFile.has_key ("Profiles", "LoadParamsFromLocation")) paramsLoadLocation   = (PPLoadLocation)keyFile.get_integer ("Profiles", "LoadParamsFromLocation");
@@ -544,7 +595,6 @@ if (keyFile.has_group ("File Browser")) {
     if (keyFile.has_key ("File Browser", "BrowserShowsHidden")) fbShowHidden       = keyFile.get_boolean ("File Browser", "BrowserShowsHidden");
     if (keyFile.has_key ("File Browser", "MaxPreviewHeight"))   maxThumbnailHeight = keyFile.get_integer ("File Browser", "MaxPreviewHeight");
     if (keyFile.has_key ("File Browser", "MaxCacheEntries"))    maxCacheEntries    = keyFile.get_integer ("File Browser", "MaxCacheEntries");
-    if (keyFile.has_key ("File Browser", "ThumbnailFormat"))    thumbnailFormat    = (ThFileType)keyFile.get_integer ("File Browser", "ThumbnailFormat");
     if (keyFile.has_key ("File Browser", "ParseExtensions"))    parseExtensions    = keyFile.get_string_list ("File Browser", "ParseExtensions");
     if (keyFile.has_key ("File Browser", "ParseExtensionsEnabled")) parseExtensionsEnabled    = keyFile.get_integer_list ("File Browser", "ParseExtensionsEnabled");
     if (keyFile.has_key ("File Browser", "ThumbnailArrangement")) fbArrangement    = keyFile.get_integer ("File Browser", "ThumbnailArrangement");
@@ -570,6 +620,10 @@ if (keyFile.has_group ("Clipping Indication")) {
     if (keyFile.has_key ("Clipping Indication", "BlinkClipped"))        blinkClipped      = keyFile.get_boolean ("Clipping Indication", "BlinkClipped");
 }
 
+if (keyFile.has_group ("Performance")) {
+    if (keyFile.has_key ("Performance", "RgbDenoiseThreadLimit")) rgbDenoiseThreadLimit = keyFile.get_integer ("Performance", "RgbDenoiseThreadLimit");
+}
+
 if (keyFile.has_group ("GUI")) { 
     if (keyFile.has_key ("GUI", "Font"))            font            = keyFile.get_string  ("GUI", "Font");
     if (keyFile.has_key ("GUI", "WindowWidth"))     windowWidth     = keyFile.get_integer ("GUI", "WindowWidth");
@@ -577,8 +631,8 @@ if (keyFile.has_group ("GUI")) {
     if (keyFile.has_key ("GUI", "WindowMaximized")) windowMaximized = keyFile.get_boolean ("GUI", "WindowMaximized");
     if (keyFile.has_key ("GUI", "DirBrowserWidth"))     dirBrowserWidth          = keyFile.get_integer ("GUI", "DirBrowserWidth");
     if (keyFile.has_key ("GUI", "DirBrowserHeight"))    dirBrowserHeight         = keyFile.get_integer ("GUI", "DirBrowserHeight");
-	if (keyFile.has_key ("GUI", "PreferencesWidth"))    preferencesWidth         = keyFile.get_integer ("GUI", "PreferencesWidth");
-	if (keyFile.has_key ("GUI", "PreferencesHeight"))   preferencesHeight        = keyFile.get_integer ("GUI", "PreferencesHeight"); 
+    if (keyFile.has_key ("GUI", "PreferencesWidth"))    preferencesWidth         = keyFile.get_integer ("GUI", "PreferencesWidth");
+    if (keyFile.has_key ("GUI", "PreferencesHeight"))   preferencesHeight        = keyFile.get_integer ("GUI", "PreferencesHeight"); 
     if (keyFile.has_key ("GUI", "SaveAsDialogWidth"))   saveAsDialogWidth        = keyFile.get_integer ("GUI", "SaveAsDialogWidth");
     if (keyFile.has_key ("GUI", "SaveAsDialogHeight"))  saveAsDialogHeight       = keyFile.get_integer ("GUI", "SaveAsDialogHeight");
     if (keyFile.has_key ("GUI", "ToolPanelWidth"))      toolPanelWidth           = keyFile.get_integer ("GUI", "ToolPanelWidth");
@@ -602,6 +656,7 @@ if (keyFile.has_group ("GUI")) {
     if (keyFile.has_key ("GUI", "CutOverlayBrush"))     cutOverlayBrush     = keyFile.get_double_list ("GUI", "CutOverlayBrush");
     if (keyFile.has_key ("GUI", "HistogramPosition"))   histogramPosition   = keyFile.get_integer ("GUI", "HistogramPosition");
     if (keyFile.has_key ("GUI", "HistogramBar"))        histogramBar        = keyFile.get_boolean ("GUI", "HistogramBar");
+    if (keyFile.has_key ("GUI", "HistogramFullMode"))   histogramFullMode   = keyFile.get_boolean ("GUI", "HistogramFullMode");
     if (keyFile.has_key ("GUI", "ShowProfileSelector")) showProfileSelector = keyFile.get_boolean ("GUI", "ShowProfileSelector");
     if (keyFile.has_key ("GUI", "SquareDetailWindow"))  squareDetailWindow  = keyFile.get_boolean ("GUI", "SquareDetailWindow");
     if (keyFile.has_key ("GUI", "FileBrowserToolbarSingleRow")) FileBrowserToolbarSingleRow = keyFile.get_boolean ("GUI", "FileBrowserToolbarSingleRow");
@@ -619,11 +674,18 @@ if (keyFile.has_group ("Color Management")) {
     if (keyFile.has_key ("Color Management", "ICCDirectory"))   rtSettings.iccDirectory         = keyFile.get_string ("Color Management", "ICCDirectory");
     if (keyFile.has_key ("Color Management", "MonitorProfile")) rtSettings.monitorProfile       = keyFile.get_string ("Color Management", "MonitorProfile");
     if (keyFile.has_key ("Color Management", "AutoMonitorProfile")) rtSettings.autoMonitorProfile = keyFile.get_boolean ("Color Management", "AutoMonitorProfile");
+    if (keyFile.has_key ("Color Management", "Autocielab")) rtSettings.autocielab = keyFile.get_boolean ("Color Management", "Autocielab");
+    if (keyFile.has_key ("Color Management", "Ciencamfloat")) rtSettings.ciecamfloat = keyFile.get_boolean ("Color Management", "Ciecamfloat");
+    if (keyFile.has_key ("Color Management", "RGBcurvesLumamode_Gamut")) rtSettings.rgbcurveslumamode_gamut = keyFile.get_boolean ("Color Management", "RGBcurvesLumamode_Gamut");
 
     if (keyFile.has_key ("Color Management", "Intent"))         rtSettings.colorimetricIntent   = keyFile.get_integer("Color Management", "Intent");
+    if (keyFile.has_key ("Color Management", "CRI"))            rtSettings.CRI_color            = keyFile.get_integer("Color Management", "CRI");
+    if (keyFile.has_key ("Color Management", "view"))           rtSettings.viewingdevice        = keyFile.get_integer("Color Management", "view");
+    if (keyFile.has_key ("Color Management", "grey"))           rtSettings.viewingdevicegrey    = keyFile.get_integer("Color Management", "grey");
 
     if (keyFile.has_key ("Color Management", "WhiteBalanceSpotSize")) whiteBalanceSpotSize      = keyFile.get_integer("Color Management", "WhiteBalanceSpotSize");
     if( keyFile.has_key ("Color Management", "GamutICC"))       rtSettings.gamutICC             = keyFile.get_boolean("Color Management", "GamutICC");
+    if( keyFile.has_key ("Color Management", "Ciecamfloat"))    rtSettings.ciecamfloat          = keyFile.get_boolean("Color Management", "Ciecamfloat");
     if( keyFile.has_key ("Color Management", "AdobeRGB"))       rtSettings.adobe                = keyFile.get_string("Color Management", "AdobeRGB");
     if( keyFile.has_key ("Color Management", "ProPhoto"))       rtSettings.prophoto             = keyFile.get_string("Color Management", "ProPhoto");
     if( keyFile.has_key ("Color Management", "ProPhoto10"))     rtSettings.prophoto10           = keyFile.get_string("Color Management", "ProPhoto10");
@@ -633,6 +695,11 @@ if (keyFile.has_group ("Color Management")) {
     if( keyFile.has_key ("Color Management", "Beta"))           rtSettings.beta                 = keyFile.get_string("Color Management", "Beta");
     if( keyFile.has_key ("Color Management", "Best"))           rtSettings.best                 = keyFile.get_string("Color Management", "Best");
     if( keyFile.has_key ("Color Management", "Bruce"))          rtSettings.bruce                = keyFile.get_string("Color Management", "Bruce");
+    if( keyFile.has_key ("Color Management", "GamutLch"))       rtSettings.gamutLch             = keyFile.get_boolean("Color Management", "GamutLch");
+    if( keyFile.has_key ("Color Management", "ProtectRed"))     rtSettings.protectred           = keyFile.get_integer("Color Management", "ProtectRed");
+    if( keyFile.has_key ("Color Management", "ProtectRedH"))    rtSettings.protectredh          = keyFile.get_double("Color Management", "ProtectRedH");
+//    if( keyFile.has_key ("Color Management", "Ciebadpixgauss")) rtSettings.ciebadpixgauss       = keyFile.get_boolean("Color Management", "Ciebadpixgauss");
+	
 }
 
 if (keyFile.has_group ("Batch Processing")) { 
@@ -656,10 +723,11 @@ if (keyFile.has_group ("Fast Export")) {
     if (keyFile.has_key ("Fast Export", "fastexport_bypass_dirpyrDenoise"     ))  fastexport_bypass_dirpyrDenoise       = keyFile.get_boolean ("Fast Export", "fastexport_bypass_dirpyrDenoise"     );
     if (keyFile.has_key ("Fast Export", "fastexport_bypass_sh_hq"             ))  fastexport_bypass_sh_hq               = keyFile.get_boolean ("Fast Export", "fastexport_bypass_sh_hq"             );
     if (keyFile.has_key ("Fast Export", "fastexport_bypass_dirpyrequalizer"   ))  fastexport_bypass_dirpyrequalizer     = keyFile.get_boolean ("Fast Export", "fastexport_bypass_dirpyrequalizer"   );
-    if (keyFile.has_key ("Fast Export", "fastexport_bypass_raw_all_enhance"   ))  fastexport_bypass_raw_all_enhance     = keyFile.get_boolean ("Fast Export", "fastexport_bypass_raw_all_enhance"   );
+    //if (keyFile.has_key ("Fast Export", "fastexport_bypass_raw_all_enhance"   ))  fastexport_bypass_raw_all_enhance     = keyFile.get_boolean ("Fast Export", "fastexport_bypass_raw_all_enhance"   );
     if (keyFile.has_key ("Fast Export", "fastexport_bypass_raw_ccSteps"       ))  fastexport_bypass_raw_ccSteps         = keyFile.get_boolean ("Fast Export", "fastexport_bypass_raw_ccSteps"       );
     if (keyFile.has_key ("Fast Export", "fastexport_bypass_raw_dcb_iterations"))  fastexport_bypass_raw_dcb_iterations  = keyFile.get_boolean ("Fast Export", "fastexport_bypass_raw_dcb_iterations");
     if (keyFile.has_key ("Fast Export", "fastexport_bypass_raw_dcb_enhance"   ))  fastexport_bypass_raw_dcb_enhance     = keyFile.get_boolean ("Fast Export", "fastexport_bypass_raw_dcb_enhance"   );
+    if (keyFile.has_key ("Fast Export", "fastexport_bypass_raw_lmmse_iterations"))  fastexport_bypass_raw_lmmse_iterations  = keyFile.get_boolean ("Fast Export", "fastexport_bypass_raw_lmmse_iterations");
     if (keyFile.has_key ("Fast Export", "fastexport_bypass_raw_ca"            ))  fastexport_bypass_raw_ca              = keyFile.get_boolean ("Fast Export", "fastexport_bypass_raw_ca"            );
     if (keyFile.has_key ("Fast Export", "fastexport_bypass_raw_linenoise"     ))  fastexport_bypass_raw_linenoise       = keyFile.get_boolean ("Fast Export", "fastexport_bypass_raw_linenoise"     );
     if (keyFile.has_key ("Fast Export", "fastexport_bypass_raw_greenthresh"   ))  fastexport_bypass_raw_greenthresh     = keyFile.get_boolean ("Fast Export", "fastexport_bypass_raw_greenthresh"   );
@@ -685,8 +753,10 @@ if (keyFile.has_group ("Dialogs")) {
     safeDirGet(keyFile, "Dialogs", "LastFlatfieldDir", lastFlatfieldDir);
     safeDirGet(keyFile, "Dialogs", "LastRgbCurvesDir", lastRgbCurvesDir);
     safeDirGet(keyFile, "Dialogs", "LastLabCurvesDir", lastLabCurvesDir);
+    safeDirGet(keyFile, "Dialogs", "LastPFCurvesDir", lastPFCurvesDir);
     safeDirGet(keyFile, "Dialogs", "LastHsvCurvesDir", lastHsvCurvesDir);
     safeDirGet(keyFile, "Dialogs", "LastToneCurvesDir", lastToneCurvesDir);
+    safeDirGet(keyFile, "Dialogs", "LastVibranceCurvesDir", lastVibranceCurvesDir);
     safeDirGet(keyFile, "Dialogs", "LastProfilingReferenceDir", lastProfilingReferenceDir);
 }
 
@@ -748,7 +818,6 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_integer ("File Browser", "ThumbnailSizeTab", thumbSizeTab);
     keyFile.set_integer ("File Browser", "MaxPreviewHeight", maxThumbnailHeight);
     keyFile.set_integer ("File Browser", "MaxCacheEntries", maxCacheEntries);
-    keyFile.set_integer ("File Browser", "ThumbnailFormat", (int)thumbnailFormat);
     Glib::ArrayHandle<Glib::ustring> pext = parseExtensions;
     keyFile.set_string_list ("File Browser", "ParseExtensions", pext);
     Glib::ArrayHandle<int> pextena = parseExtensionsEnabled;
@@ -776,8 +845,11 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_integer ("Clipping Indication", "ShadowThreshold", shadowThreshold);
     keyFile.set_boolean ("Clipping Indication", "BlinkClipped", blinkClipped);
 
+    keyFile.set_integer ("Performance", "RgbDenoiseThreadLimit", rgbDenoiseThreadLimit);
+
     keyFile.set_string  ("Output", "Format", saveFormat.format);
     keyFile.set_integer ("Output", "JpegQuality", saveFormat.jpegQuality);
+    keyFile.set_integer ("Output", "JpegSubSamp", saveFormat.jpegSubSamp);
     keyFile.set_integer ("Output", "PngCompression", saveFormat.pngCompression);
     keyFile.set_integer ("Output", "PngBps", saveFormat.pngBits);
     keyFile.set_integer ("Output", "TiffBps", saveFormat.tiffBits);
@@ -786,6 +858,7 @@ int Options::saveToFile (Glib::ustring fname) {
 
     keyFile.set_string  ("Output", "FormatBatch", saveFormatBatch.format);
     keyFile.set_integer ("Output", "JpegQualityBatch", saveFormatBatch.jpegQuality);
+    keyFile.set_integer ("Output", "JpegSubSampBatch", saveFormatBatch.jpegSubSamp);
     keyFile.set_integer ("Output", "PngCompressionBatch", saveFormatBatch.pngCompression);
     keyFile.set_integer ("Output", "PngBpsBatch", saveFormatBatch.pngBits);
     keyFile.set_integer ("Output", "TiffBpsBatch", saveFormatBatch.tiffBits);
@@ -795,9 +868,11 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_string  ("Output", "PathTemplate", savePathTemplate);
     keyFile.set_string  ("Output", "PathFolder", savePathFolder);
     keyFile.set_boolean ("Output", "AutoSuffix", autoSuffix);
+    keyFile.set_boolean ("Output", "ForceFormatOpts", forceFormatOpts);
+    keyFile.set_integer ("Output", "SaveMethodNum", saveMethodNum);
     keyFile.set_boolean ("Output", "UsePathTemplate", saveUsePathTemplate);
     keyFile.set_string  ("Output", "LastSaveAsPath", lastSaveAsPath);
-	keyFile.set_boolean ("Output", "OverwriteOutputFile", overwriteOutputFile);
+    keyFile.set_boolean ("Output", "OverwriteOutputFile", overwriteOutputFile);
     keyFile.set_boolean ("Output", "TunnelMetaData", tunnelMetaData);
 
     keyFile.set_string  ("Profiles", "Directory", profilePath);
@@ -805,6 +880,7 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_string  ("Profiles", "LoadSaveProfilePath", loadSaveProfilePath);
     keyFile.set_string  ("Profiles", "RawDefault", defProfRaw);
     keyFile.set_string  ("Profiles", "ImgDefault", defProfImg);
+    keyFile.set_boolean ("Profiles", "FilledProfile", filledProfile);
     keyFile.set_boolean ("Profiles", "SaveParamsWithFile", saveParamsFile);
     keyFile.set_boolean ("Profiles", "SaveParamsToCache", saveParamsCache);
     keyFile.set_integer ("Profiles", "LoadParamsFromLocation", paramsLoadLocation);
@@ -816,8 +892,8 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_boolean ("GUI", "WindowMaximized", windowMaximized);
     keyFile.set_integer ("GUI", "DirBrowserWidth", dirBrowserWidth);
     keyFile.set_integer ("GUI", "DirBrowserHeight", dirBrowserHeight);
-	keyFile.set_integer ("GUI", "PreferencesWidth", preferencesWidth);
-	keyFile.set_integer ("GUI", "PreferencesHeight", preferencesHeight); 
+    keyFile.set_integer ("GUI", "PreferencesWidth", preferencesWidth);
+    keyFile.set_integer ("GUI", "PreferencesHeight", preferencesHeight); 
     keyFile.set_integer ("GUI", "SaveAsDialogWidth", saveAsDialogWidth);
     keyFile.set_integer ("GUI", "SaveAsDialogHeight", saveAsDialogHeight);
     keyFile.set_integer ("GUI", "ToolPanelWidth", toolPanelWidth);
@@ -841,6 +917,7 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_double_list ("GUI", "CutOverlayBrush", cutOverlayBrush);
     keyFile.set_integer ("GUI", "HistogramPosition", histogramPosition);
     keyFile.set_boolean ("GUI", "HistogramBar", histogramBar);
+    keyFile.set_boolean ("GUI", "HistogramFullMode", histogramFullMode);
     keyFile.set_boolean ("GUI", "ShowProfileSelector", showProfileSelector);
     keyFile.set_boolean ("GUI", "SquareDetailWindow", squareDetailWindow);
     keyFile.set_boolean ("GUI", "FileBrowserToolbarSingleRow", FileBrowserToolbarSingleRow);
@@ -854,8 +931,13 @@ int Options::saveToFile (Glib::ustring fname) {
 
     keyFile.set_string  ("Color Management", "ICCDirectory", rtSettings.iccDirectory);
     keyFile.set_string  ("Color Management", "MonitorProfile", rtSettings.monitorProfile);
-	keyFile.set_boolean ("Color Management", "AutoMonitorProfile", rtSettings.autoMonitorProfile);
+    keyFile.set_boolean ("Color Management", "AutoMonitorProfile", rtSettings.autoMonitorProfile);
+    keyFile.set_boolean ("Color Management", "Autocielab", rtSettings.autocielab);
+    keyFile.set_boolean ("Color Management", "RGBcurvesLumamode_Gamut", rtSettings.rgbcurveslumamode_gamut);
     keyFile.set_integer ("Color Management", "Intent", rtSettings.colorimetricIntent);
+    keyFile.set_integer ("Color Management", "view", rtSettings.viewingdevice);	
+    keyFile.set_integer ("Color Management", "grey", rtSettings.viewingdevicegrey);	
+	
     keyFile.set_string  ("Color Management", "AdobeRGB", rtSettings.adobe);
     keyFile.set_string  ("Color Management", "ProPhoto", rtSettings.prophoto);
     keyFile.set_string  ("Color Management", "ProPhoto10", rtSettings.prophoto10);
@@ -867,6 +949,12 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_string  ("Color Management", "Bruce", rtSettings.bruce);
     keyFile.set_integer ("Color Management", "WhiteBalanceSpotSize", whiteBalanceSpotSize);
     keyFile.set_boolean ("Color Management", "GamutICC", rtSettings.gamutICC);
+    keyFile.set_boolean ("Color Management", "Ciecamfloat", rtSettings.ciecamfloat);
+    keyFile.set_boolean ("Color Management", "GamutLch", rtSettings.gamutLch);
+    keyFile.set_integer ("Color Management", "ProtectRed", rtSettings.protectred);
+    keyFile.set_double  ("Color Management", "ProtectRedH", rtSettings.protectredh);
+    keyFile.set_integer ("Color Management", "CRI", rtSettings.CRI_color);
+//    keyFile.set_boolean ("Color Management", "Ciebadpixgauss", rtSettings.ciebadpixgauss);
 
     Glib::ArrayHandle<int> bab = baBehav;
     keyFile.set_integer_list ("Batch Processing", "AdjusterBehavior", bab);
@@ -886,11 +974,12 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_boolean ("Fast Export", "fastexport_bypass_dirpyrDenoise"      , fastexport_bypass_dirpyrDenoise     );
     keyFile.set_boolean ("Fast Export", "fastexport_bypass_sh_hq"              , fastexport_bypass_sh_hq             );
     keyFile.set_boolean ("Fast Export", "fastexport_bypass_dirpyrequalizer"    , fastexport_bypass_dirpyrequalizer   );
-    keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_all_enhance"    , fastexport_bypass_raw_all_enhance   );
+    //keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_all_enhance"    , fastexport_bypass_raw_all_enhance   );
     keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_ccSteps"        , fastexport_bypass_raw_ccSteps       );
     keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_dcb_iterations" , fastexport_bypass_raw_dcb_iterations);
     keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_dcb_enhance"    , fastexport_bypass_raw_dcb_enhance   );
-    keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_ca"             , fastexport_bypass_raw_ca            );
+    keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_lmmse_iterations" , fastexport_bypass_raw_lmmse_iterations);
+	keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_ca"             , fastexport_bypass_raw_ca            );
     keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_linenoise"      , fastexport_bypass_raw_linenoise     );
     keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_greenthresh"    , fastexport_bypass_raw_greenthresh   );
     keyFile.set_boolean ("Fast Export", "fastexport_bypass_raw_df"             , fastexport_bypass_raw_df            );
@@ -913,8 +1002,10 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_string ("Dialogs", "LastFlatfieldDir", lastFlatfieldDir);
     keyFile.set_string ("Dialogs", "LastRgbCurvesDir", lastRgbCurvesDir);
     keyFile.set_string ("Dialogs", "LastLabCurvesDir", lastLabCurvesDir);
+    keyFile.set_string ("Dialogs", "LastPFCurvesDir", lastPFCurvesDir);
     keyFile.set_string ("Dialogs", "LastHsvCurvesDir", lastHsvCurvesDir);
     keyFile.set_string ("Dialogs", "LastToneCurvesDir", lastToneCurvesDir);
+    keyFile.set_string ("Dialogs", "LastVibranceCurvesDir", lastVibranceCurvesDir);
     keyFile.set_string ("Dialogs", "LastProfilingReferenceDir", lastProfilingReferenceDir);
 
     FILE *f = safe_g_fopen (fname, "wt");
@@ -1021,9 +1112,9 @@ void Options::load () {
 	// out which are the parent translations.  Furthermore, there must be a file <Language> for each locale <Language> (<LC>) -- you cannot have 
 	// 'French (CA)' unless there is a file 'French'.
 
-    Glib::ustring defaultTranslation = argv0 + "/languages/default";
-	Glib::ustring languageTranslation = "";
-	Glib::ustring localeTranslation = "";
+  Glib::ustring defaultTranslation = argv0 + "/languages/default";
+  Glib::ustring languageTranslation = "";
+  Glib::ustring localeTranslation = "";
 
     if (options.languageAutoDetect) options.language=langMgr.getOSUserLanguage();
 
