@@ -36,6 +36,9 @@ class ToneCurveParamsEdited {
 
     public:
         bool curve;
+        bool curve2;
+        bool curveMode;
+        bool curveMode2;
         bool brightness;
         bool black;
         bool contrast;
@@ -53,18 +56,23 @@ class LCurveParamsEdited {
     public:
         bool brightness;
         bool contrast;
-        bool saturation;
-        bool avoidclip;
-        bool enable_saturationlimiter;
-        bool saturationlimit;
+        bool chromaticity;
+        bool avoidcolorshift;
+        bool rstprotection;
         bool lcurve;
         bool acurve;
         bool bcurve;
+        bool bwtoning;
+		bool lcredsk;
+        bool cccurve;
+        bool chcurve;
+        bool lccurve;
 };
 
 class RGBCurvesParamsEdited {
 
     public:
+        bool lumamode;
         bool rcurve;
         bool gcurve;
         bool bcurve;
@@ -117,6 +125,7 @@ class VibranceParamsEdited {
         bool protectskins;
         bool avoidcolorshift;
         bool pastsattog;
+        bool skintonescurve;
 };
 
 /*class ColorBoostParamsEdited {
@@ -125,7 +134,7 @@ class VibranceParamsEdited {
         bool amount;
         bool avoidclip;
         bool enable_saturationlimiter;
-        bool saturationlimit;
+        bool rstprotection;
 };*/
 
 class WBParamsEdited {
@@ -161,35 +170,78 @@ class WBParamsEdited {
 class DefringeParamsEdited {
 	
 public:
-	bool enabled;
-	bool radius;
-	bool threshold;
+    bool enabled;
+    bool radius;
+    bool threshold;
+    bool huecurve;
 };
 
 class ImpulseDenoiseParamsEdited {
-	
-public:
-	bool enabled;
-	bool thresh;
 
+public:
+    bool enabled;
+    bool thresh;
 };
 
-class DirPyrDenoiseParamsEdited {
-	
+class ColorAppearanceParamsEdited {
+
 public:
-	bool enabled;
-	bool luma;
-	bool chroma;
-	bool gamma;
+    bool curve;
+    bool curve2;
+	bool curve3;
+    bool curveMode;
+    bool curveMode2;
+    bool curveMode3;
+    bool enabled;
+    bool degree;
+    bool autodegree;
+	bool autoadapscen;
+    bool surround;
+    bool adapscen;
+    bool adaplum;
+	bool badpixsl;
+    bool wbmodel;
+	bool algo;
+    bool jlight;
+    bool qbright;
+    bool chroma;
+    bool schroma;
+    bool mchroma;
+    bool contrast;
+    bool qcontrast;
+    bool colorh;
+	bool rstprotection;
+	bool surrsource;
+	bool gamut;
+//	bool badpix;
+	bool datacie;
+	bool tonecie;
+//	bool sharpcie;
+	
+};
+
+
+class DirPyrDenoiseParamsEdited {
+
+public:
+    bool enabled;
+    bool Ldetail;
+    bool luma;
+    bool chroma;
+	bool redchro;
+	bool bluechro;
+    bool gamma;
+//    bool perform;
+	bool dmethod;
 };
 
 class EPDParamsEdited{
 public:
-	bool enabled;
-	bool Strength;
-	bool EdgeStopping;
-	bool Scale;
-	bool ReweightingIterates;
+    bool enabled;
+    bool Strength;
+    bool EdgeStopping;
+    bool Scale;
+    bool ReweightingIterates;
 };
 
 
@@ -308,6 +360,7 @@ class ColorManagementParamsEdited {
 
     public:
         bool input;
+        bool toneCurve;
         bool blendCMSMatrix;
         bool preferredProfile;
         bool working;
@@ -341,7 +394,8 @@ class RAWParamsEdited {
         bool dmethod;
         bool dcbIterations;
         bool dcbEnhance;
-        bool allEnhance;
+		bool lmmseIterations;
+        //bool allEnhance;
         bool caCorrection;
         bool caRed;
         bool caBlue;
@@ -377,6 +431,7 @@ class ParamsEdited {
         SharpenEdgeParamsEdited       sharpenEdge;
         SharpenMicroParamsEdited      sharpenMicro;
         VibranceParamsEdited          vibrance;
+        ColorAppearanceParamsEdited   colorappearance;
         //ColorBoostParamsEdited        colorBoost;
         WBParamsEdited                wb;
         //ColorShiftParamsEdited        colorShift;
