@@ -50,7 +50,7 @@ PartialPasteDlg::PartialPasteDlg (Glib::ustring title) {
     sh          = Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_SHADOWSHIGHLIGHTS")));
     epd         = Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_EPD")));
     labcurve    = Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_LABCURVE")));
-
+    colorappearance= Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_COLORAPP")));
     // options in detail:
     sharpen     = Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_SHARPENING")));
     sharpenedge = Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_SHARPENEDGE")));
@@ -70,7 +70,7 @@ PartialPasteDlg::PartialPasteDlg (Glib::ustring title) {
     distortion  = Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_DISTORTION")));
     cacorr      = Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_CACORRECTION")));
     vignetting  = Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_VIGNETTING")));
-    lcp         = Gtk::manage (new Gtk::CheckButton (M("TP_LENSPROFILE_LABEL")));
+    lcp         = Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_LENSPROFILE")));
 
     // options in composition:
     coarserot   = Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_COARSETRANS")));
@@ -100,7 +100,8 @@ PartialPasteDlg::PartialPasteDlg (Glib::ustring title) {
     raw_ccSteps			= Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_RAW_FALSECOLOR")));
     raw_dcb_iterations	= Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_RAW_DCBITERATIONS")));
     raw_dcb_enhance		= Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_RAW_DCBENHANCE")));
-    raw_all_enhance		= Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_RAW_ALLENHANCE")));
+    //raw_all_enhance		= Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_RAW_ALLENHANCE")));
+    raw_lmmse_iterations= Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_RAW_LMMSEITERATIONS")));
 
 	df_file        		= Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_DARKFRAMEFILE")));
 	df_AutoSelect  		= Gtk::manage (new Gtk::CheckButton (M("PARTIALPASTE_DARKFRAMEAUTOSELECT")));
@@ -126,6 +127,7 @@ PartialPasteDlg::PartialPasteDlg (Glib::ustring title) {
     vboxes[0]->pack_start (*sh, Gtk::PACK_SHRINK, 2);
     vboxes[0]->pack_start (*epd, Gtk::PACK_SHRINK, 2);
     vboxes[0]->pack_start (*labcurve, Gtk::PACK_SHRINK, 2);
+    vboxes[0]->pack_start (*colorappearance, Gtk::PACK_SHRINK, 2);
 
     vboxes[1]->pack_start (*detail, Gtk::PACK_SHRINK, 2);
     vboxes[1]->pack_start (*hseps[1], Gtk::PACK_SHRINK, 2);
@@ -174,7 +176,8 @@ PartialPasteDlg::PartialPasteDlg (Glib::ustring title) {
 	vboxes[6]->pack_start (*raw_ccSteps, Gtk::PACK_SHRINK, 2);
 	vboxes[6]->pack_start (*raw_dcb_iterations, Gtk::PACK_SHRINK, 2);
 	vboxes[6]->pack_start (*raw_dcb_enhance, Gtk::PACK_SHRINK, 2);
-	vboxes[6]->pack_start (*raw_all_enhance, Gtk::PACK_SHRINK, 2);
+	vboxes[6]->pack_start (*raw_lmmse_iterations, Gtk::PACK_SHRINK, 2);
+	//vboxes[6]->pack_start (*raw_all_enhance, Gtk::PACK_SHRINK, 2);
 	vboxes[6]->pack_start (*Gtk::manage (new Gtk::HSeparator ()), Gtk::PACK_SHRINK, 0);
 	vboxes[6]->pack_start (*raw_linenoise, Gtk::PACK_SHRINK, 2);
 	vboxes[6]->pack_start (*raw_greenthresh, Gtk::PACK_SHRINK, 2);
@@ -244,6 +247,7 @@ PartialPasteDlg::PartialPasteDlg (Glib::ustring title) {
     shConn          = sh->signal_toggled().connect (sigc::bind (sigc::mem_fun(*basic, &Gtk::CheckButton::set_inconsistent), true));
     epdConn         = epd->signal_toggled().connect (sigc::bind (sigc::mem_fun(*basic, &Gtk::CheckButton::set_inconsistent), true));
     labcurveConn    = labcurve->signal_toggled().connect (sigc::bind (sigc::mem_fun(*basic, &Gtk::CheckButton::set_inconsistent), true));
+    colorappearanceConn=colorappearance->signal_toggled().connect (sigc::bind (sigc::mem_fun(*basic, &Gtk::CheckButton::set_inconsistent), true));
 
     sharpenConn     = sharpen->signal_toggled().connect (sigc::bind (sigc::mem_fun(*detail, &Gtk::CheckButton::set_inconsistent), true));
     gradsharpenConn = sharpenedge->signal_toggled().connect (sigc::bind (sigc::mem_fun(*detail, &Gtk::CheckButton::set_inconsistent), true));
@@ -280,7 +284,8 @@ PartialPasteDlg::PartialPasteDlg (Glib::ustring title) {
     raw_ccStepsConn         = raw_ccSteps->signal_toggled().connect (sigc::bind (sigc::mem_fun(*raw, &Gtk::CheckButton::set_inconsistent), true));
     raw_dcb_iterationsConn  = raw_dcb_iterations->signal_toggled().connect (sigc::bind (sigc::mem_fun(*raw, &Gtk::CheckButton::set_inconsistent), true));
     raw_dcb_enhanceConn     = raw_dcb_enhance->signal_toggled().connect (sigc::bind (sigc::mem_fun(*raw, &Gtk::CheckButton::set_inconsistent), true));
-    raw_all_enhanceConn     = raw_all_enhance->signal_toggled().connect (sigc::bind (sigc::mem_fun(*raw, &Gtk::CheckButton::set_inconsistent), true));
+    //raw_all_enhanceConn     = raw_all_enhance->signal_toggled().connect (sigc::bind (sigc::mem_fun(*raw, &Gtk::CheckButton::set_inconsistent), true));
+    raw_lmmse_iterationsConn  = raw_lmmse_iterations->signal_toggled().connect (sigc::bind (sigc::mem_fun(*raw, &Gtk::CheckButton::set_inconsistent), true));
 
     raw_exposConn           = raw_expos->signal_toggled().connect (sigc::bind (sigc::mem_fun(*raw, &Gtk::CheckButton::set_inconsistent), true));
     raw_preserConn          = raw_preser->signal_toggled().connect (sigc::bind (sigc::mem_fun(*raw, &Gtk::CheckButton::set_inconsistent), true));
@@ -350,7 +355,8 @@ void PartialPasteDlg::rawToggled () {
 	raw_ccStepsConn.block (true);
 	raw_dcb_iterationsConn.block (true);
 	raw_dcb_enhanceConn.block (true);
-	raw_all_enhanceConn.block (true);
+	//raw_all_enhanceConn.block (true);
+	raw_lmmse_iterationsConn.block (true);
 	raw_exposConn.block (true);
 	raw_preserConn.block (true);
 	raw_blackConn.block (true);
@@ -373,7 +379,8 @@ void PartialPasteDlg::rawToggled () {
     raw_ccSteps->set_active (raw->get_active ());
     raw_dcb_iterations->set_active (raw->get_active ());
     raw_dcb_enhance->set_active (raw->get_active ());
-    raw_all_enhance->set_active (raw->get_active ());
+    raw_lmmse_iterations->set_active (raw->get_active ());
+   //raw_all_enhance->set_active (raw->get_active ());
     raw_expos->set_active (raw->get_active ());
     raw_preser->set_active (raw->get_active ());
     raw_black->set_active (raw->get_active ());
@@ -394,7 +401,8 @@ void PartialPasteDlg::rawToggled () {
     raw_ccStepsConn.block (false);
     raw_dcb_iterationsConn.block (false);
     raw_dcb_enhanceConn.block (false);
-    raw_all_enhanceConn.block (false);
+    //raw_all_enhanceConn.block (false);
+    raw_lmmse_iterationsConn.block (false);
     raw_exposConn.block (false);
     raw_preserConn.block (false);
     raw_blackConn.block (false);
@@ -420,6 +428,7 @@ void PartialPasteDlg::basicToggled () {
     shConn.block (true);
     epdConn.block(true);
     labcurveConn.block (true);
+    colorappearanceConn.block (true);
 
     basic->set_inconsistent (false);
 
@@ -429,6 +438,7 @@ void PartialPasteDlg::basicToggled () {
     sh->set_active (basic->get_active ());
     epd->set_active (basic->get_active ());
     labcurve->set_active (basic->get_active ());
+    colorappearance->set_active (basic->get_active ());
 
     wbConn.block (false);
     exposureConn.block (false);
@@ -436,6 +446,7 @@ void PartialPasteDlg::basicToggled () {
     shConn.block (false);
     epdConn.block (false);
     labcurveConn.block (false);
+    colorappearanceConn.block (false);
 }
 
 void PartialPasteDlg::detailToggled () {
@@ -588,6 +599,7 @@ void PartialPasteDlg::applyPaste (rtengine::procparams::ProcParams* dstPP, Param
     if (!sh->get_active ())          filterPE.sh         = falsePE.sh;
     if (!epd->get_active ())         filterPE.edgePreservingDecompositionUI = falsePE.edgePreservingDecompositionUI;
     if (!labcurve->get_active ())    filterPE.labCurve   = falsePE.labCurve;
+    if (!colorappearance->get_active ()) filterPE.colorappearance= falsePE.colorappearance;
 
     if (!sharpen->get_active ())     filterPE.sharpening      = falsePE.sharpening;
     if (!sharpenedge->get_active ()) filterPE.sharpenEdge     = falsePE.sharpenEdge;
@@ -623,7 +635,8 @@ void PartialPasteDlg::applyPaste (rtengine::procparams::ProcParams* dstPP, Param
     if (!raw_ccSteps->get_active ())           filterPE.raw.ccSteps            = falsePE.raw.ccSteps;
     if (!raw_dcb_iterations->get_active ())    filterPE.raw.dcbIterations      = falsePE.raw.dcbIterations;
     if (!raw_dcb_enhance->get_active ())       filterPE.raw.dcbEnhance         = falsePE.raw.dcbEnhance;
-    if (!raw_all_enhance->get_active ())       filterPE.raw.allEnhance         = falsePE.raw.allEnhance;
+    //if (!raw_all_enhance->get_active ())       filterPE.raw.allEnhance         = falsePE.raw.allEnhance;
+    if (!raw_lmmse_iterations->get_active ())    filterPE.raw.lmmseIterations      = falsePE.raw.lmmseIterations;
 
     if (!raw_expos->get_active ())             filterPE.raw.exPos              = falsePE.raw.exPos;
     if (!raw_preser->get_active ())            filterPE.raw.exPreser           = falsePE.raw.exPreser;
