@@ -82,7 +82,11 @@ Preferences::Preferences  (RTWindow *rtwindow):parent(rtwindow)  {
     nb->append_page (*getFileBrowserPanel(),    M("PREFERENCES_TAB_BROWSER"));
     nb->append_page (*getColorManagementPanel(),M("PREFERENCES_TAB_COLORMGR"));
     nb->append_page (*getBatchProcPanel(),      M("PREFERENCES_BATCH_PROCESSING"));
+    nb->append_page (*getPerformancePanel(),    M("PREFERENCES_TAB_PERFORMANCE"));
+    // Sounds only on Windows and Linux
+#if defined(WIN32) || defined(__linux__)
     nb->append_page (*getSoundPanel(),          M("PREFERENCES_TAB_SOUND"));
+#endif
     nb->set_current_page (0);
 
     fillPreferences ();
@@ -107,8 +111,10 @@ Gtk::Widget* Preferences::getBatchProcPanel () {
     behscrollw->set_policy (Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
     behscrollw->set_border_width(8);
     behscrollw->set_size_request(-1, 60);
+    Gtk::VBox* vbbeh = Gtk::manage( new Gtk::VBox () );
+    vbbeh->pack_start (*behscrollw, Gtk::PACK_EXPAND_WIDGET);
     Gtk::Frame* behFrame = Gtk::manage (new Gtk::Frame (M("PREFERENCES_BEHAVIOR")));
-    behFrame->add (*behscrollw);
+    behFrame->add (*vbbeh);
     //mvbpp->pack_start (*behFrame);
     mvbpp->pack_start (*behFrame, Gtk::PACK_EXPAND_WIDGET, 4);
     Gtk::TreeView* behTreeView = Gtk::manage (new Gtk::TreeView ());
@@ -165,7 +171,7 @@ Gtk::Widget* Preferences::getBatchProcPanel () {
     mi->set_value (behavColumns.label, M("TP_LABCURVE_LABEL"));
     appendBehavList (mi, M("TP_LABCURVE_BRIGHTNESS"), ADDSET_LC_BRIGHTNESS, false);
     appendBehavList (mi, M("TP_LABCURVE_CONTRAST"), ADDSET_LC_CONTRAST, false);
-	appendBehavList (mi, M("TP_LABCURVE_SATURATION"), ADDSET_LC_SATURATION, false);
+	appendBehavList (mi, M("TP_LABCURVE_CHROMATICITY"), ADDSET_LC_CHROMATICITY, false);
 
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_SHARPENING_LABEL"));
@@ -183,26 +189,45 @@ Gtk::Widget* Preferences::getBatchProcPanel () {
 
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_DIRPYRDENOISE_LABEL"));
-    appendBehavList (mi, M("TP_DIRPYRDENOISE_LUMA")+", "+M("TP_DIRPYRDENOISE_CHROMA"), ADDSET_DIRPYRDN_CHLUM, true);
-    appendBehavList (mi, M("TP_DIRPYRDENOISE_GAMMA"), ADDSET_DIRPYRDN_GAMMA, true);
+  //  appendBehavList (mi, M("TP_DIRPYRDENOISE_LUMA")+", "+M("TP_DIRPYRDENOISE_CHROMA"), ADDSET_DIRPYRDN_CHLUM, true);
+    appendBehavList (mi, M("TP_DIRPYRDENOISE_LUMA"),ADDSET_DIRPYRDN_LUMA, true);
+    appendBehavList (mi, M("TP_DIRPYRDENOISE_LDETAIL"),ADDSET_DIRPYRDN_LUMDET, true);
+    appendBehavList (mi, M("TP_DIRPYRDENOISE_CHROMA"), ADDSET_DIRPYRDN_CHROMA, true);
+    appendBehavList (mi, M("TP_DIRPYRDENOISE_RED"), ADDSET_DIRPYRDN_CHROMARED, true);
+    appendBehavList (mi, M("TP_DIRPYRDENOISE_BLUE"), ADDSET_DIRPYRDN_CHROMABLUE, true);
+	appendBehavList (mi, M("TP_DIRPYRDENOISE_GAMMA"), ADDSET_DIRPYRDN_GAMMA, true);
 
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_WBALANCE_LABEL"));
     appendBehavList (mi, M("TP_WBALANCE_TEMPERATURE"), ADDSET_WB_TEMPERATURE, true);
     appendBehavList (mi, M("TP_WBALANCE_GREEN"), ADDSET_WB_GREEN, true);
 
+    mi = behModel->append ();
+    mi->set_value (behavColumns.label, M("TP_COLORAPP_LABEL"));
+    appendBehavList (mi, M("TP_COLORAPP_CIECAT_DEGREE"),ADDSET_CAT_DEGREE, true);
+    appendBehavList (mi, M("TP_COLORAPP_ADAPTSCENE"),ADDSET_CAT_ADAPTSCENE, true);
+    appendBehavList (mi, M("TP_COLORAPP_LIGHT"),ADDSET_CAT_LIGHT, true);
+    appendBehavList (mi, M("TP_COLORAPP_BRIGHT"),ADDSET_CAT_BRIGHT, true);
+    appendBehavList (mi, M("TP_COLORAPP_CHROMA"),ADDSET_CAT_CHROMA, true);
+    appendBehavList (mi, M("TP_COLORAPP_RSTPRO"),ADDSET_CAT_RSTPRO, true);
+    appendBehavList (mi, M("TP_COLORAPP_CONTRAST"),ADDSET_CAT_CONTRAST, true);
+    appendBehavList (mi, M("TP_COLORAPP_CONTRAST_Q"),ADDSET_CAT_CONTRAST_Q, true);
+    appendBehavList (mi, M("TP_COLORAPP_CHROMA_S"),ADDSET_CAT_CHROMA_S, true);
+    appendBehavList (mi, M("TP_COLORAPP_CHROMA_M"),ADDSET_CAT_CHROMA_M, true);
+    appendBehavList (mi, M("TP_COLORAPP_HUE"),ADDSET_CAT_HUE, true);
+    appendBehavList (mi, M("TP_COLORAPP_ADAPTVIEWING"),ADDSET_CAT_ADAPTVIEWING, true);
+    appendBehavList (mi, M("TP_COLORAPP_BADPIXSL"),ADDSET_CAT_BADPIX, true);
+
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_VIBRANCE_LABEL"));
     appendBehavList (mi, M("TP_VIBRANCE_PASTELS"), ADDSET_VIBRANCE_PASTELS, false);
     appendBehavList (mi, M("TP_VIBRANCE_SATURATED"), ADDSET_VIBRANCE_SATURATED, false);
-    appendBehavList (mi, M("TP_VIBRANCE_PSTHRESHOLD"), ADDSET_VIBRANCE_PSTHRESHOLD, false);
 
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_GAMMA_OUTPUT"));
     appendBehavList (mi, M("TP_GAMMA_CURV"), ADDSET_FREE_OUPUT_GAMMA, false);
     appendBehavList (mi, M("TP_GAMMA_SLOP"), ADDSET_FREE_OUTPUT_SLOPE, false);
-	
-	
+
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_CHMIXER_LABEL"));
     appendBehavList (mi, M("TP_CHMIXER_RED")+", "+M("TP_CHMIXER_GREEN")+", "+M("TP_CHMIXER_BLUE"), ADDSET_CHMIXER, false);
@@ -226,6 +251,9 @@ Gtk::Widget* Preferences::getBatchProcPanel () {
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_VIGNETTING_LABEL"));
     appendBehavList (mi, M("TP_VIGNETTING_AMOUNT"), ADDSET_VIGN_AMOUNT, false);
+    appendBehavList (mi, M("TP_VIGNETTING_RADIUS"), ADDSET_VIGN_RADIUS, false);
+    appendBehavList (mi, M("TP_VIGNETTING_STRENGTH"), ADDSET_VIGN_STRENGTH, false);
+    appendBehavList (mi, M("TP_VIGNETTING_CENTER_X")+", "+M("TP_VIGNETTING_CENTER_Y"), ADDSET_VIGN_CENTER, false);
 
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_DIRPYREQUALIZER_LABEL"));
@@ -248,6 +276,20 @@ Gtk::Widget* Preferences::getBatchProcPanel () {
 
     behTreeView->expand_all ();
 
+    behAddAll = Gtk::manage( new Gtk::Button (M("PREFERENCES_BEHADDALL")) );
+    behSetAll = Gtk::manage( new Gtk::Button (M("PREFERENCES_BEHSETALL")) );
+    behAddAll->set_tooltip_markup (M("PREFERENCES_BEHADDALLHINT"));
+    behSetAll->set_tooltip_markup (M("PREFERENCES_BEHSETALLHINT"));
+
+    behAddAll->signal_clicked().connect( sigc::mem_fun(*this, &Preferences::behAddAllPressed) );
+    behSetAll->signal_clicked().connect( sigc::mem_fun(*this, &Preferences::behSetAllPressed) );
+
+    Gtk::HBox* buttonpanel1 = Gtk::manage (new Gtk::HBox ());
+    //buttonpanel1->set_spacing(8);
+    buttonpanel1->pack_end (*behSetAll, Gtk::PACK_SHRINK, 4);
+    buttonpanel1->pack_end (*behAddAll, Gtk::PACK_SHRINK, 4);
+    vbbeh->pack_start (*buttonpanel1, Gtk::PACK_SHRINK, 4);
+
     chOverwriteOutputFile =  Gtk::manage( new Gtk::CheckButton (M("PREFERENCES_OVERWRITEOUTPUTFILE")) );
     mvbpp->pack_start(*chOverwriteOutputFile, Gtk::PACK_SHRINK, 4);
 
@@ -376,6 +418,32 @@ Gtk::Widget* Preferences::getProcParamsPanel () {
     return mvbpp;
 }
 
+Gtk::Widget* Preferences::getPerformancePanel () {
+	Gtk::VBox* mainContainer = Gtk::manage( new Gtk::VBox () );
+	mainContainer->set_spacing(4);
+
+    Gtk::HBox* threadLimitHB = Gtk::manage( new Gtk::HBox () );
+    threadLimitHB->set_border_width(4);
+    threadLimitHB->set_spacing(4);
+    threadLimitHB->set_tooltip_text(M("PREFERENCES_RGBDTL_TOOLTIP"));
+    Gtk::Label* RGBDTLl = Gtk::manage( new Gtk::Label (M("PREFERENCES_RGBDTL_LABEL") + ":", Gtk::ALIGN_LEFT));
+    rgbDenoiseTreadLimitSB = Gtk::manage( new Gtk::SpinButton () );
+    rgbDenoiseTreadLimitSB->set_digits (0);
+    rgbDenoiseTreadLimitSB->set_increments (1, 5);
+    rgbDenoiseTreadLimitSB->set_max_length(2);  // Will this be sufficient? :)
+    int maxThreadNumber = 10;
+#ifdef _OPENMP
+    maxThreadNumber = omp_get_max_threads();
+#endif
+    rgbDenoiseTreadLimitSB->set_range (0, maxThreadNumber);
+    threadLimitHB->pack_start (*RGBDTLl, Gtk::PACK_SHRINK, 0);
+    threadLimitHB->pack_end (*rgbDenoiseTreadLimitSB, Gtk::PACK_SHRINK, 0);
+
+    mainContainer->pack_start(*threadLimitHB, Gtk::PACK_SHRINK, 4);
+
+    return mainContainer;
+}
+
 Gtk::Widget* Preferences::getColorManagementPanel () {
 
     Gtk::VBox* mvbcm = Gtk::manage (new Gtk::VBox ());
@@ -408,6 +476,56 @@ Gtk::Widget* Preferences::getColorManagementPanel () {
     mvbcm->pack_start (*colt, Gtk::PACK_SHRINK, 4);
 
     autoMonProfileToggled();
+    //Gtk::Frame* fdp = Gtk::manage (new Gtk::Frame (M("PREFERENCES_OUTPUTDEVICE")));
+    Gtk::VBox* vbdp = Gtk::manage (new Gtk::VBox ());
+    vbdp->set_border_width (4);
+    //Gtk::Label* viewlab = Gtk::manage (new Gtk::Label (M("PREFERENCES_VIEW")+":"));
+    Gtk::Label* viewlab = Gtk::manage (new Gtk::Label (M("PREFERENCES_VIEW")+":", Gtk::ALIGN_LEFT));
+
+    view = Gtk::manage (new Gtk::ComboBoxText ());
+    view->append_text (M("PREFERENCES_D50"));
+    view->append_text (M("PREFERENCES_D55"));
+    view->append_text (M("PREFERENCES_D60"));
+    view->append_text (M("PREFERENCES_D65"));
+    view->append_text (M("PREFERENCES_BLACKBODY"));
+    view->append_text (M("PREFERENCES_FLUOF2"));
+    view->append_text (M("PREFERENCES_FLUOF7"));
+    view->append_text (M("PREFERENCES_FLUOF11"));
+
+    Gtk::Label* greylab = Gtk::manage (new Gtk::Label (M("PREFERENCES_GREY")+":", Gtk::ALIGN_LEFT));
+    grey = Gtk::manage (new Gtk::ComboBoxText ());
+    grey->append_text (M("PREFERENCES_GREY05"));
+    grey->append_text (M("PREFERENCES_GREY10"));
+    grey->append_text (M("PREFERENCES_GREY15"));
+    grey->append_text (M("PREFERENCES_GREY18"));
+    grey->append_text (M("PREFERENCES_GREY23"));
+    grey->append_text (M("PREFERENCES_GREY30"));
+    grey->append_text (M("PREFERENCES_GREY40"));
+
+    Gtk::Label* restartNeeded1 = Gtk::manage( new Gtk::Label (Glib::ustring(" (") + M("PREFERENCES_APPLNEXTSTARTUP") + ")") );
+    Gtk::Label* restartNeeded2 = Gtk::manage( new Gtk::Label (Glib::ustring(" (") + M("PREFERENCES_APPLNEXTSTARTUP") + ")") );
+
+    Gtk::Table* colo = Gtk::manage (new Gtk::Table (2, 3));
+    colo->attach (*viewlab, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 2, 2);
+    colo->attach (*view, 1, 2, 0, 1, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
+    colo->attach (*restartNeeded1, 2, 3, 0, 1, Gtk::FILL, Gtk::SHRINK, 2, 2);
+    colo->attach (*greylab, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 2, 2);
+    colo->attach (*grey, 1, 2, 1, 2, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
+    colo->attach (*restartNeeded2, 2, 3, 1, 2, Gtk::FILL, Gtk::SHRINK, 2, 2);
+    mvbcm->pack_start (*colo, Gtk::PACK_SHRINK, 4);
+
+    Gtk::Label* cielab = Gtk::manage (new Gtk::Label (M("PREFERENCES_CIEART")+":", Gtk::ALIGN_LEFT));
+	cbciecamfloat = Gtk::manage (new Gtk::CheckButton (M("PREFERENCES_CIEART_LABEL")));
+	//autocielabConn  = cbAutocielab->signal_toggled().connect (sigc::mem_fun(*this, &Preferences::autocielabToggled));
+	Gtk::Table* coltcie = Gtk::manage (new Gtk::Table (1, 2));	
+    coltcie->attach (*cielab, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 2, 2);
+  //  coltcie->attach (*cbAutocielab, 1, 2, 0, 1, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
+	//cbAutocielab->set_tooltip_markup (M("PREFERENCES_CIEART_TOOLTIP"));
+    coltcie->attach (*cbciecamfloat, 1, 2, 0, 1, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
+	cbciecamfloat->set_tooltip_markup (M("PREFERENCES_CIEART_TOOLTIP"));
+
+    mvbcm->pack_start (*coltcie, Gtk::PACK_SHRINK, 4);
+   // autocielabToggled();
 
     return mvbcm;
 }
@@ -481,9 +599,9 @@ Gtk::Widget* Preferences::getGeneralPanel () {
     std::vector<Glib::ustring> langs;
     parseDir (argv0 + "/languages", langs, "");
     for (size_t i=0; i<langs.size(); i++) {
-	if ("default" != langs[i] && "README" != langs[i] && "LICENSE" != langs[i]) {
-  	    languages->append_text (langs[i]);
-	}
+        if ("default" != langs[i] && "README" != langs[i] && "LICENSE" != langs[i]) {
+            languages->append_text (langs[i]);
+        }
     }
 
     Gtk::Label* langw = Gtk::manage( new Gtk::Label (Glib::ustring(" (") + M("PREFERENCES_APPLNEXTSTARTUP") + ")") );
@@ -823,18 +941,6 @@ Gtk::Widget* Preferences::getFileBrowserPanel () {
     frc->add (*vbc);  
     vbc->set_border_width (4);
 
-    Gtk::Label* cflab = Gtk::manage( new Gtk::Label (M("PREFERENCES_CACHETHUMBFORM")+":") );
-    cflab->set_alignment(Gtk::ALIGN_LEFT, Gtk::ALIGN_TOP);
-    cformat = Gtk::manage( new Gtk::ComboBoxText () );
-    cformat->set_size_request(50, -1);
-    cformat->append_text (M("PREFERENCES_CACHEFORMAT1"));
-    cformat->append_text (M("PREFERENCES_CACHEFORMAT2"));
-    cformat->append_text (M("PREFERENCES_CACHEFORMAT1")+", 16 bit");
-    cformat->signal_changed().connect( sigc::mem_fun(*this, &Preferences::cacheFormatComboChanged) );
-    cacheFormatComboChanged(); // update the tooltip
-    vbc->pack_start (*cflab, Gtk::PACK_SHRINK, 2);
-    vbc->pack_start (*cformat);
-
     Gtk::HBox* hb3 = Gtk::manage( new Gtk::HBox () );
     Gtk::Label* chlab = Gtk::manage( new Gtk::Label (M("PREFERENCES_CACHETHUMBHEIGHT")+":") );
     maxThumbSize = Gtk::manage( new Gtk::SpinButton () );
@@ -1001,7 +1107,7 @@ void Preferences::storePreferences () {
     moptions.psDir          = psDir->get_filename ();
 #elif defined __APPLE__
     moptions.psDir          = psDir->get_filename (); 
-#endif	
+#endif
     moptions.customEditorProg = editorToSendTo->get_text ();
     if (edGimp->get_active ())
         moptions.editorToSendTo = 1;
@@ -1011,16 +1117,20 @@ void Preferences::storePreferences () {
 #elif defined __APPLE__   
     else if (edPS->get_active ())
       moptions.editorToSendTo = 2; 
-#endif	
+#endif
     else if (edOther->get_active ())
         moptions.editorToSendTo = 3;
 
     moptions.customProfileBuilder = txtCustProfBuilderPath->get_text();
 
     moptions.rtSettings.monitorProfile      = monProfile->get_filename ();
-	moptions.rtSettings.autoMonitorProfile  = cbAutoMonProfile->get_active ();
-	moptions.rtSettings.iccDirectory        = iccDir->get_current_folder ();
-	moptions.rtSettings.colorimetricIntent  = intent->get_active_row_number ();
+    moptions.rtSettings.autoMonitorProfile  = cbAutoMonProfile->get_active ();
+    moptions.rtSettings.iccDirectory        = iccDir->get_filename ();
+    moptions.rtSettings.colorimetricIntent  = intent->get_active_row_number ();
+    moptions.rtSettings.viewingdevice       = view->get_active_row_number ();
+    moptions.rtSettings.viewingdevicegrey   = grey->get_active_row_number ();
+  //  moptions.rtSettings.autocielab 			= cbAutocielab->get_active ();
+    moptions.rtSettings.ciecamfloat 			= cbciecamfloat->get_active ();
 
     if (sdcurrent->get_active ()) 
         moptions.startupDir = STARTUPDIR_CURRENT;
@@ -1041,13 +1151,6 @@ void Preferences::storePreferences () {
         moptions.parseExtensionsEnabled.push_back (c[i][extensionColumns.enabled]);
     }
     
-    if (cformat->get_active_row_number() == 0)
-        moptions.thumbnailFormat = FT_Custom;
-    else if (cformat->get_active_row_number() == 1)
-        moptions.thumbnailFormat = FT_Jpeg;
-    else if (cformat->get_active_row_number() == 2)
-        moptions.thumbnailFormat = FT_Custom16;
-
     moptions.maxThumbnailHeight = (int)maxThumbSize->get_value ();
     moptions.maxCacheEntries = (int)maxCacheEntries->get_value ();
     moptions.overlayedFileNames = overlayedFileNames->get_active ();
@@ -1080,11 +1183,15 @@ void Preferences::storePreferences () {
     moptions.overwriteOutputFile = chOverwriteOutputFile->get_active ();
     moptions.UseIconNoText = ckbUseIconNoText->get_active();
 
-    // Sounds
+    moptions.rgbDenoiseThreadLimit = rgbDenoiseTreadLimitSB->get_value_as_int();
+
+    // Sounds only on Windows and Linux
+#if defined(WIN32) || defined(__linux__)
     moptions.sndEnable = ckbSndEnable->get_active ();
     moptions.sndBatchQueueDone = txtSndBatchQueueDone->get_text ();
     moptions.sndLngEditProcDone     = txtSndLngEditProcDone->get_text ();
     moptions.sndLngEditProcDoneSecs = spbSndLngEditProcDoneSecs->get_value ();
+#endif
 }
 
 void Preferences::fillPreferences () {
@@ -1101,12 +1208,16 @@ void Preferences::fillPreferences () {
     if (safe_file_test (moptions.rtSettings.monitorProfile, Glib::FILE_TEST_EXISTS)) 
         monProfile->set_filename (moptions.rtSettings.monitorProfile);
     if (moptions.rtSettings.monitorProfile.empty())
-    	monProfile->set_current_folder (moptions.rtSettings.iccDirectory);
-	cbAutoMonProfile->set_active(moptions.rtSettings.autoMonitorProfile);
+        monProfile->set_current_folder (moptions.rtSettings.iccDirectory);
+    cbAutoMonProfile->set_active(moptions.rtSettings.autoMonitorProfile);
 
     if (Glib::file_test (moptions.rtSettings.iccDirectory, Glib::FILE_TEST_IS_DIR)) 
         iccDir->set_current_folder (moptions.rtSettings.iccDirectory);
-	intent->set_active (moptions.rtSettings.colorimetricIntent);
+    intent->set_active (moptions.rtSettings.colorimetricIntent);
+    view->set_active (moptions.rtSettings.viewingdevice);
+    grey->set_active (moptions.rtSettings.viewingdevicegrey);
+//	cbAutocielab->set_active (moptions.rtSettings.autocielab);
+	cbciecamfloat->set_active (moptions.rtSettings.ciecamfloat);
     languages->set_active_text (moptions.language);
     ckbLangAutoDetect->set_active (moptions.languageAutoDetect);
     theme->set_active_text (moptions.theme);
@@ -1144,7 +1255,7 @@ void Preferences::fillPreferences () {
   edPS->set_active (moptions.editorToSendTo==2);
   if (safe_file_test (moptions.psDir, Glib::FILE_TEST_IS_DIR))
     psDir->set_filename (moptions.psDir); 
-#endif	
+#endif
     editorToSendTo->set_text (moptions.customEditorProg);
 
     txtCustProfBuilderPath->set_text(moptions.customProfileBuilder);
@@ -1167,13 +1278,6 @@ void Preferences::fillPreferences () {
         row[extensionColumns.ext]     = moptions.parseExtensions[i];
     }
        
-    if (moptions.thumbnailFormat == FT_Custom)
-        cformat->set_active (0);
-    else if (moptions.thumbnailFormat == FT_Jpeg)
-        cformat->set_active (1);
-    else if (moptions.thumbnailFormat == FT_Custom16)
-        cformat->set_active (2);
-    
     maxThumbSize->set_value (moptions.maxThumbnailHeight);
     maxCacheEntries->set_value (moptions.maxCacheEntries);
     overlayedFileNames->set_active (moptions.overlayedFileNames);
@@ -1197,6 +1301,8 @@ void Preferences::fillPreferences () {
     ckbHideTPVScrollbar->set_active(moptions.hideTPVScrollbar);
     ckbUseIconNoText->set_active(moptions.UseIconNoText);
 
+    rgbDenoiseTreadLimitSB->set_value(moptions.rgbDenoiseThreadLimit);
+
     //darkFrameDir->set_filename( moptions.rtSettings.darkFramesPath );
     //updateDFinfos();
     darkFrameDir->set_current_folder( moptions.rtSettings.darkFramesPath );
@@ -1210,14 +1316,14 @@ void Preferences::fillPreferences () {
     addc.block (true);
     setc.block (true);
     if (moptions.baBehav.size() == ADDSET_PARAM_NUM) {
-		for (size_t i=0; i<moptions.baBehav.size(); i++)
-			for (Gtk::TreeIter sections=behModel->children().begin();  sections!=behModel->children().end(); sections++) 
-				for (Gtk::TreeIter adjs=sections->children().begin();  adjs!=sections->children().end(); adjs++) 
-					if (adjs->get_value (behavColumns.addsetid) == i) {
-						adjs->set_value (behavColumns.badd, moptions.baBehav[i]==1);
-						adjs->set_value (behavColumns.bset, moptions.baBehav[i]!=1);
-						break;
-					}
+        for (size_t i=0; i<moptions.baBehav.size(); i++)
+            for (Gtk::TreeIter sections=behModel->children().begin();  sections!=behModel->children().end(); sections++) 
+                for (Gtk::TreeIter adjs=sections->children().begin();  adjs!=sections->children().end(); adjs++) 
+                    if (adjs->get_value (behavColumns.addsetid) == i) {
+                        adjs->set_value (behavColumns.badd, moptions.baBehav[i]==1);
+                        adjs->set_value (behavColumns.bset, moptions.baBehav[i]!=1);
+                        break;
+                    }
     }
     addc.block (false);
     setc.block (false);
@@ -1228,11 +1334,13 @@ void Preferences::fillPreferences () {
 
     chOverwriteOutputFile->set_active (moptions.overwriteOutputFile);
 
-    // Sounds
+    // Sounds only on Windows and Linux
+#if defined(WIN32) || defined(__linux__)
     ckbSndEnable->set_active (moptions.sndEnable);
     txtSndBatchQueueDone->set_text (moptions.sndBatchQueueDone);
     txtSndLngEditProcDone->set_text (moptions.sndLngEditProcDone);
     spbSndLngEditProcDoneSecs->set_value (moptions.sndLngEditProcDoneSecs);
+#endif
 }
 
 /*
@@ -1253,7 +1361,11 @@ void Preferences::savePressed () {
 void Preferences::autoMonProfileToggled () {
 	monProfile->set_sensitive(!cbAutoMonProfile->get_active());
 }
-
+/*
+void Preferences::autocielabToggled () {
+//	cbAutocielab->set_sensitive(cbAutocielab->get_active());
+}
+*/
 void Preferences::sndEnableToggled () {
 	txtSndBatchQueueDone->set_sensitive(ckbSndEnable->get_active());
 	txtSndLngEditProcDone->set_sensitive(ckbSndEnable->get_active());
@@ -1333,11 +1445,6 @@ void Preferences::layoutComboChanged () {
 	editorLayout->set_tooltip_text(editorLayout->get_active_text());
 }
 
-void Preferences::cacheFormatComboChanged () {
-	cformat->set_tooltip_text(cformat->get_active_text());
-}
-
-
 void Preferences::fontChanged () {
 
 	switchFontTo(fontbutton->get_font_name());
@@ -1494,3 +1601,31 @@ bool Preferences::splashClosed(GdkEventAny* event) {
 	splash = NULL;
 	return true;
 }
+
+void Preferences::behAddAllPressed () {
+
+    if (moptions.baBehav.size() == ADDSET_PARAM_NUM) {
+        for (size_t i=0; i<moptions.baBehav.size(); i++)
+            for (Gtk::TreeIter sections=behModel->children().begin();  sections!=behModel->children().end(); sections++)
+                for (Gtk::TreeIter adjs=sections->children().begin();  adjs!=sections->children().end(); adjs++)
+                    if (adjs->get_value (behavColumns.addsetid) == i) {
+                        adjs->set_value (behavColumns.badd, true);
+                        adjs->set_value (behavColumns.bset, false);
+                        break;
+                    }
+    }
+}
+
+void Preferences::behSetAllPressed () {
+
+    if (moptions.baBehav.size() == ADDSET_PARAM_NUM) {
+        for (size_t i=0; i<moptions.baBehav.size(); i++)
+            for (Gtk::TreeIter sections=behModel->children().begin();  sections!=behModel->children().end(); sections++)
+                for (Gtk::TreeIter adjs=sections->children().begin();  adjs!=sections->children().end(); adjs++)
+                    if (adjs->get_value (behavColumns.addsetid) == i) {
+                        adjs->set_value (behavColumns.badd, false);
+                        adjs->set_value (behavColumns.bset, true);
+                        break;
+                    }
+    }
+}
