@@ -80,6 +80,8 @@ class Preferences : public Gtk::Dialog {
     Gtk::FileChooserButton* iccDir;
     Gtk::FileChooserButton* monProfile;
 	Gtk::CheckButton* cbAutoMonProfile;
+	//Gtk::CheckButton* cbAutocielab;
+	Gtk::CheckButton* cbciecamfloat;
 
     Gtk::CheckButton* blinkClipped;
 	Gtk::SpinButton*  hlThresh;
@@ -88,6 +90,8 @@ class Preferences : public Gtk::Dialog {
 	Gtk::SpinButton*  panFactor;
 
     Gtk::ComboBoxText* intent;
+    Gtk::ComboBoxText* view;
+    Gtk::ComboBoxText* grey;
 
     Gtk::ComboBoxText* theme;
     Gtk::CheckButton* slimUI;
@@ -96,7 +100,6 @@ class Preferences : public Gtk::Dialog {
     Gtk::FontButton* fontbutton;
     Gtk::ColorButton* butCropCol;
 	
-    Gtk::ComboBoxText* cformat;
     Gtk::SpinButton*   maxThumbSize;
     Gtk::SpinButton*   maxCacheEntries;
     Gtk::Button*       clearThumbnails;
@@ -108,12 +111,16 @@ class Preferences : public Gtk::Dialog {
     Gtk::Button*    delExt;
     Gtk::CheckButton* overlayedFileNames;
 
+    Gtk::SpinButton*  rgbDenoiseTreadLimitSB;
+
     Gtk::CheckButton* ckbmenuGroupRank;
     Gtk::CheckButton* ckbmenuGroupLabel;
     Gtk::CheckButton* ckbmenuGroupFileOperations;
     Gtk::CheckButton* ckbmenuGroupProfileOperations;
     Gtk::CheckButton* ckbmenuGroupExtProg;
 
+    Gtk::Button*      behAddAll;
+    Gtk::Button*      behSetAll;
     Gtk::CheckButton* chOverwriteOutputFile;
 
     Gtk::CheckButton* saveParamsFile;
@@ -142,7 +149,7 @@ class Preferences : public Gtk::Dialog {
 
     Options moptions;
     sigc::connection tconn, sconn, fconn, usethcon, addc, setc, dfconn, ffconn;
-    sigc::connection autoMonProfileConn, sndEnableConn, langAutoDetectConn;
+    sigc::connection autoMonProfileConn, sndEnableConn, langAutoDetectConn, autocielabConn;
     Glib::ustring initialTheme;
     Glib::ustring initialFont;
 
@@ -158,7 +165,6 @@ class Preferences : public Gtk::Dialog {
     void forRAWComboChanged ();
     void forImageComboChanged ();
     void layoutComboChanged ();
-    void cacheFormatComboChanged ();
     void switchThemeTo (Glib::ustring newTheme, bool slimInterface);
     void switchFontTo  (Glib::ustring newFont);
     bool splashClosed(GdkEventAny* event);
@@ -170,6 +176,7 @@ class Preferences : public Gtk::Dialog {
     Gtk::Widget* getFileBrowserPanel ();
     Gtk::Widget* getGeneralPanel ();
     Gtk::Widget* getBatchProcPanel ();
+    Gtk::Widget* getPerformancePanel ();
     Gtk::Widget* getSoundPanel ();
     
   public:
@@ -184,6 +191,7 @@ class Preferences : public Gtk::Dialog {
 	void autoMonProfileToggled ();
     void sndEnableToggled ();
     void langAutoDetectToggled ();
+    void autocielabToggled ();
 
     void selectStartupDir ();
     void addExtPressed ();
@@ -196,6 +204,9 @@ class Preferences : public Gtk::Dialog {
 
     void behAddRadioToggled (const Glib::ustring& path);
     void behSetRadioToggled (const Glib::ustring& path);
+    void behAddAllPressed ();
+    void behSetAllPressed ();
+
 //    void selectICCProfileDir ();
 //    void selectMonitorProfile ();
 };
