@@ -27,8 +27,6 @@
 using namespace rtengine;
 using namespace rtengine::procparams;
 
-extern Glib::ustring argv0;
-
 PartialPasteDlg* ProfilePanel::partialProfileDlg;
 
 
@@ -40,10 +38,18 @@ void ProfilePanel::cleanup () {
     delete partialProfileDlg;
 }
 
-ProfilePanel::ProfilePanel (bool readOnly) {
+ProfilePanel::ProfilePanel (bool readOnly) : lastFilename(""), imagePath("") {
 
     tpc = NULL;
-  
+
+    profileFillModeOnImage  = new RTImage("profile-filled.png");
+    profileFillModeOffImage = new RTImage("profile-partial.png");
+    fillMode = Gtk::manage (new Gtk::ToggleButton());
+    fillMode->set_active(options.filledProfile);
+    fillMode->add( options.filledProfile ? *profileFillModeOnImage : *profileFillModeOffImage );
+    fillMode->signal_toggled().connect ( sigc::mem_fun(*this, &ProfilePanel::profileFillModeToggled) );
+    fillMode->set_tooltip_text(M("PROFILEPANEL_MODE_TIP"));
+
     profiles = Gtk::manage (new MyComboBoxText ());
     Gtk::HBox* hbox = Gtk::manage (new Gtk::HBox ());
     hbox->show ();
@@ -60,6 +66,7 @@ ProfilePanel::ProfilePanel (bool readOnly) {
     paste = Gtk::manage (new Gtk::Button ());
     paste->add (*Gtk::manage (new RTImage ("edit-paste.png")));
 
+    hbox->pack_start (*fillMode, Gtk::PACK_SHRINK, 1);
     hbox->pack_start (*profiles);
     hbox->pack_start (*load, Gtk::PACK_SHRINK, 1);
     if (!readOnly) hbox->pack_start (*save, Gtk::PACK_SHRINK, 1);
@@ -93,6 +100,8 @@ ProfilePanel::~ProfilePanel () {
 
     if (custom)    { custom->deleteInstance();    delete custom;    }
     if (lastsaved) { lastsaved->deleteInstance(); delete lastsaved; }
+    delete profileFillModeOnImage;
+    delete profileFillModeOffImage;
 }
 
 void ProfilePanel::refreshProfileList () {
@@ -126,6 +135,26 @@ void ProfilePanel::save_clicked (GdkEventButton* event) {
 
     Gtk::FileChooserDialog dialog(M("PROFILEPANEL_SAVEDLGLABEL"), Gtk::FILE_CHOOSER_ACTION_SAVE);
     FileChooserLastFolderPersister persister( &dialog, options.loadSaveProfilePath );
+    dialog.set_current_name (lastFilename);
+
+    //Add the user's default (or global if multiuser=false) profile path to the Shortcut list
+#ifdef WIN32
+    // Dirty workaround, waiting for a clean solution by using exceptions!
+    if (!safe_is_root_dir(options.getPreferredProfilePath()))
+#endif
+    try {
+        dialog.add_shortcut_folder(options.getPreferredProfilePath());
+    }
+    catch (Gtk::FileChooserError &err) {}
+    //Add the image's path to the Shortcut list
+#ifdef WIN32
+    // Dirty workaround, waiting for a clean solution by using exceptions!
+    if (!safe_is_root_dir(imagePath))
+#endif
+    try {
+        dialog.add_shortcut_folder(imagePath);
+    }
+    catch (Gtk::FileChooserError &err) {}
 
     //Add response buttons the the dialog:
     dialog.add_button(Gtk::StockID("gtk-cancel"), Gtk::RESPONSE_CANCEL);
@@ -145,41 +174,22 @@ void ProfilePanel::save_clicked (GdkEventButton* event) {
 
 //    dialog.set_do_overwrite_confirmation (true);
 
-    savedialog = &dialog;
-
     bool done = false;
     do {
-        int result = dialog.run();
-        dialog.hide();
-
-        if (result==Gtk::RESPONSE_OK) {
+        if (dialog.run()==Gtk::RESPONSE_OK) {
 
             std::string fname = dialog.get_filename();
+            Glib::ustring ext = getExtension (fname);
 
-            bool hasext = true;
-	    size_t dotpos = fname.find_last_of ('.');
-            if (dotpos==Glib::ustring::npos)
-                hasext = false;
-	    size_t dirpos1 = fname.find_last_of ('/');
-            if (dirpos1!=Glib::ustring::npos && dirpos1>dotpos)
-                hasext = false;
-	    size_t dirpos2 = fname.find_last_of ('\\');
-            if (dirpos2!=Glib::ustring::npos && dirpos2>dotpos)
-                hasext = false;
+            if (("." + ext) != paramFileExtension)
+                fname += paramFileExtension;
 
-            if (!hasext)
-                fname = fname + paramFileExtension;
+            if (!confirmOverwrite (dialog, fname))
+                continue;
 
-            if (safe_file_test (fname, Glib::FILE_TEST_EXISTS)) {
-                Glib::ustring msg_ = Glib::ustring("<b>") + fname + ": " + M("MAIN_MSG_ALREADYEXISTS") + "\n" + M("MAIN_MSG_QOVERWRITE") + "</b>";
-                Gtk::MessageDialog msgd (msg_, true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO, true);
-                int response = msgd.run ();
-                if (response==Gtk::RESPONSE_NO)
-                    // make another try
-                    continue;
-            }
+            lastFilename = Glib::path_get_basename (fname);
 
-            PartialProfile* toSave = NULL;
+            const PartialProfile* toSave;
             if (profiles->get_active_text() == Glib::ustring("(") + M("PROFILEPANEL_PCUSTOM") + ")")
                 toSave = custom;
             else if (profiles->get_active_text() == Glib::ustring("(") + M("PROFILEPANEL_PLASTSAVED") + ")")
@@ -199,17 +209,29 @@ void ProfilePanel::save_clicked (GdkEventButton* event) {
                     // saving the partial profile
                     PartialProfile ppTemp(true);
                     partialProfileDlg->applyPaste (ppTemp.pparams, ppTemp.pedited, toSave->pparams, toSave->pedited);
-                    ppTemp.pparams->save (fname, "", ppTemp.pedited);
+                    int retCode = ppTemp.pparams->save (fname, "", ppTemp.pedited);
                     ppTemp.deleteInstance();
+                    if (retCode)
+                        writeFailed(dialog, fname);
+                    else {
+                        done=true;
+                        refreshProfileList ();
+                    }
                 }
                 else {
                     // saving a full profile
-                    toSave->pparams->save (fname);
+                    int retCode = toSave->pparams->save (fname);
+                    if (retCode)
+                        writeFailed(dialog, fname);
+                    else {
+                        done=true;
+                        refreshProfileList ();
+                    }
                 }
-                refreshProfileList ();
             }
+            else done = true;
         }
-        done = true;
+        else done = true;
     } while (!done);
     return;
 }
@@ -222,7 +244,7 @@ void ProfilePanel::copy_clicked (GdkEventButton* event) {
     if (event->button != 1)
         return;
 
-    PartialProfile* toSave = NULL;
+    const PartialProfile* toSave;
     if (profiles->get_active_text() == Glib::ustring("(") + M("PROFILEPANEL_PCUSTOM") + ")") 
         toSave = custom;
     else if (profiles->get_active_text() == Glib::ustring("(") + M("PROFILEPANEL_PLASTSAVED") + ")") 
@@ -263,6 +285,26 @@ void ProfilePanel::load_clicked (GdkEventButton* event) {
     Gtk::FileChooserDialog dialog(M("PROFILEPANEL_LOADDLGLABEL"), Gtk::FILE_CHOOSER_ACTION_OPEN);
     FileChooserLastFolderPersister persister( &dialog, options.loadSaveProfilePath );
 
+    //Add the user's default (or global if multiuser=false) profile path to the Shortcut list
+#ifdef WIN32
+    // Dirty workaround, waiting for a clean solution by using exceptions!
+    if (!safe_is_root_dir(options.getPreferredProfilePath()))
+#endif
+    try {
+        dialog.add_shortcut_folder(options.getPreferredProfilePath());
+    }
+    catch (Gtk::FileChooserError &err) {}
+
+    //Add the image's path to the Shortcut list
+#ifdef WIN32
+    // Dirty workaround, waiting for a clean solution by using exceptions!
+    if (!safe_is_root_dir(imagePath))
+#endif
+    try {
+        dialog.add_shortcut_folder(imagePath);
+    }
+    catch (Gtk::FileChooserError &err) {}
+
     //Add response buttons the the dialog:
     dialog.add_button(Gtk::StockID("gtk-cancel"), Gtk::RESPONSE_CANCEL);
     dialog.add_button(Gtk::StockID("gtk-open"), Gtk::RESPONSE_OK);
@@ -299,6 +341,9 @@ void ProfilePanel::load_clicked (GdkEventButton* event) {
             custom->set(true);
             customCreated = true;
         }
+        else if (fillMode->get_active())
+            custom->pparams->setDefaults();
+
         int err = custom->load (fname);
         if (!err) {
             bool prevState = changeconn.block(true);
@@ -313,9 +358,13 @@ void ProfilePanel::load_clicked (GdkEventButton* event) {
                 PartialProfile ppTemp(true);
                 // the 2 next line modify custom->pedited without modifying custom->pparams
                 partialProfileDlg->applyPaste (ppTemp.pparams, ppTemp.pedited, custom->pparams, custom->pedited);
+                if (fillMode->get_active())
+                    *custom->pparams = *ppTemp.pparams;
                 *custom->pedited = *ppTemp.pedited;
                 ppTemp.deleteInstance();
             }
+            if (fillMode->get_active())
+                custom->pedited->set(true);
 
             changeTo (custom, M("PROFILEPANEL_PFILE"));
         }
@@ -375,7 +424,7 @@ void ProfilePanel::paste_clicked (GdkEventButton* event) {
     return;
 }
 
-void ProfilePanel::changeTo (PartialProfile* newpp, Glib::ustring profname) {
+void ProfilePanel::changeTo (const PartialProfile* newpp, Glib::ustring profname) {
 
     if (!newpp)
         return;
@@ -394,9 +443,17 @@ void ProfilePanel::selection_changed () {
     else if (profiles->get_active_text() == (entry = Glib::ustring("(") + M("PROFILEPANEL_PLASTSAVED") + ")"))
             changeTo (lastsaved, entry);
     else {
-    	PartialProfile* s = profileStore.getProfile (profiles->get_active_text());
-        if (s)
-            changeTo (s, profiles->get_active_text());
+        const PartialProfile* s = profileStore.getProfile (profiles->get_active_text());
+        if (s) {
+            if (fillMode->get_active() && s->pedited) {
+                ParamsEdited pe;
+                pe.set(true);
+                PartialProfile s2(s->pparams, &pe, false);
+                changeTo (&s2, profiles->get_active_text()+"+");
+            }
+            else
+                changeTo (s, profiles->get_active_text());
+        }
     }
     old = profiles->get_active_text ();
     dontupdate = false;
@@ -449,7 +506,7 @@ void ProfilePanel::initProfile (const Glib::ustring& profname, ProcParams* lastS
     }
 
     Glib::ustring defline = profname;
-    PartialProfile* defprofile = profileStore.getProfile (profname);
+    const PartialProfile* defprofile = profileStore.getProfile (profname);
 
     if (lastsaved) {
         defline = Glib::ustring("(") + M("PROFILEPANEL_PLASTSAVED") + ")";
@@ -471,18 +528,17 @@ void ProfilePanel::initProfile (const Glib::ustring& profname, ProcParams* lastS
             tpc->profileChange (defprofile, EvPhotoLoaded, defline);  
     }
     else {
-        bool dels = false;
         // select first valid profile
         old = "";
         profiles->set_active (0);
-        PartialProfile* s = profileStore.getProfile (profiles->get_active_text());
+        const PartialProfile* s = profileStore.getProfile (profiles->get_active_text());
         if (!s) {
             changeconn.block (false);
-            s = new PartialProfile (true);
-            s->set(true);
-            dels = true;  // we've created a temporary PartialProfile, so we set a flag to destroy it
+            PartialProfile s2(true);
+            s2.pedited->set(true);
             if (tpc)
-                tpc->profileChange (s, EvPhotoLoaded, DEFPROFILE_INTERNAL);
+                tpc->profileChange (&s2, EvPhotoLoaded, DEFPROFILE_INTERNAL);
+            s2.deleteInstance();
         }
         else {
             Glib::ustring cProfile = profiles->get_active_text();
@@ -490,13 +546,26 @@ void ProfilePanel::initProfile (const Glib::ustring& profname, ProcParams* lastS
             if (tpc)
                 tpc->profileChange (s, EvPhotoLoaded, cProfile);
         }
-
-        if (dels) {
-            s->deleteInstance();
-            delete s;
-        }
     }
 }
 
+void ProfilePanel::setInitialFileName (const Glib::ustring& filename) {
+    lastFilename = Glib::path_get_basename(filename) + paramFileExtension;
+    imagePath = Glib::path_get_dirname(filename);
+}
 
+void ProfilePanel::profileFillModeToggled() {
+    if (fillMode->get_active()) {
+        // The button is pressed, we'll use the profileFillModeOnImage
+        fillMode->set_image(*profileFillModeOnImage);
+    }
+    else {
+        // The button is released, we'll use the profileFillModeOffImage
+        fillMode->set_image(*profileFillModeOffImage);
+    }
+}
+
+void ProfilePanel::writeOptions() {
+    options.filledProfile = fillMode->get_active();
+}
 
