@@ -26,9 +26,20 @@
 #include "profilechangelistener.h"
 #include "partialpastedlg.h"
 #include "guiutils.h"
+#include "rtimage.h"
 
 class ProfilePanel : public Gtk::VBox, public PParamsChangeListener {
 
+  private:
+
+    Glib::ustring lastFilename;
+    Glib::ustring imagePath;
+    RTImage *profileFillModeOnImage;
+    RTImage *profileFillModeOffImage;
+    Gtk::ToggleButton* fillMode;
+
+    void profileFillModeToggled();
+
   protected:
 
     static PartialPasteDlg* partialProfileDlg;
@@ -44,9 +55,8 @@ class ProfilePanel : public Gtk::VBox, public PParamsChangeListener {
     ProfileChangeListener* tpc;
     bool dontupdate;
     sigc::connection changeconn;
-    Gtk::FileChooserDialog* savedialog;
 
-    void changeTo (rtengine::procparams::PartialProfile* newpp, Glib::ustring profname);
+    void changeTo (const rtengine::procparams::PartialProfile* newpp, Glib::ustring profname);
     void refreshProfileList ();
 
   public:
@@ -60,6 +70,7 @@ class ProfilePanel : public Gtk::VBox, public PParamsChangeListener {
     static void cleanup ();
 
     void initProfile (const Glib::ustring& profname, rtengine::procparams::ProcParams* lastSaved);
+    void setInitialFileName (const Glib::ustring& filename);
 
     // PParamsChangeListener interface
     void procParamsChanged (rtengine::procparams::ProcParams* params, rtengine::ProcEvent ev, Glib::ustring descr, ParamsEdited* paramsEdited=NULL);
@@ -70,6 +81,7 @@ class ProfilePanel : public Gtk::VBox, public PParamsChangeListener {
     void copy_clicked (GdkEventButton* event);
     void paste_clicked (GdkEventButton* event);
     void selection_changed ();
+    void writeOptions();
 };
 
 #endif
