@@ -24,13 +24,40 @@ using namespace rtengine::procparams;
 
 RGBCurves::RGBCurves () : Gtk::VBox(), FoldableToolPanel(this) {
 
+	set_border_width(4);
+
+	lumamode = Gtk::manage (new Gtk::CheckButton (M("TP_RGBCURVES_LUMAMODE")));
+	lumamode->set_tooltip_markup (M("TP_RGBCURVES_LUMAMODE_TOOLTIP"));
+	lumamode->set_active (false);
+	lumamode->show ();
+	pack_start (*lumamode);
+
+	Gtk::HSeparator *hsep1 = Gtk::manage (new  Gtk::HSeparator());
+	hsep1->show ();
+	pack_start (*hsep1);
+
+	lumamodeConn = lumamode->signal_toggled().connect( sigc::mem_fun(*this, &RGBCurves::lumamodeChanged) );
+
+	std::vector<GradientMilestone> milestones;
+
 	curveEditorG = new CurveEditorGroup (options.lastRgbCurvesDir, M("TP_RGBCURVES_CHANNEL"));
 	curveEditorG->setCurveListener (this);
-	curveEditorG->setColorProvider (this);
 
 	Rshape = static_cast<DiagonalCurveEditor*>(curveEditorG->addCurve(CT_Diagonal, M("TP_RGBCURVES_RED")));
+	milestones.push_back( GradientMilestone(0.0, 0.0, 0.0, 0.0) );
+	milestones.push_back( GradientMilestone(1.0, 1.0, 0.0, 0.0) );
+	Rshape->setBottomBarBgGradient(milestones);
+	Rshape->setLeftBarBgGradient(milestones);
+
+	milestones[1].r = 0.0; milestones[1].g = 1.0;
 	Gshape = static_cast<DiagonalCurveEditor*>(curveEditorG->addCurve(CT_Diagonal, M("TP_RGBCURVES_GREEN")));
+	Gshape->setBottomBarBgGradient(milestones);
+	Gshape->setLeftBarBgGradient(milestones);
+
+	milestones[1].g = 0.0; milestones[1].b = 1.0;
 	Bshape = static_cast<DiagonalCurveEditor*>(curveEditorG->addCurve(CT_Diagonal, M("TP_RGBCURVES_BLUE")));
+	Bshape->setBottomBarBgGradient(milestones);
+	Bshape->setLeftBarBgGradient(milestones);
 
 	// This will add the reset button at the end of the curveType buttons
 	curveEditorG->curveListComplete();
@@ -51,18 +78,27 @@ void RGBCurves::read (const ProcParams* pp, const ParamsEdited* pedited) {
         Rshape->setUnChanged (!pedited->rgbCurves.rcurve);
 		Gshape->setUnChanged (!pedited->rgbCurves.gcurve);
         Bshape->setUnChanged (!pedited->rgbCurves.bcurve);
+        lumamode->set_inconsistent (!pedited->rgbCurves.lumamode);
     }
 
+    lumamodeConn.block (true);
+    lumamode->set_active (pp->rgbCurves.lumamode);
+    lumamodeConn.block (false);
+
+    lastLumamode = pp->rgbCurves.lumamode;
+
     Rshape->setCurve         (pp->rgbCurves.rcurve);
 	Gshape->setCurve         (pp->rgbCurves.gcurve);
     Bshape->setCurve         (pp->rgbCurves.bcurve);
 
+    enableListener ();
+}
+
+void RGBCurves::autoOpenCurve  () {
     // Open up the first curve if selected
     bool active = Rshape->openIfNonlinear();
     if (!active) Gshape->openIfNonlinear();
     if (!active) Bshape->openIfNonlinear();
-
-    enableListener ();
 }
 
 void RGBCurves::write (ProcParams* pp, ParamsEdited* pedited) {
@@ -70,11 +106,13 @@ void RGBCurves::write (ProcParams* pp, ParamsEdited* pedited) {
     pp->rgbCurves.rcurve         = Rshape->getCurve ();
 	pp->rgbCurves.gcurve         = Gshape->getCurve ();
     pp->rgbCurves.bcurve         = Bshape->getCurve ();
+    pp->rgbCurves.lumamode       = lumamode->get_active();
 
     if (pedited) {
         pedited->rgbCurves.rcurve    = !Rshape->isUnChanged ();
 		pedited->rgbCurves.gcurve    = !Gshape->isUnChanged ();
         pedited->rgbCurves.bcurve    = !Bshape->isUnChanged ();
+        pedited->rgbCurves.lumamode  = !lumamode->get_inconsistent();
     }
 }
 
@@ -97,32 +135,27 @@ void RGBCurves::curveChanged (CurveEditor* ce) {
 	}
 }
 
+void RGBCurves::lumamodeChanged () {
 
-//!!! TODO: change
-void RGBCurves::colorForValue (double valX, double valY) {
+    if (batchMode) {
+        if (lumamode->get_inconsistent()) {
+        	lumamode->set_inconsistent (false);
+            lumamodeConn.block (true);
+            lumamode->set_active (false);
+            lumamodeConn.block (false);
+        }
+        else if (lastLumamode)
+        	lumamode->set_inconsistent (true);
 
-	CurveEditor* ce = curveEditorG->getDisplayedCurve();
-
-	if (ce == Rshape) {         // L = f(L)
-		red = (double)valY;
-		green = (double)valY;
-		blue = (double)valY;
-	}
-	else if (ce == Gshape) {    // a = f(a)
-		// TODO: To be implemented
-		red = (double)valY;
-		green = (double)valY;
-		blue = (double)valY;
-	}
-	else if (ce == Bshape) {    // b = f(b)
-		red = (double)valY;
-		green = (double)valY;
-		blue = (double)valY;
-	}
-	else {
-		printf("Error: no curve displayed!\n");
-	}
+        lastLumamode = lumamode->get_active ();
+    }
 
+    if (listener) {
+        if (lumamode->get_active ())
+            listener->panelChanged (EvRGBrCurveLumamode, M("GENERAL_ENABLED"));
+        else
+            listener->panelChanged (EvRGBrCurveLumamode, M("GENERAL_DISABLED"));
+    }
 }
 
 void RGBCurves::setBatchMode (bool batchMode) {
@@ -132,10 +165,10 @@ void RGBCurves::setBatchMode (bool batchMode) {
 }
 
 
-void RGBCurves::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma) {
+void RGBCurves::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM,  LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma) {
 
-    //Rshape->updateBackgroundHistogram (histRed);
-    //Gshape->updateBackgroundHistogram (histGreen);
-    //Bshape->updateBackgroundHistogram (histBlue);
+  //  Rshape->updateBackgroundHistogram (histRed);
+  //  Gshape->updateBackgroundHistogram (histGreen);
+  //  Bshape->updateBackgroundHistogram (histBlue);
 }
 
