@@ -29,29 +29,35 @@ class SaveAsDialog : public Gtk::Dialog, public FormatChangeListener {
   protected:
     Gtk::FileChooserWidget* fchooser;
     Gtk::CheckButton* autoSuffix;
+    Gtk::CheckButton* forceFormatOpts;
     SaveFormatPanel* formatOpts;
     Glib::ustring fname;
     Gtk::FileFilter filter_jpg;
     Gtk::FileFilter filter_tif;
     Gtk::FileFilter filter_png;
-    Gtk::RadioButton* immediately;
-    Gtk::RadioButton* putToQueueHead;
-    Gtk::RadioButton* putToQueueTail;
-    Gtk::ResponseType response;
+    Gtk::RadioButton* saveMethod[3]; /*  0 -> immediately
+                                      *  1 -> putToQueueHead
+                                      *  2 -> putToQueueTail
+                                      */
+    void  forceFmtOptsSwitched ();
+    void  saveImmediatlyClicked ();
+    void  putToQueueClicked ();
 
   public:
     SaveAsDialog (Glib::ustring initialDir);
     
-    Glib::ustring   getFileName      ();
-    Glib::ustring   getDirectory     ();
-    SaveFormat      getFormat        ();
-    bool            getAutoSuffix    ();
-    bool            getImmediately   ();
-    bool            getToHeadOfQueue ();
-    bool            getToTailOfQueue ();    
+    Glib::ustring   getFileName        ();
+    Glib::ustring   getDirectory       ();
+    SaveFormat      getFormat          ();
+    bool            getForceFormatOpts ();
+    bool            getAutoSuffix      ();
+    bool            getImmediately     ();
+    bool            getToHeadOfQueue   ();
+    bool            getToTailOfQueue   ();
+    int             getSaveMethodNum   ();
 
     void  setInitialFileName (Glib::ustring iname);
-    Gtk::ResponseType getResponse    () { return response; };
+    void  setImagePath (Glib::ustring ipath);
 
     void okPressed ();
     void cancelPressed ();
