@@ -26,7 +26,15 @@ using namespace rtengine::procparams;
 
 Sharpening::Sharpening () : Gtk::VBox(), FoldableToolPanel(this) {
 
+   set_border_width(4);
+
+   std::vector<GradientMilestone> milestones;
+   milestones.push_back( GradientMilestone(0.0, 0.0, 0.0, 0.0) );
+   milestones.push_back( GradientMilestone(1.0, 1.0, 1.0, 1.0) );
+
    enabled = Gtk::manage (new Gtk::CheckButton (M("GENERAL_ENABLED")));
+   enabled->set_tooltip_markup (M("TP_SHARPENING_TOOLTIP"));
+
    enabled->set_active (true);
    pack_start(*enabled);
    enabled->show ();
@@ -67,9 +75,10 @@ Sharpening::Sharpening () : Gtk::VBox(), FoldableToolPanel(this) {
 
    
    Gtk::HSeparator *hsep6a = Gtk::manage (new  Gtk::HSeparator());
-   amount = Gtk::manage (new Adjuster (M("TP_SHARPENING_AMOUNT"), 1, 1000, 1, 150));
-   radius = Gtk::manage (new Adjuster (M("TP_SHARPENING_RADIUS"), 0.3, 3, 0.01, 0.8));
-   threshold = Gtk::manage (new Adjuster (M("TP_SHARPENING_THRESHOLD"), 0, 16384, 50, 1));
+   amount = Gtk::manage (new Adjuster (M("TP_SHARPENING_AMOUNT"), 1, 1000, 1, 125));
+   radius = Gtk::manage (new Adjuster (M("TP_SHARPENING_RADIUS"), 0.3, 3, 0.01, 0.5));
+   threshold = Gtk::manage (new ThresholdAdjuster (M("TP_SHARPENING_THRESHOLD"), 0., 2000., 20., 80., 2000., 1200., 0, false));
+   threshold->setBgGradient(milestones);
    pack_start(*hsep6a, Gtk::PACK_SHRINK, 2);
 
    pack_start (*usm);
@@ -163,12 +172,12 @@ void Sharpening::read (const ProcParams* pp, const ParamsEdited* pedited) {
 		dradius->setEditedState 	(pedited->sharpening.deconvradius ? Edited : UnEdited);
 		diter->setEditedState 		(pedited->sharpening.deconviter ? Edited : UnEdited);
 		ddamping->setEditedState 	(pedited->sharpening.deconvdamping ? Edited : UnEdited);
-		
-        enabled->set_inconsistent 		(!pedited->sharpening.enabled);
-        halocontrol->set_inconsistent	(!pedited->sharpening.halocontrol);
-        edgesonly->set_inconsistent		(!pedited->sharpening.edgesonly);
-    }
-	
+
+		enabled->set_inconsistent 		(multiImage && !pedited->sharpening.enabled);
+		halocontrol->set_inconsistent	(multiImage && !pedited->sharpening.halocontrol);
+		edgesonly->set_inconsistent		(multiImage && !pedited->sharpening.edgesonly);
+	}
+
     enaConn.block (true);
     enabled->set_active (pp->sharpening.enabled);
     enaConn.block (false);   
@@ -186,7 +195,7 @@ void Sharpening::read (const ProcParams* pp, const ParamsEdited* pedited) {
 
     amount->setValue        (pp->sharpening.amount);
     radius->setValue        (pp->sharpening.radius);
-    threshold->setValue     (pp->sharpening.threshold);
+    threshold->setValue<int>(pp->sharpening.threshold);
     eradius->setValue       (pp->sharpening.edges_radius);
     etolerance->setValue    (pp->sharpening.edges_tolerance);
     hcamount->setValue      (pp->sharpening.halocontrol_amount);
@@ -221,7 +230,7 @@ void Sharpening::write (ProcParams* pp, ParamsEdited* pedited) {
     pp->sharpening.amount           = (int)amount->getValue();
     pp->sharpening.enabled          = enabled->get_active ();
     pp->sharpening.radius           = radius->getValue ();
-    pp->sharpening.threshold        = (int)threshold->getValue ();
+    pp->sharpening.threshold        = threshold->getValue<int> ();
     pp->sharpening.edgesonly        = edgesonly->get_active ();
     pp->sharpening.edges_radius     = eradius->getValue ();
     pp->sharpening.edges_tolerance  = (int)etolerance->getValue ();
@@ -259,7 +268,7 @@ void Sharpening::setDefaults (const ProcParams* defParams, const ParamsEdited* p
 
     amount->setDefault (defParams->sharpening.amount);
     radius->setDefault (defParams->sharpening.radius);
-    threshold->setDefault (defParams->sharpening.threshold);
+    threshold->setDefault<int> (defParams->sharpening.threshold);
     eradius->setDefault (defParams->sharpening.edges_radius);
     etolerance->setDefault (defParams->sharpening.edges_tolerance);
     hcamount->setDefault (defParams->sharpening.halocontrol_amount);
@@ -296,7 +305,7 @@ void Sharpening::setDefaults (const ProcParams* defParams, const ParamsEdited* p
 
 void Sharpening::adjusterChanged (Adjuster* a, double newval) {
 
-    if (listener && enabled->get_active()) {
+    if (listener && (multiImage||enabled->get_active()) ) {
 
         Glib::ustring costr;
         if (a==radius || a==dradius)
@@ -310,8 +319,6 @@ void Sharpening::adjusterChanged (Adjuster* a, double newval) {
             listener->panelChanged (EvShrAmount, costr);
         else if (a==radius)
             listener->panelChanged (EvShrRadius, costr);
-        else if (a==threshold)
-            listener->panelChanged (EvShrThresh, costr);
         else if (a==eradius)
             listener->panelChanged (EvShrEdgeRadius, costr);
         else if (a==etolerance)
@@ -329,9 +336,16 @@ void Sharpening::adjusterChanged (Adjuster* a, double newval) {
     }
 }
 
+//void Sharpening::adjusterChanged (ThresholdAdjuster* a, int newBottomLeft, int newTopLeft, int newBottomRight, int newTopRight) {
+void Sharpening::adjusterChanged (ThresholdAdjuster* a, int newBottomLeft, int newTopLeft, int newBottomRight, int newTopRight) {
+    if (listener && (multiImage||enabled->get_active()) ) {
+        listener->panelChanged (EvShrThresh, threshold->getHistoryString());
+    }
+}
+
 void Sharpening::enabled_toggled () {
 
-    if (batchMode) {
+    if (multiImage) {
         if (enabled->get_inconsistent()) {
             enabled->set_inconsistent (false);
             enaConn.block (true);
@@ -345,7 +359,9 @@ void Sharpening::enabled_toggled () {
     }
 
     if (listener) {
-        if (enabled->get_active ())
+        if (enabled->get_inconsistent())
+            listener->panelChanged (EvShrEnabled, M("GENERAL_UNCHANGED"));
+        else if (enabled->get_active ())
             listener->panelChanged (EvShrEnabled, M("GENERAL_ENABLED"));
         else
             listener->panelChanged (EvShrEnabled, M("GENERAL_DISABLED"));
@@ -354,7 +370,7 @@ void Sharpening::enabled_toggled () {
 
 void Sharpening::edgesonly_toggled () {
 
-    if (batchMode) {
+    if (multiImage) {
         if (edgesonly->get_inconsistent()) {
             edgesonly->set_inconsistent (false);
             eonlyConn.block (true);
@@ -367,14 +383,16 @@ void Sharpening::edgesonly_toggled () {
         lastEdgesOnly = edgesonly->get_active ();
     }
 
-	if (!batchMode) {
-		removeIfThere (edgebin, edgebox, false);
-		if (edgesonly->get_active ())
-			edgebin->pack_start (*edgebox);
-	}
-	
-    if (listener && enabled->get_active()) {
+    if (!batchMode) {
+        removeIfThere (edgebin, edgebox, false);
         if (edgesonly->get_active ())
+            edgebin->pack_start (*edgebox);
+    }
+
+    if (listener && (multiImage||enabled->get_active()) ) {
+        if (edgesonly->get_inconsistent())
+            listener->panelChanged (EvShrEdgeOnly, M("GENERAL_INITIALVALUES"));
+        else if (edgesonly->get_active ())
             listener->panelChanged (EvShrEdgeOnly, M("GENERAL_ENABLED"));
         else
             listener->panelChanged (EvShrEdgeOnly, M("GENERAL_DISABLED"));
@@ -383,7 +401,7 @@ void Sharpening::edgesonly_toggled () {
 
 void Sharpening::halocontrol_toggled () {
 
-    if (batchMode) {
+    if (multiImage) {
         if (halocontrol->get_inconsistent()) {
             halocontrol->set_inconsistent (false);
             hcConn.block (true);
@@ -396,14 +414,16 @@ void Sharpening::halocontrol_toggled () {
         lastHaloControl = halocontrol->get_active ();
     }
 
-	if (!batchMode) {
-		removeIfThere (hcbin, hcbox, false);
-		if (halocontrol->get_active ())
-			hcbin->pack_start (*hcbox);
-	}
-
-    if (listener && enabled->get_active()) {
+    if (!batchMode) {
+        removeIfThere (hcbin, hcbox, false);
         if (halocontrol->get_active ())
+            hcbin->pack_start (*hcbox);
+    }
+
+    if (listener && (multiImage||enabled->get_active()) ) {
+        if (halocontrol->get_inconsistent())
+            listener->panelChanged (EvShrHaloControl, M("GENERAL_INITIALVALUES"));
+        else if (halocontrol->get_active ())
             listener->panelChanged (EvShrHaloControl, M("GENERAL_ENABLED"));
         else
             listener->panelChanged (EvShrHaloControl, M("GENERAL_DISABLED"));
@@ -419,8 +439,8 @@ void Sharpening::method_changed () {
         pack_start (*usm);   
     else if (method->get_active_row_number()==1) 
         pack_start (*rld);
-    
-    if (listener && enabled->get_active ())
+
+    if (listener && (multiImage||enabled->get_active()) )
         listener->panelChanged (EvShrMethod, method->get_active_text ());
     
 }
@@ -429,10 +449,10 @@ void Sharpening::setBatchMode (bool batchMode) {
 
     ToolPanel::setBatchMode (batchMode);
 
-	removeIfThere (hcbin, hcbox, false);
-	hcbin->pack_start (*hcbox);
-	removeIfThere (edgebin, edgebox, false);
-	edgebin->pack_start (*edgebox);
+    removeIfThere (hcbin, hcbox, false);
+    hcbin->pack_start (*hcbox);
+    removeIfThere (edgebin, edgebox, false);
+    edgebin->pack_start (*edgebox);
 
     radius->showEditedCB ();
     amount->showEditedCB ();
@@ -449,12 +469,12 @@ void Sharpening::setBatchMode (bool batchMode) {
 
 void Sharpening::setAdjusterBehavior (bool amountadd) {
 
-	amount->setAddMode(amountadd);
-	damount->setAddMode(amountadd);
+    amount->setAddMode(amountadd);
+    damount->setAddMode(amountadd);
 }
 
 void Sharpening::trimValues (rtengine::procparams::ProcParams* pp) {
 
-	amount->trimValue(pp->sharpening.amount);
-	damount->trimValue(pp->sharpening.deconvamount);
+    amount->trimValue(pp->sharpening.amount);
+    damount->trimValue(pp->sharpening.deconvamount);
 }
