@@ -95,23 +95,24 @@ void Thumbnail::_generateThumbnailImage () {
 	cfs.exifValid = false;
 	cfs.timeValid = false;
 
-	if (ext.lowercase()=="jpg") {
-			tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1, infoFromImage (fname));
-			if (tpp)
-					cfs.format = FT_Jpeg;
+	if (ext.lowercase()=="jpg" || ext.lowercase()=="jpeg") {
+		tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1, infoFromImage (fname));
+		if (tpp)
+			cfs.format = FT_Jpeg;
 	}
 	else if (ext.lowercase()=="png") {
-			tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1);
-			if (tpp)
-					cfs.format = FT_Png;
+		tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1);
+		if (tpp)
+			cfs.format = FT_Png;
 	}
 	else if (ext.lowercase()=="tif" || ext.lowercase()=="tiff") {
-			tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1, infoFromImage (fname));
-			if (tpp)
-					cfs.format = FT_Tiff;
+		int deg = infoFromImage (fname);
+		tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1, deg);
+		if (tpp)
+			cfs.format = FT_Tiff;
 	}
-    else {
- 		// RAW works like this:
+	else {
+		// RAW works like this:
 		//  1. if we are here it's because we aren't in the cache so load the JPG
 		//     image out of the RAW. Mark as "quick".
 		//  2. if we don't find that then just grab the real image.
@@ -232,7 +233,7 @@ void Thumbnail::loadProcParams () {
 
     pparamsValid = false;
     pparams.setDefaults();
-    PartialProfile *defaultPP = profileStore.getDefaultPartialProfile(getType()==FT_Raw);
+    const PartialProfile *defaultPP = profileStore.getDefaultPartialProfile(getType()==FT_Raw);
     defaultPP->applyTo(&pparams);
 
     if (options.paramsLoadLocation==PLL_Input) {
@@ -327,6 +328,11 @@ void Thumbnail::setProcParams (const ProcParams& pp, ParamsEdited* pe, int whoCh
 	Glib::Mutex::Lock lock(mutex);
 	#endif
     
+	if (pparams.sharpening.threshold.isDouble() != pp.sharpening.threshold.isDouble())
+		printf("WARNING: Sharpening different!\n");
+	if (pparams.vibrance.psthreshold.isDouble() != pp.vibrance.psthreshold.isDouble())
+		printf("WARNING: Vibrance different!\n");
+
     if (pparams!=pp) 
         cfs.recentlySaved = false;
 
@@ -336,10 +342,7 @@ void Thumbnail::setProcParams (const ProcParams& pp, ParamsEdited* pe, int whoCh
     int inTrash = getStage();
 
     if (pe) {
-    	// coarse.rotate works in ADD mode only, so we set it to 0 first
-    	if (pe->coarse.rotate)
-    		pparams.coarse.rotate = 0;
-    	pe->combine(pparams, pp, true);
+        pe->combine(pparams, pp, true);
     }
     else pparams = pp;
     pparamsValid = true;
@@ -406,19 +409,10 @@ void Thumbnail::decreaseRef ()
 }
 
 void Thumbnail::getThumbnailSize (int &w, int &h) {
-	// TODO: Check for Linux
-	#ifdef WIN32
-	Glib::Mutex::Lock lock(mutex);
-	#endif
-
-	w=0;
-	if (!initial_ && tpp) w = tpp->getImageWidth (getProcParams(), h, imgRatio);  // this might return 0 if image was just building
-	if (w==0) {
-		if (imgRatio > 0.)
-			w = (int)(imgRatio * (float)h);
-		else
-			w = tw * h / th;
-	}
+	if (imgRatio > 0.)
+		w = (int)(imgRatio * (float)h);
+	else
+		w = tw * h / th;
 }
 
 void Thumbnail::getFinalSize (const rtengine::procparams::ProcParams& pparams, int& w, int& h) {
@@ -469,9 +463,9 @@ rtengine::IImage8* Thumbnail::processThumbImage (const rtengine::procparams::Pro
 	else
 	{
 		// Full thumbnail: apply profile
- 		image = tpp->processImage (pparams, h, rtengine::TI_Bilinear, cfs.camera, cfs.focalLen, cfs.focalLen35mm, cfs.focusDist, scale );
+ 		image = tpp->processImage (pparams, h, rtengine::TI_Bilinear, cfs.camera, cfs.focalLen, cfs.focalLen35mm, cfs.focusDist, cfs.shutter, cfs.fnumber, cfs.iso, cfs.expcomp, scale );
 	}
- 
+
     tpp->getDimensions(lastW,lastH,lastScale);
 
  	delete tpp;
@@ -494,7 +488,7 @@ rtengine::IImage8* Thumbnail::upgradeThumbImage (const rtengine::procparams::Pro
  		return 0;
  	}
  
- 	rtengine::IImage8* image = tpp->processImage (pparams, h, rtengine::TI_Bilinear, cfs.camera, cfs.focalLen, cfs.focalLen35mm, cfs.focusDist, scale );
+ 	rtengine::IImage8* image = tpp->processImage (pparams, h, rtengine::TI_Bilinear, cfs.camera, cfs.focalLen, cfs.focalLen35mm, cfs.focusDist,cfs.shutter, cfs.fnumber, cfs.iso, cfs.expcomp,  scale );
     tpp->getDimensions(lastW,lastH,lastScale);
  
  	delete tpp;
@@ -514,7 +508,6 @@ void Thumbnail::generateExifDateTimeStrings () {
 
     if (options.fbShowExpComp && cfs.expcomp!="0.00" && cfs.expcomp!="") // don't show exposure compensation if it is 0.00EV;old cache iles do not have ExpComp, so value will not be displayed. 
     	exifString = Glib::ustring::compose ("%1 %2EV", exifString, cfs.expcomp); // append exposure compensation to exifString
-
     std::string dateFormat = options.dateFormat;
     std::ostringstream ostr;
     bool spec = false;
@@ -571,7 +564,7 @@ int Thumbnail::infoFromImage (const Glib::ustring& fname, rtengine::RawMetaDataL
     cfs.exifValid = false;
     if (idata->hasExif()) {
         cfs.shutter  = idata->getShutterSpeed ();
-        cfs.fnumber  = idata->getFNumber ();
+        cfs.fnumber  = idata->getFNumber ();		
         cfs.focalLen = idata->getFocalLen ();
         cfs.focalLen35mm = idata->getFocalLen35mm ();
         cfs.focusDist = idata->getFocusDist ();
@@ -598,16 +591,16 @@ int Thumbnail::infoFromImage (const Glib::ustring& fname, rtengine::RawMetaDataL
             deg = 270;
         }
     }
-	else {
+    else {
         cfs.lens     = "Unknown";
         cfs.camera   = "Unknown";
-	}
-		// get image filetype
+    }
+    // get image filetype
     std::string::size_type idx;
     idx = fname.rfind('.');
     if(idx != std::string::npos){cfs.filetype = fname.substr(idx+1);}
     else {cfs.filetype="";}
-	
+
     delete idata;
     return deg;
 }
@@ -615,8 +608,8 @@ int Thumbnail::infoFromImage (const Glib::ustring& fname, rtengine::RawMetaDataL
 void Thumbnail::_loadThumbnail(bool firstTrial) {
 
     needsReProcessing = true;
- 	tw = -1;
- 	th = options.maxThumbnailHeight;
+    tw = -1;
+    th = options.maxThumbnailHeight;
     delete tpp;
     tpp = new rtengine::Thumbnail ();
     tpp->isRaw = (cfs.format == (int) FT_Raw);
@@ -637,7 +630,7 @@ void Thumbnail::_loadThumbnail(bool firstTrial) {
     else if (!succ) {
         delete tpp;
         tpp = NULL;
-		return;
+        return;
     }
  
     if ( cfs.thumbImgType == CacheImageData::FULL_THUMBNAIL ) {
@@ -648,15 +641,14 @@ void Thumbnail::_loadThumbnail(bool firstTrial) {
         tpp->readEmbProfile (getCacheFileName ("embprofiles")+".icc");
 
         tpp->init ();
-        
     }
  
- 	getThumbnailSize(tw,th);
+    if (!initial_ && tpp) tw = tpp->getImageWidth (getProcParams(), th, imgRatio);  // this might return 0 if image was just building
 }
 
 void Thumbnail::loadThumbnail (bool firstTrial) {
-	Glib::Mutex::Lock lock(mutex);
-	_loadThumbnail(firstTrial);
+    Glib::Mutex::Lock lock(mutex);
+    _loadThumbnail(firstTrial);
 }
 
 void Thumbnail::_saveThumbnail () {
@@ -664,17 +656,15 @@ void Thumbnail::_saveThumbnail () {
     if (!tpp)
         return;
 
-    safe_g_remove (getCacheFileName ("images")+".cust");
-    safe_g_remove (getCacheFileName ("images")+".cust16");
-    safe_g_remove (getCacheFileName ("images")+".jpg");
+    if (safe_g_remove (getCacheFileName ("images")+".rtti") == -1) {
+        // No file deleted, so we try to deleted obsolete files, if any
+        safe_g_remove (getCacheFileName ("images")+".cust");
+        safe_g_remove (getCacheFileName ("images")+".cust16");
+        safe_g_remove (getCacheFileName ("images")+".jpg");
+    }
     
     // save thumbnail image
-    if (options.thumbnailFormat == FT_Custom) 
-        tpp->writeImage (getCacheFileName ("images")+".cust", 1);
-    else if (options.thumbnailFormat == FT_Custom16) 
-        tpp->writeImage (getCacheFileName ("images")+".cust16", 2);
-    else if (options.thumbnailFormat == FT_Jpeg) 
-        tpp->writeImage (getCacheFileName ("images")+".jpg", 3);
+    tpp->writeImage (getCacheFileName ("images"), 1);
 
     // save aehistogram
     tpp->writeAEHistogram (getCacheFileName ("aehistograms"));
@@ -689,7 +679,7 @@ void Thumbnail::_saveThumbnail () {
 void Thumbnail::saveThumbnail () 
 {
    	Glib::Mutex::Lock lock(mutex);
-	_saveThumbnail(); 
+    _saveThumbnail();
 }
 
 void Thumbnail::updateCache (bool updatePParams, bool updateCacheImageData) {
@@ -705,10 +695,10 @@ void Thumbnail::updateCache (bool updatePParams, bool updateCacheImageData) {
 }
 
 Thumbnail::~Thumbnail () {
-	// TODO: Check for Linux
-	#ifdef WIN32
-	Glib::Mutex::Lock lock(mutex);
-	#endif
+    // TODO: Check for Linux
+    #ifdef WIN32
+    Glib::Mutex::Lock lock(mutex);
+    #endif
 
     delete [] lastImg;
     delete tpp;
@@ -748,7 +738,7 @@ bool Thumbnail::openDefaultViewer(int destination) {
 
 #ifdef WIN32 
     Glib::ustring openFName;
-  
+
     if (destination==1) {
             openFName = Glib::ustring::compose ("%1.%2", BatchQueue::calcAutoFileNameBase(fname), options.saveFormatBatch.format);
             if (safe_file_test (openFName, Glib::FILE_TEST_EXISTS)) {
@@ -805,12 +795,12 @@ bool Thumbnail::openDefaultViewer(int destination) {
 
 bool Thumbnail::imageLoad(bool loading)
 {
-	Glib::Mutex::Lock lock(mutex);
-	bool previous = imageLoading;
-	if( loading && !previous ){
-		imageLoading = true;
-		return true;
-	}else if( !loading )
-		imageLoading = false;
+    Glib::Mutex::Lock lock(mutex);
+    bool previous = imageLoading;
+    if( loading && !previous ){
+        imageLoading = true;
+        return true;
+    }else if( !loading )
+        imageLoading = false;
     return false;
 }
