@@ -27,6 +27,14 @@ using namespace rtengine::procparams;
 
 ToneCurve::ToneCurve () : Gtk::VBox(), FoldableToolPanel(this) {
 
+  set_border_width(4);
+
+  CurveListener::setMulti(true);
+
+  std::vector<GradientMilestone> bottomMilestones;
+  bottomMilestones.push_back( GradientMilestone(0., 0., 0., 0.) );
+  bottomMilestones.push_back( GradientMilestone(1., 1., 1., 1.) );
+
 //----------- Auto Levels ----------------------------------
   abox = Gtk::manage (new Gtk::HBox ());
   abox->set_border_width (2);
@@ -82,20 +90,55 @@ ToneCurve::ToneCurve () : Gtk::VBox(), FoldableToolPanel(this) {
   saturation = Gtk::manage (new Adjuster (M("TP_EXPOSURE_SATURATION"), -100, 100, 1, 0));
   pack_start (*saturation);
   
-//----------- Curve ------------------------------
+//----------- Curve 1 ------------------------------
   pack_start (*Gtk::manage (new  Gtk::HSeparator()));
 
-  curveEditorG = new CurveEditorGroup (options.lastToneCurvesDir, M("TP_EXPOSURE_CURVEEDITOR"));
+  toneCurveMode = Gtk::manage (new MyComboBoxText ());
+  toneCurveMode->append_text (M("TP_EXPOSURE_TCMODE_STANDARD"));
+  toneCurveMode->append_text (M("TP_EXPOSURE_TCMODE_WEIGHTEDSTD"));
+  toneCurveMode->append_text (M("TP_EXPOSURE_TCMODE_FILMLIKE"));
+  toneCurveMode->append_text (M("TP_EXPOSURE_TCMODE_SATANDVALBLENDING"));
+  toneCurveMode->set_active (0);
+  toneCurveMode->set_tooltip_text(M("TP_EXPOSURE_TCMODE_LABEL1"));
+
+  curveEditorG = new CurveEditorGroup (options.lastToneCurvesDir, M("TP_EXPOSURE_CURVEEDITOR1"));
   curveEditorG->setCurveListener (this);
 
-  shape = static_cast<DiagonalCurveEditor*>(curveEditorG->addCurve(CT_Diagonal, ""));
+  shape = static_cast<DiagonalCurveEditor*>(curveEditorG->addCurve(CT_Diagonal, "", toneCurveMode));
+  shape->setBottomBarBgGradient(bottomMilestones);
+  shape->setLeftBarBgGradient(bottomMilestones);
 
   // This will add the reset button at the end of the curveType buttons
   curveEditorG->curveListComplete();
 
-  pack_start (*curveEditorG, Gtk::PACK_SHRINK, 4);
+  pack_start( *curveEditorG, Gtk::PACK_SHRINK, 2);
 
-  //curveEditorG->show();
+  tcmodeconn = toneCurveMode->signal_changed().connect( sigc::mem_fun(*this, &ToneCurve::curveMode1Changed), true );
+
+//----------- Curve 2 ------------------------------
+
+  toneCurveMode2 = Gtk::manage (new MyComboBoxText ());
+  toneCurveMode2->append_text (M("TP_EXPOSURE_TCMODE_STANDARD"));
+  toneCurveMode2->append_text (M("TP_EXPOSURE_TCMODE_WEIGHTEDSTD"));
+  toneCurveMode2->append_text (M("TP_EXPOSURE_TCMODE_FILMLIKE"));
+  toneCurveMode2->append_text (M("TP_EXPOSURE_TCMODE_SATANDVALBLENDING"));
+  toneCurveMode2->set_active (0);
+  toneCurveMode2->set_tooltip_text(M("TP_EXPOSURE_TCMODE_LABEL2"));
+
+  curveEditorG2 = new CurveEditorGroup (options.lastToneCurvesDir, M("TP_EXPOSURE_CURVEEDITOR2"));
+  curveEditorG2->setCurveListener (this);
+
+  shape2 = static_cast<DiagonalCurveEditor*>(curveEditorG2->addCurve(CT_Diagonal, "", toneCurveMode2));
+  shape2->setBottomBarBgGradient(bottomMilestones);
+  shape2->setLeftBarBgGradient(bottomMilestones);
+
+  // This will add the reset button at the end of the curveType buttons
+  curveEditorG2->curveListComplete();
+  curveEditorG2->setTooltip(M("TP_EXPOSURE_CURVEEDITOR2_TOOLTIP"));
+
+  pack_start( *curveEditorG2, Gtk::PACK_SHRINK, 2);
+
+  tcmode2conn = toneCurveMode2->signal_changed().connect( sigc::mem_fun(*this, &ToneCurve::curveMode2Changed), true );
 
 // --------- Set Up Listeners -------------
   expcomp->setAdjusterListener (this);
@@ -108,27 +151,20 @@ ToneCurve::ToneCurve () : Gtk::VBox(), FoldableToolPanel(this) {
   saturation->setAdjusterListener (this);
 }
 
+ToneCurve::~ToneCurve () {
+    delete curveEditorG;
+    delete curveEditorG2;
+}
+
 void ToneCurve::read (const ProcParams* pp, const ParamsEdited* pedited) {
 
     disableListener ();
 
-    if (pedited) {
-        expcomp->setEditedState (pedited->toneCurve.expcomp ? Edited : UnEdited);
-        black->setEditedState (pedited->toneCurve.black ? Edited : UnEdited);
-        hlcompr->setEditedState (pedited->toneCurve.hlcompr ? Edited : UnEdited);
-        hlcomprthresh->setEditedState (pedited->toneCurve.hlcomprthresh ? Edited : UnEdited);
-        shcompr->setEditedState (pedited->toneCurve.shcompr ? Edited : UnEdited);
-        brightness->setEditedState (pedited->toneCurve.brightness ? Edited : UnEdited);
-        contrast->setEditedState (pedited->toneCurve.contrast ? Edited : UnEdited);
-		saturation->setEditedState (pedited->toneCurve.saturation ? Edited : UnEdited);
-		autolevels->set_inconsistent (!pedited->toneCurve.autoexp);
-        clipDirty = pedited->toneCurve.clip;
-        shape->setUnChanged (!pedited->toneCurve.curve);
-    }
-
+    tcmodeconn.block(true);
+    tcmode2conn.block(true);
     autoconn.block (true);
+
     autolevels->set_active (pp->toneCurve.autoexp);
-    autoconn.block (false);
     lastAuto = pp->toneCurve.autoexp;
     sclip->set_value (pp->toneCurve.clip);
 
@@ -140,14 +176,46 @@ void ToneCurve::read (const ProcParams* pp, const ParamsEdited* pedited) {
     if (!black->getAddMode()) shcompr->set_sensitive(!((int)black->getValue ()==0)); //at black=0 shcompr value has no effect
     brightness->setValue (pp->toneCurve.brightness);
     contrast->setValue (pp->toneCurve.contrast);
-	saturation->setValue (pp->toneCurve.saturation);
-	shape->setCurve (pp->toneCurve.curve);
+    saturation->setValue (pp->toneCurve.saturation);
+    shape->setCurve (pp->toneCurve.curve);
+    shape2->setCurve (pp->toneCurve.curve2);
 
-    shape->openIfNonlinear();
+    toneCurveMode->set_active(pp->toneCurve.curveMode);
+    toneCurveMode2->set_active(pp->toneCurve.curveMode2);
+
+    if (pedited) {
+        expcomp->setEditedState (pedited->toneCurve.expcomp ? Edited : UnEdited);
+        black->setEditedState (pedited->toneCurve.black ? Edited : UnEdited);
+        hlcompr->setEditedState (pedited->toneCurve.hlcompr ? Edited : UnEdited);
+        hlcomprthresh->setEditedState (pedited->toneCurve.hlcomprthresh ? Edited : UnEdited);
+        shcompr->setEditedState (pedited->toneCurve.shcompr ? Edited : UnEdited);
+        brightness->setEditedState (pedited->toneCurve.brightness ? Edited : UnEdited);
+        contrast->setEditedState (pedited->toneCurve.contrast ? Edited : UnEdited);
+        saturation->setEditedState (pedited->toneCurve.saturation ? Edited : UnEdited);
+        autolevels->set_inconsistent (!pedited->toneCurve.autoexp);
+        clipDirty = pedited->toneCurve.clip;
+        shape->setUnChanged (!pedited->toneCurve.curve);
+        shape2->setUnChanged (!pedited->toneCurve.curve2);
+        if (!pedited->toneCurve.curveMode) {
+            toneCurveMode->set_active(4);
+        }
+        if (!pedited->toneCurve.curveMode2) {
+            toneCurveMode2->set_active(4);
+        }
+    }
+
+    autoconn.block (false);
+    tcmode2conn.block(false);
+    tcmodeconn.block(false);
 
     enableListener ();
 }
 
+void ToneCurve::autoOpenCurve  () {
+		shape->openIfNonlinear();
+		shape2->openIfNonlinear();
+}
+
 void ToneCurve::write (ProcParams* pp, ParamsEdited* pedited) {
 
     pp->toneCurve.autoexp = autolevels->get_active();
@@ -159,21 +227,38 @@ void ToneCurve::write (ProcParams* pp, ParamsEdited* pedited) {
     pp->toneCurve.shcompr = (int)shcompr->getValue ();
     pp->toneCurve.brightness = (int)brightness->getValue ();
     pp->toneCurve.contrast = (int)contrast->getValue ();
-	pp->toneCurve.saturation = (int)saturation->getValue ();
+    pp->toneCurve.saturation = (int)saturation->getValue ();
     pp->toneCurve.curve = shape->getCurve ();
+    pp->toneCurve.curve2 = shape2->getCurve ();
+
+    int tcMode = toneCurveMode->get_active_row_number();
+    if      (tcMode == 0) pp->toneCurve.curveMode = ToneCurveParams::TC_MODE_STD;
+    else if (tcMode == 1) pp->toneCurve.curveMode = ToneCurveParams::TC_MODE_WEIGHTEDSTD;
+    else if (tcMode == 2) pp->toneCurve.curveMode = ToneCurveParams::TC_MODE_FILMLIKE;
+    else if (tcMode == 3) pp->toneCurve.curveMode = ToneCurveParams::TC_MODE_SATANDVALBLENDING;
+
+    tcMode = toneCurveMode2->get_active_row_number();
+    if      (tcMode == 0) pp->toneCurve.curveMode2 = ToneCurveParams::TC_MODE_STD;
+    else if (tcMode == 1) pp->toneCurve.curveMode2 = ToneCurveParams::TC_MODE_WEIGHTEDSTD;
+    else if (tcMode == 2) pp->toneCurve.curveMode2 = ToneCurveParams::TC_MODE_FILMLIKE;
+    else if (tcMode == 3) pp->toneCurve.curveMode2 = ToneCurveParams::TC_MODE_SATANDVALBLENDING;
+
 
     if (pedited) {
-        pedited->toneCurve.expcomp = expcomp->getEditedState ();
-        pedited->toneCurve.black   = black->getEditedState ();
-        pedited->toneCurve.hlcompr = hlcompr->getEditedState ();
+        pedited->toneCurve.expcomp    = expcomp->getEditedState ();
+        pedited->toneCurve.black      = black->getEditedState ();
+        pedited->toneCurve.hlcompr    = hlcompr->getEditedState ();
         pedited->toneCurve.hlcomprthresh = hlcomprthresh->getEditedState ();
-        pedited->toneCurve.shcompr = shcompr->getEditedState ();
+        pedited->toneCurve.shcompr    = shcompr->getEditedState ();
         pedited->toneCurve.brightness = brightness->getEditedState ();
-        pedited->toneCurve.contrast = contrast->getEditedState ();
-		pedited->toneCurve.saturation = saturation->getEditedState ();
-		pedited->toneCurve.autoexp  = !autolevels->get_inconsistent();
-        pedited->toneCurve.clip     = clipDirty;
-        pedited->toneCurve.curve    = !shape->isUnChanged ();
+        pedited->toneCurve.contrast   = contrast->getEditedState ();
+        pedited->toneCurve.saturation = saturation->getEditedState ();
+        pedited->toneCurve.autoexp    = !autolevels->get_inconsistent();
+        pedited->toneCurve.clip       = clipDirty;
+        pedited->toneCurve.curve      = !shape->isUnChanged ();
+        pedited->toneCurve.curve2     = !shape2->isUnChanged ();
+        pedited->toneCurve.curveMode  = toneCurveMode->get_active_row_number() != 4;
+        pedited->toneCurve.curveMode2 = toneCurveMode2->get_active_row_number() != 4;
     }
 }
 
@@ -196,7 +281,7 @@ void ToneCurve::setDefaults (const ProcParams* defParams, const ParamsEdited* pe
         shcompr->setDefaultEditedState (pedited->toneCurve.shcompr ? Edited : UnEdited);
         brightness->setDefaultEditedState (pedited->toneCurve.brightness ? Edited : UnEdited);
         contrast->setDefaultEditedState (pedited->toneCurve.contrast ? Edited : UnEdited);
-		saturation->setDefaultEditedState (pedited->toneCurve.saturation ? Edited : UnEdited);
+        saturation->setDefaultEditedState (pedited->toneCurve.saturation ? Edited : UnEdited);
     }
     else {
         expcomp->setDefaultEditedState (Irrelevant);
@@ -206,20 +291,47 @@ void ToneCurve::setDefaults (const ProcParams* defParams, const ParamsEdited* pe
         shcompr->setDefaultEditedState (Irrelevant);
         brightness->setDefaultEditedState (Irrelevant);
         contrast->setDefaultEditedState (Irrelevant);
-		saturation->setDefaultEditedState (Irrelevant);
+        saturation->setDefaultEditedState (Irrelevant);
     }
 }
 
-void ToneCurve::curveChanged () {
+void ToneCurve::curveChanged (CurveEditor* ce) {
 
-    if (listener) listener->panelChanged (EvToneCurve, M("HISTORY_CUSTOMCURVE"));
+    if (listener) {
+        if (ce == shape)
+            listener->panelChanged (EvToneCurve1, M("HISTORY_CUSTOMCURVE"));
+        else if (ce == shape2)
+            listener->panelChanged (EvToneCurve2, M("HISTORY_CUSTOMCURVE"));
+    }
+}
+
+void ToneCurve::curveMode1Changed () {
+    //if (listener)  listener->panelChanged (EvToneCurveMode, toneCurveMode->get_active_text());
+    if (listener)  Glib::signal_idle().connect (sigc::mem_fun(*this, &ToneCurve::curveMode1Changed_));
+}
+
+bool ToneCurve::curveMode1Changed_ () {
+    if (listener) listener->panelChanged (EvToneCurveMode1, toneCurveMode->get_active_text());
+    return false;
+}
+
+void ToneCurve::curveMode2Changed () {
+    //if (listener)  listener->panelChanged (EvToneCurveMode, toneCurveMode->get_active_text());
+    if (listener)  Glib::signal_idle().connect (sigc::mem_fun(*this, &ToneCurve::curveMode2Changed_));
+}
+
+bool ToneCurve::curveMode2Changed_ () {
+    if (listener) listener->panelChanged (EvToneCurveMode2, toneCurveMode2->get_active_text());
+    return false;
 }
 
 void ToneCurve::adjusterChanged (Adjuster* a, double newval) {
 
     // Switch off auto exposure if user changes sliders manually
     if (autolevels->get_active() && (a==expcomp || a==brightness || a==contrast || a==black || a==hlcompr || a==hlcomprthresh)) {
+        autoconn.block(true);
         autolevels->set_active (false);
+        autoconn.block(false);
         autolevels->set_inconsistent (false);
     }
 
@@ -274,7 +386,7 @@ void ToneCurve::neutral_pressed () {
     hlcomprthresh->setValue(0);
     brightness->setValue(0);
     black->setValue(0);
-    shcompr->setValue(0);
+    shcompr->setValue(50);
     if (!black->getAddMode()) shcompr->set_sensitive(!((int)black->getValue ()==0)); //at black=0 shcompr value has no effect
     contrast->setValue(0);
     //saturation->setValue(0);
@@ -294,35 +406,45 @@ void ToneCurve::autolevels_toggled () {
             autolevels->set_inconsistent (true);
 
         lastAuto = autolevels->get_active ();
-    }
 
-    if (!batchMode && autolevels->get_active() && listener) {
-        listener->panelChanged (EvAutoExp, M("GENERAL_ENABLED"));
-        waitForAutoExp ();
-        if (!black->getAddMode()) shcompr->set_sensitive(!((int)black->getValue ()==0)); //at black=0 shcompr value has no effect
-    } 
-    
-    if (batchMode) {
         expcomp->setEditedState (UnEdited);
-		brightness->setEditedState (UnEdited);
-		contrast->setEditedState (UnEdited);
+        brightness->setEditedState (UnEdited);
+        contrast->setEditedState (UnEdited);
         black->setEditedState (UnEdited);
         hlcompr->setEditedState (UnEdited);
         hlcomprthresh->setEditedState (UnEdited);
         if (expcomp->getAddMode())
             expcomp->setValue (0);
-		if (brightness->getAddMode())
+        if (brightness->getAddMode())
             brightness->setValue (0);
-		if (contrast->getAddMode())
+        if (contrast->getAddMode())
             contrast->setValue (0);
-		if (black->getAddMode())
+        if (black->getAddMode())
             black->setValue (0);
         if (hlcompr->getAddMode())
-        	hlcompr->setValue (0);
+            hlcompr->setValue (0);
         if (hlcomprthresh->getAddMode())
-        	hlcomprthresh->setValue (0);
-        listener->panelChanged (EvAutoExp, M("GENERAL_ENABLED"));
+            hlcomprthresh->setValue (0);
+        if (listener) {
+            if (!autolevels->get_inconsistent()) {
+                if (autolevels->get_active ())
+                    listener->panelChanged (EvAutoExp, M("GENERAL_ENABLED"));
+                else
+                    listener->panelChanged (EvFixedExp, M("GENERAL_DISABLED"));
+            }
+        }
     }
+    else if (/* !batchMode && */ listener) {
+        if (autolevels->get_active()) {
+            listener->panelChanged (EvAutoExp, M("GENERAL_ENABLED"));
+            waitForAutoExp ();
+            if (!black->getAddMode()) shcompr->set_sensitive(!((int)black->getValue ()==0)); //at black=0 shcompr value has no effect
+        }
+        else {
+            listener->panelChanged (EvFixedExp, M("GENERAL_DISABLED"));
+        }
+    }
+
 }
 
 void ToneCurve::clip_changed () {
@@ -347,14 +469,15 @@ void ToneCurve::waitForAutoExp () {
     sclip->set_sensitive (false);
     expcomp->setEnabled (false);
     brightness->setEnabled (false);
-	contrast->setEnabled (false);
+    contrast->setEnabled (false);
     black->setEnabled (false);
     hlcompr->setEnabled (false);
     hlcomprthresh->setEnabled (false);
     shcompr->setEnabled (false);
     contrast->setEnabled (false);
-	saturation->setEnabled (false);
+    saturation->setEnabled (false);
     curveEditorG->set_sensitive (false);
+    toneCurveMode->set_sensitive (false);
 }
 
 int autoExpChangedUI (void* data) {
@@ -366,8 +489,8 @@ void ToneCurve::autoExpChanged (double expcomp, int bright, int contr, int black
 
     nextBlack = black;
     nextExpcomp = expcomp;
-	nextBrightness = bright;
-	nextContrast = contr;
+    nextBrightness = bright;
+    nextContrast = contr;
     nextHlcompr = hlcompr;
     nextHlcomprthresh = hlcomprthresh;
     g_idle_add (autoExpChangedUI, this);
@@ -385,8 +508,9 @@ void ToneCurve::enableAll () {
     hlcomprthresh->setEnabled (true);
     shcompr->setEnabled (true);
     contrast->setEnabled (true);
-	saturation->setEnabled (true);
+    saturation->setEnabled (true);
     curveEditorG->set_sensitive (true);
+    toneCurveMode->set_sensitive (true);
 }
 
 bool ToneCurve::autoExpComputed_ () {
@@ -394,8 +518,8 @@ bool ToneCurve::autoExpComputed_ () {
     disableListener ();
     enableAll ();
     expcomp->setValue (nextExpcomp);
-	brightness->setValue (nextBrightness);
-	contrast->setValue (nextContrast);
+    brightness->setValue (nextBrightness);
+    contrast->setValue (nextContrast);
     black->setValue (nextBlack);
     hlcompr->setValue (nextHlcompr);
     hlcomprthresh->setValue (nextHlcomprthresh);
@@ -423,6 +547,8 @@ void ToneCurve::setBatchMode (bool batchMode) {
     contrast->showEditedCB ();
 	saturation->showEditedCB ();
 
+	toneCurveMode->append_text (M("GENERAL_UNCHANGED"));
+
     curveEditorG->setBatchMode (batchMode);
 }
 
@@ -450,7 +576,7 @@ void ToneCurve::trimValues (rtengine::procparams::ProcParams* pp) {
 	saturation->trimValue(pp->toneCurve.saturation);
 }
 
-void ToneCurve::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma) {
+void ToneCurve::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM, LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma) {
 
     shape->updateBackgroundHistogram (histToneCurve);
 }
