@@ -41,17 +41,22 @@ class ToneCurve : public Gtk::VBox, public AdjusterListener, public FoldableTool
     Adjuster* hlcomprthresh;
     Adjuster* shcompr;
     Adjuster* contrast;
-	Adjuster* saturation;
+    Adjuster* saturation;
+    MyComboBoxText* toneCurveMode;
+    MyComboBoxText* toneCurveMode2;
+
 
     bool clipDirty, lastAuto;
-    sigc::connection autoconn, neutralconn;
+    sigc::connection autoconn, neutralconn, tcmodeconn, tcmode2conn;
     CurveEditorGroup* curveEditorG;
+    CurveEditorGroup* curveEditorG2;
     DiagonalCurveEditor* shape;
+    DiagonalCurveEditor* shape2;
 
     // used temporarily in eventing
     double nextExpcomp;
-	int nextBrightness;
-	int nextContrast;
+    int nextBrightness;
+    int nextContrast;
     int nextBlack;
     int nextHlcompr;
     int nextHlcomprthresh;
@@ -59,13 +64,15 @@ class ToneCurve : public Gtk::VBox, public AdjusterListener, public FoldableTool
   public:
 
     ToneCurve ();
+    ~ToneCurve ();
 
     void read                (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited=NULL);
     void write               (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited=NULL);
     void setDefaults         (const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited=NULL);
     void setBatchMode        (bool batchMode);
     void setAdjusterBehavior (bool expadd, bool hlcompadd, bool hlcompthreshadd, bool bradd, bool blackadd, bool shcompadd, bool contradd, bool satadd);
-    void trimValues           (rtengine::procparams::ProcParams* pp);
+    void trimValues          (rtengine::procparams::ProcParams* pp);
+    void autoOpenCurve       ();
 
 
     void adjusterChanged (Adjuster* a, double newval);
@@ -77,10 +84,14 @@ class ToneCurve : public Gtk::VBox, public AdjusterListener, public FoldableTool
     void autoExpChanged (double expcomp, int bright, int contr, int black, int hlcompr, int hlcomprthresh);
     bool autoExpComputed_ ();
     void enableAll ();
-    void curveChanged ();
+    void curveChanged (CurveEditor* ce);
+    void curveMode1Changed ();
+    bool curveMode1Changed_ ();
+    void curveMode2Changed ();
+    bool curveMode2Changed_ ();
     void expandCurve (bool isExpanded);
     bool isCurveExpanded ();
-    void updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma);
+    void updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM, LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma);
 };
 
 #endif
