@@ -33,6 +33,7 @@ class ToolPanelListener {
 
     public:
     
+        virtual ~ToolPanelListener() {}
         virtual void panelChanged   (rtengine::ProcEvent event, const Glib::ustring& descr) {}
 };
 
@@ -41,19 +42,23 @@ class ToolPanel {
     protected:
         ToolPanelListener* listener;
         ToolPanelListener* tmp;
-        bool batchMode;
+        bool batchMode;  // True if the ToolPanel is used in Batch mode
+        bool multiImage; // True if more than one image are being edited at the same time (also imply that batchMode=true), false otherwise
 
     public:
 
-        ToolPanel () : listener(NULL), tmp(NULL), batchMode(false) {}
+        ToolPanel () : listener(NULL), tmp(NULL), batchMode(false), multiImage(false) {}
+        virtual ~ToolPanel() {}
 
-                void setParent   (Gtk::Box* parent) {}
-           Gtk::Box* getParent   () { return NULL; }
-                void setListener (ToolPanelListener* tpl) { listener = tpl; }
-        virtual void read        (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited=NULL) {}
-        virtual void write       (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited=NULL) {}
-        virtual void trimValues  (rtengine::procparams::ProcParams* pp) { return; }
-        virtual void setDefaults (const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited=NULL) {}
+                void setParent       (Gtk::Box* parent) {}
+           Gtk::Box* getParent       () { return NULL; }
+                void setMultiImage   (bool m) { multiImage = m; }
+                void setListener     (ToolPanelListener* tpl) { listener = tpl; }
+        virtual void read            (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited=NULL) {}
+        virtual void write           (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited=NULL) {}
+        virtual void trimValues      (rtengine::procparams::ProcParams* pp) { return; }
+        virtual void setDefaults     (const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited=NULL) {}
+        virtual void autoOpenCurve   () {}
         
                 void disableListener () { if (tmp==NULL) tmp = listener; listener = NULL; }
                 void enableListener  () { if (tmp!=NULL) listener = tmp; tmp = NULL; }
@@ -68,10 +73,10 @@ class FoldableToolPanel : public ToolPanel {
         Gtk::Box* parentContainer;
         void foldThemAll (GdkEventButton* event);
 
-	public:
-		Gtk::Expander* exp;
+    public:
+        Gtk::Expander* exp;
 
-		FoldableToolPanel(Gtk::Box* content);
+        FoldableToolPanel(Gtk::Box* content);
 
         void setParent (Gtk::Box* parent) { parentContainer = parent; }
         Gtk::Box* getParent () { return parentContainer; }
