@@ -56,6 +56,7 @@ ToolPanelCoordinator::ToolPanelCoordinator () : ipc(NULL)  {
     distortion          = Gtk::manage (new Distortion ());
     rotate              = Gtk::manage (new Rotate ());
     vibrance            = Gtk::manage (new Vibrance ());
+    colorappearance     = Gtk::manage (new ColorAppearance ());
     whitebalance        = Gtk::manage (new WhiteBalance ());
     vignetting          = Gtk::manage (new Vignetting ());
     perspective         = Gtk::manage (new PerspCorrection ());
@@ -89,6 +90,8 @@ ToolPanelCoordinator::ToolPanelCoordinator () : ipc(NULL)  {
     addPanel (colorPanel, rgbcurves,            M("TP_RGBCURVES_LABEL"));      toolPanels.push_back (rgbcurves);
     addPanel (exposurePanel, edgePreservingDecompositionUI,      M("TP_EPD_LABEL"));  toolPanels.push_back (edgePreservingDecompositionUI);
     addPanel (exposurePanel, lcurve,            M("TP_LABCURVE_LABEL"));	   toolPanels.push_back (lcurve);
+//    addPanel (exposurePanel, edgePreservingDecompositionUI,      M("TP_EPD_LABEL"));  toolPanels.push_back (edgePreservingDecompositionUI);
+    addPanel (exposurePanel, colorappearance,   M("TP_COLORAPP_LABEL"));       toolPanels.push_back (colorappearance);
     addPanel (detailsPanel, impulsedenoise,     M("TP_IMPULSEDENOISE_LABEL")); toolPanels.push_back (impulsedenoise);
     addPanel (detailsPanel, dirpyrdenoise,      M("TP_DIRPYRDENOISE_LABEL"));  toolPanels.push_back (dirpyrdenoise);
     addPanel (detailsPanel, defringe,           M("TP_DEFRINGE_LABEL"));	   toolPanels.push_back (defringe);
@@ -311,8 +314,11 @@ void ToolPanelCoordinator::profileChange  (const PartialProfile *nparams, rtengi
     crop->trim(params, fw, fh);
 
     // updating the GUI with updated values
-    for (unsigned int i=0; i<toolPanels.size(); i++)
+    for (unsigned int i=0; i<toolPanels.size(); i++) {
         toolPanels[i]->read (params);
+        if (event==rtengine::EvPhotoLoaded || event==rtengine::EvProfileChanged)
+            toolPanels[i]->autoOpenCurve();
+    }
 
     // start the IPC processing
     if (filterRawRefresh) {
@@ -351,6 +357,8 @@ void ToolPanelCoordinator::initImage (rtengine::StagedImageProcessor* ipc_, bool
 
     if (ipc) {
         ipc->setAutoExpListener (toneCurve);
+        ipc->setAutoCamListener (colorappearance);
+		
         ipc->setSizeListener (crop);
         ipc->setSizeListener (resize);
     }
@@ -376,7 +384,7 @@ void ToolPanelCoordinator::readOptions () {
     crop->readOptions (); 
     for (size_t i=0; i<options.tpOpen.size(); i++)
         if (i<expList.size())
-            expList[i]->set_expanded (options.tpOpen[i]);
+            expList.at(i)->set_expanded (options.tpOpen.at(i));
 }
 
 void ToolPanelCoordinator::writeOptions () { 
@@ -384,7 +392,7 @@ void ToolPanelCoordinator::writeOptions () {
     crop->writeOptions (); 
     options.tpOpen.clear ();
     for (size_t i=0; i<expList.size(); i++)
-        options.tpOpen.push_back (expList[i]->get_expanded ());
+        options.tpOpen.push_back (expList.at(i)->get_expanded ());
 }
 
 
@@ -512,11 +520,11 @@ int ToolPanelCoordinator::getSpotWBRectSize () {
     return whitebalance->getSize ();
 }
 
-void ToolPanelCoordinator::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma) {
-
-    toneCurve->updateCurveBackgroundHistogram (histToneCurve, histLCurve, histRed, histGreen, histBlue, histLuma);
-    rgbcurves->updateCurveBackgroundHistogram(histToneCurve, histLCurve,histRed,histGreen, histBlue, histLuma);
-    lcurve->updateCurveBackgroundHistogram (histToneCurve, histLCurve, histRed, histGreen, histBlue, histLuma);
+void ToolPanelCoordinator::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM, LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma) {
+    colorappearance->updateCurveBackgroundHistogram (histToneCurve, histLCurve, histCCurve, histLCAM,  histCCAM,histRed, histGreen, histBlue, histLuma);
+    toneCurve->updateCurveBackgroundHistogram (histToneCurve, histLCurve,histCCurve, histLCAM,  histCCAM,histRed, histGreen, histBlue, histLuma);
+    lcurve->updateCurveBackgroundHistogram (histToneCurve, histLCurve, histCCurve, histLCAM, histCCAM,histRed, histGreen, histBlue, histLuma); 
+	rgbcurves->updateCurveBackgroundHistogram(histToneCurve, histLCurve, histCCurve, histLCAM, histCCAM,histRed,histGreen, histBlue, histLuma);
 }
 
 void ToolPanelCoordinator::foldAllButOne (Gtk::Box* parent, FoldableToolPanel* openedSection) {
