@@ -30,6 +30,7 @@
 #include "coarsepanel.h"
 #include "tonecurve.h"
 #include "vibrance.h"
+#include "colorappearance.h"
 #include "shadowshighlights.h"
 #include "impulsedenoise.h"
 #include "defringe.h"
@@ -90,6 +91,7 @@ class ToolPanelCoordinator :    public ToolPanelListener,
         PerspCorrection* perspective;
         CACorrection* cacorrection;
         HLRecovery* hlrecovery;
+        ColorAppearance* colorappearance;
         Vibrance* vibrance;
         ChMixer* chmixer;
         Resize* resize;
@@ -177,7 +179,7 @@ class ToolPanelCoordinator :    public ToolPanelListener,
         virtual ~ToolPanelCoordinator ();
 
         bool getChangedState                ()                                      { return hasChanged; }
-		void updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma);
+		void updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve,LUTu & histCCurve, LUTu & histLCAM,  LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma);
         void foldAllButOne (Gtk::Box* parent, FoldableToolPanel* openedSection);
 
         // multiple listeners can be added that are notified on changes (typical: profile panel and the history)
@@ -230,6 +232,7 @@ class ToolPanelCoordinator :    public ToolPanelListener,
         void cropSelectionReady ();
         void rotateSelectionReady (double rotate_deg, Thumbnail* thm=NULL);
         ToolBar* getToolBar () { return toolBar; }
+        void removeWbTool() { if (toolBar) toolBar->removeWbTool(); }
         int  getSpotWBRectSize ();
         CropGUIListener* startCropEditing (Thumbnail* thm=NULL) { return crop; }
 
